{******************************************}
{                                          }
{             FastReport VCL               }
{            Base Memo control             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frBaseMemo;

interface

{$I frVer.inc}

uses
  {$IFNDEF FPC}
  Windows, Imm,
  {$ELSE}
  LCLType, LMessages, LCLIntf, LCLProc,
  {$ENDIF}
  Messages,
  Types, SysUtils, Classes, Graphics, Controls, StdCtrls,ExtCtrls, Forms, Menus, IniFiles,
  frPopupForm, frTypes, frVectorCanvas, frUnicodeUtils, frScrollWin, frDialectHelper
  {$IFDEF Delphi10}
  , WideStrings
  {$ENDIF}
  ;

type
  TDiffFunc = function(s1, s2: String): Integer;
{$IFDEF FPC}
  TMWSetFocus = TLMSetFocus;
{$ENDIF}

  TfrBaseSyntaxMemo = class(TfrScrollWin)
  protected
    {$IFDEF NONWINFPC}
    FCaretCreated: Boolean;
    {$ENDIF}
    FSynDialectStyles: TfrSynDialectStyles;
    FActiveLine: Integer;
    FAllowLinesChange: Boolean;
    FBlockColor: TColor;
    FBlockFontColor: TColor;
    FCommentAttr: TFont;
    FCompletionForm: TfrPopupForm;
    FCompletionLB: TListBox;
    FDoubleClicked: Boolean;
    FDown: Boolean;
    FGutterWidth: Integer;
    FIsMonoType: Boolean;
    FKeywordAttr: TFont;
    FMessage: String;
    FModified: Boolean;
    FMoved: Boolean;
    FNumberAttr: TFont;
    FOffset: TPoint;
    FOnChangePos: TNotifyEvent;
    FOnChangeText: TNotifyEvent;
    FPos: TPoint;
    FStringAttr: TFont;
    FSelEnd: TPoint;
    FSelStart: TPoint;
    FTabStops: Integer;
{$IFNDEF FPC}
    FCompSelStart: TPoint;
    FCompSelEnd: TPoint;
{$ENDIF}
    FShowGutter: boolean;
    FTempPos, FLastHintPos: TPoint;
    FText: TStringList;
    FTextAttr: TFont;
    FUndo: TStringList;
    FWindowSize: TPoint;
    FCodeCompList: TStringList;
    FStartCodeCompPos: Integer;
    FCompleationFilter: String;
    FShowLineNumber: Boolean;
    FCodeCompletionWidth: Integer;
    FCodeCompletionHeight: Integer;
    FCodeCompletionMinHeight: Integer;
    FCodeCompletionMinWidth: Integer;
    FRTLReading: Boolean;

    procedure FilterCodeListBox(aFilterStr: String; aListBox: TListBox;
      aCodeList: TStrings; aDiffFunc: TDiffFunc);
    procedure FilterCodeStringList(aFilterStr: String; aLBList: TStrings;
      aCodeList: TStrings; aDiffFunc: TDiffFunc); virtual; abstract;
    function GetLineBegin(Index: Integer): Integer;
    function GetPlainTextPos(Pos: TPoint): Integer;
    function GetPosPlainText(Pos: Integer): TPoint;
    function GetRunLine(Index: Integer): Boolean;
    function GetSelText: String; virtual;
    function GetText: TStrings;
    function LineAt(Index: Integer; UseTrim: Boolean = True): String; virtual;
    function GetIdentEnd(aPos: Integer): Integer; virtual;
    function LineLength(Index: Integer): Integer;
    function Pad(n: Integer): String;
    procedure AddSel;
    procedure AddUndo;
    procedure ClearSel;
    procedure ClearSyntax(ClearFrom: Integer); virtual;
    procedure CompletionFormClose(Sender: TObject; var Action: TCloseAction);
    procedure CompletionLBDblClick(Sender: TObject); virtual;
    procedure CompletionLBDrawItem(Control: TWinControl; Index: Integer;
      ARect: TRect; State: TOwnerDrawState); virtual;
    procedure CompletionLBKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure CompletionLBKeyPress(Sender: TObject; var Key: Char);
    procedure CorrectBookmark(Line, Delta: Integer); virtual;
    procedure DoBackspace;
    procedure DoChange; virtual;
    procedure DoChar(Ch: Char);
    procedure DoCodeCompletion; virtual;
    procedure DoCtrlI;
    procedure DoCtrlU;
    procedure DoCtrlR;
    procedure DoCtrlL;
    procedure DoDel;
    procedure DoDown; virtual;
    procedure DoEnd(Ctrl: Boolean);
    procedure DoHome(Ctrl: Boolean);
    procedure DoLeft; virtual;
    procedure DoPgUp;
    procedure DoPgDn;
    procedure DoReturn; virtual;
    procedure DoRight; virtual;
    procedure DoUp; virtual;
    procedure EnterIndent;
    procedure LinesChange(Sender: TObject);
    procedure SetActiveLine(Line: Integer);
    procedure SetCommentAttr(Value: TFont);
    procedure SetKeywordAttr(Value: TFont);
    procedure SetNumberAttr(const Value: TFont);
    procedure SetRunLine(Index: Integer; const Value: Boolean);
    procedure SetSelText(const Value: String); virtual;
    procedure SetShowGutter(Value: Boolean);
    procedure SetStringAttr(Value: TFont);
    procedure SetText(Value: TStrings);
    procedure SetTextAttr(Value: TFont);
    procedure ShiftSelected(ShiftRight: Boolean); virtual;
    procedure ShowCaretPos; virtual;
    procedure TabIndent;
    procedure UnIndent;
    procedure UpdateScrollBar; virtual;
    procedure EventFontChanged(var Message: TMessage); virtual;
    procedure EventSetFocus(var Msg: TWMSetFocus); virtual;
    procedure WMKillFocus(var Msg: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMSetFocus(var Msg: TWMSetFocus); message WM_SETFOCUS;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    { Inline IME realisation }
	{$IFNDEF FPC}
    procedure WMIMEStartComp(var Message: TMessage); message WM_IME_STARTCOMPOSITION;
    procedure WMIMEEndComp(var Message: TMessage); message WM_IME_ENDCOMPOSITION;
    procedure WMIMECOMPOSITION (var Message: TMessage); message WM_IME_COMPOSITION ;
	{$ENDIF}
    function OffsPoint(Point: TPoint): TPoint;
    function CurPosToSynPos(X, Y: Integer): TPoint; virtual;
    function GetTextSelected: Boolean;
    procedure SetGutterWidth(const Value: Integer);
    procedure SetCodeCompletionWidth(const Value: Integer);
    procedure SetCodeCompletionHeight(const Value: Integer);
    procedure SetCodeCompletionMinHeight(const Value: Integer);
    procedure SetCodeCompletionMinWidth(const Value: Integer);
    procedure PopCopy(Sender: TObject);
    procedure PopCut(Sender: TObject);
    procedure PopPaste(Sender: TObject);
    procedure PopDelete(Sender: TObject);
    procedure PopSelectAll(Sender: TObject);
    procedure QuickComment; virtual;
    procedure DblClick; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    {$IFDEF FPC}
    procedure UTF8KeyPress(var UTF8Key: TUTF8Char); override;
    {$ENDIF}
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean); virtual;
    procedure MouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean); virtual;
    procedure OnHScrollChange(Sender: TObject); override;
    procedure OnVScrollChange(Sender: TObject); override;
    procedure Resize; override;
    function GetCompletionString(Pos: TPoint): String;
    function GetFilter(aStr: String): String;
    procedure UpdateSyntaxType; virtual;
    procedure DoPPIChanged(aNewPPI: Integer); override;
    function GetIni(const IniPath: String): TCustomIniFile;
    procedure PreDrawGutter;
    procedure CheckSurrogatePairs(var x: Integer; var y: Integer; ToRight: Boolean = True);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure CopyToClipboard;
    procedure CutToClipboard;
    procedure CompletionClose; virtual;
    procedure PasteFromClipboard;
    procedure SelectAll;
    procedure SetPos(x, y: Integer); overload;
    procedure SetPos(x, y: Integer; ToRight: Boolean); overload; virtual;
    procedure SetPosByCoords(X, Y: Integer); virtual;
    procedure ShowMessage(const s: String);
    procedure Undo;
    procedure UpdateView;
    function Find(const SearchText: String; CaseSensitive: Boolean;
      var SearchFrom: Integer): Boolean;
    function GetPlainPos: Integer;
    function GetPos: TPoint;
    function IsBookmark(Line: Integer): Integer; virtual;
    procedure AddBookmark(Line, Number: Integer); virtual;
    procedure DeleteBookmark(Number: Integer); virtual;
    procedure GotoBookmark(Number: Integer); virtual;
    procedure SaveToIni(const IniPath: String; const Section: String; const FileName: String); virtual;
    procedure LoadFromIni(const IniPath: String; const Section: String; const FileName: String); virtual;

    property SynDialectStyles: TfrSynDialectStyles read FSynDialectStyles;
    property CodeCompletionMinWidth: Integer read FCodeCompletionMinWidth write SetCodeCompletionMinWidth default 300;
    property CodeCompletionMinHeight: Integer read FCodeCompletionMinHeight write SetCodeCompletionMinHeight default 100;
    property CodeCompletionWidth: Integer read FCodeCompletionWidth write SetCodeCompletionWidth default 300;
    property CodeCompletionHeight: Integer read FCodeCompletionHeight write SetCodeCompletionHeight default 100;
    property ActiveLine: Integer read FActiveLine write SetActiveLine;
    property BlockColor: TColor read FBlockColor write FBlockColor;
    property BlockFontColor: TColor read FBlockFontColor write FBlockFontColor;
    property Color;
    property CommentAttr: TFont read FCommentAttr write SetCommentAttr;
    property Font;
	{$IFNDEF FPC}
    property ImeMode;
    property ImeName;
  	{$ENDIF}
    property SelStart: TPoint read FSelStart write FSelStart;
    property SelEnd: TPoint read FSelEnd write FSelEnd;
    property TabStops: Integer read FTabStops write FTabStops;
    property GutterWidth: Integer read FGutterWidth write SetGutterWidth;
    property KeywordAttr: TFont read FKeywordAttr write SetKeywordAttr;
    property Modified: Boolean read FModified write FModified;
    property NumberAttr: TFont read FNumberAttr write SetNumberAttr;
    property RunLine[Index: Integer]: Boolean read GetRunLine write SetRunLine;
    property SelText: String read GetSelText write SetSelText;
    property StringAttr: TFont read FStringAttr write SetStringAttr;
    property TextAttr: TFont read FTextAttr write SetTextAttr;
    property Lines: TStrings read GetText write SetText;
    property ShowGutter: Boolean read FShowGutter write SetShowGutter;
    property ShowLineNumber: Boolean read FShowLineNumber write FShowLineNumber;
    property TextSelected: Boolean read GetTextSelected;
    property OnChangePos: TNotifyEvent read FOnChangePos write FOnChangePos;
    property OnChangeText: TNotifyEvent read FOnChangeText write FOnChangeText;
    property OnDragDrop;
    property OnDragOver;
    property OnKeyDown;
    property PopupMenu;
  end;

  function frSynPos(s1, s2: String): Integer;
  function Diff(s1, s2: String): Integer;

const
  {$IFNDEF FPC}
  WordChars = ['a'..'z', 'A'..'Z', ''..'', ''..'', '0'..'9', '_'];
  {$ELSE}
  WordChars = ['a'..'z', 'A'..'Z', '0'..'9', '_'];
  {$ENDIF}
  DefGutterWidth: Integer = 30;
  FCaretWidth = 2;
  FTextGutterPadding = 2;
  FHalfTextGutterPadding = 1;

implementation
end.
