unit frControls;

{$I frVer.inc}

interface

uses
{$IFDEF DELPHI16}
  System.UITypes,
{$ENDIF}
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLType, LCLIntf, LCLProc,
  {$IFNDEF LINUX}
  JwaWinUser,
  {$ENDIF}
{$ENDIF}
  Types, SysUtils, Variants, Classes, Graphics, Messages, Controls, ExtCtrls, ImgList, Forms, StdCtrls,
  frDPIAwareInt, frCoreClasses, frAppearance;

type
  TfrScrollBarPart = (sbpLeftButton, sbpLeftArea, sbpThumb, sbpRightArea, sbpRightButton);

  TfrCustomControl = class;
  TfrCustomViewInfo = class;
  TfrCustomController = class;
  TfrControlScrollBar = class;
  TfrScrollingControl = class;
  TfrScrollBarHitTest = class;

  IfrCustomViewInfoControl = interface
  ['{79F890E8-F6F9-4BFE-BF6C-047296667D63}']
    function GetBorderStyle: TBorderStyle;
    function GetCanvas: TfrCustomCanvas;
    function GetColor: TColor;
    function GetFont: TFont;
    function GetHitViewInfo: TfrCustomViewInfo;
    function GetPainter: TfrCustomPainter;
    function GetScaleHelper: TfrScaleHelper;
    function IsDesigning: Boolean;
    function IsFocused: Boolean;

    property BorderStyle: TBorderStyle read GetBorderStyle;
    property Canvas: TfrCustomCanvas read GetCanvas;
    property Color: TColor read GetColor;
    property Font: TFont read GetFont;
    property HitViewInfo: TfrCustomViewInfo read GetHitViewInfo;
    property Painter: TfrCustomPainter read GetPainter;
    property ScaleHelper: TfrScaleHelper read GetScaleHelper;
  end;

  IfrViewInfoScrollingControl = interface(IfrCustomViewInfoControl)
  ['{8C5DC9A7-6DF4-48C0-902B-868DD5CF84ED}']
    function GetHorzScrollBar: TfrControlScrollBar;
    function GetHorzScrollBarVisible: Boolean;
    function GetVertScrollBar: TfrControlScrollBar;
    function GetVertScrollBarVisible: Boolean;

    property HorzScrollBar: TfrControlScrollBar read GetHorzScrollBar;
    property HorzScrollBarVisible: Boolean read GetHorzScrollBarVisible;
    property VertScrollBar: TfrControlScrollBar read GetVertScrollBar;
    property VertScrollBarVisible: Boolean read GetVertScrollBarVisible;
  end;

  IfrInplaceEditParent = interface
  ['{33D2C741-0B8C-4D68-A56D-97EB8FDDC909}']
    procedure FinishEditing(ANeedSave: Boolean);
    function GetCanvas: TfrCustomCanvas;
    function GetContentBounds: TRect;
    function GetPainter: TfrCustomPainter;
    function GetWinControl: TWinControl;

    property Canvas: TfrCustomCanvas read GetCanvas;
    property ContentBounds: TRect read GetContentBounds;
    property Painter: TfrCustomPainter read GetPainter;
    property WinControl: TWinControl read GetWinControl;
  end;

  { TfrInplaceEdit }

  TfrInplaceEdit = class(TEdit)
  strict private
    FHiding: Boolean;
    FInplaceEditParent: IfrInplaceEditParent;

    procedure AdjustBounds;
    function GetCanvas: TfrCustomCanvas;
    function GetContentBounds: TRect;
    function GetControl: TWinControl;
    function GetPainter: TfrCustomPainter;

    procedure CMShowingChanged(var Message: TMessage); message CM_SHOWINGCHANGED;
  protected
    procedure Change; override;
    procedure KeyPress(var Key: Char); override;
    procedure WndProc(var Message: TMessage); override;

    property Canvas: TfrCustomCanvas read GetCanvas;
    property ContentBounds: TRect read GetContentBounds;
    property Control: TWinControl read GetControl;
    property InplaceEditParent: IfrInplaceEditParent read FInplaceEditParent;
    property Painter: TfrCustomPainter read GetPainter;
  public
    constructor Create(AOwner: TComponent); override;

    procedure Hide;
    procedure Show(AParent: IfrInplaceEditParent; const ABounds: TRect; const AText: string; AFont: TFont);
  end;

  { TfrCustomHitTest }

  TfrCustomHitTest = class
  strict private
    FHitPoint: TPoint;
    FHitViewInfo: TfrCustomViewInfo;
    FIsValid: Boolean;
    FOwner: TfrCustomController;

    function GetControl: TfrCustomControl;
    function GetRootViewInfo: TfrCustomViewInfo;
  protected
    procedure SetHitViewInfo(Value: TfrCustomViewInfo); virtual;

    property Control: TfrCustomControl read GetControl;
    property Owner: TfrCustomController read FOwner;
    property RootViewInfo: TfrCustomViewInfo read GetRootViewInfo;
  public
    constructor Create(AOwner: TfrCustomController);

    procedure Calculate(const P: TPoint);
    procedure Invalidate; virtual;
    procedure Recalculate; virtual;

    property HitPoint: TPoint read FHitPoint;
    property HitViewInfo: TfrCustomViewInfo read FHitViewInfo;
  end;

  { TfrCustomController }

  TfrCustomController = class
  strict private
    FHitTest: TfrCustomHitTest;
    FOwner: TfrCustomControl;
  protected
    procedure CalculateHitTest(APoint: TPoint); virtual;
    function CreateHitTest: TfrCustomHitTest; virtual; abstract;

    procedure DoClick; virtual;
    procedure DoFocused; virtual;
    procedure DoKeyDown(var Key: Word; Shift: TShiftState); virtual;
    procedure DoKeyPress(var Key: Char); virtual;
    procedure DoMouseDown(Button: TMouseButton; Shift: TShiftState; const P: TPoint); virtual;
    procedure DoMouseLeave; virtual;
    procedure DoMouseMove(Shift: TShiftState; const P: TPoint); virtual;
    procedure DoMouseUp(Button: TMouseButton; Shift: TShiftState;  const P: TPoint); virtual;

    property HitTest: TfrCustomHitTest read FHitTest;
    property Owner: TfrCustomControl read FOwner;
  public
    constructor Create(AOwner: TfrCustomControl); virtual;
    destructor Destroy; override;

    procedure Click;
    procedure KeyDown(var Key: Word; Shift: TShiftState);
    procedure KeyPress(var Key: Char);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; const P: TPoint);
    procedure MouseLeave;
    procedure MouseMove(Shift: TShiftState; const P: TPoint);
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;  const P: TPoint);
    function MouseWheel(Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint): Boolean; virtual;
  end;

  { TfrCustomViewInfo }

  TfrCustomViewInfo = class(TPersistent)
  strict private
    FBounds: TRect;
    FControl: IfrCustomViewInfoControl;

    function GetCanvas: TfrCustomCanvas;
    function GetFont: TFont;
    function GetHeight: Integer;
    function GetPainter: TfrCustomPainter;
    function GetScaleHelper: TfrScaleHelper;
    function GetWidth: Integer;
  protected
    FContentBounds: TRect;
    FSize: TSize;

    procedure CalculateContent; virtual;
    procedure CalculateContentBounds; virtual;
    procedure ChangeScale(M, D: Integer); virtual;
    procedure DoCalculateHitTest(AHitTest: TfrCustomHitTest); virtual;
    procedure DrawBackground(ACanvas: TfrCustomCanvas); virtual;
    procedure DrawContent(ACanvas: TfrCustomCanvas); virtual;
    function GetInvalidateRect: TRect; virtual;
    function HasHotTrackState: Boolean; virtual;
    procedure OffsetContent(DX, DY: Integer); virtual;

    property Canvas: TfrCustomCanvas read GetCanvas;
    property Control: IfrCustomViewInfoControl read FControl;
    property Font: TFont read GetFont;
    property Painter: TfrCustomPainter read GetPainter;
    property ScaleHelper: TfrScaleHelper read GetScaleHelper;
  public
    constructor Create(AControl: IfrCustomViewInfoControl); virtual;

    procedure Assign(Source: TPersistent); override;

    procedure Calculate(const ARect: TRect); virtual;
    function CalculateHitTest(AHitTest: TfrCustomHitTest): Boolean; virtual;
    procedure MeasureSize; virtual;
    procedure Paint(ACanvas: TfrCustomCanvas); virtual;
    procedure ResetHotTrack; virtual;

    property Bounds: TRect read FBounds;
    property ContentBounds: TRect read FContentBounds;
    property Height: Integer read GetHeight;
    property Width: Integer read GetWidth;
  end;

  { TfrCustomControl }

  TfrCustomControl = class(TCustomControl, IfrCustomViewInfoControl)
  strict private
    FBorderStyle: TBorderStyle;
    FController: TfrCustomController;
    FfrCanvas: TfrCustomCanvas;
    FLockCount: Integer;
    FPainter: TfrCustomPainter;
    FScaleChangeCounter: Integer;
    FScaleHelper: TfrScaleHelper;
    FViewInfo: TfrCustomViewInfo;

    procedure BeginScaleChange;
    procedure EndScaleChange;
    function GetIsScaleChanging: Boolean;
    procedure SetBorderStyle(AValue: TBorderStyle);

    procedure CMFocusChanged(var Message: TMessage); message CM_FOCUSCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    procedure AdjustSize; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Loaded; override;
    procedure Paint; override;
    procedure Resize; override;

    procedure Click; override;
    procedure DoMouseLeave; virtual;
    function DoMouseWheel(Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint): Boolean; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X: Integer; Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer); override;

    // IfrCustomViewInfoControl
    function GetBorderStyle: TBorderStyle;
    function GetCanvas: TfrCustomCanvas;
    function GetColor: TColor;
    function GetFont: TFont;
    function GetHitViewInfo: TfrCustomViewInfo;
    function GetPainter: TfrCustomPainter;
    function GetScaleHelper: TfrScaleHelper;
    function IsDesigning: Boolean;
    function IsFocused: Boolean;

    function IsDestroying: Boolean;
    function IsLoading: Boolean;

    function CreateCanvas: TfrCustomCanvas; virtual;
    function CreateController: TfrCustomController; virtual; abstract;
    function CreatePainter: TfrCustomPainter; virtual;
    function CreateViewInfo: TfrCustomViewInfo; virtual; abstract;

    procedure Changed; virtual;
    procedure FontChanged; virtual;
    procedure InvalidateControlRect(const R: TRect); virtual;

  {$IFDEF DELPHI24}
    procedure ChangeScale(M, D: Integer; isDpiChange: Boolean); override;
    procedure ScaleControlsForDpi(NewPPI: Integer); override;
  {$ELSE}
    procedure ChangeScale(M, D: Integer); overload; override;
  {$ENDIF}
    procedure DoChangeScale(M, D: Integer; isDpiChange: Boolean); virtual;
    procedure ScaleHelperChanging; virtual;
    procedure ScaleHelperChanged; virtual;

    procedure WndProc(var Message: TMessage); override;

    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle default bsSingle;
    property Controller: TfrCustomController read FController;
    property IsScaleChanging: Boolean read GetIsScaleChanging;
    property Painter: TfrCustomPainter read GetPainter;
    property ScaleHelper: TfrScaleHelper read GetScaleHelper;
    property ViewInfo: TfrCustomViewInfo read FViewInfo;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetFocus; override;

    procedure BeginUpdate;
    procedure EndUpdate;
    function IsUpdateLocked: Boolean;

    property frCanvas: TfrCustomCanvas read GetCanvas;
  end;

  { TfrScrollBarViewInfo }

  TfrScrollBarViewInfo = class(TfrCustomViewInfo)
  strict private
    FCaptured: Boolean;
    FLeftArea: TRect;
    FHovered: Boolean;
    FHoveredPart: TfrScrollBarPart;
    FLeftButtonBounds: TRect;
    FRatio: Double;
    FRightArea: TRect;
    FRightButtonBounds: TRect;
    FScrollBar: TfrControlScrollBar;
    FThumbBounds: TRect;

    function GetState(APart: TfrScrollBarPart): TfrButtonState;
    function IsLeftButtonDisabled: Boolean;
    function IsRightButtonDisabled: Boolean;
  protected
    procedure CalculateContent; override;
    procedure CalculateLeftArea;
    procedure CalculateLeftButtonBounds;
    procedure CalculateRatio;
    procedure CalculateRightArea;
    procedure CalculateRightButtonBounds;
    procedure CalculateThumbBounds;

    procedure DrawContent(ACanvas: TfrCustomCanvas); override;
    procedure DrawLeftButton(ACanvas: TfrCustomCanvas);
    procedure DrawRightButton(ACanvas: TfrCustomCanvas);
    procedure DrawThumb(ACanvas: TfrCustomCanvas);

    function GetKind: TScrollBarKind;
    function GetVisible: Boolean;

    property Ratio: Double read FRatio;
  public
    function CalculateHitTest(AHitTest: TfrCustomHitTest): Boolean; override;
    procedure MeasureSize; override;
    procedure ResetHotTrack; override;

    property Captured: Boolean read FCaptured write FCaptured;
    property Kind: TScrollBarKind read GetKind;
    property LeftArea: TRect read FLeftArea;
    property LeftButtonBounds: TRect read FLeftButtonBounds;
    property RightArea: TRect read FRightArea;
    property RightButtonBounds: TRect read FRightButtonBounds;
    property ScrollBar: TfrControlScrollBar read FScrollBar write FScrollBar;
    property ThumbBounds: TRect read FThumbBounds;
    property Visible: Boolean read GetVisible;
  end;

  { TfrScrollBarHitTest }

  TfrScrollBarHitTest = class(TfrCustomHitTest)
  strict private
    FHitAtLeftArea: Boolean;
    FHitAtLeftButton: Boolean;
    FHitAtRightArea: Boolean;
    FHitAtRightButton: Boolean;
    FHitAtThumb: Boolean;
    FHitPoint: TPoint;
  protected
    function GetViewInfo: TfrScrollBarViewInfo;

    procedure SetHitAtLeftArea(const Value: Boolean);
    procedure SetHitAtLeftButton(const Value: Boolean);
    procedure SetHitAtRightArea(const Value: Boolean);
    procedure SetHitAtRightButton(const Value: Boolean);
    procedure SetHitAtThumb(const Value: Boolean);

    property ViewInfo: TfrScrollBarViewInfo read GetViewInfo;
  public
    procedure Invalidate; override;
    procedure Recalculate; override;

    property HitAtLeftArea: Boolean read FHitAtLeftArea;
    property HitAtLeftButton: Boolean read FHitAtLeftButton;
    property HitAtRightArea: Boolean read FHitAtRightArea;
    property HitAtRightButton: Boolean read FHitAtRightButton;
    property HitAtThumb: Boolean read FHitAtThumb;
    property HitPoint: TPoint read FHitPoint;
  end;

  { TfrScrollBarCustomController }

  TfrScrollBarCustomController = class(TfrCustomController)
  strict private
    FCapturedPoint: TPoint;
    FChange: Integer;
    FFirstPosition: Integer;
    FIsCaptured: Boolean;
    FTimer: TTimer;

    function CalculatePositionChange: Integer;
    procedure TimerHandler(Sender: TObject);
  protected
    function CreateHitTest: TfrCustomHitTest; override;

    procedure DoKeyPress(var Key: Char); override;
    procedure DoMouseDown(Button: TMouseButton; Shift: TShiftState; const P: TPoint); override;
    procedure DoMouseMove(Shift: TShiftState; const P: TPoint); override;
    procedure DoMouseUp(Button: TMouseButton; Shift: TShiftState;  const P: TPoint); override;

    function GetControlScrollBar: TfrControlScrollBar; virtual; abstract;
    function GetScrollBarHitTest: TfrScrollBarHitTest;
    function GetViewInfo: TfrScrollBarViewInfo; virtual; abstract;

    property ControlScrollBar: TfrControlScrollBar read GetControlScrollBar;
    property ScrollBarHitTest: TfrScrollBarHitTest read GetScrollBarHitTest;
    property ViewInfo: TfrScrollBarViewInfo read GetViewInfo;
  public
    constructor Create(AOwner: TfrCustomControl); override;
    destructor Destroy; override;

    function MouseWheel(Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint): Boolean; override;

    property IsCaptured: Boolean read FIsCaptured;
  end;

  { TfrInnerHorzScrollBarController }

  TfrInnerHorzScrollBarController = class(TfrScrollBarCustomController)
  protected
    function GetControlScrollBar: TfrControlScrollBar; override;
    function GetViewInfo: TfrScrollBarViewInfo; override;
  end;

  { TfrInnerVertScrollBarController }

  TfrInnerVertScrollBarController = class(TfrScrollBarCustomController)
  protected
    function GetControlScrollBar: TfrControlScrollBar; override;
    function GetViewInfo: TfrScrollBarViewInfo; override;
  end;

  { TfrCustomControlScrollBar }

  TfrCustomControlScrollBar = class
  strict private
    FKind: TScrollBarKind;
    FLargeChange: TScrollBarInc;
    FMax: Integer;
    FMin: Integer;
    FPageSize: Integer;
    FPosition: Integer;
    FSmallChange: TScrollBarInc;

    procedure SetKind(AValue: TScrollBarKind);
    procedure SetMax(AValue: Integer);
    procedure SetMin(AValue: Integer);
    procedure SetPageSize(AValue: Integer);
    procedure SetPosition(AValue: Integer);
  protected
    procedure Changed; virtual;
    procedure ChangeScale(M, D: Integer);
  public
    constructor Create;
    procedure UpdatePosition;

    property Kind: TScrollBarKind read FKind write SetKind;
    property LargeChange: TScrollBarInc read FLargeChange write FLargeChange default 1;
    property Max: Integer read FMax write SetMax default 100;
    property Min: Integer read FMin write SetMin default 0;
    property PageSize: Integer read FPageSize write SetPageSize;
    property Position: Integer read FPosition write SetPosition default 0;
    property SmallChange: TScrollBarInc read FSmallChange write FSmallChange default 1;
  end;

  { TfrControlScrollBar }

  TfrControlScrollBar = class(TfrCustomControlScrollBar)
  strict private
    FControl: TfrCustomControl;
  protected
    procedure Changed; override;
  public
    constructor Create(AControl: TfrCustomControl); virtual;

    property Control: TfrCustomControl read FControl;
  end;

  { TfrCustomScrollingController }

  TfrCustomScrollingController = class(TfrCustomController)
  strict private
    FActualController: TfrCustomController;
    FHorzScrollBarController: TfrInnerHorzScrollBarController;
    FVertScrollBarController: TfrInnerVertScrollBarController;

    function GetScrollingControl: TfrScrollingControl;
  protected
    procedure CalculateHitTest(APoint: TPoint); override;

    procedure DoMouseDown(Button: TMouseButton; Shift: TShiftState; const P: TPoint); override;
    procedure DoMouseMove(Shift: TShiftState; const P: TPoint); override;
    procedure DoMouseUp(Button: TMouseButton; Shift: TShiftState;  const P: TPoint); override;

    function GetHorzScrollBar: TfrControlScrollBar; virtual;
    function GetHorzScrollBarViewInfo: TfrScrollBarViewInfo; virtual; abstract;
    function GetVertScrollBarViewInfo: TfrScrollBarViewInfo; virtual; abstract;
    function GetVertScrollBar: TfrControlScrollBar; virtual;

    property HorzScrollBar: TfrControlScrollBar read GetHorzScrollBar;
    property HorzScrollBarViewInfo: TfrScrollBarViewInfo read GetHorzScrollBarViewInfo;
    property ScrollingControl: TfrScrollingControl read GetScrollingControl;
    property VertScrollBar: TfrControlScrollBar read GetVertScrollBar;
    property VertScrollBarViewInfo: TfrScrollBarViewInfo read GetVertScrollBarViewInfo;
  public
    constructor Create(AOwner: TfrCustomControl); override;
    destructor Destroy; override;

    function MouseWheel(Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint): Boolean; override;
  end;

  { TfrScrollingControl }

  TfrScrollingControl = class(TfrCustomControl, IfrViewInfoScrollingControl)
  strict private
    FHorzScrollBar: TfrControlScrollBar;
    FVertScrollBar: TfrControlScrollBar;
  protected
    procedure Changed; override;
    procedure DoChangeScale(M, D: Integer; isDpiChange: Boolean); override;
    procedure ScrollBarsChange; virtual;

  // IfrViewInfoScrollingControl
    function GetHorzScrollBar: TfrControlScrollBar; virtual;
    function GetHorzScrollBarVisible: Boolean; virtual;
    function GetVertScrollBar: TfrControlScrollBar; virtual;
    function GetVertScrollBarVisible: Boolean; virtual;

    function GetHorzScrollBarViewInfo: TfrScrollBarViewInfo; virtual;
    function GetVertScrollBarViewInfo: TfrScrollBarViewInfo; virtual;

    property HorzScrollBar: TfrControlScrollBar read GetHorzScrollBar;
    property HorzScrollBarViewInfo: TfrScrollBarViewInfo read GetHorzScrollBarViewInfo;
    property HorzScrollBarVisible: Boolean read GetHorzScrollBarVisible;
    property VertScrollBar: TfrControlScrollBar read GetVertScrollBar;
    property VertScrollBarViewInfo: TfrScrollBarViewInfo read GetVertScrollBarViewInfo;
    property VertScrollBarVisible: Boolean read GetVertScrollBarVisible;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

implementation
end.
