unit frTreeView;

{$I frVer.inc}

interface

uses
{$IFDEF DELPHI16}
  System.UITypes,
{$ENDIF}
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLType, LCLIntf, LCLProc,
{$ENDIF}
  Forms,
  SysUtils, Types, Classes, Graphics, RTLConsts, ComCtrls, ImgList, Messages, ExtCtrls, Controls,
  frCore, frAppearance, frControls, frCoreClasses;

type
  TfrCustomTreeView = class;
  TfrTreeViewNodes = class;
  TfrTreeViewNode = class;
  TfrTreeViewViewInfo = class;
  TfrTreeViewNodeViewInfo = class;

  TfrTreeViewCompareProc = function(ANode1, ANode2: TfrTreeViewNode; AData: frInteger): Integer of object;

  TfrTreeViewNodeCache = record
    CacheNode: TfrTreeViewNode;
    CacheIndex: Integer;
  end;

  IfrViewInfoTreeView = interface(IfrViewInfoScrollingControl)
  ['{4F6A2428-ECC8-4F66-ACD7-C3C4AAED186F}']
    function CustomDraw(ACanvas: TfrCustomCanvas): Boolean;
    function GetHideSelection: Boolean;
    function GetHotTrack: Boolean;
    function GetImageHeight: Integer;
    function GetImageWidth: Integer;
    function GetIndent: Integer;
    function GetNodeByIndex(AIndex: Integer): TfrTreeViewNode;
    function GetRightClickNode: TfrTreeViewNode;
    function GetRootNode: TfrTreeViewNode;
    function GetRowSelect: Boolean;
    function GetShowButtons: Boolean;
    function GetShowLines: Boolean;
    function GetShowRoot: Boolean;
    function GetStateImageHeight: Integer;
    function GetStateImageWidth: Integer;

    property HideSelection: Boolean read GetHideSelection;
    property HotTrack: Boolean read GetHotTrack;
    property ImageHeight: Integer read GetImageHeight;
    property ImageWidth: Integer read GetImageWidth;
    property Indent: Integer read GetIndent;
    property Node[AIndex: Integer]: TfrTreeViewNode read GetNodeByIndex;
    property RightClickNode: TfrTreeViewNode read GetRightClickNode;
    property RootNode: TfrTreeViewNode read GetRootNode;
    property RowSelect: Boolean read GetRowSelect;
    property ShowButtons: Boolean read GetShowButtons;
    property ShowLines: Boolean read GetShowLines;
    property ShowRoot: Boolean read GetShowRoot;
    property StateImageHeight: Integer read GetStateImageHeight;
    property StateImageWidth: Integer read GetStateImageWidth;
  end;

  { TfrTreeViewNodeSortHelper }

  TfrTreeViewNodeSortHelper = class
  strict private
    FSortData: frInteger;
    FSortProc: TfrTreeViewCompareProc;

    function SortListProc(AItem1, AItem2: Pointer): Integer;
  public
    constructor Create(ASortProc: TfrTreeViewCompareProc; ASortData: frInteger);
    procedure Sort(AList: TfrList);
  end;

  { TfrTreeViewNode }

  TfrTreeViewNode = class
  strict private
    FCount: Integer;
    FCut: Boolean;
    FData: Pointer;
    FDeleting: Boolean;
    FEnabled: Boolean;
    FExpandButtonHotTrack: Boolean;
    FExpanded: Boolean;
    FExpandedImageIndex: Integer;
    FHasChildren: Boolean;
    FHidden: Boolean;
    FHotTrack: Boolean;
    FImageIndex: Integer;
    FOverlayIndex: Integer;
    FParent: TfrTreeViewNode;
    FSelectedIndex: Integer;
    FStateIndex: Integer;
    FText: string;
    FTreeView: TfrCustomTreeView;

    FFirstChild: TfrTreeViewNode;
    FLastChild: TfrTreeViewNode;
    FNextSibling: TfrTreeViewNode;
    FPrevSibling: TfrTreeViewNode;

    procedure CheckAbsoluteNodesCache;
    procedure ExpandItem(AExpand, ARecurse: Boolean);
    function GetAbsoluteIndex: Integer;
    function GetDropTarget: Boolean;
    function GetExpanded: Boolean;
    function GetFocused: Boolean;
    function GetHasChildren: Boolean;
    function GetImageIndex: Integer;
    function GetImages: TCustomImageList;
    function GetIndex: Integer;
    function GetItem(AIndex: Integer): TfrTreeViewNode;
    function GetLevel: Integer;
    function GetNext: TfrTreeViewNode;
    function GetOwner: TfrTreeViewNodes;
    function GetPrev: TfrTreeViewNode;
    function GetRoot: TfrTreeViewNode;
    function GetSelected: Boolean;
    function GetSelectedIndex: Integer;
    function GetStateImages: TCustomImageList;
    function GetVisible: Boolean;
    procedure SetCut(AValue: Boolean);
    procedure SetData(AValue: Pointer);
    procedure SetDropTarget(AValue: Boolean);
    procedure SetEnabled(AValue: Boolean);
    procedure SetExpandButtonHotTrack(AValue: Boolean);
    procedure SetExpanded(AValue: Boolean);
    procedure SetExpandedImageIndex(AValue: Integer);
    procedure SetFirstChild(AValue: TfrTreeViewNode);
    procedure SetFocused(AValue: Boolean);
    procedure SetHasChildren(AValue: Boolean);
    procedure SetHidden(AValue: Boolean);
    procedure SetHotTrack(AValue: Boolean);
    procedure SetImageIndex(AValue: Integer);
    procedure SetLastChild(AValue: TfrTreeViewNode);
    procedure SetOverlayIndex(AValue: Integer);
    procedure SetParent(AParent: TfrTreeViewNode; AMode: TAddMode);
    procedure SetSelected(AValue: Boolean);
    procedure SetSelectedIndex(AValue: Integer);
    procedure SetStateIndex(AValue: Integer);
    procedure SetText(const AValue: string);
  private
    FAbsoluteIndex: Integer;
    function DefaultTreeViewSort(AItem1, AItem2: TfrTreeViewNode; AData: frInteger): Integer;
  protected
    procedure BeginUpdate;
    procedure EndUpdate;

    procedure AddToAbsoluteNodeCashe;
    procedure Changed;
    procedure DoChange;
    function DoCustomDraw(ANodeViewInfo: TfrTreeViewNodeViewInfo; ACanvas: TfrCustomCanvas): Boolean;
    procedure ExtractFromParent;
    procedure Invalidate;
    function IsEditing: Boolean;
    function IsInLevelFirst: Boolean;
    function IsInLevelLast: Boolean;
    function IsInLevelVisibleLast: Boolean;
    function IsNodeParent(ANode: TfrTreeViewNode): Boolean;
    function IsParentCollapsed: Boolean;

    procedure ReadData(AStream: TStream);
    procedure WriteData(AStream: TStream);

    property Deliting: Boolean read FDeleting;
    property ExpandButtonHotTrack: Boolean read FExpandButtonHotTrack write SetExpandButtonHotTrack;
    property Hidden: Boolean read FHidden write SetHidden;
    property HotTrack: Boolean read FHotTrack write SetHotTrack;
    property TreeView: TfrCustomTreeView read FTreeView;
    property Visible: Boolean read GetVisible;
  public
    constructor Create(AOwner: TfrCustomTreeView);
    destructor Destroy; override;

    function AddChild(AText: string = ''; AData: Pointer = nil): TfrTreeViewNode;
    function AddChildFirst(AText: string = ''; AData: Pointer = nil): TfrTreeViewNode;
    function AddNode(ANode, ARelative: TfrTreeViewNode; AData: Pointer; AAttachMode: TNodeAttachMode): TfrTreeViewNode;
    function AlphaSort(ARecurse: Boolean = False): Boolean;
    procedure Assign(ASource: TfrTreeViewNode);
    procedure AssignFromTreeNode(ASource: TTreeNode); virtual;
    procedure Clear;
    procedure Collapse(ARecurse: Boolean);
    function CustomSort(ASortProc: TfrTreeViewCompareProc; AData: frInteger; ARecurse: Boolean = False): Boolean;
    procedure Delete;
    procedure DeleteChildren;
    function DisplayRect(ATextOnly: Boolean): TRect; // Need test (compare with TTreeView)
    function EditText: Boolean;
    procedure EndEdit(ACancel: Boolean);
    procedure Expand(ARecurse: Boolean);
    function GetNextChild(AValue: TfrTreeViewNode): TfrTreeViewNode;
    function GetNextVisible: TfrTreeViewNode; // Need test (compare with TTreeView)
    function GetPrevChild(AValue: TfrTreeViewNode): TfrTreeViewNode;
    function GetPrevVisible: TfrTreeViewNode; // Need test (compare with TTreeView)
    function HasAsParent(AValue: TfrTreeViewNode): Boolean;
    function IndexOf(AValue: TfrTreeViewNode): Integer;
    function IsFirstNode: Boolean;
    function IsRoot: Boolean;
    procedure MakeVisible;
    procedure MoveTo(ADestination: TfrTreeViewNode; AMode: TNodeAttachMode); virtual; // Need test (compare with TTreeView)

    property FirstChild: TfrTreeViewNode read FFirstChild;
    property LastChild: TfrTreeViewNode read FLastChild;
    property Next: TfrTreeViewNode read GetNext;
    property NextSibling: TfrTreeViewNode read FNextSibling;
    property Prev: TfrTreeViewNode read GetPrev;
    property PrevSibling: TfrTreeViewNode read FPrevSibling;

    property AbsoluteIndex: Integer read GetAbsoluteIndex;
    property Count: Integer read FCount;
    property Cut: Boolean read FCut write SetCut;
    property Data: Pointer read FData write SetData;
    property Deleting: Boolean read FDeleting;
    property DropTarget: Boolean read GetDropTarget write SetDropTarget;
    property Enabled: Boolean read FEnabled write SetEnabled;
    property Expanded: Boolean read GetExpanded write SetExpanded;
    property ExpandedImageIndex: Integer read FExpandedImageIndex write SetExpandedImageIndex;
    property Focused: Boolean read GetFocused write SetFocused;
    property HasChildren: Boolean read GetHasChildren write SetHasChildren;
    property ImageIndex: Integer read GetImageIndex write SetImageIndex;
    property Images: TCustomImageList read GetImages;
    property Index: Integer read GetIndex;
    property IsVisible: Boolean read GetVisible;  // Need test (compare with TTreeView)
    property Item[Index: Integer]: TfrTreeViewNode read GetItem; default;
    property Level: Integer read GetLevel;
    property OverlayIndex: Integer read FOverlayIndex write SetOverlayIndex;
    property Owner: TfrTreeViewNodes read GetOwner; // For compatibility
    property Parent: TfrTreeViewNode read FParent;
    property Root: TfrTreeViewNode read GetRoot;
    property Selected: Boolean read GetSelected write SetSelected;
    property SelectedIndex: Integer read GetSelectedIndex write SetSelectedIndex;
    property StateIndex: Integer read FStateIndex write SetStateIndex;
    property StateImages: TCustomImageList read GetStateImages;
    property Text: string read FText write SetText;
  end;

  TfrTreeViewNodeClass = class of TfrTreeViewNode;

  { TfrTreeViewNodesEnumerator }

  TfrTreeViewNodesEnumerator = class
  private
    FIndex: Integer;
    FTreeNodes: TfrTreeViewNodes;
  public
    constructor Create(ATreeNodes: TfrTreeViewNodes);
    function GetCurrent: TfrTreeViewNode;
    function MoveNext: Boolean;

    property Current: TfrTreeViewNode read GetCurrent;
  end;

  { TfrTreeViewNodes }

  TfrTreeViewNodes = class(TPersistent)
  strict private
    FRoot: TfrTreeViewNode;
    FTreeView: TfrCustomTreeView;

    function GetCount: Integer;
    function GetHandle: HWND;
    function GetItem(AIndex: Integer): TfrTreeViewNode;
    function GetLastNode: TfrTreeViewNode;
  protected
    procedure ReadData(AStreem: TStream);
    procedure WriteData(AStreem: TStream);

    property Root: TfrTreeViewNode read FRoot;
    property TreeView: TfrCustomTreeView read FTreeView;
  public
    constructor Create(ATreeView: TfrCustomTreeView); virtual;
    destructor Destroy; override;

    function Add(ASibling: TfrTreeViewNode; const S: string): TfrTreeViewNode;
    function AddChild(AParent: TfrTreeViewNode; const S: string): TfrTreeViewNode;
    function AddChildFirst(AParent: TfrTreeViewNode; const S: string): TfrTreeViewNode;
    function AddChildObject(AParent: TfrTreeViewNode; const S: string; AData: Pointer): TfrTreeViewNode;
    function AddChildObjectFirst(AParent: TfrTreeViewNode; const S: string; AData: Pointer): TfrTreeViewNode;
    function AddFirst(ASibling: TfrTreeViewNode; const S: string): TfrTreeViewNode;
    function AddNode(ANode, ARelative: TfrTreeViewNode; const S: string; AData: Pointer;
      AttachMode: TNodeAttachMode): TfrTreeViewNode;
    function AddObject(ASibling: TfrTreeViewNode; const S: string; AData: Pointer): TfrTreeViewNode;
    function AddObjectFirst(ASibling: TfrTreeViewNode; const S: string; AData: Pointer): TfrTreeViewNode;
    function AlphaSort(ARecurse: Boolean = False): Boolean;
    procedure Assign(Source: TPersistent); override;
    procedure Clear;
    function CustomSort(ASortProc: TfrTreeViewCompareProc; AData: NativeInt; ARecurse: Boolean = True): Boolean;
    procedure Delete(ANode: TfrTreeViewNode);
    function FindNodeWithText(const ANodeText: string): TfrTreeViewNode;
    function GetLastVisibleNode: TfrTreeViewNode;
    function GetEnumerator: TfrTreeViewNodesEnumerator;
    function GetFirstNode: TfrTreeViewNode;
    function Insert(ASibling: TfrTreeViewNode; const S: string): TfrTreeViewNode;
    function InsertNode(ANode, ASibling: TfrTreeViewNode; const S: string; AData: Pointer): TfrTreeViewNode;
    function InsertObject(ASibling: TfrTreeViewNode; const S: string; AData: Pointer): TfrTreeViewNode;

    procedure BeginUpdate;
    procedure EndUpdate;

    property Count: Integer read GetCount;
    property Handle: HWND read GetHandle;
    property Item[Index: Integer]: TfrTreeViewNode read GetItem; default;
  end;

  { TfrTreeViewHitTest }

  TfrTreeViewHitTest = class(TfrCustomHitTest);

  { TfrTreeViewCustomViewInfo }

  TfrTreeViewCustomViewInfo = class(TfrCustomViewInfo)
  strict private
    function GetTreeView: IfrViewInfoTreeView;
  protected
    property TreeView: IfrViewInfoTreeView read GetTreeView;
  end;

  { TfrTreeViewNodeViewInfo }

  TfrTreeViewNodeViewInfo = class(TfrTreeViewCustomViewInfo)
  strict private const
    ElementIndent: Integer = 4;
  strict private
    FDragImagePainting: Boolean;
    FEditingTextRect: TRect;
    FExpandButtonRect: TRect;
    FHintPos: TPoint;
    FHotTrackRect: TRect;
    FHTreeLineRect: TRect;
    FImageRect: TRect;
    FInvalidateRect: TRect;
    FNode: TfrTreeViewNode;
    FOtherLevelVTreeLineRect: TRect;
    FSelectRect: TRect;
    FStateImageRect: TRect;
    FTextRect: TRect;
    FVTreeLineRect: TRect;
    FVisibleIndex: Integer;

    procedure CalculateEditingTextRect;
    procedure CalculateExpandButtonRect;
    procedure CalculateHintPos;
    procedure CalculateHotTrackRect;
    procedure CalculateHTreeLineRect;
    procedure CalculateImageRect;
    procedure CalculateInvalidateRect;
    procedure CalculateSelectRect;
    procedure CalculateStateImageRect;
    procedure CalculateTextRect;
    procedure CalculateVTreeLineRect;
    procedure DrawExpandButton(ACanvas: TfrCustomCanvas);
    procedure DrawImage(ACanvas: TfrCustomCanvas);
    procedure DrawStateImage(ACanvas: TfrCustomCanvas);
    procedure DrawText(ACanvas: TfrCustomCanvas);
    procedure DrawTreeLines(ACanvas: TfrCustomCanvas);
    function GetCanvas: TfrCustomCanvas;
    function GetFirstIndent: Integer;
    function GetImageHeight: Integer;
    function GetImageWidth: Integer;
    function GetLevelIndent: Integer;
    function GetNodeIndent: Integer;
    function GetTruncated: Boolean;
    function HasExpandButton: Boolean;
    function HasImage: Boolean;
    function HasStateImage: Boolean;
    function IsExpandButtonHotTrack: Boolean;
    function IsSelected: Boolean;
    function NeedDrawVTreeLineForLevel(LNode: TfrTreeViewNode): Boolean;
    function VisibleHotTrack: Boolean;
  protected
    procedure CalculateContent; override;
    procedure DrawBackground(ACanvas: TfrCustomCanvas); override;
    procedure DrawContent(ACanvas: TfrCustomCanvas); override;
    procedure DrawDragImage(ACanvas: TfrCustomCanvas; const ARect: TRect);
    function GetDragImageSize: TSize;
    function GetInvalidateRect: TRect; override;

    function IsFocused: Boolean;
    function IsHotTrack: Boolean;
    procedure OffsetContent(DX: Integer; DY: Integer); override;
  public
    constructor Create(ATreeView: IfrViewInfoTreeView; ANode: TfrTreeViewNode; AVisibleIndex: Integer); reintroduce;

    function CalculateHitTest(AHitTest: TfrCustomHitTest): Boolean; override;
    procedure MeasureSize; override;

    procedure MeasureHeight;
    procedure MeasureWidth;

    procedure Paint(ACanvas: TfrCustomCanvas); override;
    function PointInEditingTextArea(const APoint: TPoint): Boolean;
    function PointInExpandButtonArea(const APoint: TPoint): Boolean;
    function PointInHotTrackArea(const APoint: TPoint): Boolean;
    procedure ResetHotTrack; override;

    property Canvas: TfrCustomCanvas read GetCanvas;
    property EditingTextRect: TRect read FEditingTextRect;
    property HintPos: TPoint read FHintPos;
    property ImageHeight: Integer read GetImageHeight;
    property NodeIndent: Integer read GetNodeIndent;
    property ImageWidth: Integer read GetImageWidth;
    property LevelIndent: Integer read GetLevelIndent;
    property NodeRect: TRect read FHotTrackRect;
    property Node: TfrTreeViewNode read FNode write FNode;
    property TextRect: TRect read FTextRect;
    property Truncated: Boolean read GetTruncated;
    property VisibleIndex: Integer read FVisibleIndex;
  end;

  { TfrTreeViewViewInfo }

  TfrTreeViewViewInfoChangeState = (vcsMeasure, vcsFullUpdate);
  TfrTreeViewViewInfoChangeStates = set of TfrTreeViewViewInfoChangeState;

  TfrTreeViewViewInfo = class(TfrTreeViewCustomViewInfo)
  strict private
    FChangeStates: TfrTreeViewViewInfoChangeStates;
    FHorzScrollBarViewInfo: TfrScrollBarViewInfo;
    FItemHeight: Integer;
    FOldOffset: Integer;
    FVertScrollBarViewInfo: TfrScrollBarViewInfo;
    FVisibleNodeViewInfos: TfrList;

    procedure CalculateItemHeight;
    procedure CalculateNodes;
    procedure CalculateScrollBars;
    procedure ClearNodeViewInfos;
    procedure CreateNodeViewInfos(AFirstNode: TfrTreeViewNode; AFirstNodeVisibleIndex: Integer);
    procedure FindFirstNode(var AFirstNode: TfrTreeViewNode; var AFirstVisibleIndex: Integer);

    function GetCount: Integer;
    function GetFocusedNode: TfrTreeViewNodeViewInfo;
    function GetOffsetX: Integer;
    function GetOffsetY: Integer;
    function GetVisibleNodeViewInfos(AIndex: Integer): TfrTreeViewNodeViewInfo;
    procedure SetOffsetX(AValue: Integer);
    procedure SetOffsetY(AValue: Integer);
  protected
    procedure CalculateContent; override;
    procedure ChangeScale(M, D: Integer); override;
    procedure DrawContent(ACanvas: TfrCustomCanvas); override;
    function NeedDrawHorzScrollBar: Boolean;
    function NeedDrawVertScrollBar: Boolean;

    property ChangeStates: TfrTreeViewViewInfoChangeStates read FChangeStates write FChangeStates;
    property Count: Integer read GetCount;
    property HorzScrollBarViewInfo: TfrScrollBarViewInfo read FHorzScrollBarViewInfo;
    property ItemHeight: Integer read FItemHeight;
    property VertScrollBarViewInfo: TfrScrollBarViewInfo read FVertScrollBarViewInfo;
    property VisibleNodeViewInfos[AIndex: Integer]: TfrTreeViewNodeViewInfo read GetVisibleNodeViewInfos;
  public
    constructor Create(AControl: IfrCustomViewInfoControl); override;
    destructor Destroy; override;

    function CalculateHitTest(AHitTest: TfrCustomHitTest): Boolean; override;
    procedure DrawDragImage(ACanvas: TfrCustomCanvas; const ARect: TRect);
    function GetBottomNodeViewInfo: TfrTreeViewNodeViewInfo;
    function GetDragImageSize: TSize;
    function GetDragNodeViewInfo: TfrTreeViewNodeViewInfo;
    function GetHotTrackNodeViewInfo: TfrTreeViewNodeViewInfo;
    function GetNodeViewInfoByNode(ANode: TfrTreeViewNode): TfrTreeViewNodeViewInfo;
    function GetNodeViewInfoByPoint(APoint: TPoint): TfrTreeViewNodeViewInfo;
    function GetNumberOfNodesInContentRect: Integer;
    function GetTopNodeViewInfo: TfrTreeViewNodeViewInfo;
    procedure MeasureSize; override;

    function AdjustOffsetByExpandedNode(AExpandedNode: TfrTreeViewNode): Boolean;
    function AdjustOffsetByFocusedNode(AFocusedNode: TfrTreeViewNode): Boolean;
    function AdjustOffsetByTopNode(ATopNode: TfrTreeViewNode): Boolean;
    procedure AdjustMeasureSizeByCollapsingNode(ACollapsingNode: TfrTreeViewNode; out ANeedMeasure: Boolean);
    procedure AdjustMeasureSizeByExpandedNode(AExpandedNode: TfrTreeViewNode);
    function FindNodeInvalidateRect(ANode: TfrTreeViewNode; out ABounds: TRect): Boolean;
    procedure Paint(ACanvas: TfrCustomCanvas); override;
    procedure ResetHotTrack; override;
    procedure Calculate(const ARect: TRect); override;

    property FocusedNode: TfrTreeViewNodeViewInfo read GetFocusedNode;
    property OffsetX: Integer read GetOffsetX write SetOffsetX;
    property OffsetY: Integer read GetOffsetY write SetOffsetY;
  end;

  { TfrTreeViewController }

  TfrTreeViewController = class(TfrCustomScrollingController)
  strict private
    FEditingNode: TfrTreeViewNode;
    FFocused: Boolean;
    FShowEditTimer: TTimer;
    FCaptured: Boolean;
    FCapturedPos: TPoint;

    function AllowAutoExpand(ANode: TFrTreeViewNode; AShift: TShiftState = []): Boolean;
    function AllowShowNodeEdit: Boolean;
    function CanHintActivate: Boolean;
    function CanSelChangByMouseDown(Shift: TShiftState): Boolean;
    function GetHitTest: TfrTreeViewHitTest;
    function GetTreeView: TfrCustomTreeView;
    function GetViewInfo: TfrTreeViewViewInfo;
    procedure HorizontalNavigation(AKey: Word; AShift: TShiftState);
    procedure SelectNextNode(AOffset: Integer; AShift: TShiftState);

    procedure ShowEditTimerHandler(Sender: TObject);
  protected
    procedure BeginDrag;
    function CreateHitTest: TfrCustomHitTest; override;
    procedure DoFocused; override;
    procedure DoKeyDown(var Key: Word; Shift: TShiftState); override;
    procedure DoMouseDown(Button: TMouseButton; Shift: TShiftState; const P: TPoint); override;
    procedure DoMouseLeave; override;
    procedure DoMouseMove(Shift: TShiftState; const P: TPoint); override;
    procedure DoMouseUp(Button: TMouseButton; Shift: TShiftState; const P: TPoint); override;

    function GetHorzScrollBarViewInfo: TfrScrollBarViewInfo; override;
    function GetVertScrollBarViewInfo: TfrScrollBarViewInfo; override;

    procedure BeforeNodeDelete(ANode: TfrTreeViewNode);

    procedure StartShowEditTimer;
    procedure FinishShowEditTimer;

    property HitTest: TfrTreeViewHitTest read GetHitTest;
    property TreeView: TfrCustomTreeView read GetTreeView;
    property ViewInfo: TfrTreeViewViewInfo read GetViewInfo;
  public
    constructor Create(AOwner: TfrCustomControl); override;
    destructor Destroy; override;

    function MouseWheel(Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint): Boolean; override;
  end;

  { TfrTreeViewMultiSelectHelper }

  TfrTreeViewMultiSelectHelper = class
  strict private
    FChangeDelayTimer: TTimer;
    FPreviousSelections: TfrList;
    FSelections: TfrList;
    FShiftAnchor: TfrTreeViewNode;
    FTreeView: TfrCustomTreeView;

    procedure BeginChanges;
    procedure DoChangeDelayTimer(Sender: TObject);
    procedure EndChanges;
    function GetSelection(AIndex: Integer): TfrTreeViewNode;
    function IsChanges: Boolean;
    procedure SelectRange(AStartNode, AFinishNode: TfrTreeViewNode);
  protected
    procedure Changed;

    property ChangeDelayTimer: TTimer read FChangeDelayTimer;
    property Selections[AIndex: Integer]: TfrTreeViewNode read GetSelection;
  public
    constructor Create(ATreeView: TfrCustomTreeView);
    destructor Destroy; override;

    procedure BeforeNodeDelete(ANode: TfrTreeViewNode);
    procedure NodeDeselect(AIndex: Integer);
    procedure NodeSelect(ANode: TfrTreeViewNode; APosition: Integer = -1);
    procedure Select(ANode: TfrTreeViewNode; AShift: TShiftState = []);
    procedure SelectList(ANodes: TfrList); overload;
    procedure Subselect(ANode: TfrTreeViewNode; AValidate: Boolean = False);
    procedure ValidateSelection;

    property SelectionList: TfrList read FSelections write FSelections;
  end;

  { TfrTreeViewHintHelper }

  TfrTreeViewHintHelper = class
  const
    MaxHintTextLength = 32000;
  strict private
    FHintWindow: THintWindow;
    FTreeView: TfrCustomTreeView;
    FNode: TfrTreeViewNode;
  public
    constructor Create(ATreeView: TfrCustomTreeView);
    destructor Destroy; override;
    procedure BeforeNodeDelete(ANode: TfrTreeViewNode);
    procedure HideHint;
    function IsActive: Boolean;
    procedure ShowHint(AViewInfo: TfrTreeViewNodeViewInfo; AFont: TFont);
  end;

  { TfrTreeViewAutoScrollHelper }

  TfrTreeViewAutoScrollHelper = class
  type
    TfrScrollDirection = (sdUp, sdDown);
  strict private const
    FSmallTimerInterval = 200;
    FLargeTimerInterval = 400;
  strict private
    FScrollDir: TfrScrollDirection;
    FTimer: TTimer;
    FTreeView: TfrCustomTreeView;

    procedure CancelTimer;
    function GetBottomNode: TfrTreeViewNode;
    function GetScrollBar: TfrCustomControlScrollBar;
    function GetTopNode: TfrTreeViewNode;
    procedure TimerHandler(Sender: TObject);
  protected
    property BottomNode: TfrTreeViewNode read GetBottomNode;
    property ScrollBar: TfrCustomControlScrollBar read GetScrollBar;
    property TopNode: TfrTreeViewNode read GetTopNode;
  public
    constructor Create(ATreeView: TfrCustomTreeView);
    destructor Destroy; override;

    procedure ChangedDropTargetNode;
  end;

  { TfrTreeViewCustomOptions }

  TfrTreeViewCustomOptions = class(TPersistent)
  strict private
    FTreeView: TfrCustomTreeView;
  protected
    procedure Changed; virtual;

    property TreeView: TfrCustomTreeView read FTreeView;
  public
    constructor Create(ATreeView: TfrCustomTreeView); virtual;
  end;

  { TfrTreeViewOptionsBehavior }

  TfrTreeViewOptionsBehavior = class(TfrTreeViewCustomOptions)
  strict private
    FAutoExpand: Boolean;
    FChangeDelay: Integer;
    FHotTrack: Boolean;
    FReadOnly: Boolean;
    FSortType: TSortType;
    FToolTips: Boolean;

    procedure SetChangeDelay(AValue: Integer);
    procedure SetSortType(AValue: TSortType);
    procedure SetToolTips(AValue: Boolean);
  public
    constructor Create(ATreeView: TfrCustomTreeView); override;
    procedure Assign(Source: TPersistent); override;
  published
    property AutoExpand: Boolean read FAutoExpand write FAutoExpand default False;
    property ChangeDelay: Integer read FChangeDelay write SetChangeDelay default 0;
    property HotTrack: Boolean read FHotTrack write FHotTrack default False;
    property ReadOnly: Boolean read FReadOnly write FReadOnly default False;
    property SortType: TSortType read FSortType write SetSortType default stNone;
    property ToolTips: Boolean read FToolTips write SetToolTips default True;
  end;

  { TfrTreeViewOptionsView }

  TfrTreeViewOptionsView = class(TfrTreeViewCustomOptions)
  strict private
    FIndent: Integer;
    FShowButtons: Boolean;
    FShowLines: Boolean;
    FShowRoot: Boolean;

    procedure SetIndent(AValue: Integer);
    procedure SetShowButtons(AValue: Boolean);
    procedure SetShowLines(AValue: Boolean);
    procedure SetShowRoot(const AValue: Boolean);
  protected
    procedure ChangeScale(M, D: Integer); virtual;
    procedure Changed; override;
  public
    constructor Create(ATreeView: TfrCustomTreeView); override;
    procedure Assign(Source: TPersistent); override;
  published
    property Indent: Integer read FIndent write SetIndent default 19;
    property ShowButtons: Boolean read FShowButtons write SetShowButtons default True;
    property ShowLines: Boolean read FShowLines write SetShowLines default True;
    property ShowRoot: Boolean read FShowRoot write SetShowRoot default True;
  end;

  { TfrTreeViewOptionsSelection }

  TfrTreeViewOptionsSelection = class(TfrTreeViewCustomOptions)
  strict private
    FRowSelect: Boolean;
    FHideSelection: Boolean;
    FMultiSelect: Boolean;
    FMultiSelectStyle: TMultiSelectStyle;
    FRightClickSelect: Boolean;

    procedure SetHideSelection(AValue: Boolean);
    procedure SetMultiSelect(AValue: Boolean);
    procedure SetMultiSelectStyle(AValue: TMultiSelectStyle);
    procedure SetRowSelect(AValue: Boolean);
  public
    constructor Create(ATreeView: TfrCustomTreeView); override;
    procedure Assign(Source: TPersistent); override;
  published
    property HideSelection: Boolean read FHideSelection write SetHideSelection default True;
    property MultiSelect: Boolean read FMultiSelect write SetMultiSelect default False;
    property MultiSelectStyle: TMultiSelectStyle read FMultiSelectStyle write SetMultiSelectStyle default [msControlSelect];
    property RightClickSelect: Boolean read FRightClickSelect write FRightClickSelect default False;
    property RowSelect: Boolean read FRowSelect write SetRowSelect default False;
  end;

  TfrTVChangingEvent = procedure(Sender: TObject; Node: TfrTreeViewNode; var AllowChange: Boolean) of object;
  TfrTVChangedEvent = procedure(Sender: TObject; Node: TfrTreeViewNode) of object;
  TfrTVEditingEvent = procedure(Sender: TObject; Node: TfrTreeViewNode; var AllowEdit: Boolean) of object;
  TfrTVEditedEvent = procedure(Sender: TObject; Node: TfrTreeViewNode; var S: string) of object;
  TfrTVExpandingEvent = procedure(Sender: TObject; Node: TfrTreeViewNode; var AllowExpansion: Boolean) of object;
  TfrTVCollapsingEvent = procedure(Sender: TObject; Node: TfrTreeViewNode; var AllowCollapse: Boolean) of object;
  TfrTVExpandedEvent = procedure(Sender: TObject; Node: TfrTreeViewNode) of object;
  TfrTVCompareEvent = procedure(Sender: TObject; Node1, Node2: TfrTreeViewNode; Data: Integer; var Compare: Integer) of object;
  TfrTVHintEvent = procedure(Sender: TObject; const Node: TfrTreeViewNode; var Hint: string) of object;
  TfrTVCustomDrawEvent = procedure(Sender: TfrCustomTreeView; ACanvas: TfrCustomCanvas;
     AViewInfo: TfrTreeViewViewInfo; var DefaultDraw: Boolean) of object;
  TfrTVCustomDrawItemEvent = procedure(Sender: TfrCustomTreeView; ACanvas: TfrCustomCanvas;
    ANodeViewInfo: TfrTreeViewNodeViewInfo; var DefaultDraw: Boolean) of object;
  TfrTVCreateNodeClassEvent = procedure(Sender: TfrCustomTreeView;
    var NodeClass: TfrTreeViewNodeClass) of object;

  { TfrCustomTreeView }

  TfrCustomTreeView = class(TfrScrollingControl, IfrViewInfoTreeView, IfrInplaceEditParent)
  strict private
    FAbsoluteNodesCache: TfrList;
    FAddNodeLockCount: Integer;
    FAutoScrollHelper: TfrTreeViewAutoScrollHelper;
    FDragImages: TDragImageList;
    FDropTarget: TfrTreeViewNode;
    FEditingNode: TfrTreeViewNode;
    FEncoding: TEncoding;
    FFocusedNode: TfrTreeViewNode;
    FHasExpandButtonHotTrack: Boolean;
    FHintHelper: TfrTreeViewHintHelper;
    FImages: TCustomImageList;
    FImagesChangeLink: TChangeLink;
    FInplaceEdit: TfrInplaceEdit;
    FItems: TfrTreeViewNodes;
    FLastExpandedItem: TfrTreeViewNode;
    FMultiSelectHelper: TfrTreeViewMultiSelectHelper;
    FOptionsBehavior: TfrTreeViewOptionsBehavior;
    FOptionsSelection: TfrTreeViewOptionsSelection;
    FOptionsView: TfrTreeViewOptionsView;
    FRightClickNode: TfrTreeViewNode;
    FSelections: TfrList;
    FStateImages: TCustomImageList;
    FStateImagesChangeLink: TChangeLink;
    FTopItem: TfrTreeViewNode;

    FOnCancelEdit: TfrTVChangedEvent;
    FOnChanging: TfrTVChangingEvent;
    FOnChange: TfrTVChangedEvent;
    FOnCreateNodeClass: TfrTVCreateNodeClassEvent;
    FOnCollapsed: TfrTVExpandedEvent;
    FOnCollapsing: TfrTVCollapsingEvent;
    FOnCompare: TfrTVCompareEvent;
    FOnCustomDraw: TfrTVCustomDrawEvent;
    FOnCustomDrawItem: TfrTVCustomDrawItemEvent;
    FOnDeletion: TfrTVExpandedEvent;
    FOnAddition: TfrTVExpandedEvent;
    FOnEditing: TfrTVEditingEvent;
    FOnEdited: TfrTVEditedEvent;
    FOnExpanded: TfrTVExpandedEvent;
    FOnExpanding: TfrTVExpandingEvent;
    FOnGetImageIndex: TfrTVExpandedEvent;
    FOnGetSelectedIndex: TfrTVExpandedEvent;
    FOnHint: TfrTVHintEvent;

    function CanFocusNode(AValue: TfrTreeViewNode): Boolean;
    function CreateSelectionList: TfrList;
    function GetAutoExpand: Boolean;
    function GetChangeDelay: Integer;
    function GetInplaceEdit: TfrInplaceEdit;
    function GetIsAbsoluteNodesCacheValid: Boolean;
    function GetMultiSelect: Boolean;
    function GetMultiSelectStyle: TMultiSelectStyle;
    function GetReadOnly: Boolean;
    function GetRightClickSelect: Boolean;
    function GetRoot: TfrTreeViewNode;
    function GetSelected: TfrTreeViewNode;
    function GetSelectionCount: Integer;
    function GetSelection(AIndex: Integer): TfrTreeViewNode;
    function GetSortType: TSortType;
    function GetToolTips: Boolean;
    function GetTopItem: TfrTreeViewNode;
    function GetViewInfo: TfrTreeViewViewInfo;
    function GetViewInfoChangeStates: TfrTreeViewViewInfoChangeStates;
    procedure ImagesChangeHandler(Sender: TObject);
    procedure InternalSetImages(AValue: TCustomImageList; var AImages: TCustomImageList; var AImagesChangeLink: TChangeLink);
    procedure PopulateNodesFromStringList(AList: TStringList);
  {$IFDEF DELPHI15}
    procedure SetEncoding(AValue: TEncoding);
  {$ENDIF}
    procedure SetDropTarget(AValue: TfrTreeViewNode);
    procedure SetFocusedNode(AValue: TfrTreeViewNode);
    procedure SetHasExpandButtonHotTrack(AValue: Boolean);
    procedure SetImages(AValue: TCustomImageList);
    procedure SetItems(AValue: TfrTreeViewNodes);
    procedure SetLastExpandedItem(AValue: TfrTreeViewNode);
    procedure SetOptionsBehavior(AValue: TfrTreeViewOptionsBehavior);
    procedure SetOptionsSelection(AValue: TfrTreeViewOptionsSelection);
    procedure SetOptionsView(AValue: TfrTreeViewOptionsView);
    procedure SetRightClickNode(AValue: TfrTreeViewNode);
    procedure SetSelected(AValue: TfrTreeViewNode);
    procedure SetStateImages(AValue: TCustomImageList);
    procedure SetTopItem(AValue: TfrTreeViewNode);
    procedure SetViewInfoChangeStates(AValue: TfrTreeViewViewInfoChangeStates);
    procedure StateImagesChangeHandler(Sender: TObject);

  {$IFNDEF FPC}
    procedure CMDrag(var Message: TCMDrag); message CM_DRAG;
  {$ENDIF}
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
  protected
    procedure Changed; override;
    function CreateController: TfrCustomController; override;
    function CreateViewInfo: TfrCustomViewInfo; override;
    procedure DoChangeScale(M: Integer; D: Integer; isDpiChange: Boolean); override;
    procedure FontChanged; override;
    procedure ScrollBarsChange; override;
    procedure WndProc(var Message: TMessage); override;

    procedure BeginAddNode;
    procedure EndAddNode(ANode: TfrTreeViewNode);
    function IsLockAdditionEvent: Boolean;

    procedure Added(ANode: TfrTreeViewNode); virtual;
    function AllowExpandStateChange(ANode: TfrTreeViewNode): Boolean;
    procedure AssignTreeNodes(ASource: TTreeNodes);
    procedure BeforeNodeDelete(ANode: TfrTreeViewNode);
    function CanChange(ANode: TfrTreeViewNode): Boolean; virtual;
    function CanCollapse(ANode: TfrTreeViewNode): Boolean;
    function CanEdit(ANode: TfrTreeViewNode): Boolean;
    function CanExpand(ANode: TfrTreeViewNode): Boolean;
    function CreateNode: TfrTreeViewNode; virtual;
    function CreateNodes: TfrTreeViewNodes; virtual;
    function DisplayRect(ANode: TfrTreeViewNode; ATextOnly: Boolean): TRect; //Need test (compare with TTreeView)
    procedure InvalidateNode(ANode: TfrTreeViewNode);
    function IsNodeSelected(ANode: TfrTreeViewNode): Boolean;
    procedure LoadTreeFromStream(AStream: TStream; AEncoding: TEncoding);
    procedure MultiSelectStyleChanged;
    procedure ResetAbsoluteNodesCache;
    procedure SaveTreeToStream(AStream: TStream; AEncoding: TEncoding);

    procedure DoCancelEdit(ANode: TfrTreeViewNode); virtual;
    procedure DoChange(ANode: TfrTreeViewNode); virtual;
    procedure DoCollapsed(ANode: TfrTreeViewNode); virtual;
    function DoCollapsing(ANode: TfrTreeViewNode): Boolean; virtual;
    function DoCustomDraw(ACanvas: TfrCustomCanvas): Boolean; virtual;
    function DoCustomDrawItem(ANodeViewInfo: TfrTreeViewNodeViewInfo; ACanvas: TfrCustomCanvas): Boolean; virtual;
    procedure DoDeletion(ANode: TfrTreeViewNode); virtual;
    procedure DoEdited(ANode: TfrTreeViewNode; var AText: string); virtual;
    function DoEditing(ANode: TfrTreeViewNode): Boolean; virtual;
    procedure DoExpanded(ANode: TfrTreeViewNode); virtual;
    function DoExpanding(ANode: TfrTreeViewNode): Boolean; virtual;
    procedure DoGetImageIndex(ANode: TfrTreeViewNode); virtual;
    procedure DoGetSelectedIndex(ANode: TfrTreeViewNode); virtual;
    procedure DoHint(const ANode: TfrTreeViewNode; var AHint: string); virtual;

    // IfrViewInfoTreeView
    function CustomDraw(ACanvas: TfrCustomCanvas): Boolean;
    function GetHideSelection: Boolean;
    function GetHotTrack: Boolean;
    function GetImageHeight: Integer;
    function GetImageWidth: Integer;
    function GetIndent: Integer;
    function GetNodeByIndex(AIndex: Integer): TfrTreeViewNode;
    function GetRightClickNode: TfrTreeViewNode;
    function GetRootNode: TfrTreeViewNode;
    function GetRowSelect: Boolean;
    function GetShowButtons: Boolean;
    function GetShowLines: Boolean;
    function GetShowRoot: Boolean;
    function GetStateImageHeight: Integer;
    function GetStateImageWidth: Integer;

    // IfrInplaceEditParent
    function IfrInplaceEditParent.GetCanvas = GetCanvas;
    function GetContentBounds: TRect;
    function IfrInplaceEditParent.GetPainter = GetPainter;
    function GetWinControl: TWinControl;
    procedure IfrInplaceEditParent.FinishEditing = FinishTextEditing;

    // drag and drop
    procedure DoEndDrag(Target: TObject; X, Y: Integer); override;
    procedure DoStartDrag(var DragObject: TDragObject); override;
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean); override;
    function GetDragImages: TDragImageList; override;

    // editing text
    function StartTextEditing(ANode: TfrTreeViewNode): Boolean;
    procedure FinishTextEditing(ANeedSave: Boolean = True);

    //Options Behevior
    property AutoExpand: Boolean read GetAutoExpand;
    property ChangeDelay: Integer read GetChangeDelay;
    property HotTrack: Boolean read GetHotTrack;
    property ReadOnly: Boolean read GetReadOnly;
    property SortType: TSortType read GetSortType;
    property ToolTips: Boolean read GetToolTips;

    //OptionsView
    property Indent: Integer read GetIndent;
    property ShowButtons: Boolean read GetShowButtons;
    property ShowLines: Boolean read GetShowLines;
    property ShowRoot: Boolean read GetShowRoot;

    //OptionsSelection
    property HideSelection: Boolean read GetHideSelection;
    property MultiSelect: Boolean read GetMultiSelect;
    property MultiSelectStyle: TMultiSelectStyle read GetMultiSelectStyle;
    property RightClickSelect: Boolean read GetRightClickSelect;
    property RowSelect: Boolean read GetRowSelect;
    //---

    property AbsoluteNodesCache: TfrList read FAbsoluteNodesCache;
    property EditingNode: TfrTreeViewNode read FEditingNode;
    property FocusedNode: TfrTreeViewNode read FFocusedNode write SetFocusedNode;
    property HasExpandButtonHotTrack: Boolean read FHasExpandButtonHotTrack write SetHasExpandButtonHotTrack;
    property HintHelper: TfrTreeViewHintHelper read FHintHelper;
    property InplaceEdit: TfrInplaceEdit read GetInplaceEdit;
    property IsAbsoluteNodesCacheValid: Boolean read GetIsAbsoluteNodesCacheValid;
    property LastExpandedItem: TfrTreeViewNode read FLastExpandedItem write SetLastExpandedItem;
    property MultiSelectHelper: TfrTreeViewMultiSelectHelper read FMultiSelectHelper;
    property OptionsBehavior: TfrTreeViewOptionsBehavior read FOptionsBehavior write SetOptionsBehavior;
    property OptionsSelection: TfrTreeViewOptionsSelection read FOptionsSelection write SetOptionsSelection;
    property OptionsView: TfrTreeViewOptionsView read FOptionsView write SetOptionsView;
    property RightClickNode: TfrTreeViewNode read GetRightClickNode write SetRightClickNode;
    property Root: TfrTreeViewNode read GetRoot;
    property ViewInfo: TfrTreeViewViewInfo read GetViewInfo;
    property ViewInfoChangeStates: TfrTreeViewViewInfoChangeStates read GetViewInfoChangeStates write SetViewInfoChangeStates;

    property OnAddition: TfrTVExpandedEvent read FOnAddition write FOnAddition;
    property OnCancelEdit: TfrTVChangedEvent read FOnCancelEdit write FOnCancelEdit;
    property OnChanging: TfrTVChangingEvent read FOnChanging write FOnChanging;
    property OnChange: TfrTVChangedEvent read FOnChange write FOnChange;
    property OnCollapsed: TfrTVExpandedEvent read FOnCollapsed write FOnCollapsed;
    property OnCollapsing: TfrTVCollapsingEvent read FOnCollapsing write FOnCollapsing;
    property OnCompare: TfrTVCompareEvent read FOnCompare write FOnCompare;
    property OnCustomDraw: TfrTVCustomDrawEvent read FOnCustomDraw write FOnCustomDraw;
    property OnCustomDrawItem: TfrTVCustomDrawItemEvent read FOnCustomDrawItem write FOnCustomDrawItem;
    property OnCreateNodeClass: TfrTVCreateNodeClassEvent read FOnCreateNodeClass write FOnCreateNodeClass;
    property OnDeletion: TfrTVExpandedEvent read FOnDeletion write FOnDeletion;
    property OnEditing: TfrTVEditingEvent read FOnEditing write FOnEditing;
    property OnEdited: TfrTVEditedEvent read FOnEdited write FOnEdited;
    property OnExpanding: TfrTVExpandingEvent read FOnExpanding write FOnExpanding;
    property OnExpanded: TfrTVExpandedEvent read FOnExpanded write FOnExpanded;
    property OnGetImageIndex: TfrTVExpandedEvent read FOnGetImageIndex write FOnGetImageIndex;
    property OnGetSelectedIndex: TfrTVExpandedEvent read FOnGetSelectedIndex write FOnGetSelectedIndex;
    property OnHint: TfrTVHintEvent read FOnHint write FOnHint;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function AlphaSort(ARecurse: Boolean = False): Boolean;
    function CustomSort(ASortProc: TfrTreeViewCompareProc; AData: NativeInt; ARecurse: Boolean = True): Boolean;
    procedure ExpandTo(ANode: TfrTreeViewNode);
    procedure FullCollapse;
    procedure FullExpand;
    function GetNodeAt(X, Y: Integer): TfrTreeViewNode;
    function IsEditing: Boolean;
    procedure MakeVisible(ANode: TfrTreeViewNode);
    procedure SetFocus; override;

    procedure ReadData(AStream: TStream);
    procedure WriteData(AStream: TStream);

    procedure LoadBinFromFile(const AFileName: string);
    procedure LoadBinFromStream(AStream: TStream);
    procedure SaveBinToFile(const AFileName: string);
    procedure SaveBinToStream(AStream: TStream);

    procedure LoadFromFile(const AFileName: string); overload;
    procedure LoadFromFile(const AFileName: string; AEncoding: TEncoding); overload;
    procedure LoadFromStream(AStream: TStream); overload;
    procedure LoadFromStream(AStream: TStream; AEncoding: TEncoding); overload;
    procedure SaveToFile(const AFileName: string); overload;
    procedure SaveToFile(const AFileName: string; AEncoding: TEncoding); overload;
    procedure SaveToStream(AStream: TStream); overload;
    procedure SaveToStream(AStream: TStream; AEncoding: TEncoding); overload;

    // Multiselect
    procedure ClearSelection(AKeepPrimary: Boolean = False);
    procedure Deselect(ANode: TfrTreeViewNode);
    function GetSelections(AList: TList): TfrTreeViewNode; overload;
    function GetSelections(AList: TfrList): TfrTreeViewNode; overload;
    procedure Select(ANode: TfrTreeViewNode; AShift: TShiftState = []); overload;
    procedure Select(const ANodes: array of TfrTreeViewNode); overload;
    procedure Select(ANodes: TList); overload;
    procedure Select(ANodes: TfrList); overload;
    procedure Subselect(ANode: TfrTreeViewNode; AValidate: Boolean = False);
    //

    property DropTarget: TfrTreeViewNode read FDropTarget write SetDropTarget;
    property Images: TCustomImageList read FImages write SetImages;
    property Items: TfrTreeViewNodes read FItems write SetItems;
    property Selected: TfrTreeViewNode read GetSelected write SetSelected;
    property SelectionCount: Integer read GetSelectionCount;
    property Selections[AIndex: Integer]: TfrTreeViewNode read GetSelection;
    property StateImages: TCustomImageList read FStateImages write SetStateImages;
    property TopItem: TfrTreeViewNode read GetTopItem write SetTopItem;   // Need test (compare with TTreeView)
  end;

  TfrTreeView = class(TfrCustomTreeView)
  protected
    procedure DefineProperties(Filer: TFiler); override;
  published
    property Align;
    property Anchors;
    property BiDiMode;
  {$IFDEF FPC}
    property BorderSpacing;
  {$ENDIF}
    property BorderStyle;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property Images;
    property Items;
    property OptionsBehavior;
    property OptionsSelection;
    property OptionsView;
    property ParentBiDiMode;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property StateImages;
    property TabOrder;
    property TabStop default True;
  {$IFNDEF FPC}
    property Touch;
  {$ENDIF}
    property Visible;
    property OnAddition;
    property OnCancelEdit;
    property OnChange;
    property OnChanging;
    property OnClick;
    property OnCollapsed;
    property OnCollapsing;
    property OnCompare;
    property OnContextPopup;
    property OnCreateNodeClass;
    property OnCustomDraw;
    property OnCustomDrawItem;
    property OnDblClick;
    property OnDeletion;
    property OnDragDrop;
    property OnDragOver;
    property OnEdited;
    property OnEditing;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnExpanding;
    property OnExpanded;
  {$IFNDEF FPC}
    property OnGesture;
  {$ENDIF}
    property OnGetImageIndex;
    property OnGetSelectedIndex;
    property OnHint;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
  {$IFNDEF FPC}
    property OnMouseActivate;
  {$ENDIF}
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

  TfrTreeForEachNodeProc = function(ANode: TfrTreeViewNode; AData: Pointer): Boolean;

  function frTreeForEach(ARoot: TfrTreeViewNode; AProc: TfrTreeForEachNodeProc; AData: Pointer): Integer; overload;
  function frTreeForEach(ARoot, AStarting, AFinal: TfrTreeViewNode; AProc: TfrTreeForEachNodeProc; AData: Pointer): Integer; overload;
  function frTreeForEach(ARoot, AStarting, AFinal: TfrTreeViewNode; AProc: TfrTreeForEachNodeProc; AData: Pointer; out AVisibleCount: Integer): Integer; overload;
  function frTreeForEach(ARoot, AStarting, AFinal: TfrTreeViewNode; AProc: TfrTreeForEachNodeProc; AData: Pointer; AOnlyVisible: Boolean; out AVisibleCount: Integer): Integer; overload;

implementation
end.
