{******************************************}
{                                          }
{          FastReport VCL/FMX/LCL          }
{              Core Library                }
{                                          }
{         Copyright (c) 1998-2022          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frDPIAwareUtils;

interface

{$I frVer.inc}

uses
{$IFNDEF FPC}
  Windows, Messages,
{$ENDIF}
  SysUtils,
{$IFDEF FPC}
  Types, LazarusPackageIntf,
{$ENDIF}
  Classes, Graphics, Controls,
  frTypes;

const
  FR_DefaultPPI = 96;
  FRX_WM_DPICHANGED = $02E0;
  FRX_WM_DPICHANGED_AFTERPARENT = $02E3;

type
  DPI_AWARENESS_CONTEXT_ = record
    unused_: Integer;
  end;

  FRX_DPI_AWARENESS_CONTEXT = ^DPI_AWARENESS_CONTEXT_;
{$IFNDEF FPC}
  TfrxGetDpiForWindow = function(hwnd: HWND): UINT; stdcall;
  TfrxSetProcessDpiAwarenessContext = function(value: FRX_DPI_AWARENESS_CONTEXT): BOOL; stdcall;
  TfrxSetThreadDpiAwarenessContext = function(value: FRX_DPI_AWARENESS_CONTEXT): BOOL; stdcall;
  TfrxGetThreadDpiAwarenessContext = function(): FRX_DPI_AWARENESS_CONTEXT; stdcall;
{$ENDIF}

const
  FRX_DPI_AWARENESS_CONTEXT_UNAWARE: FRX_DPI_AWARENESS_CONTEXT = FRX_DPI_AWARENESS_CONTEXT(-1);
  FRX_DPI_AWARENESS_CONTEXT_SYSTEM_AWARE: FRX_DPI_AWARENESS_CONTEXT = FRX_DPI_AWARENESS_CONTEXT(-2);
  FRX_DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE: FRX_DPI_AWARENESS_CONTEXT = FRX_DPI_AWARENESS_CONTEXT(-3);
  FRX_DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2: FRX_DPI_AWARENESS_CONTEXT = FRX_DPI_AWARENESS_CONTEXT(-4);
  FRX_DPI_AWARENESS_CONTEXT_UNAWARE_GDISCALED: FRX_DPI_AWARENESS_CONTEXT = FRX_DPI_AWARENESS_CONTEXT(-5);

function frxGetDpiForWindow(aHwnd: NativeInt): UINT;
function frxSetProcessDpiAwarenessContext(value: FRX_DPI_AWARENESS_CONTEXT): LongBool;
function frxSetThreadDpiAwarenessContext(value: FRX_DPI_AWARENESS_CONTEXT): LongBool;
function frxGetThreadDpiAwarenessContext: FRX_DPI_AWARENESS_CONTEXT;

function frGetSystemDPI: Integer;

function frApplyPPI(AValue: Integer; ANewPPI: Integer; ACurrentPPI: Integer = FR_DefaultPPI): Integer; overload;
function frApplyPPI(AValue: TSize; ANewPPI: Integer; ACurrentPPI: Integer = FR_DefaultPPI): TSize; overload;
function frApplyPPI(AValue: TPoint; ANewPPI: Integer; ACurrentPPI: Integer = FR_DefaultPPI): TPoint; overload;
function frApplyPPI(AValue: TRect; ANewPPI: Integer; ACurrentPPI: Integer = FR_DefaultPPI): TRect; overload;

function frRevertPPI(AValue: Integer; ANewPPI: Integer; ACurrentPPI: Integer = FR_DefaultPPI): Integer; overload;
function frRevertPPI(AValue: TSize; ANewPPI: Integer; ACurrentPPI: Integer = FR_DefaultPPI): TSize; overload;
function frRevertPPI(AValue: TPoint; ANewPPI: Integer; ACurrentPPI: Integer = FR_DefaultPPI): TPoint; overload;
function frRevertPPI(AValue: TRect; ANewPPI: Integer; ACurrentPPI: Integer = FR_DefaultPPI): TRect; overload;

function frApplyPPIBySystemDPI(AValue: Integer; ANewPPI: Integer): Integer; overload;
function frApplyPPIBySystemDPI(AValue: TSize; ANewPPI: Integer): TSize; overload;
function frApplyPPIBySystemDPI(AValue: TPoint; ANewPPI: Integer): TPoint; overload;
function frApplyPPIBySystemDPI(AValue: TRect; ANewPPI: Integer): TRect; overload;

function frRevertPPIBySystemDPI(AValue: Integer; ANewPPI: Integer): Integer; overload;
function frRevertPPIBySystemDPI(AValue: TSize; ANewPPI: Integer): TSize; overload;
function frRevertPPIBySystemDPI(AValue: TPoint; ANewPPI: Integer): TPoint; overload;
function frRevertPPIBySystemDPI(AValue: TRect; ANewPPI: Integer): TRect; overload;

function frCreateFontForDefaultDPI: TFont; overload;
function frCreateFontForDefaultDPI(ASize: Integer): TFont; overload;
function frGetFontHeight(AFontSize: Integer; ANewPPI: Integer; ACurrentPPI: Integer = FR_DefaultPPI): Integer;
function frGetFontHeightForDefaultDPI(AFontSize: Integer; ADesignDPI: Integer = FR_DefaultPPI): Integer;
procedure frScaleFont(AFont: TFont; M, D: Integer);

implementation
end.
