{******************************************}
{                                          }
{          FastReport VCL/FMX/LCL          }
{              Core Library                }
{         GZIP compress/decompress         }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{             Fast Reports, Inc.           }
{                                          }
{******************************************}

{$IFNDEF FMX}
unit frGZip;
{$ENDIF}

interface

{$I frVer.inc}

uses
{$IFNDEF FMX}
  {$IFDEF FPC}
    frZLib,
  {$ELSE}
    ZLib,
  {$ENDIF}
  Classes, SysUtils, frCore;
{$ELSE}
  System.Classes, System.SysUtils, FMX.frCore, ZLib;
{$ENDIF}


type
  /// <summary>
  ///   The compression level.
  /// </summary>
  TfrCompressionLevel = (gzNone, gzFastest, gzDefault, gzMax);

/// <summary>
///   Method compresses the Source stream and saves compressed data to the Dest
///   stream. gzip algorithm is used for compression. Optional parameter
///   FileName is used to store the file name into gzip header.
/// </summary>
/// <param name="Source">
///   Source stream
/// </param>
/// <param name="Dest">
///   Destination stream
/// </param>
/// <param name="Compression">
///   Compression level
/// </param>
/// <param name="FileNameW">
///   Used to store the file name into gzip header
/// </param>
procedure frCompressStream(Source, Dest: TStream;
  Compression: TfrCompressionLevel = gzDefault; {$IFDEF DELPHI12}FileNameW{$ELSE}FileName{$ENDIF}: String = '');
/// <summary>
///   Method decompresses the Source stream and saves uncompressed data to the
///   Dest stream. It returns the file name from gzip header.
/// </summary>
/// <param name="Source">
///   Source stream
/// </param>
/// <param name="Dest">
///   Destination stream
/// </param>
function frDecompressStream(Source, Dest: TStream): AnsiString;
/// <summary>
///   Method compresses the Source stream and saves compressed data to the Dest
///   stream. Deflate algorithm is used for compression.
/// </summary>
/// <param name="Source">
///   Source stream
/// </param>
/// <param name="Dest">
///   Destination stream
/// </param>
/// <param name="Compression">
///   Compression level
/// </param>
procedure frDeflateStream(Source, Dest: TStream;
  Compression: TfrCompressionLevel = gzDefault);
/// <summary>
///   Method decompresses the Source stream and saves uncompressed data to the
///   Dest stream. Inflate algorithm is used for decompression.
/// </summary>
/// <param name="Source">
///   Source stream
/// </param>
/// <param name="Dest">
///   Destination stream
/// </param>
procedure frInflateStream(Source, Dest: TStream);

implementation
end.
