{******************************************}
{                                          }
{          FastReport VCL/FMX/LCL          }
{              Core Library                }
{           Resources management           }
{                                          }
{         Copyright (c) 1998-2022          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

{$IFNDEF FMX}
unit frResources;
{$ENDIF}

interface

{$I frVer.inc}

uses
{$IFNDEF FMX}
  Types, SysUtils, Classes,
  {$IFNDEF FPC} Windows, {$ELSE} LCLType, {$ENDIF}
  Forms, Controls, Buttons, ImgList, Graphics,
  frCore, frGZip, frCoreClasses, frGraphics, frXML;
{$ELSE}
  System.Types, System.SysUtils, System.Classes,
  FMX.frCore, FMX.frCoreClasses, FMX.frXML;
{$ENDIF}

type
  TfrStringResources = class;

  { TfrStringResourcesChangedListener }

  IfrStringResourcesChangedListener = interface
  ['{17F117AE-5105-40CB-82ED-02F1CA00348A}']
    procedure ResourcesChanged(AResources: TfrStringResources);
  end;

  { TfrHelpTopics }

  TfrHelpTopic = record
    Sender: string;
    Topic: string;
  end;

  TfrHelpTopics = array of TfrHelpTopic;

  { TfrStringResources }

  TfrStringResources = class
  public type
    TAddStringResourceProc = procedure(const AName: string; const AValue: string) of object;
    TApplyLocalizationProc = procedure(const AResources: TfrStringResources);
  strict private
    class var
      FRegistredLanguages: TfrNamedObjectDictionary;
  private
    FCodePage: Integer;
    FHasChanges: Boolean;
    FHelpFile: string;
    FHelpTopics: TfrHelpTopics;
    FLanguage: string;
    FLockCount: Integer;
    FLiseners: TInterfaceList;
    FResNames: TfrStringDictionary;
    FValues: TfrStringDictionary;
    procedure SetLanguage(const Value: string);
  protected
    procedure Changed;
    function GetHelpFileName: string; virtual;
    procedure UpdateResources; virtual;

    procedure RegisterHelpTopic(const ASender, ATopic: string); overload;
    procedure RegisterHelpTopic(const ATopic: TfrHelpTopic); overload;
    procedure UnregisterHelpTopic(const ASender: string);

    class procedure Initialize; static;
    class procedure Finalize; static;

    property CodePage: Integer read FCodePage;
    property Values: TfrStringDictionary read FValues;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    function Get(const AName: string): string; overload;
    function Get(const ID: Integer): string; overload;
    procedure Add(const AName, AValue: string);
    procedure AddOrSet(const AName: string; const AValue: string);
    procedure AddStrings(const AValues: string);
    procedure AddXML(const AValue: AnsiString);
    procedure AddFormResource(AInstance: TfrHandle; const AResourceName: string);
    procedure Clear;
    procedure Help(Sender: TObject); overload;
    procedure LoadFromFile(const AFileName: string);
    procedure LoadFromStream(AStream: TStream); overload;
    class procedure LoadFromStream(AStream: TStream; AddProc: TAddStringResourceProc;
      var FCodePage: Integer; var FLanguage: string); overload;

    procedure BeginUpdate;
    procedure EndUpdate;
    function IsUpdateLocked: Boolean;

    procedure AddListener(const AListener: IfrStringResourcesChangedListener);
    procedure RemoveListener(const AListener: IfrStringResourcesChangedListener);

    class procedure Register(const ALanguage: string; AProc: TApplyLocalizationProc); static;
    class procedure Unregister(const ALanguage: string; AProc: TApplyLocalizationProc); static;
    class function RegistredLanguages: TStringDynArray; static;

    property HelpFile: string read FHelpFile write FHelpFile;
    property Language: string read FLanguage write SetLanguage;
  end;

  { TfrStringResourcesChangedListener }

  TfrStringResourcesChangedListener = class abstract(TfrInterfacedObject, IfrStringResourcesChangedListener)
  protected
    // IfrStringResourcesChangedListener
    procedure ResourcesChanged(AResources: TfrStringResources); virtual; abstract;
  end;

{$IFNDEF FMX}
  { TfrImageResources }

  TfrImageResourcesAssignProc = procedure(AImages: TBitmap; AClear: Boolean = False) of object;

  TfrImageResources = class
  private
    FImages: TStringList;
    FImagesPPI: Integer;
    procedure SetImagesPPI(AValue: Integer);
  protected
    procedure AssignImages(ABitmap: TBitmap; DX, DY: Integer; ANormal, ADisabled: TImageList; AClear: Boolean = False; PPI: Integer = 0);
    procedure ClearFields; virtual;
    function CheckImageList(AInstance: TfrHandle; var AImageList: TImageList; const AListName, AResName: string;
      AAssignProc: TfrImageResourcesAssignProc; PPI: Integer): TImageList; overload; virtual;
    function CheckImageList(AInstance: TfrHandle; var AImageList: TImageList; const AListName, AResName: string;
      AAssignProc: TfrImageResourcesAssignProc = nil; const AWidth: Integer = DefaultToolBarImageWidth;
      const AHeigh: Integer = DefaultToolBarImageHeight; PPI: Integer = 0): TImageList; overload; virtual;
    function GetImages(const AName: string; APPIScale: Integer; AImageWidth, AImageHeight: Integer): TImageList;
    procedure LoadFromResourceName(AInstance: TfrHandle; const AResName: string; AResType: PChar; AAssignProc: TfrImageResourcesAssignProc; PPI: Integer = 0);
    procedure SetSpeedButtonGlyphFromImageList(AButton: TSpeedButton; AIndex: Integer; AImages, ADisabledImages: TImageList; PPI: Integer = 0); //delete fix for Transperent when WinLaz fix it.

    property Images: TStringList read FImages;
  public
    constructor Create;
    destructor Destroy; override;

    property ImagesPPI: Integer read FImagesPPI write SetImagesPPI;
  end;
{$ENDIF}

procedure frDisplayHHTopic(AHandle: TfrHandle; const ATopic: string);
function frStringResources: TfrStringResources;
function frGet(ID: Integer): String;

implementation
end.
