
{******************************************}
{                                          }
{          FastReport VCL/FMX/LCL          }
{              Core Library                }
{                                          }
{         Copyright (c) 1998-2022          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

{$IFNDEF FMX}
unit frUtils;
{$ENDIF}

interface

{$I frVer.inc}

uses
{$IFDEF DELPHI16}
  System.UITypes,
{$ENDIF}
{$IFNDEF FMX}
  SysUtils, Types,
  frTypes,
  Graphics,
  frXML,
  {$IFNDEF FPC}
    Windows,
  {$ENDIF}
{$ELSE}
  System.SysUtils, System.Types,
  FMX.frTypes,
  FMX.frXML,
{$ENDIF}
{$IFDEF FPC}
  LCLType,
{$ENDIF}
  Classes;

type
  TfrSortPointerCompare = function(AItem1, AItem2: Pointer): Integer of object;
  TfrListSortPointerCompare = function(AItem1, AItem2: Pointer): Integer;
  {$IFNDEF FPC}
    TfrListSortCompareFunc = reference to function (Item1, Item2: Pointer): Integer;
  {$ENDIF}

function frGCD(A, B: Integer): Integer;
procedure frReduceFraction(var A, B: Integer);

procedure frStringToStream(const S: string; AStream: TStream);
function frToStringList(const S: string; const ADelimiter: Char): TStrings;
function frNormalizeScientificNotation(const S: string): string;

function frLimit(const AValue, AMinValue, AMaxValue: Single): Single; overload;
function frLimit(const AValue, AMinValue, AMaxValue: Double): Double; overload;
function frLimit(const AValue, AMinValue, AMaxValue: Integer): Integer; overload;

function frMulDiv(ANumber, ANumerator, ADenominator: Integer): Integer;
function frMax(AValues: array of Integer): Integer; overload;
function frMax(AValues: array of Single): Single; overload;

function frFloatToStr(D: Extended): string;

function frIfColor(AFlag: Boolean; const ATrue: TColor; AFalse: TColor = $1FFFFFFF): TColor;
function frIfInt(AFlag: Boolean; const ATrue: Integer; AFalse: Integer = 0): Integer;
function frIfStr(AFlag: Boolean; const ATrue: string; AFalse: string = ''): string;
function frIfReal(AFlag: Boolean; const ATrue: Extended; AFalse: Extended = 0.0): Extended;

function frFindComponent(AOwner: TComponent; const AName: String): TComponent;

{$IFNDEF FMX}
  function frFileTimeToDateTime(AFileTime: TFileTime): TDateTime;
{$ENDIF}

procedure frSort(AList: PfrPointerArray; AStart, AEnd: Integer; ACompare: TfrSortPointerCompare; ARecursive: Boolean = False); overload;
procedure frSort(AList: PfrPointerArray; AStart, AEnd: Integer; ACompare: TfrListSortPointerCompare; ARecursive: Boolean = False); overload;
{$IFNDEF FPC}
procedure frSort(AList: PfrPointerArray; AStart, AEnd: Integer; ACompare: TfrListSortCompareFunc; ARecursive: Boolean = False); overload;
{$ENDIF}

function frGetRectHeight(const ARect: TRect): Integer;
function frGetRectWidth(const ARect: TRect): Integer;
function frHalfCoordinate(ASize: Integer): Integer;
function frPointIsEqual(const P1, P2: TPoint): Boolean;
function frPointIsZero(APoint: TPoint): Boolean;
function frPointOffset(const APoint: TPoint; AOffsetX, AOffsetY: Integer): TPoint;
function frPointOffsetX(const APoint: TPoint; AOffset: Integer): TPoint;
function frPointOffsetY(const APoint: TPoint; AOffset: Integer): TPoint;
function frRect(ALeft, ATop, AWidth, AHeight: Integer): TRect;
function frRectCombain(const ARect1, ARect2: TRect): TRect;
function frRectContains(const ARect: TRect; const P: TPoint): Boolean;
function frRectGetSize(const ARect: TRect): TSize;
function frRectIsEmpty(const ARect: TRect): Boolean;
function frRectOffsetX(const ARect: TRect; AOffset: Integer): TRect;
function frRectOffsetY(const ARect: TRect; AOffset: Integer): TRect;
function frRectOffset(const ARect: TRect; AOffsetX, AOffsetY: Integer): TRect;
function frRectCenterX(const ARect, ABounds: TRect): TRect; overload;
function frRectCenterX(const ABounds: TRect): Integer; overload;
function frRectCenterY(const ARect, ABounds: TRect): TRect; overload;
function frRectCenterY(const ABounds: TRect): Integer; overload;
function frRectCenter(const ARect, ABounds: TRect): TRect; overload;
function frRectCenter(const ABounds: TRect): TPoint; overload;
function frRectContent(const ARect: TRect; ADelta: Integer): TRect; overload;
function frRectContent(const ARect: TRect; DLeft, DTop, DRight, DBottom: Integer): TRect; overload;
function frRectCrop(const ARect, ABounds: TRect): TRect;
function frRectSetOriginalX(const ARect: TRect; X: Integer): TRect;
function frRectSetOriginalY(const ARect: TRect; X: Integer): TRect;
function frRectSetOriginal(const ARect: TRect; P: TPoint): TRect; overload;
function frRectSetOriginal(const ARect: TRect; X, Y: Integer): TRect; overload;
function frRectInflate(const ARect: TRect; ADelta: Integer): TRect; overload;
function frRectInflate(const ARect: TRect; DX, DY: Integer): TRect; overload;
function frRectInflate(const ARect: TRect; DLeft, DTop, DRight, DBottom: Integer): TRect; overload;
function frSetRectHeight(const ARect: TRect; AHeight: Integer): TRect;
function frSetRectWidth(const ARect: TRect; AWidth: Integer): TRect;

procedure frSwap(var AValue1, AValue2: Integer);

var
  frNullRect: TRect;
  frNullSize: TSize;
  frNullPoint: TPoint;

implementation
end.
