unit uSignFileDemo;

interface

uses
  Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, uDemoMain, ComCtrls, Buttons,
  frxSignatureEngine, frxCertificateList;

type
  TfrmSignFileDemo = class(TfrmDemoMain)
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    DebugLogMemo: TMemo;
    PageControl1: TPageControl;
    DeprecatedTabSheet: TTabSheet;
    RecommendedTabSheet: TTabSheet;
    RecommendedNDTabSheet: TTabSheet;
    GroupBox3: TGroupBox;
    DetachedCheckBox: TCheckBox;
    ChainCheckBox: TCheckBox;
    OnlyGOSTCheckBox: TCheckBox;
    DebugLogCheckBox: TCheckBox;
    PFXCheckBox: TCheckBox;
    BESRadioButton: TRadioButton;
    TRadioButton: TRadioButton;
    XLongTime1RadioButton: TRadioButton;
    SignByCryptoProSDKButton: TButton;
    SignByCryptoAPIAndCryptoProSDKButton: TButton;
    TimeStampGroupBox: TGroupBox;
    Label8: TLabel;
    Label9: TLabel;
    Label7: TLabel;
    TSAServerEdit: TEdit;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    SubjectEdit: TEdit;
    IssuerEdit: TEdit;
    PasswordEdit: TEdit;
    DateFormatEdit: TEdit;
    NotBeforeEdit: TEdit;
    NotAfterEdit: TEdit;
    IgnoreCaseCheckBox: TCheckBox;
    SignByCryptoAPIButton: TButton;
    PFXButton: TButton;
    PFXLabel: TLabel;
    SignatureButton: TButton;
    MessageButton: TButton;
    SignatureLabel: TLabel;
    MessageLabel: TLabel;
    PropertiesMemo: TMemo;
    GroupBox1: TGroupBox;
    Button1: TButton;
    GroupBox4: TGroupBox;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    procedure SignByCryptoAPIButtonClick(Sender: TObject);
    procedure MessageButtonClick(Sender: TObject);
    procedure SignatureButtonClick(Sender: TObject);
    procedure PFXButtonClick(Sender: TObject);
    procedure BESRadioButtonClick(Sender: TObject);
    procedure SignatureCertificateButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
  private
    FSD: TfrxSignatureData;
    FCertificate: TfrxCertificate;
    FCD: TfrxCertificateData;
    FStatus: TSignatureSatus;

    procedure UpdatePropertiesMemo;
    procedure Sign;
  public
    { Public declarations }
  end;

var
  frmSignFileDemo: TfrmSignFileDemo;

implementation

{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

uses
  frxClass, frxSignature, frxFileSignature, frxSignatureErrorDialog, frxCades,
  frxSignatureEngineCrypto, frxSignatureCertificateDialog, frxRes,
  frxSignaturePropertiesDialog, frxCertificateDialog;

function IsEqualFiles(const File1, File2: TFileName): Boolean;
var
  ms1, ms2: TMemoryStream;
begin
  ms1 := TMemoryStream.Create;
  try
    ms1.LoadFromFile(File1);
    ms2 := TMemoryStream.Create;
    try
      ms2.LoadFromFile(File2);
      Result := (ms1.Size = ms2.Size) and
                CompareMem(ms1.Memory, ms2.Memory, ms1.Size);
    finally
      ms2.Free;
    end;
  finally
    ms1.Free;
  end
end;

procedure TfrmSignFileDemo.BESRadioButtonClick(Sender: TObject);
begin
  TimeStampGroupBox.Visible := Sender <> BESRadioButton;
end;

procedure TfrmSignFileDemo.Button2Click(Sender: TObject);
begin
  if SignaturePropertiesDialog(FSD) = mrOk then
    UpdatePropertiesMemo;
end;

procedure TfrmSignFileDemo.Button3Click(Sender: TObject);
begin
  if CertificateDialog(FCertificate, FCD) = mrOK then
    UpdatePropertiesMemo;
end;

procedure TfrmSignFileDemo.Button4Click(Sender: TObject);
begin
  Sign;
end;

procedure TfrmSignFileDemo.Button5Click(Sender: TObject);
begin
  FSD := frxSignatureData(sfCADES_BES, simFastReport, '', True, False);
  FCD := frxCertificateDataPFX('Data\JohnDoe.pfx', '123');
  FStatus := frxGetCertificate(FCD, FCertificate);
  if FStatus <> ssOK then
    SignatureErrorDialog(FStatus, nil, [mbOK])
  else
    Sign;
end;

procedure TfrmSignFileDemo.Button6Click(Sender: TObject);
begin
  FSD := frxSignatureDataPDF(sfCADES_T, simCryptoAPI, 'http://timestamp.comodoca.com/rfc3161');
  FCD := frxCertificateDataStore('', '');
  FStatus := frxGetCertificate(FCD, FCertificate);
  if FStatus <> ssOK then
    SignatureErrorDialog(FStatus, nil, [mbOK])
  else
    Sign;
end;

procedure TfrmSignFileDemo.FormCreate(Sender: TObject);
begin
  inherited;
  rePadding(Self);
  FSD := frxSignatureData(sfCADES_BES, simFastReport, '', True, False);
  FCD := frxCertificateDataStore('', '');

  UpdatePropertiesMemo;
end;

procedure TfrmSignFileDemo.FormDestroy(Sender: TObject);
begin
  inherited;
  FCertificate.Free;
end;

procedure TfrmSignFileDemo.MessageButtonClick(Sender: TObject);
begin
  OpenDialog1.Filter := 'Any file|*.*';
  if OpenDialog1.Execute then
    MessageLabel.Caption := OpenDialog1.FileName;
end;

procedure TfrmSignFileDemo.PFXButtonClick(Sender: TObject);
begin
  OpenDialog1.Filter := 'All Supported Certificates|*.pfx;*.p12';
  if OpenDialog1.Execute then
    PFXLabel.Caption := OpenDialog1.FileName;
end;

procedure TfrmSignFileDemo.Sign;
var
  FS: TfrxFileSignature;
begin
  DebugLogMemo.Clear;

  FS := TfrxFileSignature.Create(
    MessageLabel.Caption,
    SignatureLabel.Caption,
    FSD,
    FCertificate);
  try
    if FS.Status = ssOK then
      FS.SignFile;
  finally
    if FS.DebugLog <> nil then
      DebugLogMemo.Lines.AddStrings(FS.DebugLog);

    if FS.Status <> ssOK then
      SignatureErrorDialog(FS, [mbOK])
    else
      ShowMessage('Success');

    FS.Free;
  end;
end;

procedure TfrmSignFileDemo.SignatureButtonClick(Sender: TObject);
begin
  SaveDialog1.Filter := 'Any file|*.*';
  if SaveDialog1.Execute then
    SignatureLabel.Caption := SaveDialog1.FileName;
end;

procedure TfrmSignFileDemo.SignatureCertificateButtonClick(Sender: TObject);
begin
  if SignatureCertificateDialog(FSD, FCertificate, FCD) = mrOk then
    UpdatePropertiesMemo;
end;

procedure TfrmSignFileDemo.SignByCryptoAPIButtonClick(Sender: TObject);
var
  Lookup: TCertificateStoreLookup;
  FS: TfrxFileSignature;
  FSO: Integer;
  SignatureType: WORD;
begin
  Lookup := TCertificateStoreLookup.Create;
  Lookup.IgnoreCase := Self.IgnoreCaseCheckBox.Checked;
  Lookup.Issuer := IssuerEdit.Text;
  Lookup.NotBefore := NotBeforeEdit.Text;
  Lookup.NotAfter := NotAfterEdit.Text;
  Lookup.Subject := SubjectEdit.Text;
  Lookup.DateFormat := DateFormatEdit.Text;
  Lookup.CertificatePath := PFXLabel.Caption;

  FSO := FileSignatureOptions(
    DetachedCheckBox.Checked,
    ChainCheckBox.Checked,
    OnlyGOSTCheckBox.Checked,
    DebugLogCheckBox.Checked,
    PFXCheckBox.Checked);

  if      BESRadioButton.Checked then
    SignatureType := CADES_BES
  else if TRadioButton.Checked then
    SignatureType := CADES_T
  else // XLongTime1RadioButton.Checked
    SignatureType := CADES_X_LONG_TYPE_1;

  FS := TfrxFileSignature.Create(
    Lookup,
    MessageLabel.Caption,
    SignatureLabel.Caption,
    AnsiString(PasswordEdit.Text),
    FSO,
    SignatureType,
    TSAServerEdit.Text);

  if      Sender = SignByCryptoAPIButton then
    FS.SignByCryptoAPI
  else if Sender = SignByCryptoProSDKButton then
    FS.SignByCryptoProSDK
  else if Sender = SignByCryptoAPIAndCryptoProSDKButton then
    FS.SignByCryptoAPIStampByCryptoProSDK
    ;

  DebugLogMemo.Clear;
  if FS.DebugLog <> nil then
    DebugLogMemo.Lines.AddStrings(FS.DebugLog);

  if FS.Status <> ssOK then
    SignatureErrorDialog(FS, [mbOK])
  else
    ShowMessage('Success');

  FS.Free;
  Lookup.Free;
end;

procedure TfrmSignFileDemo.UpdatePropertiesMemo;
begin
  PropertiesMemo.Text :=
//        frxResources.Get('sdSignature') + #$D#$A +
    frxSignatureDataDescription(FSD) +
    frxResources.Get('sdCertificate') + #$D#$A +
    frxCertificateDescription(FCertificate);
end;

end.
