unit uSignWithoutDialog;

interface

uses
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  frxClass, StdCtrls, frxExportBaseDialog, frxExportPDF, frxUtils,
  frxTableObject, Buttons, ExtCtrls, uDemoMain, ImgList, ActnList, Menus;

type
  TfrmSignWithoutDialog = class(TfrmDemoMain)
    GroupBox1: TGroupBox;
    Sign1Button: TButton;
    Label2: TLabel;
    Sign2Button: TButton;
    GroupBox2: TGroupBox;
    Label3: TLabel;
    Sign3Button: TButton;
    procedure Button1Click(Sender: TObject);
  private
    procedure SignWithOneSignatureDeprecated(PDFExport: TfrxPDFExport);
    procedure SignWithOneOrMoreSignatureDeprecated(PDFExport: TfrxPDFExport);
    procedure SignWithOneOrMoreSignaturesRecomended(PDFExport: TfrxPDFExport; Report: TfrxReport);

  protected
    function GetCaption: string; override;
{$IFDEF FPC}
public
    constructor Create(AOwner: TComponent); override;
{$ENDIF}
  end;

var
  frmSignWithoutDialog: TfrmSignWithoutDialog;

implementation

uses
  frxSignatureEngine, frxCertificateList, frxSignatureErrorDialog;

{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

{$IFDEF FPC}
constructor TfrmSignWithoutDialog.Create(AOwner: TComponent);
begin
  inherited;
  rePadding(Self);
end;
{$ENDIF}

procedure TfrmSignWithoutDialog.Button1Click(Sender: TObject);
const
  FR3FileName = 'Signatures.fr3';
var
  PDFExport: TfrxPDFExport;
  Report: TfrxReport;
begin
  Report := TfrxReport.Create(nil);
  try
    Report.LoadFromFile(FR3FileName);
    Report.PrepareReport;

    PDFExport := TfrxPDFExport.Create(nil);
    try
      PDFExport.Report := Report;
      PDFExport.ShowDialog := False;
      PDFExport.OpenAfterExport := True;

      if      Sender = Sign1Button then
      begin
        SignWithOneSignatureDeprecated(PDFExport);
        PDFExport.FileName := ExtractFileName(FR3FileName) + '-1.pdf';
      end
      else if Sender = Sign2Button then
      begin
        SignWithOneOrMoreSignatureDeprecated(PDFExport);
        PDFExport.FileName := ExtractFileName(FR3FileName) + '-2.pdf';
      end
      else
      begin
        SignWithOneOrMoreSignaturesRecomended(PDFExport, Report);
        PDFExport.FileName := ExtractFileName(FR3FileName) + '-3.pdf';
      end;

      Report.Export(PDFExport);
    finally
      PDFExport.Free;
    end;
  finally
    Report.Free;
  end;
end;

function TfrmSignWithoutDialog.GetCaption: string;
begin
  Result := 'PDF Sign Without Dialog Demo';
end;

procedure TfrmSignWithoutDialog.SignWithOneOrMoreSignatureDeprecated(PDFExport: TfrxPDFExport);
var
  SD: TSignatureData;
  Status: TSignatureSatus;
begin
  PDFExport.SignatureInfoList.Init;

  SD.Name := 'DigitalSignature1';
  SD.Location := 'Unknown';
  SD.Reason := 'Completely unknown';
  SD.ContactInfo := 'Absolutely unknown';
  SD.CertificatePath := 'JohnDoe.pfx';
  SD.CertificatePassword := '123';

  Status := PDFExport.SignatureInfoList.AddData(SD);
  if Status <> ssOK then
    SignatureErrorDialog(Status, nil, [mbOK]);

  SD.Name := 'DigitalSignature2';
  SD.Location := 'Unknown';
  SD.Reason := 'Completely unknown';
  SD.ContactInfo := 'Absolutely unknown';
  SD.CertificatePath := 'JaneDoe.pfx';
  SD.CertificatePassword := '123';

  Status := PDFExport.SignatureInfoList.AddData(SD);
  if Status <> ssOK then
    SignatureErrorDialog(Status, nil, [mbOK]);
end;

procedure TfrmSignWithoutDialog.SignWithOneOrMoreSignaturesRecomended(PDFExport: TfrxPDFExport; Report: TfrxReport);
var
  DigitalSignatureView: TfrxDigitalSignatureView;
  SD: TfrxSignatureData;
  CD: TfrxCertificateData;
  PSD: TfrxPDFSignatureData;
  Status: TSignatureSatus;
begin
  PDFExport.SignatureInfoList.Init;

  DigitalSignatureView := Report.FindObject('DigitalSignature1') as TfrxDigitalSignatureView;
  SD := frxSignatureDataPDF(sfCADES_BES, simFastReport, '');
  PSD := frxPDFSignatureData(DigitalSignatureView, SD);
  CD := frxCertificateDataPFX('JohnDoe.pfx', '123');

  Status := PDFExport.SignatureInfoList.AddData(PSD, CD);
  if Status <> ssOK then
    SignatureErrorDialog(Status, nil, [mbOK]);

  DigitalSignatureView := Report.FindObject('DigitalSignature2') as TfrxDigitalSignatureView;
  SD := frxSignatureDataPDF(sfCADES_BES, simCryptoAPI, '');
  PSD := frxPDFSignatureData(DigitalSignatureView, SD);
  CD := frxCertificateDataPFX('JaneDoe.pfx', '123');

  Status := PDFExport.SignatureInfoList.AddData(PSD, CD);
  if Status <> ssOK then
    SignatureErrorDialog(Status, nil, [mbOK]);

  DigitalSignatureView := Report.FindObject('DigitalSignature3') as TfrxDigitalSignatureView;
  SD := frxSignatureDataPDF(sfCADES_T, simCryptoAPI, 'http://timestamp.comodoca.com/rfc3161');
  PSD := frxPDFSignatureData(DigitalSignatureView, SD);
  CD := frxCertificateDataStore('', '');

  Status := PDFExport.SignatureInfoList.AddData(PSD, CD);
  if Status <> ssOK then
    SignatureErrorDialog(Status, nil, [mbOK]);
end;

procedure TfrmSignWithoutDialog.SignWithOneSignatureDeprecated(PDFExport: TfrxPDFExport);
begin
  PDFExport.DigitalSignLocation := 'Unknown';
  PDFExport.DigitalSignReason := 'Completely unknown';
  PDFExport.DigitalSignContactInfo := 'Absolutely unknown';
  PDFExport.DigitalSignCertificatePath := 'JohnDoe.pfx';
  PDFExport.DigitalSignCertificatePassword := '123';
end;

end.
