
{******************************************}
{                                          }
{             FastReport VCL               }
{            Server cahce module           }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxServerCache;

{$I frx.inc}

interface

uses
     {$IFNDEF Linux}
     Windows,
     {$ENDIF}
     {$IFDEF LCLGTK2}
     gdk2,
     {$ENDIF}
     Classes, SysUtils, frxUtils, frxServerUtils, frxNetUtils,
     frxVariables, frxClass, frxServerLog, SyncObjs;

type
  TfrxServerCacheItem = class(TCollectionItem)
  private
    FReportName: String;
    FVariables: TfrxVariables;
    FFileName: String;
    FExpTime: TDateTime;
    FSessionId: String;
    FStream: TStream;
    FInternalId: String;
    FFileType: TfrxServerFormat;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
  published
    property ReportName: String read FReportName write FReportName;
    property Variables: TfrxVariables read FVariables write FVariables;
    property FileName: String read FFileName write FFileName;
    property FileType: TfrxServerFormat read FFileType write FFileType;
    property ExpTime: TDateTime read FExpTime write FExpTime;
    property SessionId: String read FSessionId write FSessionId;
    property Stream: TStream read FStream write FStream;
    property InternalId: String read FInternalId write FInternalId;
  end;

  TfrxServerCacheSpool = class(TCollection)
  private
    function GetItems(Index: Integer): TfrxServerCacheItem;
    function EqualVariables(const Var1: TfrxVariables; const Var2: TfrxVariables): Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    property Items[Index: Integer]: TfrxServerCacheItem read GetItems;
    procedure Clear;
  published
    function Add: TfrxServerCacheItem;
    function Insert(Index: Integer): TfrxServerCacheItem;
    function IndexOf(const ReportName: String; const Variables: TfrxVariables; const SessionId: String; const aFileFormat: TfrxServerFormat): Integer;
  end;

  TfrxServerCache = class (TThread)
  protected
    FCACHE_PREFIX: String;
    FCachePath: String;
    FActive: Boolean;
    FLatency: Integer;
    FHeap: TfrxServerCacheSpool;
    FMemoryCache: Boolean;
    FThreadActive: Boolean;
    procedure SetActive(const Value: Boolean);
    procedure CleanUpFiles; virtual;
    procedure RemoveExpired; virtual;
    procedure Execute; override;
  public
    constructor Create(WithInitialization: Boolean = True);
    destructor Destroy; override;
    function GetCachedStream(const Report: TfrxReport; const ReportName: String; const Variables: TfrxVariables; const Id: String): Boolean; overload;
    function GetCachedReportById(const Report: TfrxReport; const Id: String): Boolean;
    function IsReportInCache(const ReportName: String; const Variables: TfrxVariables; const Id: String; const aFileFormat: TfrxServerFormat = sfFRP): Boolean;
    function CopyCacheFileIfExist(const aFileName: String; const ReportName: String; const Variables: TfrxVariables; const Id: String; const aFileFormat: TfrxServerFormat = sfFRP): Boolean;
    procedure Open;
    procedure Close;
    procedure Clear;
    procedure AddReport(const Report: TfrxReport;
      const ReportName: String; const Variables: TfrxVariables; const Id: String; const InternalId: String);
    procedure AddFileFormat(const sStream: TStream; const aFileFormat: TfrxServerFormat;
      const ReportName: String; const Variables: TfrxVariables; const Id: String; const InternalId: String); virtual;

    property Active: Boolean read FActive write SetActive;
    property CachePath: String read FCachePath write FCachePath;
    property DefaultLatency: Integer read FLatency write FLatency;
    property Heap: TfrxServerCacheSpool read FHeap;
    property MemoryCache: Boolean read FMemoryCache write FMemoryCache;
    property CACHE_PREFIX: String read FCACHE_PREFIX write FCACHE_PREFIX;
  end;

  TfrxResultCache = class (TfrxServerCache)  //for caching fp3 export results
  protected
    procedure CleanUpFiles; override;
  public
    constructor Create(aCachePath, aCACHE_PREFIX: String); overload;
    procedure AddFileFormat(const sStream: TStream; const aFileFormat: TfrxServerFormat;
      const ReportName: String; const Variables: TfrxVariables; const Id: String; const InternalId: String); override;
    procedure RemoveExpired; override;
    procedure UpdExpTime(const ReportName: String; const Variables: TfrxVariables; const SessionId: String; const aFileFormat: TfrxServerFormat);
  end;

var
  CacheCS1: TCriticalSection;
  ReportCache: TfrxServerCache;
  ResultCache: TfrxResultCache;

  procedure RemoveAll(path: string);
  function GetFileCount(Dir: string): Integer;

implementation
end.
