
{******************************************}
{                                          }
{             FastReport VCL               }
{               Debug Module               }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxDebug;

interface

uses
  frXML, frxExportMatrix, MemInfo;

{ Profiling routines }

function DbgTimerCreate(Name: string): Integer;
procedure DbgTimerStart(TimerId: Integer);
procedure DbgTimerStop(TimerId: Integer);
function DbgTimerValue(TimerId: Integer): Cardinal;
procedure DbgTimerReset(TimerId: Integer);
function DbgTimerInfo(MainTimerId: Integer): string;

{ Replaces invisible characters with hex-codes. Examples:

  abc def       abc<20>def
  abcdef        abcdef
                <empty>
  abc<20><def   abc<3c>20<3e>def }

function DbgStr(s: string): string;

{ Dumping routines.

  Dumps are a list of XML documents that are created dynamically
  during runtime and are saved at the shutdown. Each XML document
  is associated with a file where it will be saved and a unique id.

  - DbgDumpGet
      Returns an existing or creates a new dump.

  - DbgDump
      Adds to a specified dump a new XML node with information
      about a passed object.

  - DbgDumpSave
      Saves a specified dump to to the associated file.

  - DbgDumpSaveAll
      Saves all dumps. }

type

  TDbgDumpId = LongInt;

  TDbgDump = class
  public

    Id:   TDbgDumpId;
    Path: string;
    Doc:  TfrXMLDocument;

    constructor Create(Id: TDbgDumpId);
    destructor Destroy; override;

  end;

function DbgDumpGet(Id: TDbgDumpId): TDbgDump;
procedure DbgDump(Obj: TObject; Id: TDbgDumpId);
procedure DbgDumpObj(Obj: TObject; Node: TfrXMLItem);
procedure DbgDumpSave(Id: TDbgDumpId);
procedure DbgDumpSaveAll;
procedure DbgDumpToFile(const s: AnsiString; const FileName: string);

{ This routine prints a text to a global file.
  This routine can be used by a few threads.
  Avoid calling this routine because it's very slow. }

procedure DbgPrint(const Fmt: string; const Args: array of const); overload
procedure DbgPrint(const s: AnsiString); overload
procedure DbgPrint(const s: string); overload
procedure DbgPrintln(const Fmt: string; const Args: array of const); overload
procedure DbgPrintln(const s: AnsiString); overload
procedure DbgPrintln(const s: string = ''); overload
procedure DbgPrintTitle(const s: string); overload;
procedure DbgPrintTitle(const Fmt: string; const Args: array of const); overload;
procedure DbgPrintMatrix(m: TfrxIEMatrix; RowFirst, RowLast: Integer);
procedure DbgPrintMatrixObjects(m: TfrxIEMatrix);

{ Writes data from internal buffers to the disk file }

procedure DbgPrintFlush;

{ Logging features. }

type
  TDbgListener = procedure(s: string);

procedure DbgLog(s: string); overload;
procedure DbgLog(Fmt: string; Args: array of const); overload;

procedure DbgAddListener(p: TDbgListener);

{ Memory info }

function DbgGetMemInfo: PROCESS_MEMORY_COUNTERS_EX;

implementation
end.
