
{******************************************}
{                                          }
{             FastReport VCL               }
{           EMF Abstract Export            }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxEMFAbstractExport;

interface

{$I frx.inc}

uses
  Types, Classes, Windows, frxEMFFormat, {The right order: "Windows, frxEMFFormat"}
  Contnrs, Graphics, SysUtils, frBaseGraphicsTypes, frxClass, frxUtils,
  frxAnaliticGeometry;

type
  TDeviceContextData = record
    DeviceDPI: TDoublePoint;
    DeviceTopLeft: TPoint;
    rTopLeft: TfrxPoint;
    Layout: LongWord;
    MapMode: LongWord;
    WindowOrgEx, ViewPortOrgEx, BrushOrgEx: TPoint;
    WindowExtEx, ViewPortExtEx: TSize;
    Pen: TEnhMetaObj;
    Brush: TEnhMetaObj;
    Font: TEnhMetaObj;
    Palette: TEnhMetaObj;
    ColorSpace: TEnhMetaObj;
    ColorAdjustment: TEnhMetaObj;
    PositionCurrent, PositionNext: TPoint;
    BkMode: LongWord;
    MiterLimit: Single;
    BkColor: LongWord;
    TextColor: LongWord;
    SetRop2: LongWord;
    PolyFillMode: LongWord;
    ClipHRgn: HRGN;
    EOFPalette: array of TColor;
    StretchMode: LongWord;
    IsPathBracketOpened: Boolean;
    TextAlignmentMode: LongWord;
    ICMMode: LongWord;
    XForm: TXForm;
    MapperFlags: LongWord;
    iArcDirection: LongWord;
  end;

  TEMFAbstractExport = class;
  TEnhMetaObjArray = array of TEnhMetaObj;

  TDeviceContext = class
  private
    FRotationXForm, FShiftXForm, FScaleXForm: TXForm;
    FIsDecomposited: Boolean;

    function GetLogPenStyle: LongWord;
    function GetPenType: LongWord;
    function GetPenStyle: LongWord;
    function GetPenEndCap: LongWord;
    function GetPenLineJoin: LongWord;
    function GetPenWidth: Extended;
    function GetPenNumEntries: Cardinal;
    function GetPenStyleEntry: TCardinalDynArray;
    function GetMiterLimit: Single;
    function GetPenColor: TColor;

    function GetBrushColor: TColor;
    function GetBrushStyle: LongWord;
    function GetBrushHatch: LongWord;

    function GetTextColor: TColor;
    function GetFontFamily: string;
    function GetFontSize: Integer;
    function GetFontWeight: Integer;
    function GetFontItalic: Boolean;
    function GetFontStrikeOut: Boolean;
    function GetFontCharSet: byte;
    function GetFontUnderline: Boolean;
    function GetFontOrientation: LongInt;
    function GetFontRadian: Extended;
    function GetBkColor: TColor;
    function GetHAlign: TfrxHAlign;
    function GetVAlign: TfrxVAlign;
    procedure SetXForm(const Value: TXForm);
    function GetLineOrientation: LongInt;
    procedure Decomposition;
    function GetXFormScale: TfrxPoint;
    function GetXFormAverageScale: Extended;
    procedure SetClipHRGN(const Value: HRGN);
    function GetDeviceDPI: TDoublePoint;
  protected
    FData: TDeviceContextData;

    procedure DeleteObject(ih: LongWord);
    procedure SelectObject(ih: LongWord; EnhMetaObjArray: TEnhMetaObjArray);


    function MonoBrushAverageColor(ih: LongWord; EnhMetaObjArray: TEnhMetaObjArray): TColor;
    function IndirectBrushAverageColor(ih: LongWord; EnhMetaObjArray: TEnhMetaObjArray): TColor;
    function DIBPatternBrushAverageColor(ih: LongWord; EnhMetaObjArray: TEnhMetaObjArray): TColor;
  public
    destructor Destroy; override;
    procedure CopyFrom(ADC: TObject); virtual;
    procedure Init; virtual;
    function IsCellHeight: Boolean; // In Windows specified with a positive elfLogFont.lfHeight value (negative - point size).
    function BrushAverageColor(ih: LongWord; EnhMetaObjArray: TEnhMetaObjArray): TColor;

    property ClipRgn: HRGN read FData.ClipHRgn write SetClipHRGN;
    property IsPathBracketOpened: Boolean read FData.IsPathBracketOpened;
    property Layout: LongWord read FData.Layout;
    property MapMode: LongWord read FData.MapMode;
    property PolyFillMode: LongWord read FData.PolyFillMode;
    property PositionCurrent: TPoint read FData.PositionCurrent;
    property PositionNext: TPoint read FData.PositionNext;
    property ICMMode: LongWord read FData.ICMMode;
    property MapperFlags: LongWord read FData.MapperFlags;
    property iArcDirection: LongWord read FData.iArcDirection;

    property LogPenStyle: LongWord read GetLogPenStyle;
    property PenType: LongWord read GetPenType;
    property PenStyle: LongWord read GetPenStyle;
    property PenEndCap: LongWord read GetPenEndCap;
    property PenLineJoin: LongWord read GetPenLineJoin;
    property PenWidth: Extended read GetPenWidth;
    property PenColor: TColor read GetPenColor;
    property PenNumEntries: Cardinal read GetPenNumEntries;
    property PenStyleEntry: TCardinalDynArray read GetPenStyleEntry;

    property MiterLimit: Single read GetMiterLimit;
    property SetRop2: LongWord read FData.SetRop2;
    property StretchMode: LongWord read FData.StretchMode;

    property TextAlignmentMode: LongWord read FData.TextAlignmentMode;
    property TextColor: TColor read GetTextColor;
    property BkColor: TColor read GetBkColor;
    property FontFamily: string read GetFontFamily;
    property FontSize: Integer read GetFontSize;
    property FontWeight: Integer read GetFontWeight;
    property FontItalic: Boolean read GetFontItalic;
    property FontCharSet: byte read GetFontCharSet;
    property FontUnderline: Boolean read GetFontUnderline;
    property FontStrikeOut: Boolean read GetFontStrikeOut;
    property FontOrientation: LongInt read GetFontOrientation; // specifies the angle, in tenths of degrees
    property FontRadian: Extended read GetFontRadian;

    property LineOrientation: LongInt read GetLineOrientation; // specifies the angle, in tenths of degrees
    property XFormScale: TfrxPoint read GetXFormScale;
    property XFormAverageScale: Extended read GetXFormAverageScale;

    property BrushColor: TColor read GetBrushColor;
    property BrushStyle: LongWord read GetBrushStyle;
    property BrushHatch: LongWord read GetBrushHatch;
    property Brush: TEnhMetaObj read FData.Brush;

    property BkMode: LongWord read FData.BkMode;

    property DeviceTopLeft: TPoint read FData.DeviceTopLeft;
    property rTopLeft: TfrxPoint read FData.rTopLeft;
    property WindowOrgEx: TPoint read FData.WindowOrgEx;
    property ViewPortOrgEx: TPoint read FData.ViewPortOrgEx;
    property BrushOrgEx: TPoint read FData.BrushOrgEx;
    property WindowExtEx: TSize read FData.WindowExtEx;
    property ViewPortExtEx: TSize read FData.ViewPortExtEx;
    property XForm: TXForm read FData.XForm write SetXForm;

    property VAlign: TfrxVAlign read GetVAlign;
    property HAlign: TfrxHAlign read GetHAlign;

    property DeviceDPI: TDoublePoint read GetDeviceDPI;
  end;

  PWideCharArray = ^TWideCharArray;
  TWideCharArray = array[0..0] of WideChar;

  TMemoExternalParams = record
    IsExternal: Boolean;
    Margins: TfrxRect;
    Width, Height: Extended;
    Shift: TfrxPoint;
  end;

  TEMFFont = class(TFont)
  private
    FPreciseSize: Double;
    FDownSizeFactor: Double;
  public
    property PreciseSize: Double read FPreciseSize write FPreciseSize;
    property DownSizeFactor: Double read FDownSizeFactor write FDownSizeFactor;
  end;

  TEMFAbstractExport = class
  private
    FShowComments: Boolean;
    FFormatted: Boolean;
    FEnableTransform: Boolean;
    FParsing: WideString;
    FDCList: TObjectList; // Device Context List

    procedure SetParsing(const Value: WideString);

    procedure PlayMetaCommand;
    procedure AddLastRecord;
    function ByteToHex(B: byte): string;
    procedure ReadEoFPalette;

    procedure TransformPoint(var DP: TfrxPoint);

    procedure Parse_Poly(Name: string);
    procedure Parse_Poly16(Name: string);
    procedure Parse_PolyPoly(Name: string);
    procedure Parse_PolyPoly16(Name: string);

    function CommentRect(Rect: TRect): string;
    function CommentPoint(Point: TPoint): string;

    function LogToDevX(LX: Extended): Extended;
    function LogToDevY(LY: Extended): Extended;
  protected
    FInStream: TStream;
    FOutStream: TStream;
    FLastRecord: TEMR;
    FEMRList: TObjectList;
    FEMRLastCreated: TEnhMetaObjArray;
    FDC: TDeviceContext;
    FScalingOnly: Boolean;
    FIsPreviousAlpha: Boolean;
    FMetafileDescriptiom: WideString;

    FMEP: TMemoExternalParams;
    procedure CalcMemoExternalParams(Obj: TfrxView);

    procedure InStreamInit;
    function InStreamEOF: Boolean;
    procedure InStreamLastRecord;
    function IsBitmapAsAlpha: Boolean;
    function GlueBitmap32(EMRBitmapAlpha, EMRBitmap: TEMRBitmap): TBitmap;
    function GetNextEnhMetaObj: TEMRBitmapROP;

    function BrushAverageColor(ih: LongWord): TColor; // used in frxEMFtoSVGExport

    procedure Comment(CommentString: string = ''); virtual; {Empty}

    function CommentArray(A: TByteDynArray): string; overload;
    function CommentArray(A: TWordDynArray): string; overload;
    function CommentArray(A: TLongWordDynArray): string; overload;
    function CommentArray(A: TIntegerDynArray): string; overload;
    function CommentArray(A: TDoubleArray; Prec: Integer = 1): string; overload;
    function ArraySum(A: TLongWordDynArray): LongWord;

    function CommentArraySum(A: TLongWordDynArray): string; overload;
    function CommentArraySum(A: TDoubleArray; Prec: Integer = 1): string; overload;

    function PLast: PEnhMetaData;

    procedure DCCreate; virtual; {Empty}
    function FontCreate: TEMFFont; virtual; // Must be overrided for Font.Size

    function LogToDevPoint(LP: TSmallPoint): TfrxPoint; overload;
    function LogToDevPoint(LP: TPoint): TfrxPoint; overload;
    function LogToDevPoint(LP: TfrxPoint): TfrxPoint; overload;
    function LogToDevPoint(LP: TDoublePoint): TfrxPoint; overload;
    function LogToDevPoint(X, Y: Extended): TfrxPoint; overload;

    function LogToDevRect(LR: TRect): TfrxRect;

    function LogToDevSizeX(Value: Extended): Extended;
    function LogToDevSizeY(Value: Extended): Extended;
    function LogToDevSize(Value: Extended): Extended;

    procedure DoEMR_AbortPath; virtual;
    procedure DoEMR_AlphaBlend; virtual;
    procedure DoEMR_AngleArc; virtual;
    procedure DoEMR_Arc; virtual;
    procedure DoEMR_ArcTo; virtual;
    procedure DoEMR_BeginPath; virtual;
    procedure DoEMR_BitBlt; virtual;
    procedure DoEMR_Chord; virtual;
    procedure DoEMR_CloseFigure; virtual;
    procedure DoEMR_ColorCorrectPalette; virtual;
    procedure DoEMR_ColorMatchToTargetW; virtual;
    procedure DoEMR_CreateBrushIndirect; virtual;
    procedure DoEMR_CreateColorSpace; virtual;
    procedure DoEMR_CreateColorSpaceW; virtual;
    procedure DoEMR_CreateDIBPatternBrushPt; virtual;
    procedure DoEMR_CreateMonoBrush; virtual;
    procedure DoEMR_CreatePalette; virtual;
    procedure DoEMR_CreatePen; virtual;
    procedure DoEMR_DeleteColorSpace; virtual;
    procedure DoEMR_DeleteObject; virtual;
    procedure DoEMR_DrawEscape; virtual;
    procedure DoEMR_Ellipse; virtual;
    procedure DoEMR_EndPath; virtual;
    procedure DoEMR_EoF; virtual;
    procedure DoEMR_ExcludeClipRect; virtual;
    procedure DoEMR_ExtCreateFontIndirectW; virtual;
    procedure DoEMR_ExtCreatePen; virtual;
    procedure DoEMR_ExtEscape; virtual;
    procedure DoEMR_ExtFloodFill; virtual;
    procedure DoEMR_ExtSelectClipRgn; virtual;
    procedure DoEMR_ExtTextOutA; virtual;
    procedure DoEMR_ExtTextOutW; virtual;
    procedure DoEMR_FillPath; virtual;
    procedure DoEMR_FillRgn; virtual;
    procedure DoEMR_FlattenPath; virtual;
    procedure DoEMR_ForceUFIMapping; virtual;
    procedure DoEMR_FrameRgn; virtual;
    procedure DoEMR_GDIComment; virtual;
    procedure DoEMR_GLSBoundedRecord; virtual;
    procedure DoEMR_GLSRecord; virtual;
    procedure DoEMR_GradientFill; virtual;
    procedure DoEMR_Header; virtual;
    procedure DoEMR_IntersectClipRect; virtual;
    procedure DoEMR_InvertRgn; virtual;
    procedure DoEMR_LineTo; virtual;
    procedure DoEMR_MaskBlt; virtual;
    procedure DoEMR_ModifyWorldTransform; virtual;
    procedure DoEMR_MoveToEx; virtual;
    procedure DoEMR_NamedEscape; virtual;
    procedure DoEMR_OffsetClipRgn; virtual;
    procedure DoEMR_PaintRgn; virtual;
    procedure DoEMR_Pie; virtual;
    procedure DoEMR_PixelFormat; virtual;
    procedure DoEMR_PLGBlt; virtual;
    procedure DoEMR_PolyBezier; virtual;
    procedure DoEMR_PolyBezier16; virtual;
    procedure DoEMR_PolyBezierTo; virtual;
    procedure DoEMR_PolyBezierTo16; virtual;
    procedure DoEMR_PolyDraw; virtual;
    procedure DoEMR_PolyDraw16; virtual;
    procedure DoEMR_Polygon; virtual;
    procedure DoEMR_Polygon16; virtual;
    procedure DoEMR_Polyline; virtual;
    procedure DoEMR_Polyline16; virtual;
    procedure DoEMR_PolylineTo; virtual;
    procedure DoEMR_PolylineTo16; virtual;
    procedure DoEMR_PolyPolygon; virtual;
    procedure DoEMR_PolyPolygon16; virtual;
    procedure DoEMR_PolyPolyline; virtual;
    procedure DoEMR_PolyPolyline16; virtual;
    procedure DoEMR_PolyTextOutA; virtual;
    procedure DoEMR_PolyTextOutW; virtual;
    procedure DoEMR_RealizePalette; virtual;
    procedure DoEMR_Rectangle; virtual;
    procedure DoEMR_Reserved_69; virtual;
    procedure DoEMR_ResizePalette; virtual;
    procedure DoEMR_RestoreDC; virtual;
    procedure DoEMR_RoundRect; virtual;
    procedure DoEMR_SaveDC; virtual;
    procedure DoEMR_ScaleViewportExtEx; virtual;
    procedure DoEMR_ScaleWindowExtEx; virtual;
    procedure DoEMR_SelectClipPath; virtual;
    procedure DoEMR_SelectObject; virtual;
    procedure DoEMR_SelectPalette; virtual;
    procedure DoEMR_SetArcDirection; virtual;
    procedure DoEMR_SetBkColor; virtual;
    procedure DoEMR_SetBkMode; virtual;
    procedure DoEMR_SetBrushOrgEx; virtual;
    procedure DoEMR_SetColorSpace; virtual;
    procedure DoEMR_SetColorAdjustment; virtual;
    procedure DoEMR_SetDIBitsToDevice; virtual;
    procedure DoEMR_SetICMMode; virtual;
    procedure DoEMR_SetIcmProfileA; virtual;
    procedure DoEMR_SetIcmProfileW; virtual;
    procedure DoEMR_SetLayout; virtual;
    procedure DoEMR_SetLinkedUFIs; virtual;
    procedure DoEMR_SetMapMode; virtual;
    procedure DoEMR_SetMapperFlags; virtual;
    procedure DoEMR_SetMetaRgn; virtual;
    procedure DoEMR_SetMiterLimit; virtual;
    procedure DoEMR_SetPaletteEntries; virtual;
    procedure DoEMR_SetPixelV; virtual;
    procedure DoEMR_SetPolyFillMode; virtual;
    procedure DoEMR_SetRop2; virtual;
    procedure DoEMR_SetStretchBltMode; virtual;
    procedure DoEMR_SetTextAlign; virtual;
    procedure DoEMR_SetTextColor; virtual;
    procedure DoEMR_SetTextJustification; virtual;
    procedure DoEMR_SetViewPortExtEx; virtual;
    procedure DoEMR_SetViewPortOrgEx; virtual;
    procedure DoEMR_SetWindowExtEx; virtual;
    procedure DoEMR_SetWindowOrgEx; virtual;
    procedure DoEMR_SetWorldTransform; virtual;
    procedure DoEMR_SmallTextOut; virtual;
    procedure DoEMR_StartDoc; virtual;
    procedure DoEMR_StretchBlt; virtual;
    procedure DoEMR_StretchDIBits; virtual;
    procedure DoEMR_StrokeAndFillPath; virtual;
    procedure DoEMR_StrokePath; virtual;
    procedure DoEMR_TransparentBlt; virtual;
    procedure DoEMR_TransparentDIB; virtual;
    procedure DoEMR_WidenPath; virtual;

    procedure DoStart; virtual; {Empty}
    procedure DoFinish; virtual; {Empty}
    procedure DoUnknown; virtual;
  public
    constructor Create(InStream, OutStream: TStream);
    destructor Destroy; override;
    procedure PlayMetaFile;

    property ShowComments: Boolean read FShowComments write FShowComments;
    property Formatted: Boolean read FFormatted write FFormatted;
    property Parsing: WideString read FParsing write SetParsing;
    property EnableTransform: Boolean read FEnableTransform write FEnableTransform;
  end;

function CreateMetaStream(const Obj: TfrxView; IsFullOpacity: Boolean = False): TMemoryStream;

implementation
end.

