
{******************************************}
{                                          }
{             FastReport VCL               }
{               EMF Format                 }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxEMFFormat;

interface

{$I frx.inc}

uses
  Windows, Graphics, Classes, Types,
  frxTrueTypeFont, frxTrueTypeCollection, frxUtils;

const
{ Enhanced metafile record types.}
  EMR_Reserved_69 = $00000045;
  EMR_SetLayout =   $00000073;
  EMR_LAYOUT_RTL = 1; // https://msdn.microsoft.com/en-us/library/cc230689.aspx
  EMR_ColorMatchToTargetW = $00000079;
  EMR_CreateColorSpaceW = $0000007A;

  EMR_EMR = $0000007B;
  EMR_Bytes = $0000007C;
  EMR_CreateHandle = $0000007D;
  EMR_LongWords = $0000007E;

  MWT_SET = 4; // https://msdn.microsoft.com/en-us/library/cc230538.aspx

  ETO_NO_RECT = $00000100; // https://msdn.microsoft.com/en-us/library/cc231172.aspx
  ETO_SMALL_CHARS = $00000200;

  // https://docs.microsoft.com/en-us/openspecs/windows_protocols/ms-emf/cd57866b-d2ef-49f0-9dd1-d928ddc02885
  EMR_COMMENT_WINDOWS_METAFILE = $80000001;
  EMR_COMMENT_BEGINGROUP = $00000002;
  EMR_COMMENT_ENDGROUP = $00000003;
  EMR_COMMENT_MULTIFORMATS = $40000004;
  EMR_COMMENT_UNICODE_STRING = $00000040;
  EMR_COMMENT_UNICODE_END = $00000080;

{ Record structures for the enhanced metafile.}
type
//  PEnhMetaHeader = ^TEnhMetaHeader;
//  TEnhMetaHeader = record
//    iType: DWORD;          { Record type EMR_HEADER}
//    nSize: DWORD;          { Record size in bytes.  This may be greater than the sizeof(TEnhMetaHeader). }
//    rclBounds: TRect;      { Inclusive-inclusive bounds in device units}
//    rclFrame: TRect;       { Inclusive-inclusive Picture Frame of metafile in .01 mm units}
//    dSignature: DWORD;     { Signature.  Must be ENHMETA_SIGNATURE.}
//    nVersion: DWORD;       { Version number}
//    nBytes: DWORD;         { Size of the metafile in bytes}
//    nRecords: DWORD;       { Number of records in the metafile}
//    nHandles: Word;        { Number of handles in the handle table Handle index zero is reserved. }
//    sReserved: Word;       { Reserved.  Must be zero.}
//    nDescription: DWORD;   { Number of chars in the unicode description string This is 0 if there is no description string }
//    offDescription: DWORD; { Offset to the metafile description record. This is 0 if there is no description string }
//    nPalEntries: DWORD;    { Number of entries in the metafile palette.}
//    szlDevice: TSize;      { Size of the reference device in pels}
//    szlMillimeters: TSize; { Size of the reference device in millimeters}
//{ Extension 1 }
//    cbPixelFormat: DWORD;  { Size of TPixelFormatDescriptor information This is 0 if no pixel format is set }
//    offPixelFormat: DWORD; { Offset to TPixelFormatDescriptor This is 0 if no pixel format is set }
//    bOpenGL: DWORD;        { True if OpenGL commands are present in the metafile, otherwise FALSE }
// {Extension 2 }
//    szlMicrometers: TSize; { Size of the reference device in micrometers }
//  end;

  TEMRPolyPolygon16 = TEMRPolyPolyline16;

  TEMRPolyBezier16 = TEMRPolyLine16;
  TEMRPolyBezierTo16 = TEMRPolyLine16;
  TEMRPolygon16 = TEMRPolyline16;
  TEMRPolylineTo16 = TEMRPolyLine16;

  TEMRDeleteObject = TEMRSelectObject;

  TEMRRectangle = TEMREllipse;

  TEMRMoveToEx = TEMRLineTo;

  TEMRExtCreateFontIndirectW = TEMRExtCreateFontIndirect;

  TEMRStrokeAndFillPath = TEMRFillPath;
  TEMRStrokePath = TEMRFillPath;

  TEMRExtTextOutW = TEMRExtTextOut;
  TEMRExtTextOutA = TEMRExtTextOut;

  TSetBrushOrgEx = TEMRSetViewportOrgEx;

  TEMRIntersectClipRect = TEMRExcludeClipRect;

  TEMRScaleWindowExtEx = TEMRScaleViewportExtEx;

  TEMRChord = TEMRArc;
  TEMRPie = TEMRArc;
  TEMRArcTo = TEMRArc;

  TEMRPaintRgn = TEMRInvertRgn;

  TEMRPolyTextOutA = TEMRPolyTextOut;
  TEMRPolyTextOutW = TEMRPolyTextOut;

  TEMRDeleteColorSpace = TEMRSelectColorSpace;
  TEMRSetColorSpace = TEMRSelectColorSpace;

  TEMRextEscape = record
    emr: TEMR;
    iEscape: LongInt; // Escape code
    cbEscData: LongInt; // Size of escape data
    EscData: array [0..0] of Byte; // Escape data
  end;
  TEMRDrawEsc = TEMRextEscape;

  TEMRColorCorrectPalette = record
    emr: TEMR;
    ihPalette: LongWord;   // Palette handle index
    nFirstEntry: LongWord; // Index of first entry to correct
    nPalEntries: LongWord; // Number of palette entries to correct
    nReserved: LongWord;   // Reserved
  end;

  TEMRSetIcmProfile = record
    emr: TEMR;
    dwFlags: LongWord; // flags
    cbName: LongWord; // Size of desired profile name
    cbData: LongWord; // Size of raw profile data if attached
    Data: array [0..0] of Byte; // Array size is cbName + cbData
  end;

  TEMRSetLayout = TEMRSetBkMode;

  TEMRPolyBezier = TEMRPolyLine;
  TEMRPolygon = TEMRPolyLine;
  TEMRPolyBezierTo = TEMRPolyLine;
  TEMRPolylineTo = TEMRPolyLine;

  TEMRPolyPolygon = TEMRPolyPolyline;

  TEMRForceUFIMapping = record
    emr: TEMR;
    Checksum: LongWord;
    Index: LongWord;
  end;

  TEMRSmallTextOut = record
    emr: TEMR;
    ptlReference: TPoint;
    nChars: LongWord;
    fuOptions: LongWord;
    iGraphicsMode: LongWord;
    exScale: Single;
    eyScale: Single;
    rclClip: TRect; {optional}
  end;

  TEMRGradientFill = TEMGradientFill;

  TEMRColorMatchToTarget = record
    emr: TEMR;
    dwAction: LongWord;  // CS_ENABLE, CS_DISABLE or CS_DELETE_TRANSFORM
    dwFlags: LongWord;   // flags
    cbName: LongWord;    // Size of desired target profile name
    cbData: LongWord;    // Size of raw target profile data if attached
    Data: array [0..0] of Byte; // Array size is cbName + cbData
  end;

//  PBytes = ^TBytes;
  TBytes = array[0..0] of Byte;
  TLongWords = array[0..0] of LongWord;
  {$NODEFINE TEnhMetaData}
  (*$HPPEMIT '  struct TEnhMetaData;'*)
  PEnhMetaData = ^TEnhMetaData;
  TEnhMetaData = record
    case LongWord of
      EMR_EMR: (EMR: TEMR);
      EMR_Bytes: (Bytes: TBytes);
      EMR_LongWords: (LongWords: TLongWords);
      EMR_Header: (Header: TEnhMetaHeader); // -- TEnhMetaHeader
      EMR_PolyBezier: (PolyBezier: TEMRPolyBezier);
      EMR_Polygon: (Polygon: TEMRPolygon);
      EMR_Polyline: (Polyline: TEMRPolyline);
      EMR_PolyBezierTo: (PolyBezierTo: TEMRPolyBezierTo);
      EMR_PolylineTo: (PolylineTo: TEMRPolylineTo);
      EMR_PolyPolyline: (PolyPolyline: TEMRPolyPolyline);
      EMR_PolyPolygon: (PolyPolygon: TEMRPolyPolygon);
      EMR_SetWindowExtEx: (SetWindowExtEx: TEMRSetViewportExtEx);
      EMR_SetWindowOrgEx: (SetWindowOrgEx: TEMRSetViewportOrgEx);
      EMR_SetViewPortExtEx: (SetViewPortExtEx: TEMRSetViewportExtEx);
      EMR_SetViewPortOrgEx: (SetViewPortOrgEx: TEMRSetViewportOrgEx);
      EMR_SetBrushOrgEx: (SetBrushOrgEx: TSetBrushOrgEx);
      EMR_EoF: (EoF: TEMREoF);
      EMR_SetPixelV: (SetPixelV: TEMRSetPixelV);
      EMR_SetMapperFlags: (SetMapperFlags: TEMRSetMapperFlags);
      EMR_SetMapMode: (SetMapMode: TEMRSetMapMode);
      EMR_SetBkMode: (SetBkMode: TEMRSetBkMode);
      EMR_SetPolyFillMode: (SetPolyFillMode: TEMRSetPolyFillMode);
      EMR_SetRop2: (SetRop2: TEMRSetRop2);
      EMR_SetStretchBltMode: (SetStretchBltMode: TEMRSetStretchBltMode);
      EMR_SetTextAlign: (SetTextAlign: TEMRSetTextAlign);
      EMR_SetColorAdjustment: (SetColorAdjustment: TEMRSetColorAdjustment);
      EMR_SetTextColor: (SetTextColor: TEMRSetTextColor);
      EMR_SetBkColor: (SetBkColor: TEMRSetBkColor);
      EMR_OffsetClipRgn: (OffsetClipRgn: TEMROffsetClipRgn);
      EMR_MoveToEx: (MoveToEx: TEMRMoveToEx);
      EMR_SetMetaRgn: (SetMetaRgn: TEMRSetMetaRgn);
      EMR_ExcludeClipRect: (ExcludeClipRect: TEMRExcludeClipRect);
      EMR_IntersectClipRect: (IntersectClipRect: TEMRIntersectClipRect);
      EMR_ScaleViewportExtEx: (ScaleViewportExtEx: TEMRScaleViewportExtEx);
      EMR_ScaleWindowExtEx: (ScaleWindowExtEx: TEMRScaleWindowExtEx);
      EMR_SaveDC: (SaveDC: TEMRSaveDC);
      EMR_RestoreDC: (RestoreDC: TEMRRestoreDC);
      EMR_SetWorldTransform: (SetWorldTransform: TEMRSetWorldTransform);
      EMR_ModifyWorldTransform: (ModifyWorldTransform: TEMRModifyWorldTransform);
      EMR_SelectObject: (SelectObject: TEMRSelectObject);
      EMR_CreatePen: (CreatePen: TEMRCreatePen);
      EMR_CreateBrushIndirect: (CreateBrushIndirect: TEMRCreateBrushIndirect);
      EMR_DeleteObject: (DeleteObject: TEMRDeleteObject);
      EMR_AngleArc: (AngleArc: TEMRAngleArc);
      EMR_Ellipse: (Ellipse: TEMREllipse);
      EMR_Rectangle: (Rectangle: TEMRRectangle);
      EMR_RoundRect: (RoundRect: TEMRRoundRect);
      EMR_Arc: (Arc: TEMRArc);
      EMR_Chord: (Chord: TEMRChord);
      EMR_Pie: (Pie: TEMRPie);
      EMR_SelectPalette: (SelectPalette: TEMRSelectPalette);
      EMR_CreatePalette: (CreatePalette: TEMRCreatePalette);
      EMR_SetPaletteEntries: (SetPaletteEntries: TEMRSetPaletteEntries);
      EMR_ResizePalette: (ResizePalette: TEMRResizePalette);
      EMR_RealizePalette: (RealizePalette: TEMR);
      EMR_ExtFloodFill: (ExtFloodFill: TEMRExtFloodFill);
      EMR_LineTo: (LineTo: TEMRLineTo);
      EMR_ArcTo: (ArcTo: TEMRArcTo);
      EMR_PolyDraw: (PolyDraw: TEMRPolyDraw);
      EMR_SetArcDirection: (SetArcDirection: TEMRSetArcDirection);
      EMR_SetMiterLimit: (SetMiterLimit: TEMRSetMiterLimit);
      EMR_BeginPath: (BeginPath: TEMRBeginPath);
      EMR_EndPath: (EndPath: TEMREndPath);
      EMR_CloseFigure: (CloseFigure: TEMRCloseFigure);
      EMR_FillPath: (FillPath: TEMRFillPath);
      EMR_StrokeAndFillPath: (StrokeAndFillPath: TEMRStrokeAndFillPath);
      EMR_StrokePath: (StrokePath: TEMRStrokePath);
      EMR_FlattenPath: (FlattenPath: TEMRFlattenPath);
      EMR_WidenPath: (WidenPath: TEMRWidenPath);
      EMR_SelectClipPath: (SelectClipPath: TEMRSelectClipPath);
      EMR_AbortPath: (AbortPath: TEMRAbortPath);
      EMR_Reserved_69: (Reserved_69: TEMR);
      EMR_GDIComment: (GDIComment: TEMRGDIComment);
      EMR_FillRgn: (FillRgn: TEMRFillRgn);
      EMR_FrameRgn: (FrameRgn: TEMRFrameRgn);
      EMR_InvertRgn: (InvertRgn: TEMRInvertRgn);
      EMR_PaintRgn: (PaintRgn: TEMRPaintRgn);
      EMR_ExtSelectClipRgn: (ExtSelectClipRgn: TEMRExtSelectClipRgn);
      EMR_BitBlt: (BitBlt: TEMRBitBlt);
      EMR_StretchBlt: (StretchBlt: TEMRStretchBlt);
      EMR_MaskBlt: (MaskBlt: TEMRMaskBlt);
      EMR_PLGBlt: (PLGBlt: TEMRPLGBlt);
      EMR_SetDIBitsToDevice: (SetDIBitsToDevice: TEMRSetDIBitsToDevice);
      EMR_StretchDIBits: (StretchDIBits: TEMRStretchDIBits);
      EMR_ExtCreateFontIndirectW: (ExtCreateFontIndirectW: TEMRExtCreateFontIndirectW);
      EMR_ExtTextOutA: (ExtTextOutA: TEMRExtTextOutA);
      EMR_ExtTextOutW: (ExtTextOutW: TEMRExtTextOutW);
      EMR_PolyBezier16: (PolyBezier16: TEMRPolyBezier16);
      EMR_Polygon16: (Polygon16: TEMRPolygon16);
      EMR_Polyline16: (Polyline16: TEMRPolyline16);
      EMR_PolyBezierTo16: (PolyBezierTo16: TEMRPolyBezierTo16);
      EMR_PolylineTo16: (PolylineTo16: TEMRPolylineTo16);
      EMR_PolyPolyline16: (PolyPolyline16: TEMRPolyPolyline16);
      EMR_PolyPolygon16: (PolyPolygon16: TEMRPolyPolygon16);
      EMR_PolyDraw16: (PolyDraw16: TEMRPolyDraw16);
      EMR_CreateMonoBrush: (CreateMonoBrush: TEMRCreateMonoBrush);
      EMR_CreateDIBPatternBrushPt: (CreateDIBPatternBrushPt: TEMRCreateDIBPatternBrushPt);
      EMR_ExtCreatePen: (ExtCreatePen: TEMRExtCreatePen);
      EMR_PolyTextOutA: (PolyTextOutA: TEMRPolyTextOutA);
      EMR_PolyTextOutW: (PolyTextOutW: TEMRPolyTextOutW);
      EMR_SetICMMode: (SetICMMode: TEMRSetICMMode);
      EMR_CreateColorSpace: (CreateColorSpace: TEMRCreateColorSpace);
      EMR_SetColorSpace: (SetColorSpace: TEMRSetColorSpace);
      EMR_DeleteColorSpace: (DeleteColorSpace: TEMRDeleteColorSpace);
      EMR_GLSRecord: (GLSRecord: TEMRGLSRecord);
      EMR_GLSBoundedRecord: (GLSBoundedRecord: TEMRGLSBoundedRecord);
      EMR_PixelFormat: (PixelFormat: TEMRPixelFormat);
      EMR_DrawEscape: (DrawEscape: TEMRDrawEsc);
      EMR_ExtEscape: (ExtEscape: TEMRExtEscape);
      EMR_StartDoc: (StartDoc: TEMR); // ? https://msdn.microsoft.com/en-us/library/windows/desktop/dd145114%28v=vs.85%29.aspx
      EMR_SmallTextOut: (SmallTextOut: TEMRSmallTextOut);
      EMR_ForceUFIMapping: (ForceUFIMapping: TEMRForceUFIMapping);
      EMR_NamedEscape: (NamedEscape: TEMR);
      EMR_ColorCorrectPalette: (ColorCorrectPalette: TEMRColorCorrectPalette);
      EMR_SetIcmProfileA: (SetIcmProfileA: TEmrSetIcmProfile);
      EMR_SetIcmProfileW: (SetIcmProfileW: TEmrSetIcmProfile);
      EMR_AlphaBlend: (AlphaBlend: TEMRAlphaBlend);
      EMR_SetLayout: (SetLayout: TEMRSetLayout);
      EMR_TransparentBlt: (TransparentBlt: TEMRTransparentBlt);
      EMR_TransparentDIB: (TransparentDIB: TEMR);
      EMR_GradientFill: (GradientFill: TEMRGradientFill);
      EMR_SetLinkedUFIs: (SetLinkedUFIs: TEMR);
      EMR_SetTextJustification: (SetTextJustification: TEMR);
      EMR_ColorMatchToTargetW: (ColorMatchToTargetW: TEMRColorMatchToTarget);
      EMR_CreateColorSpaceW: (CreateColorSpaceW: TEMRCreateColorSpace);
  end;

  TEnhMetaObj = class
  protected
    FP: PEnhMetaData;
    function GetWideString(Length, Offset: Integer): WideString;
    function GetANSIString(Length, Offset: Integer): AnsiString;
  public
    constructor Create(Stream: TStream; Size: Integer);
    constructor CreateRec(var EMR);
    destructor Destroy; override;

    property P: PEnhMetaData read FP;
  end;

const
{ TEnhMetaHeader Type}
  ehUnknown = -1;
  ehOriginal = 0;
  ehExtension1 = 1;
  ehExtension2 = 2;

type
  TEnhMetaHeaderObj = class(TEnhMetaObj)
  private
    function GetExtension: Integer;
    function GetDescription: WideString;
    function GetPixelFormatDescriptor: TPixelFormatDescriptor;
  public
    property Extension: Integer read GetExtension;
    property Description: WideString read GetDescription;
{Extension 1}
    property PixelFormatDescriptor: TPixelFormatDescriptor read GetPixelFormatDescriptor;
  end;

  TEoFObj = class(TEnhMetaObj)
  private
    function GetPaletteEntry(Index: LongWord): TPaletteEntry;
    function GetnSizeLast: LongWord;
  public
    property PaletteEntry[Index: LongWord]: TPaletteEntry read GetPaletteEntry;
    property nSizeLast: LongWord read GetnSizeLast;
  end;

  TEMRPolyPolygon16Obj = class(TEnhMetaObj)
  private
    function GetPolyPoint(Poly, Point: Integer): TSmallPoint;
  public
    property PolyPoint[Poly, Point: Integer]: TSmallPoint read GetPolyPoint;
  end;
  TEMRPolyPolyline16Obj = class(TEMRPolyPolygon16Obj);

  TEMRPolyPolygonObj = class(TEnhMetaObj)
  private
    function GetPolyPoint(Poly, Point: Integer): TPoint;
  public
    property PolyPoint[Poly, Point: Integer]: TPoint read GetPolyPoint;
  end;
  TEMRPolyPolylineObj = class(TEMRPolyPolygon16Obj);

  PRectArray = ^TRectArray;
  TRectArray = array[0..0] of TRect;

  TEMRExtSelectClipRgnObj = class(TEnhMetaObj)
  private
    function GetRegionData: PRgnData;
    function GetRegion(Index: LongWord): TRect;
  public
    property PRegionData: PRgnData read GetRegionData;
    property Region[Index: LongWord]: TRect read GetRegion;
  end;

  TEMRBitmap  = class(TEnhMetaObj)
  protected
    FSourceBitmap: TBitmap;

    procedure SureSourceBitmap;
    function OffsetBmi: LongWord; virtual; abstract;
  public
    destructor Destroy; override;
    function GetBitmap: TBitmap;
    function cbBitsSrc: Cardinal; virtual; abstract;
  end;

  TEMRBitmapROP = class(TEMRBitmap)
  public
    function dwRop: Cardinal; virtual; abstract;
  end;

  TEMRBitBltObj = class(TEMRBitmapROP)
  protected
    function OffsetBmi: LongWord; override;
  public
    function cbBitsSrc: Cardinal; override;
    function dwRop: Cardinal; override;
  end;

  TEMRStretchBltObj = class(TEMRBitmapROP)
  protected
    function OffsetBmi: LongWord; override;
  public
    function cbBitsSrc: Cardinal; override;
    function dwRop: Cardinal; override;
  end;

  TEMRStretchDIBitsObj = class(TEMRBitmapROP)
  protected
    function OffsetBmi: LongWord; override;
  public
    procedure FlipVertical(Bitmap: TBitmap);
    function cbBitsSrc: Cardinal; override;
    function dwRop: Cardinal; override;
  end;

  TEMRMaskBltObj = class(TEMRBitmapROP)
  protected
    function OffsetBmi: LongWord; override;
  public
    function cbBitsSrc: Cardinal; override;
    function dwRop: Cardinal; override;
  end;

  TEMRPLGBltObj = class(TEMRBitmap)
  protected
    function OffsetBmi: LongWord; override;
  public
    function cbBitsSrc: Cardinal; override;
  end;

  TEMRSetDIBitsToDeviceObj = class(TEMRBitmap)
  protected
    function OffsetBmi: LongWord; override;
  public
    function cbBitsSrc: Cardinal; override;
  end;

  TEMRAlphaBlendObj = class(TEMRBitmapROP)
  protected
    function OffsetBmi: LongWord; override;
  public
    function GetARGBGraphic: TGraphic;
    function GetBitmap24: TBitmap;
    function cbBitsSrc: Cardinal; override;
    function dwRop: Cardinal; override;
  end;

  TEMRTransparentBltObj = class(TEMRBitmapROP)
  protected
    function OffsetBmi: LongWord; override;
  public
    function cbBitsSrc: Cardinal; override;
    function dwRop: Cardinal; override;
  end;

  TMonoColors = (mcAllZero, mcAllOne, mcBoth);
  TBoolArray = array of Boolean;
  TEMRCreateBrushObj = class(TEMRBitmap)
  private
    function GetPixelFormat: TPixelFormat;
  protected
    FLen: Integer;

    procedure TestBitmapData;
    function CalcBG: TBoolArray;
    function OffsetBits: LongWord; virtual; abstract;
    function iUsage: DWord; virtual; abstract;
  public
    constructor Create(Stream: TStream; Size: Integer);
    function CountMonoColors: TMonoColors;
    function GetBitmap24OutOf1(BG: TColor = -1; FG: TColor = -1): TBitmap;
    function GetBitmap32OutOf1: TBitmap;

    property Len: Integer read FLen;
    property PixelFormat: TPixelFormat read GetPixelFormat;
  end;

  TEMRCreateMonoBrushObj = class(TEMRCreateBrushObj)
  protected
    function OffsetBmi: LongWord; override;
    function OffsetBits: LongWord; override;
    function iUsage: DWord; override;
  public
    function cbBitsSrc: Cardinal; override;
  end;

  TEMRCreateDIBPatternBrushPtObj = class(TEMRCreateBrushObj)
  protected
    function OffsetBmi: LongWord; override;
    function OffsetBits: LongWord; override;
    function iUsage: DWord; override;
  public
    function cbBitsSrc: Cardinal; override;
  end;

  TEMRExtCreatePenObj = class(TEMRBitmap)
  protected
    function OffsetBmi: LongWord; override;
  public
    function cbBitsSrc: Cardinal; override;
  end;

  TEMRExtTextOutWObj = class(TEnhMetaObj)
  private
    function GetTextLength: LongWord;
  public
    function OutputString(FontName: string; IsRTL: Boolean = False): WideString;
    function OutputDx: TLongWordDynArray;
    function OutputDy: TLongWordDynArray;
    function IsOption(O: LongWord): Boolean;

    property TextLength: LongWord read GetTextLength;
  end;

  TEMRExtTextOutAObj = class(TEnhMetaObj)
  private
    function GetOutputString: AnsiString;
    function GetTextLength: LongWord;
  public
    function OutputDx: TLongWordDynArray;
    function OutputDy: TLongWordDynArray;
    function IsOption(O: LongWord): Boolean;

    property TextLength: LongWord read GetTextLength;
    property OutputString: AnsiString read GetOutputString;
  end;

  TEMRPolyTextOutWObj = class(TEnhMetaObj)
  private
    function GetOutputString(Index: Integer): WideString;
  public
    property OutputString[Index: Integer]: WideString read GetOutputString;
  end;

  TEMRPolyTextOutAObj = class(TEnhMetaObj)
  private
    function GetOutputString(Index: Integer): AnsiString;
  public
    property OutputString[Index: Integer]: AnsiString read GetOutputString;
  end;

  TEMRPolyDrawObj =  class (TEnhMetaObj)
  private
    function GetTypes(Index: LongWord): byte;
  public
    property Types[Index: LongWord]: byte read GetTypes;
  end;

  TEMRPolyDraw16Obj =  class (TEnhMetaObj)
  private
    function GetTypes(Index: LongWord): byte;
  public
    property Types[Index: LongWord]: byte read GetTypes;
  end;

  TEMRSmallTextOutObj =  class (TEnhMetaObj)
  private
    function GetOutputStringWide: WideString;
    function GetOutputStringANSI: AnsiString;

    function StringOffset: Integer;
  public
    function IsNoRect: boolean;
    function IsANSI: boolean;

    property OutputStringWide: WideString read GetOutputStringWide;
    property OutputStringANSI: AnsiString read GetOutputStringANSI;
  end;

function IsStockObject(const ih: LongWord): Boolean; overload;
function IsStockObject(EnhMetaObj: TEnhMetaObj): Boolean; overload;
function IsStockBrush(const ih: LongWord): Boolean; overload;
function IsStockBrush(EnhMetaObj: TEnhMetaObj): Boolean; overload;
function IsStockPen(const ih: LongWord): Boolean; overload;
function IsStockPen(EnhMetaObj: TEnhMetaObj): Boolean; overload;
function IsStockFont(const ih: LongWord): Boolean; overload;
function IsStockFont(EnhMetaObj: TEnhMetaObj): Boolean; overload;
function IsStockPalette(const ih: LongWord): Boolean; overload;
function IsStockPalette(EnhMetaObj: TEnhMetaObj): Boolean; overload;
function StockObject(const ih: LongWord): TEnhMetaObj;

implementation
end.
