
{******************************************}
{                                          }
{             FastReport VCL               }
{            EMF to SVG Export             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxEMFtoSVGExport;

interface

{$I frx.inc}

uses
  Windows, Graphics, Classes, Types,
  frxClass, frxExportHelpers, frxEMFAbstractExport, frxEMFFormat, frxUtils,
  frxAnaliticGeometry, frCSSStyle;

type
  TSVGDeviceContext = class (TDeviceContext)
  protected
    FLastClipPathName: string;
    FLastPatternName: string;
  public
    procedure CopyFrom(ADC: TObject); override;
    procedure Init; override;
  end;

  TEMFtoSVGExport = class (TEMFAbstractExport)
  private
    FCSS: TfrxCSSList;
    FPath: string;
    FEmbedded: Boolean;
    FX, FY: Extended;
    FForceMitterLineJoin: Boolean;
    FLinearBarcode: Boolean;
    FRich: Boolean;
    FForceWidth: Boolean;

    procedure NextClipPath;
    procedure NextPattern;

    function MeasureUnit(r: Extended; DefaultUnits: string = ''): string;
    function MU(r: Extended): string;
    function IsVerticalFont: Boolean;
    function IsAverageDx: Boolean;

    function MeasureAngleArc(Center: TPoint; Radius: Integer; StartAngle, SweepAngle: Single): string;
    function MeasureArc(LR: TRect; LStart, LEnd: TPoint; NeedClosePath: Boolean = False): string;
    function MeasureClipPath: string;
    function MeasureClipPathDefine(ClipPath: string): string;
    function MeasureClipRule: string;
    function MeasureDirection(RTL: Boolean): string;
    function MeasureDominantBaseline: string;
    function MeasureDx(OutputDx: TLongWordDynArray; OutputString: WideString; IsAverageDX: Boolean): string;
    function MeasureDy(OutputDy: TLongWordDynArray; OutputString: WideString): string;
    function MeasureEllipse(LR: TRect): string;
    function MeasureEndCap: string;
    function MeasureFill(Options: byte): string;
    function MeasureFontFamily: string;
    function MeasureFontStyle: string;
    function MeasureFontSize: string;
    function MeasureFontOrientation(LP: TPoint; IsAdvanced: Boolean = False): string; overload;
    function MeasureFontOrientation(LP: TDoublePoint; IsAdvanced: Boolean = False): string; overload;
    function MeasureLine: string;
    function MeasureLineJoin: string;
    function MeasurePenWidth: string;
    function MeasurePie(LR: TRect; LStart, LEnd: TPoint): string;
    function MeasurePoint(LP: TfrxPoint; dX: Extended = 0): string; overload;
    function MeasurePoint(LP: TDoublePoint; dX: Extended = 0): string; overload;
    function MeasurePoint(LP: TPoint; dX: Extended = 0): string; overload;
    function MeasurePoint(LP: TSmallPoint; dX: Extended = 0): string; overload;
    function MeasurePolyPoints: string; overload;
    function MeasurePoly16Points: string; overload;
    function MeasurePolyPoints(var MovToPos: String): string; overload;
    function MeasurePoly16Points(var MovToPos: String): string; overload;
    function MeasurePolyFillMode: string;
    function MeasureRect(LR: TRect; dHeight: Double = 0.0): string; overload;
    function MeasureRect(x, y, cx, cy: Integer): string; overload;
    function MeasureRectAsPath(LR: TRect): string;
    function MeasureRoundRect(LR: TRect; LS: TSize): string; overload;
    function MeasureStroke(Options: byte): string;
    function MeasureStrokeMiterLimit: string;
    function MeasureStrokeDasharray: string;
    function MeasureTextAnchor(IsRTL: Boolean): string;
    function MeasureTextDecoration: string;
//    function MeasureTextLength: string;
    function MeasureWritingMode: string;
    function MeasureXY(LP: TDoublePoint): string;
    function MeasureXYText(LP: TDoublePoint): string; overload;
    function MeasureXYText(LP: TPoint): string; overload;
    function MeasureXYText(X, Y: Double): string; overload;
    function MeasureWidth(Size: TfrxPoint): string;
    function SpaceDlm(st: string): string;

    procedure Puts(const s: WideString); overload;
    procedure Puts(const Fmt: WideString; const Args: array of const); overload;
    procedure PutsA(const s: AnsiString);
    procedure PutsRaw(const s: AnsiString);

    function CSSPaintStyleName(Options: byte): string;

    procedure Do_BitMap(DestRect: string; dwRop: LongWord; EMRBitmap: TEMRBitmap; Bitmap: TBitmap = nil);
    procedure Do_Pattern(XLine, YLine, Turn: Boolean);
    procedure Do_PolyPoly(Name: string; Options: byte);
    procedure Do_PolyPoly16(Name: string; Options: byte);
  protected
    procedure Comment(CommentString: string = ''); override;
    function NormalizeRect(frxRect: TfrxRect): TfrxRect; overload;
    function NormalizeRect(const Rect: TRect): TRect; overload;
    procedure DCCreate; override;
    function FontCreate: TEMFFont; override;
    function SVGDeviceContext: TSVGDeviceContext;
    function IsNonZero(A: TIntegerDynArray): Boolean;
    function SureFirstM(Path: string): string;

    procedure DoEMR_AbortPath; override;
    procedure DoEMR_AlphaBlend; override;
    procedure DoEMR_AngleArc; override;
    procedure DoEMR_Arc; override;
    procedure DoEMR_ArcTo; override;
    procedure DoEMR_BeginPath; override;
    procedure DoEMR_BitBlt; override;
    procedure DoEMR_Chord; override;
    procedure DoEMR_CloseFigure; override;
    procedure DoEMR_Ellipse; override;
    procedure DoEMR_EoF; override;
    procedure DoEMR_ExtSelectClipRgn; override;
    procedure DoEMR_ExtTextOutA; override;
    procedure DoEMR_ExtTextOutW; override;
    procedure DoEMR_FillPath; override;
    procedure DoEMR_FillRgn; override;
    procedure DoEMR_FlattenPath; override;
    procedure DoEMR_FrameRgn; override;
    procedure DoEMR_Header; override;
    procedure DoEMR_IntersectClipRect; override;
    procedure DoEMR_LineTo; override;
    procedure DoEMR_MaskBlt; override;
    procedure DoEMR_MoveToEx; override;
    procedure DoEMR_PaintRgn; override;
    procedure DoEMR_Pie; override;
    procedure DoEMR_PLGBlt; override;
    procedure DoEMR_PolyBezier; override;
    procedure DoEMR_PolyBezier16; override;
    procedure DoEMR_PolyBezierTo; override;
    procedure DoEMR_PolyBezierTo16; override;
    procedure DoEMR_PolyDraw; override;
    procedure DoEMR_PolyDraw16; override;
    procedure DoEMR_Polygon; override;
    procedure DoEMR_Polygon16; override;
    procedure DoEMR_Polyline; override;
    procedure DoEMR_Polyline16; override;
    procedure DoEMR_PolylineTo; override;
    procedure DoEMR_PolylineTo16; override;
    procedure DoEMR_PolyPolygon; override;
    procedure DoEMR_PolyPolygon16; override;
    procedure DoEMR_PolyPolyline; override;
    procedure DoEMR_PolyPolyline16; override;
    procedure DoEMR_PolyTextOutA; override;
    procedure DoEMR_PolyTextOutW; override;
    procedure DoEMR_Rectangle; override;
    procedure DoEMR_RoundRect; override;
    procedure DoEMR_SelectClipPath; override;
    procedure DoEMR_SetDIBitsToDevice; override;
    procedure DoEMR_SetMetaRgn; override;
    procedure DoEMR_SetPixelV; override;
    procedure DoEMR_SmallTextOut; override;
    procedure DoEMR_StretchBlt; override;
    procedure DoEMR_StretchDIBits; override;
    procedure DoEMR_StrokeAndFillPath; override;
    procedure DoEMR_StrokePath; override;
    procedure DoEMR_TransparentBlt; override;
    procedure DoEMR_WidenPath; override;

    procedure DoStart; override;
    procedure DoFinish; override;
  public
    procedure AfterConstruction; override;
    procedure SetEmbedded(CSS: TfrxCSSList; Obj: TfrxView);

    property ForceWidth: Boolean read FForceWidth write FForceWidth;
  end;

implementation
end.


