
{******************************************}
{                                          }
{             FastReport VCL               }
{            PDF export filter             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{         Copyright (c) 1998-2021          }
{          PDF/A by Anton Khayrudinov      }
{                                          }
{      After Service support by alman      }
{                                          }
{             Fast Reports Inc.            }
{                                          }
{******************************************}

unit frxExportAPDF;

interface

{$I frx.inc}

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  ComObj,
  Printers,
  ShellAPI,
  ComCtrls,

  {$IFDEF DELPHI10}
  WideStrings,
  {$ENDIF}

  {$IFDEF DELPHI12}
  AnsiStrings,
  {$ENDIF}

  {$IFDEF DELPHI6}
  Variants,
  {$ENDIF}

  JPEG,
  frCore,
  frxStorage,
  frxOTF,
  frxClass,
  frxRC4;

type
  TfrxPDFEncBit = (ePrint, eModify, eCopy, eAnnot);
  TfrxPDFEncBits = set of TfrxPDFEncBit;

  TfrxAPDFExportDialog = class(TForm)
    PageControl1: TPageControl;
    ExportPage: TTabSheet;
    InfoPage: TTabSheet;
    SecurityPage: TTabSheet;
    ViewerPage: TTabSheet;
    OkB: TButton;
    CancelB: TButton;
    SaveDialog1: TSaveDialog;
    OpenCB: TCheckBox;
    GroupQuality: TGroupBox;
    CompressedCB: TCheckBox;
    EmbeddedCB: TCheckBox;
    PrintOptCB: TCheckBox;
    OutlineCB: TCheckBox;
    BackgrCB: TCheckBox;
    GroupPageRange: TGroupBox;
    DescrL: TLabel;
    AllRB: TRadioButton;
    CurPageRB: TRadioButton;
    PageNumbersRB: TRadioButton;
    PageNumbersE: TEdit;
    SecGB: TGroupBox;
    OwnPassL: TLabel;
    UserPassL: TLabel;
    OwnPassE: TEdit;
    UserPassE: TEdit;
    PermGB: TGroupBox;
    PrintCB: TCheckBox;
    ModCB: TCheckBox;
    CopyCB: TCheckBox;
    AnnotCB: TCheckBox;
    DocInfoGB: TGroupBox;
    TitleL: TLabel;
    TitleE: TEdit;
    AuthorE: TEdit;
    AuthorL: TLabel;
    SubjectL: TLabel;
    SubjectE: TEdit;
    KeywordsL: TLabel;
    KeywordsE: TEdit;
    CreatorE: TEdit;
    CreatorL: TLabel;
    ProducerL: TLabel;
    ProducerE: TEdit;
    ViewerGB: TGroupBox;
    HideToolbarCB: TCheckBox;
    HideMenubarCB: TCheckBox;
    HideWindowUICB: TCheckBox;
    FitWindowCB: TCheckBox;
    CenterWindowCB: TCheckBox;
    PrintScalingCB: TCheckBox;
    cbPDFA: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure PageNumbersEChange(Sender: TObject);
    procedure PageNumbersEKeyPress(Sender: TObject; var Key: Char);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  end;

  TfrxPDFCharClass = (ccRegular, ccWhitespace, ccDelimiter);

  TfrxPDFFontInfo = record
    FontBox:      TFontBox;
    Ascent:       Integer;
    Descent:      Integer;
    AvgWidth:     Integer;
    MaxWidth:     Integer;
    ItalicAngle:  Integer;
    MissingWidth: Integer;
    StemV:        Integer;
    CapHeight:    Integer;
    Leading:      Integer;
    Flags:        Word;
  end;

  TfrxPDFFontView = class
  private
    FView: TView;
    FCollection: Boolean;
    FData: TMemoryStream;

    function  FindFont(Name: string): TFontView;
    procedure CreateView;
  public
    constructor Create;
    destructor Destroy; override;

    procedure Load(Handle: HFont); overload;
    procedure Load(Src: TStream); overload;

    function  GetFontsCount: Integer;
    function  GetFont(Index: Integer): TFontView; overload;
    function  GetFont(Name: string): TFontView; overload;
  end;

  TfrxPDFFont = class
  private
    FEmbedSubset: Boolean;
    FSourceFont: TFont;
    FReference: Longint;
    FSaved: Boolean;
    FName: AnsiString;
    FSubsetTag: AnsiString;
    FFontName: AnsiString;
    FView: TfrxPDFFontView;
    FFontView: TFontView;
    FUsedChars: TBitArray;
    FUsedGlyphs: TBitArray;
    FFontInfo: TfrxPDFFontInfo;
    FChars: array of Word;
    FGlyphs: array of Word;

    procedure MarkCharAsUsed(Char: Word);
    procedure MarkAllCharsAsUsed(const s: WideString);
    procedure LoadFontInfo;
    procedure CreateMapping;

    function GetFontFile: TStream;
    function GetFontName: AnsiString;
    function GetFontInfo: TfrxPDFFontInfo;
    function GetSubsetTag: AnsiString;
    function IsBuilt: Boolean;

    function GetCMapType: Integer;
    function GetCMapName: string;
    function GetCMapRegistry: string;
    function GetCMapOrdering: string;
    function GetCMapSupplement: Integer;
  public
    constructor Create(Font: TFont; EmbedSubset: Boolean);
    destructor Destroy; override;

    function RemapString(str: WideString; rtl: Boolean): WideString;
    procedure BuildFont;

    { Writes mapping from charcodes to CIDs.
      This stream is the content of the /Encoding key. }

    procedure WriteCharToCIDMap(Stream: TStream);

    { Writes a mapping from charcodes to unicodes }

    procedure WriteCharToUnicodeMap(Stream: TStream);

    { Writes mapping from CID to GID indices.
      This stream is the contents of the /CIDToGID key. }

    procedure WriteCIDToGIDMap(Stream: TStream);

    { Writes widths of used characters.
      This data is the value of the /W key. }

    procedure WriteCharWidths(Stream: TStream);

    { Writes a bit array of used CIDs }

    procedure WriteCIDSet(Stream: TStream);

    property FontName: AnsiString read GetFontName;
    property FontFile: TStream read GetFontFile;
    property FontInfo: TfrxPDFFontInfo read GetFontInfo;

    property CMapName: string read GetCMapName;
    property CMapOrdering: string read GetCMapOrdering;
    property CMapRegistry: string read GetCMapRegistry;
    property CMapSupplement: Integer read GetCMapSupplement;

    property PDFName: AnsiString read FName write FName;
    property Saved: Boolean read FSaved write FSaved;
    property Reference: Integer read FReference write FReference;
    property SourceFont: TFont read FSourceFont;
  end;

  TfrxPDFOutlineNode = class
  public
    Number:     Integer;
    Dest:       Integer;  // Index to a page referred to by this outline node
    Top:        Integer;  // Position on the referred to page
    CountTree:  Integer;  // Number of all descendant nodes
    Count:      Integer;  // Number of all first-level descendants
    Title:      string;

    First:  TfrxPDFOutlineNode; // The first first-level descendant
    Last:   TfrxPDFOutlineNode; // The last first-level descendant
    Next:   TfrxPDFOutlineNode; // The next neighbouring node
    Prev:   TfrxPDFOutlineNode; // The previous neighbouring node
    Parent: TfrxPDFOutlineNode; // The parent node of this node

    constructor Create;
    destructor Destroy; override;
  end;

  TfrxPDFPage = class
  public
    Height: Double;
  end;

  TfrxPDFXObjectHash = array[0..15] of Byte; // MD5

  TfrxPDFXObject = record
    ObjId: Integer; // id that appears in 'id 0 R'
    Hash: TfrxPDFXObjectHash;
  end;

  TfrxPDFBinaryStreamOption = (bsoEncrypt, bsoCompress, bsoHexEncode);
  TfrxPDFBinaryStreamOptions = set of TfrxPDFBinaryStreamOption;

  TfrxAPDFExport = class(TfrxCustomExportFilter)
  private
    FCompressed: Boolean;
    FEmbedded: Boolean;
    FOpenAfterExport: Boolean;
    FPrintOpt: Boolean;
    FPages: TList;
    FOutline: Boolean;
    FPreviewOutline: TfrxCustomOutline;

    FSubject: WideString;
    FAuthor: WideString;
    FBackground: Boolean;
    FCreator: WideString;
    FKeywords: WideString;
    FTitle: WideString;
    FProducer: WideString;

    FTags: Boolean;
    FProtection: Boolean;
    FUserPassword: AnsiString;
    FOwnerPassword: AnsiString;
    FProtectionFlags: TfrxPDFEncBits;
    FPrintScaling: Boolean;
    FFitWindow: Boolean;
    FHideMenubar: Boolean;
    FCenterWindow: Boolean;
    FHideWindowUI: Boolean;
    FHideToolbar: Boolean;

    pdf: TStream;

    FRootNumber: longint;
    FPagesNumber: longint;
    FInfoNumber: longint;
    FStartXRef: longint;

    FFonts: TList;
    FPageFonts: TList;
    FXRef: TStringList;
    FPagesRef: TStringList;

    FWidth: Extended;
    FHeight: Extended;
    FMarginLeft: Extended;
    FMarginWoBottom: Extended;
    FMarginTop: Extended;

    FEncKey: AnsiString;
    FOPass: AnsiString;
    FUPass: AnsiString;

    FEncBits: Cardinal;
    FFileID: AnsiString;

    FDivider: Extended;
    FLastColor: TColor;
    FLastColorResult: String;
    FID: AnsiString;

    FPDFA: Boolean;

    OutStream: TMemoryStream;
    FXObjects: array of TfrxPDFXObject;
    FUsedXObjects: array of Integer; // XObjects' ids used within the current page

    { These fields are used for debugging/profiling }

    FPicTotalSize: Cardinal;  // size occupied by all pictures
    FFontTotalSize: Cardinal; // size occupied by all embedded fonts

    { When an anchor is being added, two changes are made:

        - a link object is written to the document
        - a reference to the link object is added to /Annots field of the page

      FAnnots contains text of /Annots field.
      This stream is updated by WriteLink and its auxiliary routines. }

    FAnnots: TMemoryStream;

    { Writes to Res an object with Src inside.
      Performs compression and encryption if needed.
      Returns the id of the written object. }

    function WriteDataStream(Res, Src: TStream; const Ext: string;
      IsText: Boolean): Integer;

    function GetBinaryStreamOptions: TfrxPDFBinaryStreamOptions;

    function PrepXrefPos(pos: Longint): String;
    function GetID: AnsiString;
    function CryptStr(Source: AnsiString; Key: AnsiString; Enc: Boolean; id: Integer): AnsiString;
    function CryptStream(Source: TStream; Target: TStream; Key: AnsiString; id: Integer): AnsiString;
    function PrepareString(const Text: WideString; Key: AnsiString; Enc: Boolean; id: Integer): AnsiString;
    function EscapeSpecialChar(TextStr: AnsiString): AnsiString;
    function StrToUTF16(const Value: WideString): AnsiString;
    function PMD52Str(p: Pointer): AnsiString;
    function PadPassword(Password: AnsiString): AnsiString;
    procedure PrepareKeys;
    procedure SetProtectionFlags(const Value: TfrxPDFEncBits);
    procedure Clear;
    procedure WriteFont(pdfFont: TfrxPDFFont);
    procedure AddObject(const Obj: TfrxView);
    function StrToHex(const Value: WideString): AnsiString;
    function AddPage(Page: TfrxReportPage): TfrxPDFPage;
    function ObjNumber(FNumber: longint): String;
    function ObjNumberRef(FNumber: longint): String;
    function UpdateXRef: longint;
    function GetPDFColor(const Color: TColor): String;
    procedure GetStreamHash(out Hash: TfrxPDFXObjectHash; S: TStream);
    function FindXObject(const Hash: TfrxPDFXObjectHash): Integer;
    function AddXObject(Id: Integer; const Hash: TfrxPDFXObjectHash): Integer;

    { Writes the output profile.
      Returns id of the written PDF object. }

    function WriteOutputProfile: Integer;

    { Writes a PDF object with a specified id to the stream
      and writes to this object XMP metadata. }

    function WriteMetaData: Integer;

    { Formats a time in the PDF format.
      Acceptable modes:

      Mode    Sample formatted time

        D     D:19950325125439
        DZ    D:19950325125439Z
        TZ    1995-03-25T12:54:39Z }

    function FormatTime(Time: Extended; Mode: string): string;

    { Writes /OutputIntents entry.
      The entry has a subentry /DestOutputProfile.
      If OutputProfileId is not negative, then this subentry is written. }

    procedure WriteOutputIntents(OutputProfileId: Integer);

    { Writes /ViewerPreferences entry }

    procedure WriteViewerPreferences;

    { Writes /StructTreeRoot enrty }

    procedure WriteStructTreeRoot;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function GetDescription: String; override;
    function ShowModal: TModalResult; override;
    function Start: Boolean; override;
    procedure ExportObject(Obj: TfrxComponent); override;
    procedure Finish; override;
    procedure StartPage(Page: TfrxReportPage; Index: Integer); override;
    procedure FinishPage(Page: TfrxReportPage; Index: Integer); override;
  published
    property Compressed: Boolean read FCompressed write FCompressed default True;
    property EmbeddedFonts: Boolean read FEmbedded write FEmbedded default False;
    property OpenAfterExport: Boolean read FOpenAfterExport
      write FOpenAfterExport default False;
    property PrintOptimized: Boolean read FPrintOpt write FPrintOpt;
    property Outline: Boolean read FOutline write FOutline;
    property Background: Boolean read FBackground write FBackground;
    property HTMLTags: Boolean read FTags write FTags;
    property OverwritePrompt;
    property PDFA: Boolean read FPDFA write FPDFA;
    property ID: AnsiString read FID write FID;

    property Title: WideString read FTitle write FTitle;
    property Author: WideString read FAuthor write FAuthor;
    property Subject: WideString read FSubject write FSubject;
    property Keywords: WideString read FKeywords write FKeywords;
    property Creator: WideString read FCreator write FCreator;
    property Producer: WideString read FProducer write FProducer;

    property UserPassword: AnsiString read FUserPassword write FUserPassword;
    property OwnerPassword: AnsiString read FOwnerPassword write FOwnerPassword;
    property ProtectionFlags: TfrxPDFEncBits read FProtectionFlags write SetProtectionFlags;

    property HideToolbar: Boolean read FHideToolbar write FHideToolbar;
    property HideMenubar: Boolean read FHideMenubar write FHideMenubar;
    property HideWindowUI: Boolean read FHideWindowUI write FHideWindowUI;
    property FitWindow: Boolean read FFitWindow write FFitWindow;
    property CenterWindow: Boolean read FCenterWindow write FCenterWindow;
    property PrintScaling: Boolean read FPrintScaling write FPrintScaling;
  end;

{ Returns a color in PDF form. }

function PdfColor(Color: TColor): AnsiString;

{ Returns a pair of coordinates in PDF form. }

function PdfPoint(x, y: Double): AnsiString;

{ Moves the pen to the specified point. }

function PdfMove(x, y: Double): AnsiString;

{ Draws a line to the specified point. }

function PdfLine(x, y: Double): AnsiString;

{ Changes the current color. }

function PdfSetColor(Color: TColor): AnsiString;

{ Changes width of the line drawed by the pen.
  The width is measured in points (1/72 of an inch). }

function PdfSetLineWidth(Width: Double): AnsiString;

{ Changes the color of the pen. }

function PdfSetLineColor(Color: TColor): AnsiString;

{ Fills the latest contoured area. }

function PdfFill: AnsiString;

{ Fills a rectangle area. }

function PdfFillRect(Left, Bottom, Right, Top: Double; Color: TColor): AnsiString;

{ Returns either (...) or <...> sequence. }

function PdfString(const Str: WideString): AnsiString;

implementation
end.
