
{******************************************}
{                                          }
{             FastReport VCL               }
{            DBF export filter             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxExportDBF;

interface

{$I frx.inc}

uses
  {$IFNDEF Linux}
  Windows,
  {$ELSE}
  LCLType, LCLIntf, LCLProc,
  {$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, extctrls, frxClass, frxExportMatrix, frCore
{$IFNDEF Linux}, ShellAPI {$ENDIF}
{$IFDEF DELPHI6}, Variants {$ENDIF}
{$IFDEF FPC}, LazHelper {$ENDIF};
type
  TfrxDBFExportDialog = class(TForm)
    OkB: TButton;
    CancelB: TButton;
    sd: TSaveDialog;
    GroupPageRange: TGroupBox;
    DescrL: TLabel;
    AllRB: TRadioButton;
    CurPageRB: TRadioButton;
    PageNumbersRB: TRadioButton;
    PageNumbersE: TEdit;
    GroupQuality: TGroupBox;
    OpenCB: TCheckBox;
    OEMCB: TCheckBox;
    gbFNames: TGroupBox;
    rbFNAuto: TRadioButton;
    rbFNManual: TRadioButton;
    btFNLoad: TButton;
    odFN: TOpenDialog;
    mmFN: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure PageNumbersEChange(Sender: TObject);
    procedure PageNumbersEKeyPress(Sender: TObject; var Key: Char);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btFNLoadClick(Sender: TObject);
  end;

{$IFDEF DELPHI16}
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxDBFExport = class(TfrxCustomExportFilter)
  private
    FOpenAfterExport: Boolean;
    FMatrix:      TfrxIEMatrix;
    Exp:          TStream;
    FOEM:         Boolean;
    FFieldNames:  TStrings;
    FFieldPrefix: AnsiString;

    procedure SetFieldNames(Value: TStrings);
    procedure ExportMatrix(Stream: TStream; mx: TfrxIEMatrix);
    procedure SetFieldPrefix(const Value: AnsiString);

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function GetDescription: String; override;
    function ShowModal: TModalResult; override;
    function Start: Boolean; override;
    procedure Finish; override;
    procedure FinishPage(Page: TfrxReportPage; Index: Integer); override;
    procedure StartPage(Page: TfrxReportPage; Index: Integer); override;
    procedure ExportObject(Obj: TfrxComponent); override;

  published
    property OEMCodepage: Boolean read FOEM write FOEM;
    property OpenAfterExport: Boolean read FOpenAfterExport write FOpenAfterExport default False;
    property OverwritePrompt;

    // If FieldNames is empty, then
    // FieldPrefix value specifies a short prefix for every DBF
    // field name. Note, that the length of FieldPrefix should not
    // exceed 6 characters. If it does, it will be truncated to 6
    // characters.
    property FieldPrefix: AnsiString read FFieldPrefix write SetFieldPrefix;

    // If FieldNames is empty, names of DBF fields are assigned automatically
    // using FieldPrefix value. If FieldNames is not empty, then it specifies
    // a list of names for DBF fields. Since the exporter canot forecast
    // names of fields in a report. The number of items in FieldNames can
    // be less than the actual number of columns in a report, in this case
    // the missing names of fields are generated automatically using FieldPrefix
    // value due to the common scheme.
    property FieldNames: TStrings read FFieldNames write SetFieldNames;
  end;

implementation
end.
