
{******************************************}
{                                          }
{             FastReport VCL               }
{               DOCX export                }
{                                          }
{         Copyright (c) 1998-2021          }
{                                          }
{******************************************}

unit frxExportDOCX;

interface

{$I frx.inc}

uses
{$IFNDEF FPC}
  Windows, Messages, ShellAPI,
{$ELSE}
  LCLType, LCLIntf, LazHelper,
{$ENDIF}
  SysUtils, Classes, Graphics,
  frxClass, frZip, frxExportMatrix,
  frxImageConverter, frxExportBaseDialog, frPlatformServices, frGraphicUtils,
  frxOfficeOpen, frBaseGraphicsTypes
{$IFDEF DELPHI16}
, System.UITypes
{$ENDIF}
;

type
  TfrxDocxPage = record
    Width: Integer;
    Height: Integer;
    Margins: TRect;
  end;

  TExportType = (dxTable, dxObjects);  //TODO dxText

{$IFDEF DELPHI16}
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxDOCXExport = class(TfrxBaseDialogExportFilter)
  private
    FDocFolder: string;
    FDocument: TStream; // word/document.xml
    FDocRels: TStream; // word/_rels/document.xml.rels
    FMatrix: TfrxIEMatrix;
    FLastPage: TfrxDocxPage;
    FURLNum: Integer;
    FFirstPage: Boolean;
    FPictureType: TfrxPictureType;
    FExportType: TExportType;
    FLeftCurPage, FTopCurPage: Extended;
    FCharSpacing: Integer;
    function SubPath(const s: string): string;
    function SecPr: string;
    procedure WriteRels(URL: string);
  protected
    function EnableCalculateHash: Boolean; override;
  public
    constructor Create(Owner: TComponent); override;
    class function GetDescription: string; override;
    class function ExportDialogClass: TfrxBaseExportDialogClass; override;
    function Start: Boolean; override;
    procedure Finish; override;
    procedure StartPage(Page: TfrxReportPage; Index: Integer); override;
    procedure FinishPage(Page: TfrxReportPage; Index: Integer); override;
    procedure ExportObject(Obj: TfrxComponent); override;
    property CharSpacing: Integer read FCharSpacing write FCharSpacing default -4;
  published
    property OpenAfterExport;
    property OverwritePrompt;
    property PictureType: TfrxPictureType read FPictureType write FPictureType;
    property ExportType: TExportType read FExportType write FExportType;
  end;

  //suport:
  function GetVAlign(a: TfrxVAlign): string;
  function GetHAlign(a: TfrxHAlign): string;
  function GetFont(f: TFont; SubType: TSubStyle = ssNormal; RTL: Boolean = false; AdditionAttr: String = ''): string;
  function BStr(b: Boolean; s: string): string;
  function GetText(const s: string): string;
  function WriteTaggedText(Text: WideString; HTMLTags, RTL: boolean; AStyle: TfrxIEMStyle; AWriter: TfrxWriter): Boolean;
  function StrokedToString(frame: TfrxFrame): String;

implementation
end.
