
{******************************************}
{                                          }
{             FastReport VCL               }
{              SVG 1.1 Export              }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxExportSVG;

interface

{$I frx.inc}

uses
  Windows,
  Classes,
  StrUtils,
  Graphics,
  frxClass,
  frxExportBaseDialog,
{$IFDEF DELPHI16}
  System.UITypes,
{$ENDIF}
{$IFDEF FPC}frUnicodeUtils{$ELSE}WideStrings{$ENDIF},
  frxExportHelpers,
  frCSSStyle,
  frBaseGraphicsTypes;

type
  { SVG export filter }

{$IFDEF DELPHI16}
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxSVGExport = class(TExportHTMLDivSVGParent)
  private
    type
      TTextParams = record
        Center: TfrxPoint;
        Width, Height, Top, Start, Finish: Extended;
        Interval: Extended;
      end;
  private
    FShadowStyle: TfrxCSSStyle;
    FEndCurrentFile: TStream;
    FTargetCurrentFile: TStream;
    FDefs: TStream;

    function GetShadowStyle: TfrxCSSStyle;
    procedure PutImage(Obj: TfrxView; Pic: TGraphic);
    procedure StartSVG(Width, Height: Extended);
    procedure FinishSVG;

    { Handlers for specific kinds of TfrxView objects.
      They return "True" if they succeed to export an object, or "False"
      if they want to pass the object further along the handlers chain. }
    function ExportAsPicture  (Obj: TfrxView): Boolean;
    function ExportPicture    (Obj: TfrxView): Boolean;
    function ExportViaEMF     (Obj: TfrxView): Boolean;
    function ExportGradient   (Obj: TfrxView): Boolean;
    function ExportLine       (Obj: TfrxView): Boolean;
    function ExportShape      (Obj: TfrxView): Boolean;
    function ExportMemo       (Obj: TfrxView): Boolean;

    function CalcTextParams(Memo: TfrxCustomMemoView; LineCount: Integer): TTextParams;
    function MeasureDirection(Memo: TfrxCustomMemoView): string;
    function MeasureDominantBaseline(Memo: TfrxCustomMemoView): string;
    function MeasureDy(i: Integer; Dy: Extended): string;
    function MeasureFontFamily(Memo: TfrxCustomMemoView): string;
    function MeasureTextAnchor(Memo: TfrxCustomMemoView): string;
    function MeasureTextDecoration(Memo: TfrxCustomMemoView): string;
  protected
    FGlobalPageY: Extended;

    procedure RunExportsChain(Obj: TfrxView); override;

    procedure DoGradient(Obj: TfrxView; BeginValue, EndValue: string; Style: TfrxGradientStyle; ClipValue: string = '');
    procedure DoFrameLine(x1, y1, x2, y2: Extended; frxFrameLine: TfrxFrameLine);
    procedure DoFill(Obj: TfrxView);
    procedure DoFrame(Obj: TfrxView);
    procedure DoFilledRect(x, y, Width, Height: integer; FillValue: string; ClipValue: string = ''); overload;
    procedure DoFilledRect(Obj: TfrxView; FillValue: string; ClipValue: string = ''); overload;
    function DoHyperLink(Obj: TfrxView): boolean;
    procedure DoExportAsPicture(Obj: TfrxView; Transparent: boolean; IsVectorSource: Boolean; IsAlphaSource: Boolean; TransparentColor: TColor = clNone);
    function WrapByTSpan(const TextList: TWideStrings; Memo: TfrxCustomMemoView;
                         const TP: TTextParams; Arabic: Boolean): WideString;
    function DefineShapeClipPath(Obj: TfrxView): string;
    function DefineRectClipPath(Obj: TfrxView): string;

    procedure StartAnchors;
    procedure StartNavigator;
    procedure CreateCSS; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    class function GetDescription: string; override;
    class function ExportDialogClass: TfrxBaseExportDialogClass; override;

    procedure StartPage(Page: TfrxReportPage; Index: Integer); override;
    procedure FinishPage(Page: TfrxReportPage; Index: Integer); override;
    procedure Finish; override;

    property ShadowStyle: TfrxCSSStyle read GetShadowStyle;
  published
  end;

implementation
end.
