unit frxExportZPL;

interface

{$I frx.inc}

uses
{$IFNDEF Linux}
  Windows,
{$ELSE}
  LCLType, LCLIntf, LCLProc,
{$ENDIF}
  Messages, frxExportBaseDialog, SysUtils, Classes, Graphics, frCore, frxClass,
  frxExportMatrix, Math, frxBarcod, frxBarcode, frxBarcode2D,
  frxBarcodeMaxiCode, frxTableObject, frxRes, frxExportZPLDialog,
  frBaseGraphicsTypes;

type
    TZplDensity = (d6_dpmm_152_dpi, d8_dpmm_203_dpi, d12_dpmm_300_dpi, d24_dpmm_600_dpi, test1to1);

type TZplScale = record
    PageScale, BarcodeScale: Double;
    TwoDCodeScale: Integer;
  end;

const defaultFontScale: double = 1.4;
const Dictionary: String = 'GHIJKLMNOPQRSTUVWXYghijklmnopqrstuvwxyz';
const counts: array[0..38] of Integer = (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,
                                         20,40,60,80,100,120,140,160,180,200,220,240,260,280,300,320,340,360,380,400);
const ZplScale: array[0..4] of TZplScale = (
                                            (PageScale : 1.5833 ; BarcodeScale : 2;   TwoDCodeScale : 7),     //6 dpmm(152 dpi)
                                            (PageScale : 2.11458; BarcodeScale : 2.5; TwoDCodeScale : 8),    //8 dpmm(203 dpi)
                                            (PageScale : 3.16667; BarcodeScale : 4;   TwoDCodeScale : 10),    //12 dpmm(300 dpi)
                                            (PageScale : 6.34375; BarcodeScale : 8;   TwoDCodeScale : 10),    //24 dpmm(600 dpi)
                                            (PageScale : 1      ; BarcodeScale : 1;   TwoDCodeScale : 1 ));   //test

type

{$IFDEF DELPHI16}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxZPLExport = class(TfrxBaseDialogExportFilter)
  private
    FPage: TfrxReportPage;
    FPrintAsBitmap, FBMillimeters, FBreakLines, FHexText: Boolean;
    Exp: TStream;
    FFontScale, leftMargin, topMargin: double;
    PageBitmap: TBitmap;
    FZplDensity: TZplDensity;
  public
    PrinterInit, CodePage, PrinterFinish, PageInit, PrinterFont: String;
    scaleIndex: Byte;
  private
    procedure ZWrite  (s: String);
    procedure ZWriteLn(s: String);
    procedure ZFDWriteLn(s: String; prefix: String = '');

    procedure SetPosition      (left, top: Integer);
    procedure SetTextAttributes(width, lines, leading: Integer; HAlign :TfrxHAlign; gap: Integer);
    procedure DrawRectangle    (width, height, lineWidth: Integer);
    procedure DrawEllipse      (width, height, lineWidth: Integer);
    procedure DrawLine         (width, height, lineWidth: Integer; direction: Boolean);
    procedure DrawText         (fontHeight, fontWidth: Integer; text: String);
    procedure DrawPictureObject(pic: TfrxPictureView);
    procedure DrawBorders      (Frame :TfrxFrame; AbsLeft, AbsTop, Width, Height: Double);
    procedure DrawBWPicture    (tbm: TBitMap);
    procedure WriteRotation    (rot: Integer);

    procedure ExportTextObject     (FfrxCMV: TfrxCustomMemoView);
    procedure ExportRectangle      (Left, Top, Width, Height, LineWidth: Double);
    procedure ExportEllipse        (Left, Top, Width, Height, LineWidth: Double);
    procedure ExportLine           (Left, Top, Width, Height, LineWidth: Double);
    procedure ExportPicture        (pic: TfrxPictureView);
    procedure ExportLineObject     (line: TfrxLineView);
    procedure ExportBarcodeObject  (b: TfrxBarCodeView);
    procedure Export2DBarcodeObject(b: TfrxBarCode2DView);
    procedure ExportShapeObject    (ShapeObject: TfrxShapeView);

    function  GetWidth (width, height:Double; rotation: Integer): Integer;
    function  GetTop   (top: Double): Integer;
    function  GetHeight(width, height: Double; rotation: Integer): Integer;
    function  GetLeft  (left: Double): Integer;

    function  GetZPLText(source: String) : String;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    class function GetDescription: String; override;
    class function ExportDialogClass: TfrxBaseExportDialogClass; override;
    function Start: Boolean; override;
    procedure Finish; override;
    procedure FinishPage(Page: TfrxReportPage; Index: Integer); override;
    procedure StartPage(Page: TfrxReportPage; Index: Integer); override;
    procedure ExportObject(Obj: TfrxComponent); override;

    procedure SetFZplDensity(ZPLD: TZplDensity);
  published
    property PrintAsBitmap: Boolean read FPrintAsBitmap write FPrintAsBitmap default True;
    property BMillimeters: Boolean read FBMillimeters write FBMillimeters default True;
    property BreakLines: Boolean read FBreakLines write FBreakLines default True;
    property FontScale: double read FFontScale write FFontScale;
    property ZplDensity: TZplDensity read FZplDensity write SetFZplDensity default d8_dpmm_203_dpi;
    property HexText: Boolean read FHexText write FHexText default False;
end;


{$IFNDEF FPC}
    procedure WriteToPrinter(PrinterIndex: Integer; PrinterName, SText: String); overload;
    procedure WriteToPrinter(PrinterIndex: Integer; PrinterName: String; mem: TMemoryStream); overload;
{$ENDIF}
implementation
end.
