unit frxTrueTypeFont;

interface

{$I frx.inc}

uses
      TTFHelpers, frxCmapTableClass, frxFontHeaderClass, frxGlyphTableClass,
      frxGlyphSubstitutionClass, frxHorizontalHeaderClass,
      frxHorizontalMetrixClass, frxIndexToLocationClass,
      frxKerningTableClass, frxNameTableClass, frxPostScriptClass,
      frxMaximumProfileClass, frxOS2WindowsMetricsClass,
      Classes, frxTrueTypeTable,
      frxPreProgramClass
    {$IFNDEF Linux}
    {$ELSE}
      , LCLType, LCLIntf, LCLProc, Controls, Dialogs, LazHelper
    {$ENDIF};

type
    ChecksumFaultAction = (ChecksumFaultAction_IgnoreChecksum=0, ChecksumFaultAction_ThrowException=1, ChecksumFaultAction_Warn=2);
    TablesID = (TablesID_CMAP=$70616d63, TablesID_ControlValueTable=$20747663, TablesID_DigitalSignature=$47495344,
      TablesID_EmbedBitmapLocation=$434c4245, TablesID_EmbededBitmapData=$54444245, TablesID_FontHeader=$64616568,
      TablesID_FontProgram=$6d677066, TablesID_Glyph=$66796c67, TablesID_GlyphDefinition=$46454447, TablesID_GlyphPosition=$534f5047,
      TablesID_GlyphSubstitution=$42555347, TablesID_GridFittingAndScanConversion=$70736167, TablesID_HorizontakDeviceMetrix=$786d6468,
      TablesID_HorizontalHeader=$61656868, TablesID_HorizontalMetrix=$78746d68, TablesID_IndexToLocation=$61636f6c,
      TablesID_Justification=$4654534a, TablesID_KerningTable=$6e72656b, TablesID_LinearThreshold=$4853544c,
      TablesID_MaximumProfile=$7078616d, TablesID_Name=$656d616e, TablesID_OS2Table=$322f534f, TablesID_PCL5Table=$544c4350,
      TablesID_Postscript=$74736f70, TablesID_PreProgram=$70657270, TablesID_VerticalDeviceMetrix=$584d4456,
      TablesID_VerticalMetrix=$78746d76, TablesID_VertivalMetrixHeader=$61656876);

    TByteArray = Array of Byte;
    TTagList = TList;

    TTableSortHelper = record
      Offset: Cardinal;
      TAG:    Cardinal;
    end;

    PTableSortHelper = ^TTableSortHelper;

    TableDirectory = packed record
        sfntversion: Cardinal;
        numTables: Word;
        searchRange: Word;
        entrySelector: Word;
        rangeShift: Word;
    end;

    TrueTypeFont = class(TTF_Helpers)
      // Fields
    private
      beginfile_ptr: Pointer;
      cmap_table: CmapTableClass;
      dir: TableDirectory;
      font_header: FontHeaderClass;
      glyph_table: GlyphTableClass;
      gsub_table: GlyphSubstitutionClass;
      horizontal_header: HorizontalHeaderClass;
      horizontal_metrix_table: HorizontalMetrixClass;
      index_to_location: IndexToLocationClass;
      kerning_table: KerningTableClass;
      name_table: NameTableClass;
      postscript: PostScriptClass;
      profile: MaximumProfileClass;
      selector_ptr: Pointer;
      windows_metrix: OS2WindowsMetricsClass;
      preprogram_table: PreProgramClass;
      ListOfTables: TList;
      ListOfUsedGlyphs: TList;
      // private ListOfUsedWidths: TList;
      GlyphWidths: TIntegerList;
      Indices: TIntegerList;
      FSubstitutionName: String;
      function CalcTableChecksum(font: Pointer; entry: TrueTypeTable;
        debug: boolean): Cardinal;
      procedure CalculateFontChecksum(start_offset: Pointer;
        font_length: Cardinal);
      procedure ChangeEndian;
      procedure CheckTablesChecksum;
      {$IFDEF Linux}
      public procedure GetOutlineTextMetrics(var FTextMetric: OUTLINETEXTMETRIC);
      public function GetGlyphIndices(text: WideString; glyphs: PWord; widths: PInteger; rtl: boolean): Integer;
      {$ENDIF}
      function GetTablesOrder: TTagList;
      procedure LoadCoreTables;
      procedure LoadDescriptors(skip_array: TIntegerList);
      procedure ReorderGlyphTable(position: Pointer; uniscribe: boolean);
      procedure SaveDescriptors(position: Pointer);
      procedure set_UsedGlyphs(dict: TList);
      function get_GlyphWidth(glph: Integer): Cardinal;
      function get_Glyph(idx: Integer): Cardinal;
      function GetFamilyName: WideString;
      function GetPostScriptName: WideString;
      // Methods
    public
      checksum_action: ChecksumFaultAction;
      constructor Create(bgn: Pointer; font: Pointer;
        action: ChecksumFaultAction);
      destructor Destroy; override;
      procedure AddCharacterToKeepList(ch: WideChar);
      procedure ParseSingleGlyph(idx: Word);
      procedure BuildGlyphIndexList(used_glyphs: TList; uniscribe: boolean;
        decompose: boolean; collate: boolean; use_kerning: boolean
        {$IFDEF Linux}; SkipComposite: Boolean = False; SkipSpace: Boolean = True{$ENDIF}
        );
      function PackFont(translate_to: FontType; uniscribe: boolean): TByteArray;
      procedure PrepareFont(skip_list: TIntegerList);
      property Names: NameTableClass read name_table;
      property UsedGlyphs: TList write set_UsedGlyphs;
      property Widths: TList read GlyphWidths;
      property Width[glph: Integer]: Cardinal read get_GlyphWidth;
      property Glyph[idx: Integer]: Cardinal read get_Glyph;
      property SubstitutionName: String read FSubstitutionName
        write FSubstitutionName;
      property FamilyName: WideString read GetFamilyName;
      property PostScriptName: WideString read GetPostScriptName;
    end;

procedure AddIgnoreChecksum(FamilyName: String);
procedure DeleteIgnoreChecksum(FamilyName: String);

implementation
end.
