{******************************************}
{                                          }
{             FastReport VCL               }
{             PDFView class                }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxPDFViewer;

interface

{$I frx.inc}

uses
{$IFNDEF Linux}
  Windows,
{$ELSE}
  LCLType, LCLIntf, LCLProc,
  {$IFDEF LCLGTK2}Printers, {$ENDIF}
{$ENDIF}
  Controls, Classes, Types, Graphics, SysUtils, frxClass, frxDsgnIntf, frxPDFium, frxProtocolFactory;

type
{$IFDEF DELPHI16}
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxPDFObject = class(TComponent);  // fake component

  TDetailStretchMode = (
  pdOneToOneStrongStretch = 0,
  pdOneToOneNormalize = 1,
  pdManyToOneNormalize = 2
  //, pdManyToOneSmart //TODO
  );

  TfrxDrawOptions = class;

  TfrxPDFView = class(TfrxStretcheable, IfrxDataLinkObject, IfrxDataBindingComponent)
  private
    FPDFDocument: TPdfDocument;
    FMemoryPDF, FTempStream: TMemoryStream;
    FDrawPartIndex: Integer;
    FDefHeight, FLastPadding: Extended;
    FHasNextDataPart: Boolean;

    FRotation: Integer;
    FProp: TfrxDrawOptions;
    FPassword: String;
    //FRange: String; //TODO
    FFileLink: String;
    FDetailStretchMode: TDetailStretchMode;
    FDataLink: TfrxDataLink;

    procedure PDFDraw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended; DrawRect: TRect);
    procedure DrawError(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended; DrawRect: TRect);
    procedure ReadData(Stream: TStream);
    procedure WriteData(Stream: TStream);
    procedure Normalize(CX, CY: Integer; var DX, DY: Integer);
    procedure CheckEmpty;

    procedure SetProp(v: TfrxDrawOptions);
    procedure SetPassword(val: String);

    procedure ClearObjectData;
    function LoadDataStream(Stream: TStream; const NewLink: String): Boolean;
    function GetLink(LoadMethod: TfrxDataLinkLoadMethod): String;
    function IsExpressionLink: Boolean;
    function GetDataLink: TfrxDataLink;
    procedure SetDataLink(const Value: TfrxDataLink);
    function IsDataLinkStored: Boolean;

  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    PDFDrawOptions: TPdfDrawOptions;
    constructor Create(AComponent:TComponent); override;
    destructor Destroy; override;
    function IsOneToOne: Boolean;
    class function GetDescription: String; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    procedure BeforePrint; override;
    procedure AfterPrint; override;
    procedure LoadPDFFromStream(AStream: TStream; const APassword: String = '');
    procedure LoadFromFile(const AFileName: string);
    function CalcHeight: Extended; override;
    function DrawPart: Extended; override;
    function HasNextDataPart(aFreeSpace: Extended): Boolean; override;
    procedure GetData; override;
    function GetLastShiftAmount(aFreeSpace: Extended): Extended; override;
    procedure EditorDraw(Canvas: TCanvas; DrawRect: TRect);
    procedure ClearPDF;

    property PDFDocument: TPdfDocument read FPDFDocument;
    property MemoryPDF: TMemoryStream read FMemoryPDF;
  published
    property Rotation: Integer read FRotation write FRotation default 0;
    property FileLink: String read FFileLink write FFileLink;
    property DataLink: TfrxDataLink read GetDataLink write SetDataLink stored IsDataLinkStored;
    property DetailStretchMode: TDetailStretchMode read FDetailStretchMode write FDetailStretchMode;
    property Password: String read FPassword write SetPassword;
    property DrawOptions: TfrxDrawOptions read FProp write SetProp;
    //property Range: Integer read FRange write FRange;
    property DataField;
    property DataSet;
    property DataSetName;
    property FillType;
    property Fill;
    property Frame;
    property BrushStyle;
    property Color;
    property Cursor;
    property Style;
  end;

  TfrxDrawOptions = class(TPersistent)
  private
    FWhose: TfrxPDFView;
    function GetValue(const Index: Integer): Boolean;
    procedure SetValue(const Index: Integer; const Value: Boolean);
  public
    procedure Assign(Source: TPersistent); override;
    procedure SetWhose(w: TfrxPDFView);
  published
    property Annotations: Boolean index Integer(pdoAnnotations) read GetValue write SetValue;
    property LCDOptimized: Boolean index Integer(pdoLCDOptimized) read GetValue write SetValue;
    property NoNativeText: Boolean index Integer(pdoNoNativeText) read GetValue write SetValue;
    property GrayScale: Boolean index Integer(pdoGrayScale) read GetValue write SetValue;
    property NoCatch: Boolean index Integer(pdoNoCatch) read GetValue write SetValue;
    property LimitedImageCacheSize: Boolean index Integer(pdoLimitedImageCacheSize) read GetValue write SetValue;
    property ForceHalftone: Boolean index Integer(pdoForceHalftone) read GetValue write SetValue;
    property FastDraw: Boolean index Integer(pdoFastDraw) read GetValue write SetValue;
    property PrintTextWithGDI: Boolean index Integer(pdoPrintTextWithGDI) read GetValue write SetValue;
  end;

  procedure frxAssignPDF(PDFFrom, PDFTo: TfrxPDFView);
  procedure frxSetPDFiumDLLPath(Path: String);

implementation
end.
