
{******************************************}
{                                          }
{             FastReport VCL               }
{           Aggregate Functions            }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxAggregate;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}
  Windows,
  {$ENDIF}
  SysUtils, Classes, Dialogs, frxClass
{$IFDEF DELPHI6}
, Variants
{$ENDIF};


type
  TfrxAggregateFunction = (agSum, agAvg, agMin, agMax, agCount);

  TfrxAggregateItem = class(TObject)
  private
    FAggregateFunction: TfrxAggregateFunction;
    FDataRow: TfrxComponent;
    FCountInvisibleBands: Boolean;
    FDontReset: Boolean;
    FExpression: String;
    FIsPageFooter: Boolean;
    FItemsArray: Variant;      { used for vbands }
    FItemsCount: Integer;
    FItemsCountArray: Variant; { used for vbands }
    FItemsValue: Variant;
    FKeeping: Boolean;
    FLastCount: Integer;
    FLastValue: Variant;
    { used in Save/Restore }
    FSavedCount: Integer;
    FSavedValue: Variant;
    FMemoName: String;
    FOriginalName: String;
    FParentContainer: TfrxComponent;
    FReport: TfrxReport;
    FTempItemsCount: Integer;
    FTempItemsValue: Variant;
    FVColumn: Integer;         { used for vbands }
  public
    procedure Calc(AValue: Variant);
    procedure DeleteValue;
    procedure SaveValue;
    procedure RestoreValue;
    procedure Reset;
    procedure StartKeep;
    procedure EndKeep;
    function Value: Variant;
    property DataRow: TfrxComponent read FDataRow;
  end;

  TfrxAggregateList = class(TfrxCustomAggregateList)
  private
    FList: TList;
    FReport: TfrxReport;
    function GetItem(Index: Integer): TfrxAggregateItem;
    procedure FindAggregate(AComponent: TfrxComponent; const Text: String; const ExpressionDelimiters: String; DataRow: TfrxComponent; AList: TList = nil; MainParent: TfrxComponent = nil; IsAggregatedObj: Boolean = False); overload;
    procedure FindAggregate(Memo: TfrxCustomMemoView; DataRow: TfrxComponent); overload;
    procedure ParseName(const ComplexName: String; var Func: TfrxAggregateFunction;
      var Expr: String; var DataRow: TfrxComponent; var CountInvisible, DontReset: Boolean; MainParent: TfrxComponent = nil; IsAggregatedObj: Boolean = False);
    property Items[Index: Integer]: TfrxAggregateItem read GetItem; default;
  public
    constructor Create(AReport: TfrxReport);
    destructor Destroy; override;
    procedure Clear; override;
    procedure ClearValues; override;
    procedure AddItems(Page: TfrxReportPage); override;
    function AddAggregatedItem(AObject: IfrxAggregateObject; Container: TfrxComponent; AList: TList = nil): Boolean; override;
    procedure AddValue(DataRow: TfrxComponent; VColumn: Integer = 0); override;
    function AddCalculatedValue(DataRow: TfrxComponent; const Value: Variant; VColumn: Integer = 0): Boolean; override;
    procedure DeleteValue(DataRow: TfrxComponent); override;
    procedure SaveValue(DataRow: TfrxComponent); override;
    procedure RestoreValue(DataRow: TfrxComponent); override;
    procedure EndKeep; override;
    procedure Reset(ParentContainer: TfrxComponent); override;
    procedure StartKeep; override;
    function GetValue(ParentContainer: TfrxComponent; const ComplexName: String;
      VColumn: Integer = 0): Variant; overload; override;
    function GetValue(ParentContainer: TfrxComponent; VColumn: Integer;
      const Name, Expression: String; DataRow: TfrxComponent; Flags: Integer): Variant; overload; override;
  end;


implementation
end.
