
{******************************************}
{                                          }
{             FastReport VCL               }
{            Analitic Geometry             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxAnaliticGeometry;

{$I frx.inc}

interface

uses
  Types,
  frxClass;

type
  TDoublePoint = record
    X, Y: Double;
  end;

  TDoubleRect = record
    case Integer of
      0: (Left, Top, Right, Bottom: Double);
      1: (TopLeft, BottomRight: TDoublePoint);
  end;

  TPointArray = array of TPoint;
  TDoubleArray = array of Double;
  TDoublePointArray = array of TDoublePoint;
  TDoublePointMatrix = array of TDoublePointArray;

type
  TCircle = record
    X, Y, Radius: Extended;
  end;

  TSegment = record
    First, Second: TfrxPoint;
  end;

function Circle(X, Y, Radius: Extended): TCircle;

function Segment(P1, P2: TPoint): TSegment; overload;
function Segment(x1, y1, x2, y2: Extended): TSegment; overload;
function Segment(P1, P2: TDoublePoint): TSegment; overload;
function Segment(P1, P2: TfrxPoint): TSegment; overload;

type
  TMinDistance = class
  private
  protected
    FValue: Extended;
    FIndex: Integer;
    FTreshold: Extended;
  public
    constructor Create(ATreshold: Extended = 0.0);
    procedure Init(ATreshold: Extended);
    procedure Add(AValue: Extended; AIndex: Integer);
    function IsNear: Boolean;
    function IsZero: Boolean;

    property Index: Integer read FIndex;
  end;

function DoublePoint(X, Y: Extended): TDoublePoint; overload;
function DoublePoint(P: TfrxPoint): TDoublePoint; overload;
function DoublePoint(P: TPoint): TDoublePoint; overload;
function DoubleRect(TL, BR: TfrxPoint): TDoubleRect; overload;
function DoubleRect(Left, Top, Right, Bottom: Double): TDoubleRect; overload;
function DoubleRect(R: TfrxRect): TDoubleRect; overload;
function DoubleRect(R: TRect): TDoubleRect; overload;
function DoubleRectExpand(DR: TDoubleRect; dX, dY: Double): TDoubleRect; overload;
function DoubleRectExpand(DR: TDoubleRect; d: Double): TDoubleRect; overload;
function ConstrainedDR(DR: TDoubleRect; ConstrainWidth, ConstrainHeight: Double): TDoubleRect;
function ToRect(DR: TDoubleRect): TRect;
function ToPoint(X, Y: Extended): TPoint; overload;
function ToPoint(frxPoint: TfrxPoint): TPoint; overload;
function ToFrxPoint(Point: TPoint): TfrxPoint;
function CenterRect(R: TfrxRect): TfrxPoint;
procedure InitRect(var R: TfrxRect; DP: TDoublePoint);
procedure ExpandRect(var R: TfrxRect; X, Y: Extended); overload;
procedure ExpandRect(var R: TfrxRect; DP: TDoublePoint); overload;
procedure ExpandRect(var R: TfrxRect; R1: TfrxRect); overload;
function RectWidth(R: TfrxRect): Extended; overload;
function RectWidth(R: TRect): Integer; overload;
function RectHeight(R: TfrxRect): Extended;
function Boundary(Value, MinValue, MaxValue: Extended): Extended;
function frxCanonicalRect(ALeft, ATop, ARight, ABottom: Extended): TfrxRect; overload;
function frxCanonicalRect(DR: TDoubleRect): TfrxRect; overload;
function IsPointInRect(P: TfrxPoint; R: TfrxRect): Boolean;
function IsInside(Value, Start, Finish: Extended): Boolean;
function IsInsideRect(R: TDoubleRect; P: TfrxPoint): Boolean;
function IsInsideEllipse(R: TDoubleRect; P: TfrxPoint): Boolean;
function IsInsidePolyline(Poly: TDoublePointArray; P: TfrxPoint): Boolean;
function IsInsidePolygon(Poly: TDoublePointArray; P: TfrxPoint): Boolean;
function IsInsideDiamond(R: TDoubleRect; P: TfrxPoint): Boolean;
function IsInsideMultiPolygon(Matrix: TDoublePointMatrix; P: TfrxPoint): Boolean;
function IsInsideMultiPolyline(Matrix: TDoublePointMatrix; P: TfrxPoint): Boolean;

function IsSegmentsIntersect(S1, S2: TSegment): Boolean;
function IsPointInPolygon(x, y: Extended; PolyPoints: TPointArray): Boolean;
function Distance(P1, P2: TfrxPoint): Extended; overload;
function Distance(P1: TDoublePoint; P2: TfrxPoint): Extended; overload;
function Distance(P: TfrxPoint; X, Y: Extended): Extended; overload;
function Distance(P: TPoint; X, Y: Extended): Extended; overload;
function DistancePolyline(Poly: TDoublePointArray; P: TfrxPoint): Extended;
function DistancePolygon(Poly: TDoublePointArray; P: TfrxPoint): Extended;
function DistanceSegment(Segment: TSegment; P: TfrxPoint): Extended;
function DistanceRect(Rect: TDoubleRect; P: TfrxPoint): Extended;
function DistanceDiamond(Rect: TDoubleRect; P: TfrxPoint): Extended;
function DistanceEllipse(Rect: TDoubleRect; P: TfrxPoint): Extended;
function DistancePicture(Rect: TDoubleRect; P: TfrxPoint): Extended;
function DistanceTemplate(Poly: TDoublePointArray; P: TfrxPoint): Extended;
function DistanceMultiPoint(Matrix: TDoublePointMatrix; P: TfrxPoint): Extended;
function DistanceMultiPolyline(Matrix: TDoublePointMatrix; P: TfrxPoint): Extended;
function DistanceMultiPolygon(Matrix: TDoublePointMatrix; P: TfrxPoint): Extended;

function IntersectionEllipse(Rect: TDoubleRect; P: TfrxPoint): TfrxPoint;

function IsEqual(P1, P2: TDoublePoint): Boolean;

const
  Unknown = -1;
  MinSelectDistance = 8;

implementation
end.
