
{******************************************}
{                                          }
{             FastReport VCL               }
{          Barcode Add-in object           }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxBarcode;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}Windows, Messages, {$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, frxBarcod, frxClass, ExtCtrls, frxBarcodeInterface,
  frBaseGraphicsTypes
{$IFDEF FPC}
  , LCLType, LCLIntf, LazarusPackageIntf, LazHelper, frLazGraphicClasses
{$ENDIF}
{$IFDEF DELPHI6}
, Variants
{$ENDIF}
{$IFDEF DELPHI16}
, System.Types
{$ENDIF}
;

const
  frxBarcodeCategoryName = 'Barcode';

type
{$IFDEF DELPHI16}
/// <summary>
///   The TfrxBarCodeObject allows the use of the BarCode component in your
///   report. TfrxBarCodeObject is an empty component. It is used for adding
///   the frxBarCode.pas file to the "uses" list. The main component is
///   TfrxBarCodeView.
/// </summary>
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxBarCodeObject = class(TComponent);  // fake component

  TfrxDigitsAutoFillMode = (dafmLeftOnly, dafmRightChecksum);

  /// <summary>
  ///   The TfrxBarCodeView component represents a barcode. it shows the Text
  ///   property value as a barcode. It can also show a barcode from the DB
  ///   field (you should set DataSet, DataField properties) or from an
  ///   expression (Expression property).
  /// </summary>
  TfrxBarCodeView = class(TfrxStyledView, IfrxBarCodeView, IfrxDataBindingComponent)
  private
    FBarCode: TfrxBarCode;
    FBarType: TfrxBarcodeType;
    FCalcCheckSum: Boolean;
    FExpression: String;
    FHAlign: TfrxHAlign;
    FRotation: Integer;
    FShowText: Boolean;
    FTestLine: Boolean;
    FText: String;
    FTextSup: String;
    FWideBarRatio: Extended;
    FZoom: Extended;
    FMacroLoaded: Boolean;
    FAutoSize: Boolean;
    FFirstTryAfterSBT: boolean;
    procedure BcFontChanged(Sender: TObject);
    procedure TypeDefStr(v: TfrxBarcodeType);
    procedure SetBarType(v: TfrxBarcodeType);
    function GetBarcodeText: TfrxBarcodeText;
    procedure SetBarcodeText(const Value: TfrxBarcodeText);
    function GetBarcodeType: Integer;
    procedure SetBarcodeType(const Value: Integer);
    function GetDigitsAutoFillMode: TfrxDigitsAutoFillMode;
    procedure SetDigitsAutoFillMode(const Value: TfrxDigitsAutoFillMode);
  public
    constructor Create(AOwner: TComponent); override;
    constructor DesignCreate(AOwner: TComponent; Flags: Word); override;
    destructor Destroy; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    procedure GetData; override;
    function GetText: String;
    procedure SetText(Value: String);
    function GetTextSup: String;
    procedure SetTextSup(Value: String);
    procedure SetZoom(v: Extended);
    function GetZoom: Extended;
    function GetFont: TFont;

    function GetFontHeightTop: Integer;
    function GetFontHeightBottom: Integer;

    class function GetDescription: String; override;
    function GetRealBounds: TfrxRect; override;
    procedure SetColorBar(c: TColor);
    function GetColorBar: TColor;

    procedure SaveContentToDictionary(aReport: TfrxReport; PostProcessor: TfrxPostProcessor); override;
    function LoadContentFromDictionary(aReport: TfrxReport; aItem: TfrxMacrosItem): Boolean; override;
    procedure ProcessDictionary(aItem: TfrxMacrosItem; aReport: TfrxReport; PostProcessor: TfrxPostProcessor); override;

    /// <summary>
    ///   Reference to internal TfrxBarCode object.
    /// </summary>
    property BarCode: TfrxBarCode read FBarCode;
  published
    property AutoSize: Boolean read FAutoSize write FAutoSize default True;
    /// <summary>
    ///   Type of the barcode.
    /// </summary>
    property BarType: TfrxBarcodeType read FBarType write SetBarType;
    property BrushStyle;
    /// <summary>
    ///   Determines if a checksum calc is required.
    /// </summary>
    property CalcCheckSum: Boolean read FCalcCheckSum write FCalcCheckSum default False;
    property FillType;
    property Fill;
    property Color;
    property Cursor;
    property DataField;
    property DataSet;
    property DataSetName;
    property DigitsAutoFillMode: TfrxDigitsAutoFillMode read GetDigitsAutoFillMode write SetDigitsAutoFillMode default dafmLeftOnly;
    /// <summary>
    ///   Value of the expression is printed as a barcode.
    /// </summary>
    property Expression: String read FExpression write FExpression;
    property Frame;
    /// <summary>
    ///   Determines how the Left property will be changed depending on the
    ///   barcode width. Valid values are: <br />haLeft - don't change Left; <br />
    ///   haRight - change Left so right edge is locked; <br />haCenter -
    ///   change Left so center is locked.
    /// </summary>
    property HAlign: TfrxHAlign read FHAlign write FHAlign default haLeft;
    property Processing;
    /// <summary>
    ///   Rotation of the barcode, in degrees.
    /// </summary>
    property Rotation: Integer read FRotation write FRotation;
    /// <summary>
    ///   Determines if we show the barcode text.
    /// </summary>
    property ShowText: Boolean read FShowText write FShowText default True;
    property Style;
    property TagStr;
    property TestLine: Boolean read FTestLine write FTestLine;
    /// <summary>
    ///   Text of the barcode.
    /// </summary>
    property Text: String read GetText write SetText;
    property TextSup: String read GetTextSup write SetTextSup;
    property URL;
    property WideBarRatio: Extended read FWideBarRatio write FWideBarRatio;
    /// <summary>
    ///   Zoom of the barcode, can be non-integer.
    /// </summary>
    property Zoom: Extended read GetZoom write SetZoom;
    property Font: TFont read GetFont stored False;
    property ColorBar: TColor read GetColorBar write SetColorBar;
    property BarcodeText: TfrxBarcodeText read GetBarcodeText write SetBarcodeText;
  end;

{$IFDEF FPC}
//  procedure Register;
{$ENDIF}
implementation
end.
