
{ ****************************************** }
{                                            }
{             FastReport VCL                 }
{          Barcode Add-in object             }
{                                            }
{         Copyright (c) 1998-2021            }
{            by Fast Reports Inc.            }
{                                            }
{ ****************************************** }

unit frxBarcode2D;

interface

{$I frx.inc}

uses
{$IFDEF FPC}
  frLazGraphicClasses, LCLType, LMessages, LazHelper, LCLIntf,
{$ELSE}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Types, Controls, Variants, Forms, Dialogs,
  StdCtrls, Menus, ExtCtrls,
  frxClass, frxBarcodePDF417, frxBarcodeDataMatrix, frxBarcodeQR,
  frxBarcodeAztec, frxBarcodeMaxiCode, frxGS1Databar, frxBarcodePharmacodeTT,
  frxBarcode2DBase, frxBarcodeProperties, frx2DBarcodesPresets,
  frxBarcodeInterface, frBaseGraphicsTypes;

type

  //  - ////////////////////////////////////////////////////////////////////////
  TfrxBarcode2DType = (bcCodePDF417, bcCodeDataMatrix, bcCodeQR, bcCodeAztec, bcCodeMaxiCode, bcGS1DatabarE, bcGS1DatabarES, bcPharmacodeTT);

  TfrxBarcode2DView = class(TfrxStretcheable, IInspectedProperties, IfrxBarCodeView, IfrxDataBindingComponent)
  private
    FBarCode: TfrxBarcode2DBase;
    FBarType: TfrxBarcode2DType;
    FHAlign: TfrxHAlign;
    FProp: TfrxBarcode2DProperties;
    //       
    FExpression: String;
    FMacroLoaded: Boolean;
    FAutoSize: Boolean;
    FExpressionPreset: TfrxObjectDataPreset;
    FExtPropList: TfrxExtPropList;
    procedure SetZoom(z: Extended);
    function GetZoom: Extended;
    procedure SetRotation(v: Integer);
    function GetRotation: Integer;
    procedure SetShowText(v: Boolean);
    function GetShowText: Boolean;
    procedure SetText(v: String);
    function GetText: String;
    procedure SetTextSup(v: String);
    function GetTextSup: String;
    procedure SetFontScaled(v: Boolean);
    function GetFontScaled: Boolean;
    procedure SetErrorText(v: String);
    function GetErrorText: String;
    procedure SetQuietZone(v: Integer);
    function GetQuietZone: Integer;
    procedure SetColorBar(c: TColor);
    function GetColorBar: TColor;

    procedure SetProp(v: TfrxBarcode2DProperties);
    procedure SetBarType(v: TfrxBarcode2DType);
    procedure CalcSize;
    function CalcAddSize: TfrxPoint;
    function GetHexData: String;
    procedure SetHexData(const Value: String);
    procedure WritePreset(Writer: TWriter);
    procedure ReadPreset(Reader: TReader);
    function GetDataPreset: TfrxObjectDataPreset; virtual;
    function GetBarcodeText: TfrxBarcodeText;
    procedure SetBarcodeText(const Value: TfrxBarcodeText);
    function GetBarcodeType: Integer;
    procedure SetBarcodeType(const Value: Integer);
    function GetFont: TFont;
    procedure UpdateZoom;
  protected
    procedure SetWidth(Value: Extended); override;
    procedure SetHeight(Value: Extended); override;
    procedure DefineProperties(Filer: TFiler); override;
    function GetProperies: TfrxExtPropList;
  public
    constructor Create(AOwner: TComponent); override;
    constructor DesignCreate(AOwner: TComponent; Flags: Word); override;
    destructor Destroy; override;

    function GetFontHeightTop: Integer;
    function GetFontHeightBottom: Integer;

    function GetVectorGraphic(DrawFill: Boolean; AScaleX, AScaleY: Double): TGraphic; overload; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX,
      OffsetY: Extended); override;
    class function GetDescription: String; override;
    procedure GetData; override;
    function GetRealBounds: TfrxRect; override;
    procedure GetScaleFactor(var ScaleX: Extended; var ScaleY: Extended); override;
  	function CalcHeight: Extended; override;
    function DrawPart: Extended; override;
    procedure SaveContentToDictionary(aReport: TfrxReport; PostProcessor: TfrxPostProcessor); override;
    function LoadContentFromDictionary(aReport: TfrxReport; aItem: TfrxMacrosItem): Boolean; override;
    procedure ProcessDictionary(aItem: TfrxMacrosItem; aReport: TfrxReport; PostProcessor: TfrxPostProcessor); override;
    property ExpressionPreset: TfrxObjectDataPreset read FExpressionPreset;
  published
    property AutoSize: Boolean read FAutoSize write FAutoSize default True;
    property Expression: String read FExpression write FExpression;
    property BarType: TfrxBarcode2DType read FBarType write SetBarType;
    property BarProperties: TfrxBarcode2DProperties read FProp write SetProp;
    property BrushStyle;
    property Color;
    property Cursor;
    property DataField;
    property DataSet;
    property DataSetName;
    property Fill;
    property Frame;
    property HAlign: TfrxHAlign read FHAlign write FHAlign default haLeft;
    property Processing;
    property Rotation: Integer read GetRotation write SetRotation;
    property ShowText: Boolean read GetShowText write SetShowText;
    property Style;
    property TagStr;
    property Text: String read GetText write SetText stored False;
    property HexData: String read GetHexData write SetHexData;
    property URL;
    property Zoom: Extended read GetZoom write SetZoom;
    property Font: TFont read GetFont stored False;
    property FontScaled: Boolean read GetFontScaled write SetFontScaled;
    property ErrorText: String read GetErrorText write SetErrorText;
    property QuietZone: Integer read GetQuietZone write SetQuietZone;
    property ColorBar: TColor read GetColorBar write SetColorBar;
    property TextSup: String read GetTextSup write SetTextSup;
    property BarcodeText: TfrxBarcodeText read GetBarcodeText write SetBarcodeText;
  end;

implementation
end.

