
{ ****************************************** }
{                                            }
{             FastReport VCL                 }
{          2D Barcode base class             }
{                                            }
{         Copyright (c) 1998-2021            }
{            by Fast Reports Inc.            }
{                                            }
{ ****************************************** }

unit frxBarcode2DBase;
//{$Define RandVertCol}
interface

{$I frx.inc}

uses
  Classes, SysUtils,
{$IFDEF FPC}
  LResources, LMessages, LCLType, LCLIntf, LazarusPackageIntf,
  LCLProc, FileUtil, LazHelper,
{$ELSE}
  Windows,
{$ENDIF}
  Graphics, Types, frPrinter, frxBarcodeInterface;

type

  T2DBooleanArray = array of array of Boolean;

  /// ////////////////////////////////////////////////////////////////////////////////////////////
  // base 2D barcode class
  /// ////////////////////////////////////////////////////////////////////////////////////////////

{$M+}
  TfrxBarcode2DBase = class(TObject)
  protected
    FBarcodeText: TfrxBarcodeText;
    FHeight: Integer; // height of barcode matrix
    FWidth: Integer; // Width of barcode matrix
    FPixelWidth: Integer;
    FPixelHeight: Integer;
    FShowText: Boolean;
    FRotation: Integer;
    FText: String;
    FTextSup: String;
    FZoom: Extended;
    FFontScaled: Boolean; // scale font with barcode
    FColor: TColor;
    FColorBar: TColor;
    FErrorText: String;
    FQuietZone: Integer; // paddings
    FOriginalHeight: Integer;  //for linear multi-line barcodes

    procedure GenerateLM(Text: string; rebase: Boolean = True); virtual;  //for linear multi-line barcodes
    procedure SetShowText(v: Boolean); virtual;
    procedure SetRotation(v: Integer); virtual;
    procedure SetText(v: String); virtual;
    procedure SetZoom(v: Extended); virtual;
    procedure SetFontScaled(v: Boolean); virtual;
    function GetFont: TFont;
    procedure SetColor(v: TColor); virtual;
    procedure SetColorBar(v: TColor); virtual;
    procedure SetErrorText(v: String); virtual;
    procedure SetHeight(const Value: Integer); virtual;

    function GetWidth: Integer; virtual;
    function GetHeight: Integer; virtual;
    procedure InternalDraw(aCanvas: TCanvas; scalex, scaley: Extended;
      x, y, footerHeight, dx, dy, paddingX, paddingY: Integer;
      kx, ky: Extended); virtual;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Assign(src: TfrxBarcode2DBase); virtual;
    function GetFooterHeight: Integer; virtual;
    function GetRotatedHeigh: Integer;
    function GetRotatedWidth: Integer;

    procedure Draw2DBarcode(var g: TCanvas; scalex, scaley: Extended;
      x, y: Integer); virtual;
    procedure FillRect(Canv: TCanvas; Rect: TRect);
    function IsScaled: Boolean; virtual;

    property ShowText: Boolean read FShowText write SetShowText;
    property Rotation: Integer read FRotation write SetRotation;

    property Text: String read FText write SetText;
    property TextSup: String read FTextSup write FTextSup;
    property Zoom: Extended read FZoom write SetZoom;
    property FontScaled: Boolean read FFontScaled write SetFontScaled;
    property Font: TFont read GetFont;
    property Color: TColor read FColor write SetColor;
    property ColorBar: TColor read FColorBar write SetColorBar;
    property ErrorText: String read FErrorText write SetErrorText;
    property Width: Integer read GetWidth;
    property Height: Integer read GetHeight write SetHeight;
    property PixelWidth: Integer read FPixelWidth write FPixelWidth;
    property PixelHeight: Integer read FPixelHeight write FPixelHeight;
    property QuietZone: Integer read FQuietZone write FQuietZone;
    property BarcodeText: TfrxBarcodeText read FBarcodeText;
  end;

  TfrxBarcode2DBaseWithUnion = class(TfrxBarcode2DBase)
  protected
    FVectorPrimitives: TList;
    DotsR, DotsL: array of Integer;
  public
    procedure FVectorPrimitivesAdd(ALeft, ATop, AWidth, AHeight: Integer);

    constructor Create; override;
    destructor Destroy; override;

    procedure ClearFigures();
    procedure T2DBooleanArrayToVectorPrimitives(GlArr: T2DBooleanArray; x, y: Integer; QuietZone: Byte = 0);
    procedure InternalDraw(aCanvas: TCanvas; scalex, scaley: Extended;
      x, y, footerHeight, dx, dy, paddingX, paddingY: Integer;
      kx, ky: Extended); override;
  end;

  TfrxBarcode2DBaseWithOutUnion = class(TfrxBarcode2DBase)
  protected
    FImage: array of Byte; // boolean array of barcode data
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure InternalDraw(aCanvas: TCanvas; scalex, scaley: Extended;
      x, y, footerHeight, dx, dy, paddingX, paddingY: Integer;
      kx, ky: Extended); override;
  end;

const
  cbDefaultText = '12345678';

implementation
end.
