{******************************************}
{                                          }
{             FastReport VCL               }
{            Composite Barcode             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxBarcodeComposite;

interface

{$I frx.inc}

uses
  Classes, Types, SysUtils, Graphics, frxClass, frxBarcode, frxBarcod,
  frx2DBarcodesPresets, frxBarcodeInterface, Controls
{$IFDEF OFF2DBARS}, frxBarcode2D{$ENDIF}
;

type
  TRelativityTopPosition = (rtAllAvailable, rtTextPos, rtPrevPos);
  TRelativityBootomPosition = (rbAllAvailable, rbTextPos, rbPrevPos);
  TfrxContentType = (ctRightToLeft{$IFDEF TESTALLBARMODE}, ctTopToBottom{$ENDIF});

  IfrxBarCodeInternal = interface;
  TfrxBarCodeCompositeView = class;
  TfrxBarCodeInternalView = class;
{$IFDEF OFF2DBARS}
  TfrxBarCodeInternal2DView =class;
{$ENDIF}

  TfrxPaddings = class(TPersistent)
  private
    FLeft: Extended;
    FRight: Extended;
    FTop: Extended;
    FBottom: Extended;
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
    property Left: Extended read FLeft write FLeft;
    property Top: Extended read FTop write FTop;
    property Right: Extended read FRight write FRight;
    property Bottom: Extended read FBottom write FBottom;
  end;

  TfrxBaseRelativityPositions = class(TPersistent)
  public
    constructor Create; virtual;
  end;

  TfrxLeftToRightRelativityPositions = class(TfrxBaseRelativityPositions)
  private
    FRelativityTopPosition: TRelativityTopPosition;
    FRelativityBootomPosition: TRelativityBootomPosition;
  public
    constructor Create; override;
    procedure Assign(Source: TPersistent); override;
  published
    property RelativityTopPosition: TRelativityTopPosition read FRelativityTopPosition write FRelativityTopPosition;
    property RelativityBootomPosition: TRelativityBootomPosition read FRelativityBootomPosition write FRelativityBootomPosition;
  end;

  TfrxTopToBottomRelativityPositions = class(TfrxBaseRelativityPositions)

  end;

  TfrxBasePosition = class(TPersistent)
  protected
    FPadding: TfrxPaddings;
    FRelativityPositions: TfrxBaseRelativityPositions;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    property Padding: TfrxPaddings read FPadding write FPadding;
    property RelativityPositions: TfrxBaseRelativityPositions read FRelativityPositions write FRelativityPositions;
  end;

  TfrxBasePositionClass = class of TfrxBasePosition;

  TfrxLeftToRightPosition = class(TfrxBasePosition)
  private
    function GetLeft: Extended;
    procedure SetLeft(val: Extended);
  public
    constructor Create; override;
  published
    property LeftPadding: Extended read GetLeft write SetLeft;
    property RelativityPositions;
  end;

  TfrxTopToBottomPosition = class(TfrxBasePosition)
  private
    function GetTop: Extended;
    procedure SetTop(val: Extended);
  published
    property TopPadding: Extended read GetTop write SetTop;
  end;

  IfrxBarCodeInternal = interface
  ['{1A676D61-012A-40A7-B027-D8080E1743A8}']
    procedure LockPos;
    procedure UnlockPos;
    function BarParent: TfrxBarCodeCompositeView;

    function GetPosition: TfrxBasePosition;
    procedure SetPosition(val: TfrxBasePosition);

    property Positions: TfrxBasePosition read GetPosition write SetPosition;
  end;

  TfrxGetBaseData = function (bar: IfrxBarCodeView): String of object;
  TfrxsetBaseData = procedure (bar: IfrxBarCodeView; val: String) of object;

  TfrxBarCodeCompositeView = class(TfrxView)
  private
    FShowMoveArrow: Boolean;
    FContentType: TfrxContentType;
    FCatchCycle: Boolean;
    FColor: TColor;
    FDataDelimiter: String;
    function GetPrevBar(val: TfrxReportComponent): TfrxReportComponent;
    procedure SetColor(const Value: TColor);
    function GetBarCount: Integer;
    procedure SetBarCount(val: Integer);
    procedure SetContentType(vContentType: TfrxContentType);

    function GetTextFrom(bar: IfrxBarCodeView): String;
    function GetTextSupFrom(bar: IfrxBarCodeView): String;
    function GetBaseData(AText: TfrxGetBaseData): String;
    procedure SetTextFrom(bar: IfrxBarCodeView; val: String);
    procedure SetTextSupFrom(bar: IfrxBarCodeView; val: String);
    procedure SetBaseData(AText: TfrxSetBaseData; val: String);

    function GetText: String;
    procedure SetText(val: String);
    function GetTextSup: String;
    procedure SetTextSup(val: String);
    function GetSubItem(index: Integer): TfrxReportComponent;
  protected
    procedure SetWidth(Value: Extended); override;
    procedure SetHeight(Value: Extended); override;
    function CheckMoveArrow(X, Y: Extended): Boolean;
    function AddInternalBarcode(ABarcodeClass: TfrxComponentClass; const ABarcodeType: Integer): TfrxView;
  public
    constructor Create(AOwner: TComponent); override;
    function IsContain(X, Y: Extended): Boolean; override;
    procedure DoMouseEnter(aPreviousObject: TfrxComponent; var EventParams: TfrxInteractiveEventsParams); override;
    procedure DoMouseLeave(aNextObject: TfrxComponent; var EventParams: TfrxInteractiveEventsParams); override;
    function GetRealBounds: TfrxRect; override;
    class function ContentTypeToClass(val: TfrxContentType): TfrxBasePositionClass;
    function AddLinearBarcode(ABarcodeType: TfrxBarcodeType): TfrxBarCodeInternalView; overload;
    function AddLinearBarcode(ABarcodeType: Integer): TfrxBarCodeInternalView; overload;
{$IFDEF OFF2DBARS}
    function Add2DBarcode(ABarcodeType: Integer): TfrxBarCodeInternal2DView; overload;
    function Add2DBarcode(ABarcodeType: TfrxBarcode2DType): TfrxBarCodeInternal2DView; overload;
{$ENDIF}
    function GetFirstBar: TfrxReportComponent;
    function GetLastBar: TfrxReportComponent;
    procedure UpdatePos(EventOwner: TfrxReportComponent);
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    class function GetDescription: String; override;
  published
    property ContentType: TfrxContentType read FContentType write SetContentType;
    property Color: TColor read FColor write SetColor;
    property BarCount: Integer read GetBarCount write SetBarCount stored False;
    property DataDelimiter: String read FDataDelimiter write FDataDelimiter;
    property Text: String read GetText write SetText stored False;
    property SupText: String read GetTextSup write SetTextSup stored False;
  end;

  TfrxBarCodeInternalView = class(TfrxBarCodeView, IfrxBarCodeInternal)
  private
    FCatchPos: Boolean;
    FPosition: TfrxBasePosition;
    function GetPosition: TfrxBasePosition;
    procedure SetPosition(val: TfrxBasePosition);
    procedure LockPos;
    procedure UnlockPos;
    function IsLockPos: Boolean;
    function BarParent: TfrxBarCodeCompositeView;
  protected
    procedure DoMouseUp(X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState;
      var EventParams: TfrxInteractiveEventsParams); override;
    procedure DoMouseMove(X, Y: Integer; Shift: TShiftState;
      var EventParams: TfrxInteractiveEventsParams); override;
    function DoMouseDown(X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams)
      : Boolean; override;
    procedure SetParent(AParent: TfrxComponent); override;
    procedure SetLeft(Value: Extended); override;
    procedure SetTop(Value: Extended); override;
    procedure SetWidth(Value: Extended); override;
    procedure SetHeight(Value: Extended); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
  published
    property Position: TfrxBasePosition read GetPosition write SetPosition;
  end;

{$IFDEF OFF2DBARS}
  TfrxBarCodeInternal2DView = class(TfrxBarcode2DView, IfrxBarCodeInternal)
  private
    FCatchPos: Boolean;
    FPosition: TfrxBasePosition;
    function GetPosition: TfrxBasePosition;
    procedure SetPosition(val: TfrxBasePosition);
    procedure LockPos;
    procedure UnlockPos;
    function IsLockPos: Boolean;
    function BarParent: TfrxBarCodeCompositeView;
  protected
    procedure DoMouseUp(X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState;
      var EventParams: TfrxInteractiveEventsParams); override;
    procedure DoMouseMove(X, Y: Integer; Shift: TShiftState;
      var EventParams: TfrxInteractiveEventsParams); override;
    function DoMouseDown(X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams)
      : Boolean; override;
    procedure SetParent(AParent: TfrxComponent); override;
    procedure SetLeft(Value: Extended); override;
    procedure SetTop(Value: Extended); override;
    procedure SetWidth(Value: Extended); override;
    procedure SetHeight(Value: Extended); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
  published
    property Position: TfrxBasePosition read GetPosition write SetPosition;
  end;
{$ENDIF}

implementation
end.
