
{******************************************}
{                                          }
{             FastReport VCL               }
{         Composite Barcode Editor         }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxBarcodeCompositeEditor;

interface

{$I frx.inc}

uses
{$IFNDEF NONWINFPC}
  Windows,
{$ENDIF}	
  Classes, Types, Menus, Graphics, Controls, frxClass, frxBarcode, frxCustomEditors, frxBarcodeComposite
{$IFDEF OFF2DBARS}, frxBarcode2D{$ENDIF}, frxInPlaceEditors;

type
  TfrxBaseEvent = class
  private
    FRect: TRect;
  public
    constructor Create(vRect: TRect);
    function IsInRect(vPoint: TPoint): Boolean;
    procedure Execute(c: TfrxBarCodeCompositeView); virtual; abstract;
  end;

  TfrxBaseInternalEvent = class(TfrxBaseEvent)
  protected
    FView: TfrxView;
  public
    constructor Create(vRect: TRect; AView: TfrxView);
    procedure Execute(c: TfrxBarCodeCompositeView); override; abstract;
  end;

  TfrxSwitchPosEvent = class(TfrxBaseInternalEvent)
  public
    procedure Execute(c: TfrxBarCodeCompositeView); override;
  end;

  TfrxListEvents = class
  private
    FList: TList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure Add(vBaseEvent: TfrxBaseEvent);
    function FindEventInPoint(vPoint: TPoint): TfrxBaseEvent;
  end;

  TfrxInPlaceBarcodeCompositePresetEditor = class(TfrxInPlaceCustomListBoxEditor)
  protected
    function ListBoxClicked(Sender: TObject): Boolean; override;
    function IsFillListBox: Boolean; override;
    procedure ListBoxDrawItem(ACanvas: TCanvas; AControl: TWinControl; AIndex: Integer; ARect: TRect;
      AState: TfrOwnerDrawState); override;
    function IsDropDownButtonVisible: Boolean; override;
    procedure DrawButton(ACanvas: TCanvas; ARect: TRect); override;
  public
    constructor Create(AClassRef: TfrxComponentClass; AOwner: TWinControl); override;
  end;

  TfrxInPlaceBarcodeCompositeBaseTypeEditor = class(TfrxInPlaceCustomListBoxEditor)
  private
    FItemsList: TStringList;
  protected
    procedure FreeListItems;
    function IsFillListBox: Boolean; override;
    function IsDropDownButtonVisible: Boolean; override;
    procedure ListBoxDrawItem(ACanvas: TCanvas; AControl: TWinControl; AIndex: Integer; ARect: TRect;
      AState: TfrOwnerDrawState); override;
    procedure DrawButton(ACanvas: TCanvas; ARect: TRect); override;
    function GetImageIndex: Integer; virtual; abstract;
  public
    constructor Create(aClassRef: TfrxComponentClass; aOwner: TWinControl); override;
    destructor Destroy; override;
  end;

  TfrxInPlaceBarcodeCompEditor = class(TfrxInPlaceBarcodeCompositeBaseTypeEditor)
  private
    FShowGUI: Boolean;
    FListEvents: TfrxListEvents;
    FLastMousePos: TPoint;
    FLastComponent: TfrxComponent;
    procedure DrawGUI(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended);
    function CheckGUIClick(X, Y: Integer; Button: TMouseButton; Shift: TShiftState;
      var EventParams: TfrxInteractiveEventsParams): Boolean;
  protected
    function ListBoxClicked(Sender: TObject): Boolean; override;
    function GetButtonTopLeft: TPoint; override;
    function GetImageIndex: Integer; override;
  public
    constructor Create(aClassRef: TfrxComponentClass; aOwner: TWinControl); override;
    destructor Destroy; override;
    procedure DrawCustomEditor(aCanvas: TCanvas; aRect: TRect); override;
    procedure InitializeUI(var EventParams: TfrxInteractiveEventsParams); override;
    procedure FinalizeUI(var EventParams: TfrxInteractiveEventsParams); override;
    function DoMouseUp(X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams): Boolean; override;
    function DoMouseMove(X, Y: Integer; Shift: TShiftState;
      var EventParams: TfrxInteractiveEventsParams): Boolean; override;
  end;

  TfrxInPlaceBarcodeCompositeAddEditor = class(TfrxInPlaceBarcodeCompositeBaseTypeEditor)
  protected
    function ListBoxClicked(Sender: TObject): Boolean; override;
    function GetButtonTopLeft: TPoint; override;
    function GetImageIndex: Integer; override;
  end;

  TfrxBarCodeCustomTypeEditor = class(TfrxViewEditor)
  protected
    procedure CreateBarTypesSubmenu(AParentItem: TMenuItem);
  end;

  TfrxBarCodeCompositeEditor = class(TfrxBarCodeCustomTypeEditor)
  private
    procedure CreatePreset(PresetB: TMenuItem);
  public
    procedure GetMenuItems; override;
    function Execute(Tag: Integer; Checked: Boolean): Boolean; override;
  end;

{$IFDEF OFF2DBARS}
  TfrxBarCodeInternalEditor = class(TfrxBarCodeCustomTypeEditor)
  public
    procedure GetMenuItems; override;
    function Execute(ATag: Integer; Checked: Boolean): Boolean; override;
  end;

  function CheckIs2D(c: TfrxComponent): Boolean;
{$ENDIF}

implementation
end.
