//
//
//
// Copyright 2007 by Paulo Soares.
//
// The contents of this file are subject to the Mozilla Public License Version 1.1
// (the "License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the License.
//
// The Original Code is 'iText, a free JAVA-PDF library'.
//
// The Initial Developer of the Original Code is Bruno Lowagie. Portions created by
// the Initial Developer are Copyright (C) 1999, 2000, 2001, 2002 by Bruno Lowagie.
// All Rights Reserved.
// Co-Developer of the code is Paulo Soares. Portions created by the Co-Developer
// are Copyright (C) 2000, 2001, 2002 by Paulo Soares. All Rights Reserved.
// Modifications: Alexander Tzyganenko
//
//
//
//


unit frxBarcodeDataMatrix;


interface

{$I frx.inc}

uses
{$IFDEF FPC}
  LCLType, LMessages, LazHelper, LCLIntf,
{$ELSE}
  Windows, Messages,
{$ENDIF}
  SysUtils, Types, StrUtils, Classes, Graphics, Controls, Forms, Dialogs, frxBarcode2DBase, frUnicodeUtils, frxGS1Helper;

type


// Specifies the Datamatrix encoding. ///////////////////////////////////////////////////////////////////////

DatamatrixEncoding =
(
    Auto,
    Ascii,
    C40,
    Txt,
    Base256,
    X12,
    Edifact
);

// Specifies the Datamatrix symbol size. /////////////////////////////////////////////////////////////////////

DatamatrixSymbolSize =
(
    AutoSize,
    Size10x10,
    Size12x12,
    Size8x18,
    Size14x14,
    Size8x32,
    Size16x16,
    Size12x26,
    Size18x18,
    Size20x20,
    Size12x36,
    Size22x22,
    Size16x36,
    Size24x24,
    Size26x26,
    Size16x48,
    Size32x32,
    Size36x36,
    Size40x40,
    Size44x44,
    Size48x48,
    Size52x52,
    Size64x64,
    Size72x72,
    Size80x80,
    Size88x88,
    Size96x96,
    Size104x104,
    Size120x120,
    Size132x132,
    Size144x144
);

// ////////////////////////////////////////////////////////////////////////////////////////////////////
//
//
// ////////////////////////////////////////////////////////////////////////////////////////////////////

DmParams = record

    height,
    width,
    heightSection,
    widthSection,
    dataSize,
    dataBlock,
    errorBlock : integer;

end;

const

    dmSizes : array[Size10x10..Size144x144] of DmParams =
    (
        ( height: 10; width: 10; heightSection: 10; widthSection: 10; dataSize: 3; dataBlock: 3; errorBlock: 5 ),
        ( height: 12; width: 12; heightSection: 12; widthSection: 12; dataSize: 5; dataBlock: 5; errorBlock: 7 ),
        ( height:  8; width: 18; heightSection:  8; widthSection: 18; dataSize: 5; dataBlock: 5; errorBlock: 7 ),
        ( height: 14; width: 14; heightSection: 14; widthSection: 14; dataSize: 8; dataBlock: 8; errorBlock:10 ),
        ( height:  8; width: 32; heightSection:  8; widthSection: 16; dataSize:10; dataBlock:10; errorBlock:11 ),
        ( height: 16; width: 16; heightSection: 16; widthSection: 16; dataSize:12; dataBlock:12; errorBlock:12 ),
        ( height: 12; width: 26; heightSection: 12; widthSection: 26; dataSize:16; dataBlock:16; errorBlock:14 ),
        ( height: 18; width: 18; heightSection: 18; widthSection: 18; dataSize:18; dataBlock:18; errorBlock:14 ),
        ( height: 20; width: 20; heightSection: 20; widthSection: 20; dataSize:22; dataBlock:22; errorBlock:18 ),
   ( height: 12; width: 36; heightSection: 12; widthSection: 18; dataSize:22; dataBlock:22; errorBlock:18 ),
   ( height: 22; width: 22; heightSection: 22; widthSection: 22; dataSize:30; dataBlock:30; errorBlock:20 ),
   ( height: 16; width: 36; heightSection: 16; widthSection: 18; dataSize:32; dataBlock:32; errorBlock:24 ),
   ( height: 24; width: 24; heightSection: 24; widthSection: 24; dataSize:36; dataBlock:36; errorBlock:24 ),
   ( height: 26; width: 26; heightSection: 26; widthSection: 26; dataSize:44; dataBlock:44; errorBlock:28 ),
   ( height: 16; width: 48; heightSection: 16; widthSection: 24; dataSize:49; dataBlock:49; errorBlock:28 ),
   ( height: 32; width: 32; heightSection: 16; widthSection: 16; dataSize:62; dataBlock:62; errorBlock:36 ),
   ( height: 36; width: 36; heightSection: 18; widthSection: 18; dataSize:86; dataBlock:86; errorBlock:42 ),
   ( height: 40; width: 40; heightSection: 20; widthSection: 20; dataSize:114; dataBlock:114; errorBlock: 48 ),
   ( height: 44; width: 44; heightSection: 22; widthSection: 22; dataSize:144; dataBlock:144; errorBlock: 56 ),
   ( height: 48; width: 48; heightSection: 24; widthSection: 24; dataSize:174; dataBlock:174; errorBlock: 68 ),
   ( height: 52; width: 52; heightSection: 26; widthSection: 26; dataSize:204; dataBlock:102; errorBlock: 42 ),
   ( height: 64; width: 64; heightSection: 16; widthSection: 16; dataSize:280; dataBlock:140; errorBlock: 56 ),
   ( height: 72; width: 72; heightSection: 18; widthSection: 18; dataSize:368; dataBlock:92; errorBlock: 36 ),
   ( height: 80; width: 80; heightSection: 20; widthSection: 20; dataSize:456; dataBlock:114; errorBlock: 48 ),
   ( height: 88; width: 88; heightSection: 22; widthSection: 22; dataSize:576; dataBlock:144; errorBlock: 56 ),
   ( height: 96; width: 96; heightSection: 24; widthSection: 24; dataSize:696; dataBlock:174; errorBlock: 68 ),
   ( height:104; width:104; heightSection: 26; widthSection: 26; dataSize:816; dataBlock:136; errorBlock: 56 ),
   ( height:120; width:120; heightSection: 20; widthSection: 20; dataSize:1050; dataBlock:175; errorBlock: 68 ),
   ( height:132; width:132; heightSection: 22; widthSection: 22; dataSize:1304; dataBlock:163; errorBlock: 62 ),
   ( height:144; width:144; heightSection: 24; widthSection: 24; dataSize:1558; dataBlock:156; errorBlock: 62 )
);

 log : array[0..255] of integer =
              ( 0, 255,   1, 240,   2, 225, 241,  53,   3,  38, 226, 133, 242,  43,  54, 210,
                4, 195,  39, 114, 227, 106, 134,  28, 243, 140,  44,  23,  55, 118, 211, 234,
                5, 219, 196,  96,  40, 222, 115, 103, 228,  78, 107, 125, 135,   8,  29, 162,
                244, 186, 141, 180,  45,  99,  24,  49,  56,  13, 119, 153, 212, 199, 235,  91,
                6,  76, 220, 217, 197,  11,  97, 184,  41,  36, 223, 253, 116, 138, 104, 193,
                229,  86,  79, 171, 108, 165, 126, 145, 136,  34,   9,  74,  30,  32, 163,  84,
                245, 173, 187, 204, 142,  81, 181, 190,  46,  88, 100, 159,  25, 231,  50, 207,
                57, 147,  14,  67, 120, 128, 154, 248, 213, 167, 200,  63, 236, 110,  92, 176,
                7, 161,  77, 124, 221, 102, 218,  95, 198,  90,  12, 152,  98,  48, 185, 179,
                42, 209,  37, 132, 224,  52, 254, 239, 117, 233, 139,  22, 105,  27, 194, 113,
                230, 206,  87, 158,  80, 189, 172, 203, 109, 175, 166,  62, 127, 247, 146,  66,
                137, 192,  35, 252,  10, 183,  75, 216,  31,  83,  33,  73, 164, 144,  85, 170,
                246,  65, 174,  61, 188, 202, 205, 157, 143, 169,  82,  72, 182, 215, 191, 251,
                47, 178,  89, 151, 101,  94, 160, 123,  26, 112, 232,  21,  51, 238, 208, 131,
                58,  69, 148,  18,  15,  16,  68,  17, 121, 149, 129,  19, 155,  59, 249,  70,
                214, 250, 168,  71, 201, 156,  64,  60, 237, 130, 111,  20,  93, 122, 177, 150 );

      alog : array[0..255] of integer =
              ( 1,   2,   4,   8,  16,  32,  64, 128,  45,  90, 180,  69, 138,  57, 114, 228,
                229, 231, 227, 235, 251, 219, 155,  27,  54, 108, 216, 157,  23,  46,  92, 184,
                93, 186,  89, 178,  73, 146,   9,  18,  36,  72, 144,  13,  26,  52, 104, 208,
                141,  55, 110, 220, 149,   7,  14,  28,  56, 112, 224, 237, 247, 195, 171, 123,
                246, 193, 175, 115, 230, 225, 239, 243, 203, 187,  91, 182,  65, 130,  41,  82,
                164, 101, 202, 185,  95, 190,  81, 162, 105, 210, 137,  63, 126, 252, 213, 135,
                35,  70, 140,  53, 106, 212, 133,  39,  78, 156,  21,  42,  84, 168, 125, 250,
                217, 159,  19,  38,  76, 152,  29,  58, 116, 232, 253, 215, 131,  43,  86, 172,
                117, 234, 249, 223, 147,  11,  22,  44,  88, 176,  77, 154,  25,  50, 100, 200,
                189,  87, 174, 113, 226, 233, 255, 211, 139,  59, 118, 236, 245, 199, 163, 107,
                214, 129,  47,  94, 188,  85, 170, 121, 242, 201, 191,  83, 166,  97, 194, 169,
                127, 254, 209, 143,  51, 102, 204, 181,  71, 142,  49,  98, 196, 165, 103, 206,
                177,  79, 158,  17,  34,  68, 136,  61, 122, 244, 197, 167,  99, 198, 161, 111,
                222, 145,  15,  30,  60, 120, 240, 205, 183,  67, 134,  33,  66, 132,  37,  74,
                148,   5,  10,  20,  40,  80, 160, 109, 218, 153,  31,  62, 124, 248, 221, 151,
                3,   6,  12,  24,  48,  96, 192, 173, 119, 238, 241, 207, 179,  75, 150,   1 );
    poly5  : array[0..4] of integer = ( 228,  48,  15, 111,  62 );
    poly7  : array[0..6] of integer = ( 23,  68, 144, 134, 240,  92, 254 );
    poly10 : array[0..9] of integer = ( 28,  24, 185, 166, 223, 248, 116, 255, 110,  61 );
    poly11 : array[0..10] of integer = ( 175, 138, 205,  12, 194, 168,  39, 245,  60,  97, 120 );
    poly12 : array[0..11] of integer = ( 41, 153, 158,  91,  61,  42, 142, 213,  97, 178, 100, 242 );
    poly14 : array[0..13] of integer = ( 156,  97, 192, 252,  95,   9, 157, 119, 138,  45,  18, 186,  83, 185 );
    poly18 : array[0..17] of integer = ( 83, 195, 100,  39, 188,  75,  66,  61, 241, 213, 109, 129,  94, 254, 225,  48, 90, 188 );
    poly20 : array[0..19] of integer = ( 15, 195, 244,   9, 233,  71, 168,   2, 188, 160, 153, 145, 253,  79,
                                         108,  82, 27, 174, 186, 172 );
    poly24 : array[0..23] of integer = ( 52, 190,  88, 205, 109,  39, 176,  21, 155, 197, 251, 223, 155,  21,   5, 172,
                                         254, 124,  12, 181, 184,  96,  50, 193 );
    poly28 : array[0..27] of integer = ( 211, 231,  43,  97,  71,  96, 103, 174,  37, 151, 170,  53,  75,  34, 249, 121,
                                         17, 138, 110, 213, 141, 136, 120, 151, 233, 168,  93, 255 );
    poly36 : array[0..35] of integer = ( 245, 127, 242, 218, 130, 250, 162, 181, 102, 120,  84, 179, 220, 251,  80, 182,
                                         229,  18,   2,   4,  68,  33, 101, 137,  95, 119, 115,  44, 175, 184,  59,  25,
                                         225,  98,  81, 112 );
    poly42 : array[0..41] of integer = ( 77, 193, 137,  31,  19,  38,  22, 153, 247, 105, 122,   2, 245, 133, 242,   8,
                                        175,  95, 100,   9, 167, 105, 214, 111,  57, 121,  21,   1, 253,  57,  54, 101,
                                         248, 202,  69,  50, 150, 177, 226,   5,   9,   5 );
    poly48 : array[0..47] of integer = ( 245, 132, 172, 223,  96,  32, 117,  22, 238, 133, 238, 231, 205, 188, 237,  87,
                                         191, 106,  16, 147, 118,  23,  37,  90, 170, 205, 131,  88, 120, 100,  66, 138,
                                         186, 240,  82,  44, 176,  87, 187, 147, 160, 175,  69, 213,  92, 253, 225,  19 );
    poly56 : array[0..55] of integer = ( 175,   9, 223, 238,  12,  17, 220, 208, 100,  29, 175, 170, 230, 192, 215, 235,
                                         150, 159,  36, 223,  38, 200, 132,  54, 228, 146, 218, 234, 117, 203,  29, 232,
                                         144, 238,  22, 150, 201, 117,  62, 207, 164,  13, 137, 245, 127,  67, 247,  28,
                                         155,  43, 203, 107, 233,  53, 143,  46 );
    poly62 : array[0..61] of integer = ( 242,  93, 169,  50, 144, 210,  39, 118, 202, 188, 201, 189, 143, 108, 196,  37,
                                         185, 112, 134, 230, 245,  63, 197, 190, 250, 106, 185, 221, 175,  64, 114,  71,
                                         161,  44, 147,   6,  27, 218,  51,  63,  87,  10,  40, 130, 188,  17, 163,  31,
                                         176, 170,   4, 107, 232,   7,  94, 166, 224, 124,  86,  47,  11, 204 );
    poly68 : array[0..67] of integer = ( 220, 228, 173,  89, 251, 149, 159,  56,  89,  33, 147, 244, 154,  36,  73, 127,
                                         213, 136, 248, 180, 234, 197, 158, 177,  68, 122,  93, 213,  15, 160, 227, 236,
                                         66, 139, 153, 185, 202, 167, 179,  25, 220, 232,  96, 210, 231, 136, 223, 239,
                                         181, 241,  59,  52, 172,  25,  49, 232, 211, 189,  64,  54, 108, 153, 132,  63,
                                         96, 103,  82, 186 );

       _x12  = #13 + '*> 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';
       cbDefaultText = '12345678';

type

    SizeF = record
        height : extended;
        width  : extended;
    end;

    TInts = array of integer;

//                                         /////////////////////////////////////////////////////////////////////////
// Generates the 2D Data Matrix barcode.   /////////////////////////////////////////////////////////////////////////
//                                         /////////////////////////////////////////////////////////////////////////

{$M+}
TfrxBarcodeDataMatrix = class( TfrxBarcode2DBaseWithUnion )

  private

    FPlace            : TInts;
    FSymbolSize       : DatamatrixSymbolSize;
    FEncoding         : DatamatrixEncoding;
    FCodePage         : integer;
    FGS1Rule           : Boolean;
    FIsLastEncodingAscii: Boolean;

    ArrOB: T2DBooleanArray;

    procedure SetGS1Rule(b: Boolean);
    procedure SetBit(x, y : integer);
    procedure Generate(var text : string); overload;
    procedure Generate( var text: array of byte; textOffset, textSize : integer); overload;
    function  GetEncodation(var Text : array of byte; textOffset, textSize : integer; var data : array of byte;
                                     dataOffset, dataSize : integer; firstMatch : boolean) : integer;

    procedure Draw(var data : array of byte; dataSize : integer; const dm: DmParams);
    procedure SetCodePage( cp : integer );
    procedure SetEncoding( v : DatamatrixEncoding);
    procedure Ecc200;
    procedure SetSymbolSize( s : DatamatrixSymbolSize);
    function  GetPixelSize : integer;
    procedure SetPixelSize(v : integer);
    function AsciiEncodation(var text: array of byte; textOffset, textLength: Integer; var data: array of byte; dataOffset, dataLength: Integer): Integer;
    function B256Encodation(var text: array of byte; textOffset, textLength: Integer; var data: array of byte; dataOffset, dataLength: Integer): Integer;
    function X12Encodation(var text: array of byte; textOffset, textLength: Integer; var data: array of byte; dataOffset, dataLength : Integer) : Integer;
    function EdifactEncodation(var text: array of byte; textOffset, textLength: Integer; var data: array of byte; dataOffset, dataLength: Integer): Integer;
    function C40OrTextEncodation(var Text: array of byte; textOffset, textLength: Integer; var data: array of byte; dataOffset, dataLength: Integer; C40 : boolean ): Integer;
    procedure MakePadding(var data: array of byte; position, count: Integer);
protected
    procedure SetText(v : string); override;

public

    constructor Create; override;
    destructor  Destroy; override;
    procedure   Assign(src: TfrxBarcode2DBase);override;
//    procedure   Draw2DBarcode(var g : TCanvas; scalex, scaley : extended; x, y : integer ); override;

published


    property SymbolSize : DatamatrixSymbolSize read FSymbolSize write SetSymbolSize;
    property Encoding :  DatamatrixEncoding read    FEncoding    write    SetEncoding;
    property CodePage : integer read FCodePage write SetCodePage;
    property PixelSize : integer read GetPixelSize write SetPixelSize;
    property GS1Rule : Boolean read FGS1Rule write SetGS1Rule;

end;


procedure GenerateECC(var wd : array of byte; nd, datablock, nc : integer );


implementation
end.
