
{******************************************}
{                                          }
{             FastReport VCL               }
{            Barcode Interfaxe             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxBarcodeInterface;

interface

{$I frx.inc}

uses
  Graphics, Classes, frxClass;

type
  TfrxBarTextPos = (btpTop = 0, btpBottom = 1);

  TfrxBarcodeText = class;

  TfrxTextSettings = class(TPersistent)
  private
    FBarcodeText: TfrxBarcodeText;
    FFont: TFont;
    FBarTextPos: TfrxBarTextPos;
    procedure SetBarTextPos(val: TfrxBarTextPos);
  public
    constructor Create(vBarcodeText: TfrxBarcodeText; vSize: Integer; vBarTextPos: TfrxBarTextPos);
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Font: TFont read FFont;
    property BarTextPos: TfrxBarTextPos read FBarTextPos write SetBarTextPos;
  end;

  TfrxBarcodeText = class(TPersistent)
  private
    FTextSettings: TfrxTextSettings;
    FSupSettings: TfrxTextSettings;
    FLock: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure Lock;
    procedure UnLock;
    function IsLock: Boolean;
  published
    property TextSettings: TfrxTextSettings read FTextSettings write FTextSettings;
    property SupSettings: TfrxTextSettings read FSupSettings write FSupSettings;
  end;


  IfrxBarCodeView = interface
  ['{CF6CB462-96D3-43AA-9FBD-EB949BBB51C0}']
    function GetBarcodeText: TfrxBarcodeText;
    procedure SetBarcodeText(const Value: TfrxBarcodeText);
    function GetBarcodeType: Integer;
    procedure SetBarcodeType(const Value: Integer);
    function GetText: String;
    procedure SetText(Value: String);
    function GetTextSup: String;
    procedure SetTextSup(Value: String);
    procedure SetZoom(v: Extended);
    function GetZoom: Extended;

    function GetFontHeightTop: Integer;
    function GetFontHeightBottom: Integer;

    property BarcodeText: TfrxBarcodeText read GetBarcodeText write SetBarcodeText;
    property BarcodeType: Integer read GetBarcodeType write SetBarcodeType;
    property Text: String read GetText write SetText;
    property TextSup: String read GetTextSup write SetTextSup;
    property Zoom: Extended read GetZoom write SetZoom;
  end;

function GetFontHeight(comp: TfrxComponent; val: TfrxBarTextPos): Integer;
function RotPTB(valPTB: TfrxBarTextPos; Angle: Double): Integer;

implementation
end.
