unit frxBarcodeProperties;

{$I frx.inc}

interface

uses
{$IFDEF FPC}
  LCLType, LMessages, LazHelper, LCLIntf,
{$ELSE}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Types, Controls, Forms, Dialogs, StdCtrls, Menus,
  ExtCtrls,
  frxClass, frxDesgn, frxBarcodePDF417, frxBarcodeDataMatrix, frxBarcodeQR,
  frxDelphiZXingQRCode, frxBarcode2DBase, frxBarcodeAztec, frxBarcodeMaxiCode, frxGS1Databar;

type

{$M+}
  // ,   
  TfrxBarcode2DProperties = class(TPersistent)
  private
    FOnChange: TNotifyEvent;
  public
    FWhose: TObject; //  
    procedure Changed;
    procedure Assign(Source: TPersistent); override; abstract;
    procedure SetWhose(w: TObject);
  end;

  TfrxPDF417Properties = class(TfrxBarcode2DProperties)
  private
    function GetAspectRatio: Extended;
    function GetColumns: Integer;
    function GetRows: Integer;
    function GetErrorCorrection: PDF417ErrorCorrection;
    function GetCodePage: Integer;
    function GetCompactionMode: PDF417CompactionMode;
    function GetPixelWidth: Integer;
    function GetPixelHeight: Integer;

    procedure SetAspectRatio(v: Extended);
    procedure SetColumns(v: Integer);
    procedure SetRows(v: Integer);
    procedure SetErrorCorrection(v: PDF417ErrorCorrection);
    procedure SetCodePage(v: Integer);
    procedure SetCompactionMode(v: PDF417CompactionMode);
    procedure SetPixelWidth(v: Integer);
    procedure SetPixelHeight(v: Integer);
  public
    procedure Assign(Source: TPersistent); override;
  published
    property AspectRatio: Extended read GetAspectRatio write SetAspectRatio;
    property Columns: Integer read GetColumns write SetColumns;
    property Rows: Integer read GetRows write SetRows;
    property ErrorCorrection: PDF417ErrorCorrection read GetErrorCorrection
      write SetErrorCorrection;
    property CodePage: Integer read GetCodePage write SetCodePage;
    property CompactionMode: PDF417CompactionMode read GetCompactionMode
      write SetCompactionMode;
    property PixelWidth: Integer read GetPixelWidth write SetPixelWidth;
    property PixelHeight: Integer read GetPixelHeight write SetPixelHeight;
  end;

  TfrxDataMatrixProperties = class(TfrxBarcode2DProperties)
  private
    function GetCodePage: Integer;
    function GetPixelSize: Integer;
    function GetSymbolSize: DatamatrixSymbolSize;
    function GetEncoding: DatamatrixEncoding;
    function GetGS1Rule: Boolean;

    procedure SetCodePage(v: Integer);
    procedure SetPixelSize(v: Integer);
    procedure SetSymbolSize(v: DatamatrixSymbolSize);
    procedure SetEncoding(v: DatamatrixEncoding);
    procedure SetGS1Rule(b: Boolean);
  public
    procedure Assign(Source: TPersistent); override;
  published
    property CodePage: Integer read GetCodePage write SetCodePage;
    property PixelSize: Integer read GetPixelSize write SetPixelSize;
    property SymbolSize: DatamatrixSymbolSize read GetSymbolSize
      write SetSymbolSize;
    property Encoding: DatamatrixEncoding read GetEncoding write SetEncoding;
    property GS1Rule: Boolean read GetGS1Rule write SetGS1Rule;
  end;

  TfrxQRProperties = class(TfrxBarcode2DProperties)
  private
    function GetEncoding: TQRCodeEncoding;
    function GetQuietZone: Integer;
    function GetPixelSize: Integer;
    procedure SetPixelSize(v: Integer);
    procedure SetEncoding(const Value: TQRCodeEncoding);
    procedure SetQuietZone(const Value: Integer);
    function GetErrorLevels: TQRErrorLevels;
    procedure SetErrorLevels(const Value: TQRErrorLevels);
    function GetCodePage: Longint;
    procedure SetCodePage(const Value: Longint);
    function GetLogo: TfrxBarcodeLogo;
    function  GetGraphicMarker : TfrxBarcodeGraphicMarker;
  public
    procedure Assign(Source: TPersistent); override;
  published
    property Encoding: TQRCodeEncoding read GetEncoding write SetEncoding;
    property QuietZone: Integer read GetQuietZone write SetQuietZone;
    property ErrorLevels: TQRErrorLevels read GetErrorLevels
      write SetErrorLevels;
    property PixelSize: Integer read GetPixelSize write SetPixelSize;
    property CodePage: Longint read GetCodePage write SetCodePage;
    property Logo: TfrxBarcodeLogo read GetLogo;
    property GraphicMarker : TfrxBarcodeGraphicMarker read GetGraphicMarker;
  end;

  TfrxAztecProperties = class(TfrxBarcode2DProperties)
  private
    function GetMinECCPercent: integer;
    procedure SetMinECCPercent(const Value: integer);
    function GetPixelSize: integer;
    procedure SetPixelSize(const Value: integer);
  public
    procedure Assign(Source: TPersistent); override;
  published
    property MinECCPercent: integer read GetMinECCPercent
      write SetMinECCPercent;
    property PixelSize: integer read GetPixelSize write SetPixelSize;
  end;

  TfrxMaxiCodeProperties = class(TfrxBarcode2DProperties)
  private
    function GetMode: Integer;
    procedure SetMode(const Value: Integer);
//    function GetMinECCPercent: integer;
//    procedure SetMinECCPercent(const Value: integer);
//    function GetPixelSize: integer;
//    procedure SetPixelSize(const Value: integer);
  public
    procedure Assign(Source: TPersistent); override;
  published
    property Mode: Integer read GetMode write SetMode;
  end;

  TfrxGS1DatabarProperties = class(TfrxBarcode2DProperties)
  public
    procedure Assign(Source: TPersistent); override;
  end;

  TfrxGS1DatabarEProperties = class(TfrxGS1DatabarProperties)
  public
    procedure Assign(Source: TPersistent); override;
  end;

  TfrxGS1DatabarESProperties = class(TfrxGS1DatabarProperties)
  private
    procedure SetColumns(Col: Integer);
    function GetColumns(): Integer;
  public
    procedure Assign(Source: TPersistent); override;
  published
    property Columns: Integer read GetColumns write SetColumns;
  end;

  TfrxPTTDatabarProperties = class(TfrxBarcode2DProperties)
  public
    procedure Assign(Source: TPersistent); override;
  end;

implementation
end.
