
{******************************************}
{                                          }
{             FastReport VCL               }
{                 QR code                  }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxBarcodeQR;


interface

{$I frx.inc}

uses
{$IFDEF FPC}
  LCLType, LMessages, LazHelper, LCLIntf,
{$ELSE}
  Windows, Messages,
{$ENDIF}
  SysUtils, Types, StrUtils, Classes, Graphics, Controls, Forms, Dialogs, frxBarcode2DBase, frxDelphiZXingQRCode;

type
  TfrxBarcodeLogo = class(TPersistent)
  private
    FLogo: TPicture;
    FWidth: Integer;
    FHeight: Integer;
    procedure SetLogo(const Value: TPicture);
    function GetLogo: TPicture;
    function GetLogoInst: TPicture;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure Draw(DrawRect: TRect; ScaleX, ScaleY: Extended; Canvas: TCanvas; Zoom: Extended);
  published
    property Logo: TPicture read GetLogo write SetLogo;
    property Width: Integer read FWidth write FWidth;
    property Height: Integer read FHeight write FHeight;
  end;

  TfrxBarcodeGraphicMarker = class(TPersistent)
  private
    FShowGraphicMarker : Boolean;
    FWidthLine: Integer;
    FDistance: Integer;
    FPixelSize: Integer;
    FScaleX: Extended;
    FScaleY: Extended;
    procedure SetWidthLine(const Value: Integer);
    procedure SetDistance(const Value: Integer);
    procedure SetPixelSize(const Value: Integer);
  public
    constructor Create(PixelSize : Integer);
    procedure Draw(DrawRect: TRect; ScaleX, ScaleY: Extended; var Canvas: TCanvas;
      Zoom: Extended; FooterHeight: Integer);
  published
    property ShowGraphicMarker: Boolean read FShowGraphicMarker write FShowGraphicMarker;
    property WidthLine: Integer read FWidthLine write SetWidthLine;
    property Distance: Integer read FDistance write SetDistance;
  end;

  TfrxBarcodeQR = class( TfrxBarcode2DBaseWithUnion )
  private
    FDelphiZXingQRCode: TDelphiZXingQRCode;
    FLogo: TfrxBarcodeLogo;
    FGraphicMarker: TfrxBarcodeGraphicMarker;
    procedure Generate();
    function GetEncoding: TQRCodeEncoding;
    function GetQuietZone: Integer;
    procedure SetEncoding(const Value: TQRCodeEncoding);
    procedure SetQuietZone(const Value: Integer);
    function GetErrorLevels: TQRErrorLevels;
    procedure SetErrorLevels(const Value: TQRErrorLevels);
    function  GetPixelSize : integer;
    procedure SetPixelSize(v : integer);
    function GetCodepage: Longint;
    procedure SetCodepage(const Value: Longint);
  protected
    procedure SetText( v : string ); override;
    function GetWidth: Integer; override;
    function GetHeight: Integer; override;
  public
    constructor Create; override;
    destructor  Destroy; override;
    procedure   Assign(src: TfrxBarcode2DBase);override;
    procedure Draw2DBarcode(var g: TCanvas; scalex, scaley: Extended;
      x, y: Integer); override;
  published
     property Encoding: TQRCodeEncoding read GetEncoding write SetEncoding;
     property QuietZone: Integer read GetQuietZone write SetQuietZone;
     property ErrorLevels: TQRErrorLevels read GetErrorLevels write SetErrorLevels;
     property PixelSize : integer read GetPixelSize write SetPixelSize;
     property Codepage : Longint read GetCodepage write SetCodepage;
     property Logo: TfrxBarcodeLogo read FLogo;
     property GraphicMarker : TfrxBarcodeGraphicMarker read FGraphicMarker;
  end;


implementation
end.
