
{******************************************}
{                                          }
{              FastReport v6.0             }
{              frxBaseGoogleApi            }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxBaseMailApi;

interface

{$I frx.inc}

uses
  Classes, Forms, Controls, StdCtrls, ComCtrls, Dialogs,
  frxIOTransportHelpers, frxBaseTransportConnection, frxIOTransportOAuthDialog,
  frxBaseForm, frxIOTransportIntDialog, frxBaseTransportDialogForm;

type
  TfrxBaseMailIOTransportForm = class(TfrxBaseTransportDialogForm)
    ReqLB: TLabel;
    AddressE: TEdit;
    AddressLB: TLabel;
    MessageLB: TLabel;
    MessageM: TMemo;
    SubjectE: TEdit;
    SubjectLB: TLabel;
    FileLB: TLabel;
    FileE: TEdit;
    OkB: TButton;
    CancelB: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure OkBClick(Sender: TObject);
  protected
    procedure FormCloseAct;
    function GetFileName: String; override;
    procedure SetFileName(const Value: String); override;
    procedure SetDialogMode(const Value: TfrxIOInternetDialogMode); override;
  public
    procedure UpdateResouces; override;
  end;

  TMailOpenForm = class(TfrxIOTransportDialogIntForm)
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TfrxLabel = class
  protected
    FId: String;
    FName: String;
  public
    constructor Create;
    property Id: String read FId write FId;
    property Name: String read FName write FName;
  end;

  TfrxMessage = class
  protected
    FId: String;
    FName: String;
  public
    constructor Create;
    property Id: String read FId write FId;
    property Name: String read FName write FName;
  end;

  TfrxMessageStack = class
  protected
    FfrxLabel: TfrxLabel;
    FfrxMessage: TfrxMessage;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Push(Id, Name: String);
    procedure Pop;
    property frxLabel: TfrxLabel read FfrxLabel write FfrxLabel;
    property frxMessage: TfrxMessage read FfrxMessage write FfrxMessage;
  end;

{$IFDEF DELPHI16}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxBaseMailIOTransport = class(TfrxHTTPIOTransport)
  private
    FAddress, FSubject: String;
    FMessageText: TStringList;
  protected
    FfrxMessageStack: TfrxMessageStack;

    procedure ChangeDirUP; override;
    procedure DialogDirCreate(Name: String; DirItems: TStrings); override;
    procedure DialogDirChange(Name, Id: String; DirItems: TStrings); override;
    procedure DialogDirDelete(Name, Id: String; DirItems: TStrings); override;
    procedure DialogFileDelete(Name, Id: String; DirItems: TStrings); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function TransportPostDialogClass(DM: TfrxIOInternetDialogMode): TfrxBaseFormClass; override;

    property Address: String read FAddress write FAddress;
    property Subject: String read FSubject write FSubject;
    property MessageText: TStringList read FMessageText;
  end;

  function LoadStringFromStream(Stream: TStream): String;
  function FormatMailName(Subject, Date, id: String): String;

implementation
end.
