
{******************************************}
{                                          }
{             FastReport VCL               }
{          HTTP base IO handler            }
{                                          }
{         Copyright (c) 1998-2021          }
{                                          }
{******************************************}

unit frxBaseSocketIOHandler;

{$I frx.inc}
interface

uses
  Windows, SysUtils, Classes, ScktComp,
  WinSock;


type
  TfrxBaseSockErrors = (bseNone, bseSocketErr, bseWantRead, bseWantWrite, bseWantLookup, bseSYSCall, bseZeroRet, bseWantConnect, bseWantAccept, bseWantAsync);
  TfrxSecurityProtocol = (spNone, spSSLv2, spSSLv3, spSSLv23, spTLSv1, spTLSv1_1, spTLSv1_2, spTLS_Auto);
  TfrxSecurityProtocols = set of TfrxSecurityProtocol;

  TfrxCustomIOHandler = class
  protected
    FLastError: Integer;
    FBuffer: PAnsiChar;
    FBuffSize: Integer;
    FStream: TMemoryStream;
    FConnected: Boolean;
    FSecurityProtocol: TfrxSecurityProtocol;
    function DoRead(var Buffer; Count: Longint): Integer; virtual; abstract;
    function DoWrite(var Buffer; Count: Longint): Integer; virtual; abstract;
    function GetErrorCode(ErrCode: Integer): Integer; virtual; abstract;
    procedure SetSecurityProtocol(const Value: TfrxSecurityProtocol); virtual;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    function GetLastError: String; virtual;
    function InitializeHandler: Boolean; virtual;
    procedure BindSocket(Socket: TCustomWinSocket; const Host: AnsiString); virtual;
    function TryConnect: Boolean; virtual;
    procedure Disconnect; virtual;
    function Read(Stream: TStream): Boolean; virtual;
    function Write(Stream: TStream): Boolean; virtual;
    function ProcessIO: Boolean; virtual;
    procedure RunIO; virtual;
    function SupportedSecurityProtocol: TfrxSecurityProtocols; virtual;
    property SecurityProtocol: TfrxSecurityProtocol read FSecurityProtocol write SetSecurityProtocol;
  end;

  TfrxWinSockIOHandler = Class(TfrxCustomIOHandler)
  private
    FWinSock: TCustomWinSocket;
  protected
    function DoRead(var Buffer; Count: Longint): Integer; override;
    function DoWrite(var Buffer; Count: Longint): Integer; override;
    function GetErrorCode(ErrCode: Integer): Integer; override;
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure BindSocket(Socket: TCustomWinSocket; const Host: AnsiString); override;
    procedure Connect;
    function TryConnect: Boolean; override;
    procedure Disconnect; override;
    function ProcessIO: Boolean; override;
  end;

  TfrxCustomIOHandlerClass = class of TfrxCustomIOHandler;

var
  frxDefaultSocketIOHandlerClass: TfrxCustomIOHandlerClass;
  frxHTTPSSocketIOHandlerClass: TfrxCustomIOHandlerClass;
  frxHTTPSocketIOHandlerClass: TfrxCustomIOHandlerClass;

implementation
end.
