{******************************************}
{                                          }
{             FastReport VCL               }
{           Module for using               }
{     TChromiumWindow as a browser         }
{              (CEF4Delphi)                }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{                                          }
{******************************************}
unit frxBrowserChromium;

// ************************************************************************
// ***************************** CEF4Delphi *******************************
// ************************************************************************
//
// CEF4Delphi is based on DCEF3 which uses CEF to embed a chromium-based
// browser in Delphi applications.
//
// The original license of DCEF3 still applies to CEF4Delphi.
//
// For more information about CEF4Delphi visit :
//         https://www.briskbard.com/index.php?lang=en&pageid=cef
//
//        Copyright  2021 Salvador Diaz Fau. All rights reserved.
//
// ************************************************************************
// ************ vvvv Original license and comments below vvvv *************
// ************************************************************************
(*
 *                       Delphi Chromium Embedded 3
 *
 * Usage allowed under the restrictions of the Lesser GNU General Public License
 * or alternatively the restrictions of the Mozilla Public License 1.1
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * Unit owner : Henri Gourvest <hgourvest@gmail.com>
 * Web site   : http://www.progdigy.com
 * Repository : http://code.google.com/p/delphichromiumembedded/
 * Group      : http://groups.google.com/group/delphichromiumembedded
 *
 * Fast Reports, Inc is not permitted to use or redistribute
 * this source code without explicit permission.
 *
 *)
interface

{$I frx.inc}
{$I cef.inc}

uses
  {$IFDEF DELPHI16_UP}
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls,
  {$ELSE}
  Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
  {$ENDIF}
  frxBrowser, frTypes,
  uCEFChromium, uCEFWindowParent, uCEFChromiumWindow, uCEFTypes, uCEFInterfaces,
  uCEFWinControl, uCEFSentinel;

type

  TfrxBrowserChromium = class(TChromiumWindow,IfrxWebBrowser)
    private
      FURL: String;
      FNavigateHistory: TStringList;
      FOnTestURL: TTestURLEvent;
      FOnDocumentComplet: TNotifyEvent;
      FOnNavComplete: TNavComplete;
      FTimer: TTimer;
      procedure FTimerOnTimer(Sender: TObject);
      function GetNavigateHistory : TStringList; stdcall;
      function GetOnDocumentComplet : TNotifyEvent; stdcall;
      function GetOnTestURL : TTestURLEvent; stdcall;
      function GetURL : String; stdcall;
      function GetDocument: IDispatch; stdcall;
      procedure SetNavigateHistory(const Value: TStringList); stdcall;
      procedure SetOnDocumentComplet(const Value: TNotifyEvent); stdcall;
      procedure SetOnTestURL(const Value: TTestURLEvent); stdcall;
      procedure SetURL(const Value: String); stdcall;
      procedure SetOnNavComplete(const Value: TNavComplete); stdcall;
      procedure BrowserConfiguration; stdcall;
      procedure CloseQuery(var CanClose: Boolean); stdcall;
      procedure ChromiumWindowAfterCreated(Sender: TObject);
      procedure ChromiumWindowClose(Sender: TObject);
      procedure ChromiumWindowBeforeClose(Sender: TObject);
      // You have to handle this two messages to call NotifyMoveOrResizeStarted or some page elements will be misaligned.
      procedure WMMove(var aMessage : TWMMove); message WM_MOVE;
      procedure WMMoving(var aMessage : TMessage); message WM_MOVING;
      // You also have to handle these two messages to set GlobalCEFApp.OsmodalLoop
      procedure WMEnterMenuLoop(var aMessage: TMessage); message WM_ENTERMENULOOP;
      procedure WMExitMenuLoop(var aMessage: TMessage); message WM_EXITMENULOOP;
      procedure ChromiumOnLoadEnd (Sender: TObject; const browser: ICefBrowser; const frame: ICefFrame; httpStatusCode: Integer);
      procedure ChromiumOnLoadError(Sender: TObject; const browser: ICefBrowser; const frame: ICefFrame; errorCode: TCefErrorCode; const errorText, failedUrl: ustring);
    protected
      // Variables to control when can we destroy the form safely
      FCanClose : boolean;  // Set to True in TChromium.OnBeforeClose
      FClosing  : boolean;  // Set to True in the CloseQuery event.
      procedure NavigateURL(aURL : String); stdcall;
      procedure Chromium_OnBeforePopup(Sender: TObject; const browser: ICefBrowser; const frame: ICefFrame; const targetUrl, targetFrameName: ustring; targetDisposition: TCefWindowOpenDisposition; userGesture: Boolean; const popupFeatures: TCefPopupFeatures; var windowInfo: TCefWindowInfo; var client: ICefClient; var settings: TCefBrowserSettings; var extra_info: ICefDictionaryValue; var noJavascriptAccess: Boolean; var Result: Boolean);
    public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
  end;

implementation
end.
