{******************************************}
{                                          }
{             FastReport VCL               }
{           CellularText Object            }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxCellularTextObject;

interface

{$I frx.inc}

uses
{$IFDEF FPC}
  LazHelper,
{$ELSE}
  WideStrings,
{$ENDIF}
  SysUtils, Types, Classes, Graphics, Variants, Controls,
  frCoreClasses, frxClass, frxTableObject, frUnicodeUtils;

type

{$IFDEF DELPHI16}
/// <summary>
///   The TfrxReportCellularTextObject allows use of the CellularText component
///   in a report. TfrxReportCellularTextObject is an empty component. It is
///   used to add CellularText unit to uses clause of an application.
/// </summary>
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxReportCellularTextObject = class(TComponent)  // fake component
  end;

  /// <summary>
  ///   The CellularText report object class. A report objects of this class
  ///   automatically breaks text to table cell where each symbol of the text
  ///   place in sperate cell. It supports align and text wrapping.
  /// </summary>
  TfrxCellularText = class(TfrxCustomMemoView, IfrxDataBindingComponent)
  private
    FCellWidth: Extended;
    FCellHeight: Extended;
    FHorzSpacing: Extended;
    FVertSpacing: Extended;
    function GetCellWidth: Extended;
    function GetCellHeight: Extended;
    procedure SetCellWidth(Value: Extended);
    procedure SetCellHeight(Value: Extended);
    procedure WrapText(sLines: TWideStrings; ColumnCount: Integer);
  protected
    procedure Loaded; override;
    procedure UpdateSize;

    procedure BuildArray(aTable: TfrxTableObject; var aCellHeight: Extended;
      var sLines: TWideStrings; var sText: WideString; var ColColumn: Integer);
    procedure BuildTableFromArray(aTable: TfrxTableObject; aCellHeight: Extended;
      sLines: TWideStrings; sText: WideString; ColColumn: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    function CalcHeight: Extended; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    function DrawPart: Extended; override;
    function ExportInternal(Filter: TfrxCustomExportFilter): Boolean; override;
    class function GetDescription: String; override;
    procedure UpdateBounds; override;
    procedure BuildTable(aTable: TfrxTableObject);
  published
    property AllowExpressions;
    property BrushStyle;
    property Color;
    /// <summary>
    ///   Width of the cell.
    /// </summary>
    property CellWidth: Extended read FCellWidth write SetCellWidth;
    /// <summary>
    ///   Height of the cell.
    /// </summary>
    property CellHeight: Extended read FCellHeight write SetCellHeight;
    /// <summary>
    ///   Horizontal spacing between cells.
    /// </summary>
    property HorzSpacing: Extended read FHorzSpacing write FHorzSpacing;
    /// <summary>
    ///   Vertical spacing between cells.
    /// </summary>
    property VertSpacing: Extended read FVertSpacing write FVertSpacing;
    property DataField;
    property DataSet;
    property DataSetName;
    property DisplayFormat;
    property ExpressionDelimiters;
    property Font;
    property Frame;
    property FillType;
    property Fill;
    property HAlign;
    property HideZeros;
    property Highlight;
    property Memo;
    property ParentFont;
    property RTLReading;
    property SuppressRepeated;
    property Style;
    property UseDefaultCharset;
    property WordWrap;
    property VAlign;
  end;




implementation
end.
