
{******************************************}
{                                          }
{             FastReport VCL               }
{         Checkbox Add-In Object           }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxChBox;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}
  Windows, Messages,
  {$ENDIF}
  SysUtils, Classes, Graphics, Menus, frxClass
  {$IFDEF FPC}
  , LCLType, LMessages, LCLIntf, LazarusPackageIntf, LazHelper
  {$ENDIF}
{$IFDEF DELPHI6}
, Variants
{$ENDIF}
{$IFDEF DELPHI16}
 , Vcl.Controls
{$ELSE}
 , Controls
{$ENDIF};


type
  /// <summary>
  ///   Checked style of the checkbox - cross, check, crossline, plus.
  /// </summary>
  TfrxCheckStyle = (csCross, csCheck, csLineCross, csPlus);
  /// <summary>
  ///   UnChecked style of the checkbox - empty, cross, crossline, minus.
  /// </summary>
  TfrxUncheckStyle = (usEmpty, usCross, usLineCross, usMinus);

{$IFDEF DELPHI16}
  /// <summary>
  ///   The TfrxCheckBoxObject allows the use of the CheckBox component in your
  ///   <br />report. TfrxCheckBoxObject is an empty component. It is used to
  ///   add the <br />frxChBox.pas file to the "uses" list. The main component
  ///   is <br />TfrxCheckBoxView.
  /// </summary>
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxCheckBoxObject = class(TComponent)  // fake component
  end;

  /// <summary>
  ///   The TfrxCheckBoxView component represents a box containing a mark -
  ///   check or cross. Component shows the check if Checked property value is
  ///   True. Also it can show a check from the DB field (you should set
  ///   DataSet, DataField properties) or from an expression(Expression
  ///   property). <br />
  /// </summary>
  TfrxCheckBoxView = class(TfrxStyledView)
  private
    FCheckColor: TColor;
    FChecked: Boolean;
    FCheckStyle: TfrxCheckStyle;
    FUncheckStyle: TfrxUncheckStyle;
    FExpression: String;
    procedure DrawCheck(ARect: TRect);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    procedure GetData; override;
    class function GetDescription: String; override;
    function DoMouseDown(X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams): Boolean; override;
  published
    property BrushStyle;
    /// <summary>
    ///   Color of the check.
    /// </summary>
    property CheckColor: TColor read FCheckColor write FCheckColor;
    /// <summary>
    ///   Determines whether to show the check.
    /// </summary>
    property Checked: Boolean read FChecked write FChecked default True;
    /// <summary>
    ///   Checked style of the check.
    /// </summary>
    property CheckStyle: TfrxCheckStyle read FCheckStyle write FCheckStyle;
    property Color;
    property Cursor;
    property DataField;
    property DataSet;
    property DataSetName;
    /// <summary>
    ///   Value of the expression determines whether to show the check.
    /// </summary>
    property Expression: String read FExpression write FExpression;
    property Editable default [ferAllowInPreview];
    property FillType;
    property Fill;
    property Frame;
    property Style;
    property TagStr;
    /// <summary>
    ///   Unchecked style of the check.
    /// </summary>
    property UncheckStyle: TfrxUncheckStyle read FUncheckStyle write FUncheckStyle default usEmpty;
    property URL;
  end;

  {$IFDEF FPC}
  //procedure Register;
  {$ENDIF}

implementation
end.
