unit frxCipher;

{$I frx.inc}

interface

uses
  {$IFNDEF Linux}
  Windows,
  {$ENDIF}
  SysUtils, Classes, Graphics, Math,
  frxASN1, frxHelpers;

type
  TfrxCipher = class
  private
    FKeyLen: Cardinal;
  protected
    procedure Init(Key, InitVector: AnsiString); virtual; abstract;
    procedure ExpandLastBlock(var Str: AnsiString); virtual; // Empty
    procedure RemoveExpansion(var Str: AnsiString); virtual; // Empty
    procedure Decode(Buf: Pointer; Len: Cardinal); virtual; abstract;
    procedure Encode(Buf: Pointer; Len: Cardinal); virtual; abstract;
 public
    constructor Create(Key, InitVector: AnsiString);
    function DecodeToStr(Str: AnsiString): AnsiString;
    function EncodeToStr(Str: AnsiString): AnsiString;
  end;

  TfrxCipherClass = class of TfrxCipher;

  TfrxBlockCipher = class(TfrxCipher)
  private
    FInitVector: array[0..63] of byte;
    function GetBlockSize: Integer; virtual; abstract;
    function GetIVSize: Integer; virtual; abstract;
  protected
    procedure Init(Key, InitVector: AnsiString); override;
    procedure ExpandLastBlock(var Str: AnsiString); override;
    procedure RemoveExpansion(var Str: AnsiString); override;
    procedure XORBuffer(const Buf1, Buf2: Pointer; const BufferSize: Integer);
    procedure DecodeBlock(Buf: Pointer); virtual; abstract;
    procedure EncodeBlock(Buf: Pointer); virtual; abstract;
    procedure Decode(Buf: Pointer; Len: Cardinal); override;
    procedure Encode(Buf: Pointer; Len: Cardinal); override;

    property BlockSize: Integer read GetBlockSize;
    property IVSize: Integer read GetIVSize;
  public
  end;

  TfrxRC2CipherKey = packed record
    case Byte of
      0 : (Bytes: array[0..127] of Byte);
      1 : (Words: array[0..63] of Word);
  end;

  TfrxRC2Cipher = class(TfrxBlockCipher)
  private
    FKey: TfrxRC2CipherKey;
    function GetBlockSize: Integer; override;
    function GetIVSize: Integer; override;
  protected
    procedure Init(Key, InitVector: AnsiString); override;
    procedure DecodeBlock(Buf: Pointer);override;
    procedure EncodeBlock(Buf: Pointer);override;
  public
  end;

  TfrxDES3Cipher = class(TfrxBlockCipher)
  private
    FUserInfo: array[0..192 - 1] of DWord;
    function GetBlockSize: Integer; override;
    function GetIVSize: Integer; override;
  protected
    procedure Init(Key, InitVector: AnsiString); override;
    procedure DES_Func(Buf: Pointer; Index: Integer);
    procedure DES3_Func(Buf: Pointer; i1, i2, i3: Integer);
    procedure DecodeBlock(Buf: Pointer); override;
    procedure EncodeBlock(Buf: Pointer); override;
    procedure MakeKey(const Data: array of Byte; Key: PDWord; Reverse: Boolean);
  end;

implementation
end.
