unit frxComCtrls;
{$I frx.inc}
interface

uses
  {$IFNDEF NONWINFPC}Windows, CommCtrl,{$ENDIF}
  {$IFDEF FPC}LCLType, LMessages, LCLIntf, LazHelper, LazarusPackageIntf,{$ENDIF}
  Messages, SysUtils, Variants, Classes, Graphics,
  Controls, ComCtrls, frxDPIAwareBaseControls, frxUtils, frxCtrls, ImgList,
  frDPIAwareUtils, frDPIAwareInt, frTypes, frAppearance, frControls, frTreeView;

type
  TfrxTreeView = class;
  TfrxTreeViewExpandButton = class;

  { IfrxViewInfoTreeView }

  IfrxViewInfoTreeView = interface(IfrViewInfoTreeView)
  ['{124939E9-6262-4523-9419-097889086B23}']
    function GetExpandButton: TfrxTreeViewExpandButton;

    property ExpandButton: TfrxTreeViewExpandButton read GetExpandButton;
  end;

  TfrxFilterAlign = (fraLeft, fraRight, fraWidth);
  TfrxToolAlign = (ftaTop, ftaBottom);
  TfrxTreeSortType = (dtsUnsorted, dtsAscending, dtsDescending);

  TfrxTreeNodes = class(TfrTreeViewNodes)
  protected
    procedure DefineProperties(Filer: TFiler); override;
  public
    procedure Assign(Source: TPersistent); override;
  end;

  TfrxTVFilterCompareEvent = function(Sender: TObject; Node: TfrTreeViewNode; const aFilter: string): Boolean of object;

  { TfrxTreeViewExpandButton }

  TfrxTreeViewExpandButton = class
  strict private
    FAlign: TfrxToolAlign;
    FExpanded: Boolean;
    FHeight: Integer;
    FState: TfrButtonState;
    FTreeView: TfrxTreeView;
    FVisible: Boolean;
    FWidth: Integer;

    procedure SetAlign(AValue: TfrxToolAlign);
    procedure SetExpanded(AValue: Boolean);
    procedure SetHeight(AValue: Integer);
    procedure SetState(AValue: TfrButtonState);
    procedure SetVisible(AValue: Boolean);
    procedure SetWidth(AValue: Integer);
  protected
    procedure Changed;
    procedure ChangeScale(M, D: Integer);
    procedure Invalidate;
  public
    constructor Create(ATreeView: TfrxTreeView);

    property Align: TfrxToolAlign read FAlign write SetAlign;
    property Expanded: Boolean read FExpanded write SetExpanded;
    property Height: Integer read FHeight write SetHeight;
    property State: TfrButtonState read FState write SetState;
    property Visible: Boolean read FVisible write SetVisible;
    property Width: Integer read FWidth write SetWidth;
  end;

  { TfrxTreeViewViewInfo }

  TfrxTreeViewViewInfo = class(TfrTreeViewViewInfo)
  strict private
    FExpandButtonRect: TRect;

    procedure CalculateExpandButtonRect;
    procedure DrawExpandButton(ACanvas: TfrCustomCanvas);
    function GetTreeView: IfrxViewInfoTreeView;
  protected
    procedure CalculateContent; override;
    procedure DrawContent(ACanvas: TfrCustomCanvas); override;
    function PointInExpandButtonArea(const APoint: TPoint): Boolean;

    property ExpandButtonRect: TRect read FExpandButtonRect;
    property TreeView: IfrxViewInfoTreeView read GetTreeView;
  public
    function CalculateHitTest(AHitTest: TfrCustomHitTest): Boolean; override;
  end;

  { TfrxTreeViewController }

  TfrxTreeViewController = class(TfrTreeViewController)
  protected
    procedure DoMouseDown(Button: TMouseButton; Shift: TShiftState; const P: TPoint); override;
  end;

  { TfrxTreeView }

  TfrxTreeView = class(TfrTreeView, IfrxViewInfoTreeView)
  strict private
    FExpandButton: TfrxTreeViewExpandButton;
    FExpandLevel: Integer;
    FLastFilter: String;

    FOnFilterCompare: TfrxTVFilterCompareEvent;
    FOnExpandButtonClick: TNotifyEvent;
  protected
    procedure DoChangeScale(M: Integer; D: Integer; isDpiChange: Boolean); override;
    function CreateController: TfrCustomController; override;
    function CreateViewInfo: TfrCustomViewInfo; override;
    procedure DestroyWnd; override;

    procedure DoExpandButtonClick;
    procedure InvalidateExpandButton(HideButton: Boolean = False);

    // IfrxViewInfoTreeView
    function GetExpandButton: TfrxTreeViewExpandButton;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Filter(const AFilter: string);
    procedure ResetFilter;
    procedure UpdateFilter;

    property ExpandLevel: Integer read FExpandLevel write FExpandLevel;
  published
    property ExpandButton: TfrxTreeViewExpandButton read FExpandButton;
    property OnFilterCompare: TfrxTVFilterCompareEvent read FOnFilterCompare write FOnFilterCompare;
    property OnExpandButtonClick: TNotifyEvent read FOnExpandButtonClick write FOnExpandButtonClick;
  end;

  TfrxToolPanel = class;

  TfrxToolButtonStyle = (fbsButton, fbsCheckButton, fbsRadioButton, fbsDropDownButton);
  TfrxToolButtonKind = (fbkButton, fbkSeparator);

  TfrxToolPanelCustomButton = class(TGraphicControl)
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TfrxToolPanelSeparator = class(TfrxToolPanelCustomButton)
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  { TfrxToolPanelButton }

  TfrxToolPanelButton = class(TfrxToolPanelCustomButton)
  private
    FIsActive: Boolean;
    FIsDown: Boolean;
    FImageIndex: Integer;
    FButtonStyle: TfrxToolButtonStyle;
    FGroup: Integer;
    function GetParentPanel: TfrxToolPanel;
    procedure SetButtonStyle(const Value: TfrxToolButtonStyle);
    procedure SetIsDown(const Value: Boolean);
  protected
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  protected
    procedure Paint; override;
    function GetArrowWidth: Integer;
  public
    constructor Create(AOwner: TComponent); override;
    //function IsDown: Boolean;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
{$IFDEF FPC}
    procedure ChangeBounds(ALeft, ATop, AWidth, AHeight: integer; KeepBase: boolean); override;
{$ENDIF}
    property ButtonStyle: TfrxToolButtonStyle read FButtonStyle write SetButtonStyle;
    property ImageIndex: Integer read FImageIndex write FImageIndex;
    property IsDown: Boolean read FIsDown write SetIsDown;
  end;

  TfrxToolPanel = class(TfrxDPIAwarePanel)
  private
    FButtons: TList;
    FImageList: TCustomImageList;
    FOnBtnClick: TNotifyEvent;
    procedure SetImageList(const Value: TCustomImageList);
    procedure ResetRadioGroup(GroupIndex: Integer);
  protected
    function CalcButtonsWidth: Integer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function AddButton(ImageIndex: Integer; HintText: String = ''; Tag: NativeInt = -1;
      ButtonStyle: TfrxToolButtonStyle = fbsButton; Group: Integer = -1): TfrxToolPanelButton; overload;
    function AddCustomButton(aKind: TfrxToolButtonKind): TfrxToolPanelCustomButton;
    procedure ChangeButtonHint(AIndex: Integer; const AHint: string);
    procedure SetBounds(aLeft, aTop, aWidth, aHeight: integer); override;
{$IFDEF FPC}
    procedure ChangeBounds(ALeft, ATop, AWidth, AHeight: integer; KeepBase: boolean); override;
{$ENDIF}
    function GetArrowWidth: Integer;
    procedure Paint; override;
    procedure RealignButtons;
    property ImageList: TCustomImageList read FImageList write SetImageList;
    property OnBtnClick: TNotifyEvent read FOnBtnClick write FOnBtnClick;
  end;

  TfrxToolWithFilterPanel = class(TfrxDPIAwarePanel)
  private
    FToolPanel: TfrxToolPanel;
    FPanel: TfrxDPIAwarePanel;
    FFilter: TfrxFilterEdit;
    FFileterMinWidth: Integer;
    FFileterMaxWidth: Integer;
    FFilterActiveImageIndex: Integer;
    FFilterUnactiveImageIndex: Integer;
    FFilterAlign: TfrxFilterAlign;
    FToolPanelAlign: TfrxToolAlign;
    procedure SetFilterVisible(const Value: Boolean);
    function GetFilterVisible: Boolean;
    function GetToolVisible: Boolean;
    procedure SetToolVisible(const Value: Boolean);
    function GetImageList: TCustomImageList;
    procedure SetImageList(const Value: TCustomImageList);
    procedure SetFilterActiveImageIndex(const Value: Integer);
    procedure SetFilterUnactiveImageIndex(const Value: Integer);
    procedure SetFileterMaxWidth(const Value: Integer);
    procedure SetFileterMinWidth(const Value: Integer);
    procedure ExpandButtonClick(Sender: TObject); virtual;
    procedure SetFilterAlign(const Value: TfrxFilterAlign);
    procedure SetToolPanelAlign(const Value: TfrxToolAlign); virtual;
    procedure SetFilterColor(const Value: TColor);
    function GetFilterColor: TColor;
    procedure SetOnFilterChanged(const Value: TNotifyEvent);
  protected
    procedure UpdateSize; virtual;
    function CalcButtonsWidth: Integer;
    procedure Resize; override;
{$IFDEF FPC}
    procedure DoOnResize; override;
{$ENDIF}
    procedure SetParent(AParent: TWinControl); override;
    procedure DoPPIChanged(aNewPPI: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property FilterColor: TColor read GetFilterColor write SetFilterColor;
    property FilterEdit: TfrxFilterEdit read FFilter;
    property OnFilterChanged: TNotifyEvent write SetOnFilterChanged;
    property FilterVisible: Boolean read GetFilterVisible write SetFilterVisible;
    property FilterActiveImageIndex: Integer read FFilterActiveImageIndex write SetFilterActiveImageIndex;
    property FilterUnactiveImageIndex: Integer read FFilterUnactiveImageIndex write SetFilterUnactiveImageIndex;
    property FileterMinWidth: Integer read FFileterMinWidth write SetFileterMinWidth;
    property FileterMaxWidth: Integer read FFileterMaxWidth write SetFileterMaxWidth;
    property FilterAlign: TfrxFilterAlign read FFilterAlign write SetFilterAlign;
    property ToolPanel: TfrxToolPanel read FToolPanel;
    property ToolVisible: Boolean read GetToolVisible write SetToolVisible;
    property ToolImageList: TCustomImageList read GetImageList write SetImageList;
    property ToolPanelAlign: TfrxToolAlign read FToolPanelAlign write SetToolPanelAlign;
  end;

  { TfrxTreePanel }

  TfrxTreePanel = class(TfrxToolWithFilterPanel)
  private
    FTreeView: TfrxTreeView;
    procedure ExpandButtonClick(Sender: TObject); override;
    procedure SetToolPanelAlign(const Value: TfrxToolAlign); override;
  protected
    procedure UpdateSize; override;
    procedure SetParent(AParent: TWinControl); override;
    procedure EditChange(Sender: TObject); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property TreeView: TfrxTreeView read FTreeView;
  end;


implementation
end.
