
{******************************************}
{                                          }
{             FastReport VCL               }
{         ComboBox Add-In Object           }
{                                          }
{         Copyright (c) 1998-2022          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxComboBox;

interface

{$I frx.inc}

uses
  {$IFNDEF Linux}
  Windows,
  {$ENDIF}
  Classes, Controls, Graphics, Types,
  frUnicodeUtils, frxClass, frxListControl, frxRes;

type
{$IFDEF DELPHI16}
  /// <summary>
  ///   The TfrxComboBoxObject allows the use of the TfrxComboBoxView component
  ///   in your report. TfrxComboBoxObject is an empty component. It is used to
  ///   add the frxComboBox.pas file to the "uses" list.
  /// </summary>
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxComboBoxObject = class(TComponent)  // fake component
  end;

  /// <summary>
  ///   TfrxComboBoxView combines an edit box with a scrollable list.
  ///   A TfrxComboBoxView component is an edit box with a scrollable drop-down
  ///   list attached to it. Users can select an item from the list or type
  ///   directly into the edit box.
  /// </summary>
  TfrxComboBoxView = class(TfrxCustomListControlView)
  private
    FDropDownCount: Integer;
    procedure SetDropDownCount(const Value: Integer);
  protected
    FContentHeight: Extended;
    FText: WideString;
    FNeedReDraw: Boolean;

    procedure SetItemIndex(const Value: Integer); override;

    procedure DrawText;
    procedure SetDrawParams(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended);
    procedure FillMemoText(Memo: TfrxCustomMemoView); override;
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: string; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;

    procedure DoMouseMove(X, Y: Integer; Shift: TShiftState;
      var EventParams: TfrxInteractiveEventsParams); override;
    procedure PreviewCoordinates(var X, Y: Integer);

    property ContentHeight: Extended read FContentHeight;
    property NeedReDraw: Boolean read FNeedReDraw write FNeedReDraw;
  published
    property AllowExpressions;
    property BrushStyle;
    property Color;
    property Cursor;
    property DataField;
    property DataSet;
    property DataSetName;
    property DropDownCount: Integer read FDropDownCount write SetDropDownCount;
    property Editable default [ferAllowInPreview];
    property ExpressionDelimiters;
    property Font;
    property Frame;
    property FillType;
    property Fill;
    property ItemIndex; // hidden
    property Items;
    property ItemsText; // hidden
    property Style;
    property TagStr;
    property Text: WideString read FText write FText;
    property ParentFont;
  end;

implementation
end.
