
{******************************************}
{                                          }
{     FastReport VCL  - Dialog designer    }
{         Standard Dialog controls         }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxDCtrl;

interface

{$I frx.inc}

uses
{$IFNDEF FPC}
  Windows, Messages, CommCtrl,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, StdCtrls,
  ExtCtrls, Forms, Menus, Dialogs, Comctrls, Buttons,
{$IFDEF FPC}
  LMessages,MaskEdit, LCLType, LCLIntf, LCLProc,LazHelper, LazarusPackageIntf, DateTimePicker,
{$ELSE}
  Mask,
{$ENDIF}
{$IFDEF DELPHI6}
  Variants,
{$ENDIF}
  CheckLst, frxClass, frCore;

type
{$IFDEF DELPHI16}
/// <summary>
///   The TfrxDialogControls lets you use the Frdialog controls in your report.
///   TfrxDialogControls is an empty component. It is used to add the
///   frxDCtrl.pas file to the "uses" list.
/// </summary>
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxDialogControls = class(TComponent) // fake component
  end;

  /// <summary>
  ///   The TfrxLabelControl component represents a TLabel adapter.
  /// </summary>
  TfrxLabelControl = class(TfrxDialogControl)
  private
    FLabel: TLabel;
    function GetAlignment: TAlignment;
    function GetAutoSize: Boolean;
    function GetWordWrap: Boolean;
    procedure SetAlignment(const Value: TAlignment);
    procedure SetAutoSize(const Value: Boolean);
    procedure SetWordWrap(const Value: Boolean);
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    procedure BeforeStartReport; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    /// <summary>
    ///   Reference to internal TLabel object.
    /// </summary>
    property LabelCtl: TLabel read FLabel;
  published
    /// <summary>
    ///   Controls the horizontal placement of the text within the label.
    /// </summary>
    property Alignment: TAlignment read GetAlignment write SetAlignment
      default taLeftJustify;
    /// <summary>
    ///   Determines whether the size of the label automatically resizes to
    ///   accommodate the text.
    /// </summary>
    property AutoSize: Boolean read GetAutoSize write SetAutoSize default True;
    property Caption;
    property Color;
    /// <summary>
    ///   Specifies whether the label text wraps when it is too long for the
    ///   width of the label.
    /// </summary>
    property WordWrap: Boolean read GetWordWrap write SetWordWrap default False;

    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  TfrxOnChangingEvent = type String;
  
  TfrxPageControl = class(TfrxDialogControl)
  private
    FPageControl: TPageControl;
    FOnChange: TfrxNotifyEvent;
    FOnChanging: TfrxOnChangingEvent;
    function GetMultiLine: Boolean;
    function GetPageCount: Integer;
    {$IFNDEF FPC}
    function GetRaggedRight: Boolean;
    function GetScrollOpposite: Boolean;
    function GetStyle: TTabStyle;
    {$ENDIF}
    function GetTabHeight: Smallint;
    function GetTabIndex: Integer;
    function GetTabOrder: Integer;
    function GetTabPosition: TTabPosition;
    function GetTabWidth: Smallint;
    procedure SetMultiLine(const Value: Boolean);
    {$IFNDEF FPC}
    procedure SetRaggedRight(const Value: Boolean);
    procedure SetScrollOpposite(const Value: Boolean);
    procedure SetStyle(const Value: TTabStyle);
    {$ENDIF}
    procedure SetTabHeight(const Value: Smallint);
    procedure SetTabIndex(const Value: Integer);
    procedure SetTabOrder(const Value: Integer);
    procedure SetTabPosition(const Value: TTabPosition);
    procedure SetTabWidth(const Value: Smallint);
    {$IFNDEF FPC}
    function GetHotTrack: Boolean;
    procedure SetHotTrack(const Value: Boolean);
    {$ENDIF}
    procedure DoOnChange(Sender: TObject);
    procedure DoOnChanging(Sender: TObject; var AllowChange: Boolean);
  protected
    procedure SetParent(AParent: TfrxComponent); override;
    procedure SetWidth(Value: Extended); override;
    procedure SetHeight(Value: Extended); override;
    procedure UpdateSize;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    function IsAcceptControl(aControl: TfrxComponent): Boolean; override;
    function DoMouseDown(X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams)
      : Boolean; override;
    property PageControl: TPageControl read FPageControl;
    property PageCount: Integer read GetPageCount;
  published
    {$IFNDEF FPC}
    property HotTrack: Boolean read GetHotTrack write SetHotTrack default False;
    {$ENDIF}
    property MultiLine: Boolean read GetMultiLine write SetMultiLine default False;
    {$IFNDEF FPC}
    property RaggedRight: Boolean read GetRaggedRight write SetRaggedRight default False;
    property ScrollOpposite: Boolean read GetScrollOpposite
      write SetScrollOpposite default False;
    property Style: TTabStyle read GetStyle write SetStyle default tsTabs;
    {$ENDIF}
    property TabHeight: Smallint read GetTabHeight write SetTabHeight default 0;
    property TabIndex: Integer read GetTabIndex write SetTabIndex stored False;
    property TabPosition: TTabPosition read GetTabPosition write SetTabPosition
      default tpTop;
    property TabWidth: Smallint read GetTabWidth write SetTabWidth default 0;
    property TabOrder: Integer read GetTabOrder write SetTabOrder;
    property TabStop;
    property OnChange: TfrxNotifyEvent read FOnChange write FOnChange;
    property OnChanging: TfrxOnChangingEvent read FOnChanging write FOnChanging;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  { TfrxTabSheet }

  TfrxTabSheet = class(TfrxDialogControl)
  private
    FTabSheet: TTabSheet;
    function GetBorderWidth: Integer;
    {$IFNDEF FPC}
    function GetHighlighted: Boolean;
    {$ENDIF}
    function GetImageIndex: Integer;
    function GetPageIndex: Integer;
    function GetTabVisible: Boolean;
    procedure SetBorderWidth(const Value: Integer);
    {$IFNDEF FPC}
    procedure SetHighlighted(const Value: Boolean);
    {$ENDIF}
    procedure SetImageIndex(const Value: Integer);
    procedure SetPageIndex(const Value: Integer);
    procedure SetTabVisible(const Value: Boolean);
  protected
    procedure CorrectControlCoordinates; override;
    procedure SetParent(AParent: TfrxComponent); override;
    procedure DoSetParent;
    procedure SetLeft(Value: Extended); override;
    procedure SetTop(Value: Extended); override;
    procedure SetWidth(Value: Extended); override;
    procedure SetHeight(Value: Extended); override;
    procedure UpdateSize;
  public
    constructor Create(AOwner: TComponent); override;
    function IsAcceptAsChild(aParent: TfrxComponent): Boolean; override;
    function IsOwnerDraw: Boolean; override;
    function IsContain(X, Y: Extended): Boolean; override;
    function GetContainedComponent(X, Y: Extended; IsCanContain: TfrxComponent = nil): TfrxComponent; override;
    property TabSheet: TTabSheet read FTabSheet;
    // todo
    property ImageIndex: Integer read GetImageIndex write SetImageIndex default 0;
  published
    property Caption;
    property TabStop;
    property BorderWidth: Integer read GetBorderWidth write SetBorderWidth ;
    property TabVisible: Boolean read GetTabVisible write SetTabVisible default True;
    property PageIndex: Integer read GetPageIndex write SetPageIndex stored False;
    {$IFNDEF FPC}
    property Highlighted: Boolean read GetHighlighted write SetHighlighted default False;
    {$ENDIF}
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxCustomEditControl is the base class for TCustomEdit-based
  ///   controls.
  /// </summary>
  TfrxCustomEditControl = class(TfrxDialogControl)
  private
    FOnChange: TfrxNotifyEvent;
    function GetMaxLength: Integer;
    function GetPasswordChar: Char;
    function GetReadOnly: Boolean;
    function GetText: String;
    procedure DoOnChange(Sender: TObject);
    procedure SetMaxLength(const Value: Integer);
    procedure SetPasswordChar(const Value: Char);
    procedure SetReadOnly(const Value: Boolean);
    procedure SetText(const Value: String);
  protected
    FCustomEdit: TCustomEdit;
    function GetCharCase: TEditCharCase; virtual;
    procedure SetCharCase(const Value: TEditCharCase); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    property CharCase: TEditCharCase read GetCharCase write SetCharCase default ecNormal;   
    /// <summary>
    ///   Specifies the maximum number of characters the user can enter into
    ///   the edit control.
    /// </summary>
    property MaxLength: Integer read GetMaxLength write SetMaxLength;
    /// <summary>
    ///   Indicates the character, if any, to display in place of the actual
    ///   characters typed in the control.
    /// </summary>
    property PasswordChar: Char read GetPasswordChar write SetPasswordChar;
    /// <summary>
    ///   Determines whether the user can change the text of the edit control.
    /// </summary>
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    /// <summary>
    ///   Contains a text string associated with the control.
    /// </summary>
    property Text: String read GetText write SetText;
    /// <summary>
    ///   Name of the script procedure that will be executed when the text for
    ///   the edit control may have changed.
    /// </summary>
    property OnChange: TfrxNotifyEvent read FOnChange write FOnChange;
  end;

  /// <summary>
  ///   The TfrxEditControl component represents a TEdit adapter.
  /// </summary>
  TfrxEditControl = class(TfrxCustomEditControl)
  private
    FEdit: TEdit;
  protected
    function GetCharCase: TEditCharCase; override;
    procedure SetCharCase(const Value: TEditCharCase); override;
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TEdit object.
    /// </summary>
    property Edit: TEdit read FEdit;
  published
    property CharCase;
    property Color;
    property MaxLength;
    property PasswordChar;
    property ReadOnly;
    property TabStop;
    property Text;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxMemoControl component represents a TMemo adapter.
  /// </summary>
  TfrxMemoControl = class(TfrxCustomEditControl)
  private
    FMemo: TMemo;
    function GetLines: TStrings;
    procedure SetLines(const Value: TStrings);
    function GetScrollStyle: TScrollStyle;
    function GetWordWrap: Boolean;
    procedure SetScrollStyle(const Value: TScrollStyle);
    procedure SetWordWrap(const Value: Boolean);
 protected
    function GetCharCase: TEditCharCase; override;
    procedure SetCharCase(const Value: TEditCharCase); override;
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TMemo object.
    /// </summary>
    property Memo: TMemo read FMemo;
  published
    property CharCase;
    property Color;
    /// <summary>
    ///   Contains the individual lines of text in the memo control.
    /// </summary>
    property Lines: TStrings read GetLines write SetLines;
    property MaxLength;
    property ReadOnly;
    property ScrollBars: TScrollStyle read GetScrollStyle write SetScrollStyle default ssNone;
    property TabStop;
    /// <summary>
    ///   Specifies whether the label text wraps when it is too long for the
    ///   width of the label.
    /// </summary>
    property WordWrap: Boolean read GetWordWrap write SetWordWrap default True;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxButtonControl component represents a TButton adapter.
  /// </summary>
  TfrxButtonControl = class(TfrxDialogControl)
  private
    FButton: TButton;
    function GetCancel: Boolean;
    function GetDefault: Boolean;
    function GetModalResult: TModalResult;
    procedure SetCancel(const Value: Boolean);
    procedure SetDefault(const Value: Boolean);
    procedure SetModalResult(const Value: TModalResult);
{$IFDEF DELPHI7}
    function GetWordWrap: Boolean;
    procedure SetWordWrap(const Value: Boolean);
{$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TButton object.
    /// </summary>
    property Button: TButton read FButton;
  published
    /// <summary>
    ///   Determines whether the button's OnClick event handler executes when
    ///   the Escape key is pressed.
    /// </summary>
    property Cancel: Boolean read GetCancel write SetCancel default False;
    property Caption;
    /// <summary>
    ///   Determines whether the button's OnClick event handler executes when
    ///   the Enter key is pressed.
    /// </summary>
    property Default: Boolean read GetDefault write SetDefault default False;
    /// <summary>
    ///   Determines if and how the button closes its (modal) parent form.
    /// </summary>
    property ModalResult: TModalResult read GetModalResult write SetModalResult default mrNone;
{$IFDEF DELPHI7}
    /// <summary>
    ///   Specifies whether the label text wraps when it is too long for the
    ///   width of the label.
    /// </summary>
    property WordWrap: Boolean read GetWordWrap write SetWordWrap default False;
{$ENDIF}
    property TabStop;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxCheckBoxControl component represents a TCheckBox adapter.
  /// </summary>
  TfrxCheckBoxControl = class(TfrxDialogControl)
  private
    FCheckBox: TCheckBox;
    function GetAlignment: TAlignment;
    function GetAllowGrayed: Boolean;
    function GetChecked: Boolean;
    function GetState: TCheckBoxState;
    procedure SetAlignment(const Value: TAlignment);
    procedure SetAllowGrayed(const Value: Boolean);
    procedure SetChecked(const Value: Boolean);
    procedure SetState(const Value: TCheckBoxState);
{$IFDEF DELPHI7}
    function GetWordWrap: Boolean;
    procedure SetWordWrap(const Value: Boolean);
{$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TCheckBox object.
    /// </summary>
    property CheckBox: TCheckBox read FCheckBox;
  published
    /// <summary>
    ///   Controls the position of the check box's caption.
    /// </summary>
    property Alignment: TAlignment read GetAlignment write SetAlignment
      default taRightJustify;
    property Caption;
    /// <summary>
    ///   Specifies lf the check box appears checked.
    /// </summary>
    property Checked: Boolean read GetChecked write SetChecked default False;
    /// <summary>
    ///   Determines if the check box can be in a "grayed" state.
    /// </summary>
    property AllowGrayed: Boolean read GetAllowGrayed write SetAllowGrayed default False;
    /// <summary>
    ///   Indicates if the check box is selected, deselected, or grayed.
    /// </summary>
    property State: TCheckBoxState read GetState write SetState default cbUnchecked;
    property TabStop;
{$IFDEF DELPHI7}
    /// <summary>
    ///   Specifies if the label text wraps when it is too long for the width
    ///   of the label.
    /// </summary>
    property WordWrap: Boolean read GetWordWrap write SetWordWrap default False;
{$ENDIF}
    property Color;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxRadioButtonControl component represents a TRadioButton adapter.
  /// </summary>
  TfrxRadioButtonControl = class(TfrxDialogControl)
  private
    FRadioButton: TRadioButton;
    function GetAlignment: TAlignment;
    function GetChecked: Boolean;
    procedure SetAlignment(const Value: TAlignment);
    procedure SetChecked(const Value: Boolean);
{$IFDEF DELPHI7}
    function GetWordWrap: Boolean;
    procedure SetWordWrap(const Value: Boolean);
{$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TRadioButton object.
    /// </summary>
    property RadioButton: TRadioButton read FRadioButton;
  published
    /// <summary>
    ///   Controls the position of the radio button's caption.
    /// </summary>
    property Alignment: TAlignment read GetAlignment write SetAlignment
      default taRightJustify;
    property Caption;
    /// <summary>
    ///   Determines if the option represented by the radio button is selected.
    /// </summary>
    property Checked: Boolean read GetChecked write SetChecked default False;
    property TabStop;
{$IFDEF DELPHI7}
    /// <summary>
    ///   Specifies if the label text wraps when it is too long for the width
    ///   of the label.
    /// </summary>
    property WordWrap: Boolean read GetWordWrap write SetWordWrap default False;
{$ENDIF}
    property Color;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxListBoxControl component represents a TListBox adapter.
  /// </summary>
  TfrxListBoxControl = class(TfrxDialogControl)
  private
    FListBox: TListBox;
    function GetItems: TStrings;
    procedure SetItems(const Value: TStrings);
    function GetItemIndex: Integer;
    procedure SetItemIndex(const Value: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TListBox object.
    /// </summary>
    property ListBox: TListBox read FListBox;
    property ItemIndex: Integer read GetItemIndex write SetItemIndex;
  published
    property Color;
    /// <summary>
    ///   Contains the strings that appear in the list box.
    /// </summary>
    property Items: TStrings read GetItems write SetItems;
    property TabStop;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxComboBoxControl component represents a TComboBox adapter.
  /// </summary>
  TfrxComboBoxControl = class(TfrxDialogControl)
  private
    FComboBox: TComboBox;
    FOnChange: TfrxNotifyEvent;
    function GetItemIndex: Integer;
    function GetItems: TStrings;
    function GetStyle: TComboBoxStyle;
    function GetText: String;
    procedure DoOnChange(Sender: TObject);
    procedure SetItemIndex(const Value: Integer);
    procedure SetItems(const Value: TStrings);
    procedure SetStyle(const Value: TComboBoxStyle);
    procedure SetText(const Value: String);
  protected
    function GetCharCase: TEditCharCase; virtual;
    procedure SetCharCase(const Value: TEditCharCase); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TComboBox object.
    /// </summary>
    property ComboBox: TComboBox read FComboBox;
  published
    property CharCase: TEditCharCase read GetCharCase write SetCharCase default ecNormal;
    property Color;
    /// <summary>
    ///   Contains the strings that appear in the combo box.
    /// </summary>
    property Items: TStrings read GetItems write SetItems;
    /// <summary>
    ///   Determines the display style of the combo box.
    /// </summary>
    property Style: TComboBoxStyle read GetStyle write SetStyle default csDropDown;
    property TabStop;
    property Text: String read GetText write SetText;
    property ItemIndex: Integer read GetItemIndex write SetItemIndex;
    /// <summary>
    ///   Name of the script procedure that will be executed when the user
    ///   changes the text displayed in the edit region.
    /// </summary>
    property OnChange: TfrxNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
  end;

  /// <summary>
  ///   The TfrxPanelControl component represents a TPanel adapter.
  /// </summary>
  TfrxPanelControl = class(TfrxDialogControl)
  private
    FPanel: TPanel;
    function GetAlignment: TAlignment;
    function GetBevelInner: TPanelBevel;
    function GetBevelOuter: TPanelBevel;
    function GetBevelWidth: Integer;
    procedure SetAlignment(const Value: TAlignment);
    procedure SetBevelInner(const Value: TPanelBevel);
    procedure SetBevelOuter(const Value: TPanelBevel);
    procedure SetBevelWidth(const Value: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TPanel object.
    /// </summary>
    property Panel: TPanel read FPanel;
  published
    /// <summary>
    ///   Determines how caption is aligned within the panel.
    /// </summary>
    property Alignment: TAlignment read GetAlignment write SetAlignment default taCenter;
    /// <summary>
    ///   Determines the style of the inner bevel of a panel.
    /// </summary>
    property BevelInner: TPanelBevel read GetBevelInner write SetBevelInner default bvNone;
    /// <summary>
    ///   Determines the style of the outer bevel of a panel.
    /// </summary>
    property BevelOuter: TPanelBevel read GetBevelOuter write SetBevelOuter default bvRaised;
    /// <summary>
    ///   Determines the width, in pixels, of both the inner and outer bevels
    ///   of a panel.
    /// </summary>
    property BevelWidth: Integer read GetBevelWidth write SetBevelWidth default 1;
    property Caption;
    property Color;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxGroupBoxControl component represents a TGroupBox adapter.
  /// </summary>
  TfrxGroupBoxControl = class(TfrxDialogControl)
  private
    FGroupBox: TGroupBox;
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    /// <summary>
    ///   Reference to internal TGroupBox object.
    /// </summary>
    property GroupBox: TGroupBox read FGroupBox;
  published
    property Caption;
    property Color;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxDateEditControl component represents a TDateTimePicker adapter.
  /// </summary>
  TfrxDateEditControl = class(TfrxDialogControl)
  private
    FDateEdit: TDateTimePicker;
    FOnChange: TfrxNotifyEvent;
    FWeekNumbers: Boolean;
    function GetDate: TDate;
    function GetTime: TTime;
    function GetDateFormat: TDTDateFormat;
    function GetKind: TDateTimeKind;
    procedure DoOnChange(Sender: TObject);
    procedure SetDate(const Value: TDate);
    procedure SetTime(const Value: TTime);
    procedure SetDateFormat(const Value: TDTDateFormat);
    procedure SetKind(const Value: TDateTimeKind);
    procedure DropDown(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TDateTimePicker object.
    /// </summary>
    property DateEdit: TDateTimePicker read FDateEdit;
  published
    property Color;
    /// <summary>
    ///   Indicates the date that is marked on the calendar.
    /// </summary>
    property Date: TDate read GetDate write SetDate;
    /// <summary>
    ///   Specifies format in which the date is presented.
    /// </summary>
    property DateFormat: TDTDateFormat read GetDateFormat write SetDateFormat
      default dfShort;
    property Kind: TDateTimeKind read GetKind write SetKind default dtkDate;
    property TabStop;
    property Time: TTime read GetTime write SetTime;
    property WeekNumbers: Boolean read FWeekNumbers write FWeekNumbers;
    /// <summary>
    ///   Name of the script procedure that will be executed when a date or
    ///   time is entered.
    /// </summary>
    property OnChange: TfrxNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
  end;

  /// <summary>
  ///   The TfrxImageControl component represents a TImage adapter.
  /// </summary>
  TfrxImageControl = class(TfrxDialogControl)
  private
    FImage: TImage;
    function GetAutoSize: Boolean;
    function GetCenter: Boolean;
    function GetPicture: TPicture;
    function GetStretch: Boolean;
    procedure SetAutoSize(const Value: Boolean);
    procedure SetCenter(const Value: Boolean);
    procedure SetPicture(const Value: TPicture);
    procedure SetStretch(const Value: Boolean);
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TImage object.
    /// </summary>
    property Image: TImage read FImage;
  published
    /// <summary>
    ///   Determines whether the image control resizes to accommodate the image
    ///   it displays.
    /// </summary>
    property AutoSize: Boolean read GetAutoSize write SetAutoSize default False;
    /// <summary>
    ///   Indicates whether the image is centered in the image control.
    /// </summary>
    property Center: Boolean read GetCenter write SetCenter default False;
    /// <summary>
    ///   Specifies the image that appears on the image control.
    /// </summary>
    property Picture: TPicture read GetPicture write SetPicture;
    /// <summary>
    ///   Indicates whether the image should be changed so that it exactly fits
    ///   the bounds of the image control.
    /// </summary>
    property Stretch: Boolean read GetStretch write SetStretch default False;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxBevelControl component represents a TBevel adapter.
  /// </summary>
  TfrxBevelControl = class(TfrxDialogControl)
  private
    FBevel: TBevel;
    function GetBevelShape: TBevelShape;
    function GetBevelStyle: TBevelStyle;
    procedure SetBevelShape(const Value: TBevelShape);
    procedure SetBevelStyle(const Value: TBevelStyle);
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TBevel object.
    /// </summary>
    property Bevel: TBevel read FBevel;
  published
    /// <summary>
    ///   Determines the shape of the bevel.
    /// </summary>
    property Shape: TBevelShape read GetBevelShape write SetBevelShape default bsBox;
    /// <summary>
    ///   Determines whether the bevel appears raised or lowered.
    /// </summary>
    property Style: TBevelStyle read GetBevelStyle write SetBevelStyle default bsLowered;
  end;

  /// <summary>
  ///   The TfrxBitBtnControl component represents a TBitBtn adapter.
  /// </summary>
  TfrxBitBtnControl = class(TfrxDialogControl)
  private
    FBitBtn: TBitBtn;
    function GetGlyph: TBitmap;
    function GetKind: TBitBtnKind;
    function GetLayout: TButtonLayout;
    function GetMargin: Integer;
    function GetModalResult: TModalResult;
    function GetSpacing: Integer;
    procedure SetGlyph(const Value: TBitmap);
    procedure SetKind(const Value: TBitBtnKind);
    procedure SetLayout(const Value: TButtonLayout);
    procedure SetMargin(const Value: Integer);
    procedure SetModalResult(const Value: TModalResult);
    procedure SetSpacing(const Value: Integer);
    function GetNumGlyphs: Integer;
    procedure SetNumGlyphs(const Value: Integer);
{$IFDEF DELPHI7}
    function GetWordWrap: Boolean;
    procedure SetWordWrap(const Value: Boolean);
{$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TBitBtn object.
    /// </summary>
    property BitBtn: TBitBtn read FBitBtn;
  published
    /// <summary>
    ///   Specifies the bitmap that appears on the selected bitmap button.
    /// </summary>
    property Glyph: TBitmap read GetGlyph write SetGlyph;
    /// <summary>
    ///   Determines the kind of bitmap button.
    /// </summary>
    property Kind: TBitBtnKind read GetKind write SetKind default bkCustom;
    property Caption; // should be after Kind prop
    /// <summary>
    ///   Determines where the image appears on the bitmap button.
    /// </summary>
    property Layout: TButtonLayout read GetLayout write SetLayout default blGlyphLeft;
    /// <summary>
    ///   Determines the number of pixels between the edge of the image
    ///   (specified in the Glyph property) and the edge of the button.
    /// </summary>
    property Margin: Integer read GetMargin write SetMargin default -1;
    /// <summary>
    ///   Determines if and how the button closes its (modal) parent form.
    /// </summary>
    property ModalResult: TModalResult read GetModalResult write SetModalResult default mrNone;
    /// <summary>
    ///   Indicates the number of images that are in the graphic specified in
    ///   the Glyph property for use on a bitmap button or speed button.
    /// </summary>
    property NumGlyphs: Integer read GetNumGlyphs write SetNumGlyphs default 1;
    /// <summary>
    ///   Determines where the image and text appear on a bitmap or speed
    ///   button.
    /// </summary>
    property Spacing: Integer read GetSpacing write SetSpacing default 4;
{$IFDEF DELPHI7}
    /// <summary>
    ///   Specifies whether the label text wraps when it is too long for the
    ///   width of the label.
    /// </summary>
    property WordWrap: Boolean read GetWordWrap write SetWordWrap default False;
{$ENDIF}
    property TabStop;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxSpeedButtonControl component represents a TSpeedButton adapter.
  /// </summary>
  TfrxSpeedButtonControl = class(TfrxDialogControl)
  private
    FSpeedButton: TSpeedButton;
    function GetAllowAllUp: Boolean;
    function GetDown: Boolean;
    function GetFlat: Boolean;
    function GetGlyph: TBitmap;
    function GetGroupIndex: Integer;
    function GetLayout: TButtonLayout;
    function GetMargin: Integer;
    function GetSpacing: Integer;
    procedure SetAllowAllUp(const Value: Boolean);
    procedure SetDown(const Value: Boolean);
    procedure SetFlat(const Value: Boolean);
    procedure SetGlyph(const Value: TBitmap);
    procedure SetGroupIndex(const Value: Integer);
    procedure SetLayout(const Value: TButtonLayout);
    procedure SetMargin(const Value: Integer);
    procedure SetSpacing(const Value: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TSpeedButton object.
    /// </summary>
    property SpeedButton: TSpeedButton read FSpeedButton;
  published
    /// <summary>
    ///   Specifies if all speed buttons in the group that contains this speed
    ///   button can be unselected at the same time.
    /// </summary>
    property AllowAllUp: Boolean read GetAllowAllUp write SetAllowAllUp default False;
    property Caption;
    /// <summary>
    ///   Specifies if the button is selected (down) or unselected (up).
    /// </summary>
    property Down: Boolean read GetDown write SetDown default False;
    /// <summary>
    ///   Determines if the button has a 3D border that provides a raised or
    ///   lowered look.
    /// </summary>
    property Flat: Boolean read GetFlat write SetFlat default False;
    /// <summary>
    ///   Specifies the bitmap that appears on the speed button.
    /// </summary>
    property Glyph: TBitmap read GetGlyph write SetGlyph;
    /// <summary>
    ///   Allows speed buttons to work together as a group.
    /// </summary>
    property GroupIndex: Integer read GetGroupIndex write SetGroupIndex;
    /// <summary>
    ///   Determines where the image or text appears on the speed button.
    /// </summary>
    property Layout: TButtonLayout read GetLayout write SetLayout default blGlyphLeft;
    /// <summary>
    ///   Specifies the number of pixels between the edge of the button and the
    ///   image or caption drawn on its surface.
    /// </summary>
    property Margin: Integer read GetMargin write SetMargin default -1;
    /// <summary>
    ///   Determines where the image and text appear on a speed button
    /// </summary>
    property Spacing: Integer read GetSpacing write SetSpacing default 4;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxMaskEditControl component represents a TMaskEdit adapter.
  /// </summary>
  TfrxMaskEditControl = class(TfrxCustomEditControl)
  private
    FMaskEdit: TMaskEdit;
    function GetEditMask: String;
    procedure SetEditMask(const Value: String);
protected
    function GetCharCase: TEditCharCase; override;
    procedure SetCharCase(const Value: TEditCharCase); override;
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TMaskEdit object.
    /// </summary>
    property MaskEdit: TMaskEdit read FMaskEdit;
  published
    property CharCase;
    property Color;
    /// <summary>
    ///   Specifies the mask that represents what text is valid for the masked
    ///   edit control.
    /// </summary>
    property EditMask: String read GetEditMask write SetEditMask;
    property MaxLength;
    property ReadOnly;
    property TabStop;
    property Text;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  /// <summary>
  ///   The TfrxCheckListBoxControl component represents a TCheckListBox
  ///   adapter.
  /// </summary>
  TfrxCheckListBoxControl = class(TfrxDialogControl)
  private
    FCheckListBox: TCheckListBox;
    function GetAllowGrayed: Boolean;
    function GetItems: TStrings;
    function GetSorted: Boolean;
    function GetChecked(Index: Integer): Boolean;
    function GetState(Index: Integer): TCheckBoxState;
    procedure SetAllowGrayed(const Value: Boolean);
    procedure SetItems(const Value: TStrings);
    procedure SetSorted(const Value: Boolean);
    procedure SetChecked(Index: Integer; const Value: Boolean);
    procedure SetState(Index: Integer; const Value: TCheckBoxState);
    function GetItemIndex: Integer;
    procedure SetItemIndex(const Value: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TCheckListBox object.
    /// </summary>
    property CheckListBox: TCheckListBox read FCheckListBox;
    property Checked[Index: Integer]: Boolean read GetChecked write SetChecked;
    property ItemIndex: Integer read GetItemIndex write SetItemIndex;
    property State[Index: Integer]: TCheckBoxState read GetState write SetState;
  published
    /// <summary>
    ///   Determines if the check box can be in a "grayed" state.
    /// </summary>
    property AllowGrayed: Boolean read GetAllowGrayed write SetAllowGrayed default False;
    property Color;
    /// <summary>
    ///   Contains the strings that appear in the list box.
    /// </summary>
    property Items: TStrings read GetItems write SetItems;
    /// <summary>
    ///   Specifies if the items in a list box are arranged alphabetically.
    /// </summary>
    property Sorted: Boolean read GetSorted write SetSorted default False;
    property TabStop;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

{$IFDEF FPC}
  procedure Register;
{$ENDIF}

implementation
end.
