unit frxDelphiMaxiCode;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}Windows,{$ENDIF} Graphics, SysUtils;

(* Copyright 2014-2015 Robin Stuart, Daniel Gredler
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 * Implements MaxiCode according to ISO 16023:2000.
 *
 * MaxiCode employs a pattern of hexagons around a central 'bulls-eye'
 * finder pattern. Encoding in several modes is supported, but encoding in
 * Mode 2 and 3 require primary messages to be set. Input characters can be
 * any from the ISO 8859-1 (Latin-1) character set.
 *
 * TODO: Add ECI functionality.
 *
 * @author <a href="mailto:rstuart114@gmail.com">Robin Stuart</a>
 * @author Daniel Gredler
 *)

type
  TMaxicodeEncoder = class
  private
    FData: WideString;
    FMode: Integer;

    procedure SetData(const Value: WideString);
    procedure SetMode(const Value: Integer);
    function GetIsBlack(Row, Column: integer): Boolean;
  protected
    FBitmap: TBitmap;

    procedure Update;
  public
    constructor Create;
    destructor Destroy; override;
    function Width: Integer;
    function Height: Integer;
    function GetScanLine(Column: Integer): PByteArray;

    property IsBlack[Row, Column: integer]: Boolean read GetIsBlack;
    property Data: WideString read FData write SetData;
    property Mode: Integer read FMode write SetMode;
    property Bitmap: TBitmap read FBitmap;
  end;

implementation
end.
