unit frxDelphiZXIngCode;

(*
 * Copyright 2008 ZXing authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *)

{$I frx.inc}

interface

uses Contnrs;

type
  TIntegerArray = array of Integer;

  TGenericGFPoly = class;

  TGenericGF = class
  private
    FExpTable: TIntegerArray;
    FLogTable: TIntegerArray;
    FZero: TGenericGFPoly;
    FOne: TGenericGFPoly;
    FSize: Integer;
    FPrimitive: Integer;
    FGeneratorBase: Integer;
    FInitialized: Boolean;
    FPolyList: array of TGenericGFPoly;

    procedure CheckInit;
    procedure Initialize;
  public
    class function CreateQRCodeField256: TGenericGF;

    class function CreateAztecData12: TGenericGF; // x^12 + x^6 + x^5 + x^3 + 1
    class function CreateAztecData10: TGenericGF; // x^10 + x^3 + 1
    class function CreateAztecData8: TGenericGF;
    class function CreateMatrixField256: TGenericGF; // x^8 + x^5 + x^3 + x^2 + 1
    class function CreateAztecData6: TGenericGF;
    class function CreateMaxicodeField64: TGenericGF; // x^6 + x + 1
    class function CreateAztecParam: TGenericGF; // x^4 + x + 1

    class function AddOrSubtract(A, B: Integer): Integer;

    constructor Create(Primitive, Size, B: Integer);
    destructor Destroy; override;
    function GetZero: TGenericGFPoly;
    function Exp(A: Integer): Integer;
    function GetGeneratorBase: Integer;
    function Inverse(A: Integer): Integer;
    function Multiply(A, B: Integer): Integer;
    function BuildMonomial(Degree, Coefficient: Integer): TGenericGFPoly;
  end;

  TGenericGFPolyArray = array of TGenericGFPoly;

  TGenericGFPoly = class
  private
    FField: TGenericGF;
    FCoefficients: TIntegerArray;
  public
    constructor Create(AField: TGenericGF; ACoefficients: TIntegerArray);
    destructor Destroy; override;
    function Coefficients: TIntegerArray;
    function Multiply(Other: TGenericGFPoly): TGenericGFPoly;
    function MultiplyByMonomial(Degree, Coefficient: Integer): TGenericGFPoly;
    function Divide(Other: TGenericGFPoly): TGenericGFPolyArray;
    function GetCoefficients: TIntegerArray;
    function IsZero: Boolean;
    function GetCoefficient(Degree: Integer): Integer;
    function GetDegree: Integer;
    function AddOrSubtract(Other: TGenericGFPoly): TGenericGFPoly;
  end;

  TReedSolomonEncoder = class
  private
    FField: TGenericGF;
    FCachedGenerators: TObjectList;
  public
    constructor Create(AField: TGenericGF);
    destructor Destroy; override;
    procedure Encode(ToEncode: TIntegerArray; ECBytes: Integer);
    function BuildGenerator(Degree: Integer): TGenericGFPoly;
  end;

  TByteArray = array of Byte;

  TBitArray = class
  private
    FSize: Integer;
    Bits: array of Integer;

    procedure SetItem(i: Integer; const Value: Boolean);
  protected
    procedure EnsureCapacity(Size: Integer);
  public
    constructor Create; overload;
    constructor Create(const Size: Integer); overload;
    procedure Clear;
    function GetSizeInBytes: Integer;
    function GetSize: Integer;
    function Get(i: Integer): Boolean;
    procedure SetBit(Index: Integer);
    procedure AppendBit(Bit: Boolean);
    procedure AppendBits(Value, NumBits: Integer);
    procedure AppendBitArray(NewBitArray: TBitArray);
    procedure ToBytes(BitOffset: Integer; Source: TByteArray;
      Offset, NumBytes: Integer);
    procedure XorOperation(Other: TBitArray);

    function ToString: string; {$IFDEF DELPHI12} override; {$ENDIF}

    property Size: Integer read FSize;
    property Item[i: Integer]: Boolean read Get write SetItem; default;
  end;

function IfValue(IsTrue: Boolean; TrueValue, FalseValue: Integer)
  : Integer; overload;
function IfValue(IsTrue: Boolean; TrueValue, FalseValue: string)
  : string; overload;
function IfValue(IsTrue: Boolean; TrueValue, FalseValue: AnsiChar)
  : AnsiChar; overload;

//function ToBinaryString(x: LongInt): string;
procedure FreeAndSetBitArray(var BitArray1: TBitArray; BitArray2: TBitArray);

implementation
end.

