
{******************************************}
{                                          }
{             FastReport VCL               }
{            Designer controls             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxDesgnCtrls;

interface

{$I frx.inc}

uses
  SysUtils, {$IFNDEF FPC}Windows, Messages,{$ENDIF}
  Types, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, ComCtrls, ToolWin, ImgList, frxClass,
  frPictureCache, frxDPIAwareBaseControls,
{$IFDEF FPC}
  LResources, LCLType, LazHelper, LCLIntf, LazarusPackageIntf,
{$ENDIF}
{$IFDEF DELPHI6}
  Variants,
{$ENDIF}
  frTypes;


const
  ClipboardPrefix: String = '#FR3 clipboard#';

type
  TfrxRulerUnits = (ruCM, ruInches, ruPixels, ruChars);
  TfrxRulerPointerAction = (rpAdd, rpDelete);
  TfrxRulerSign = (rsNone, rsPlus, rsMinus);
  TfrxUpdateVirtualLines = procedure (Sender: TObject; Position: Extended) of object;
  TfrxPointerChanged = procedure (Sender: TObject; Action: TfrxRulerPointerAction; Position: Extended) of object;
  TfrxPointerCheck = function (Sender: TObject; Position: Extended): Boolean of object;

{$IFDEF DELPHI16}
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxRuler = class(TfrxDPIAwarePanel)
  private
    FOffset: Integer;
    FScale: Extended;
    FStart: Integer;
    FUnits: TfrxRulerUnits;
    FPosition: Extended;
    FSize: Integer;
    FGuides: TStrings;
    FVirtualGuid: Extended;
    FMouseDown: Boolean;
    FOnPointerAdded: TNotifyEvent;
    FOnUpdateVirtualLines: TfrxUpdateVirtualLines;
    FOnPointerChanged: TfrxPointerChanged;
    FOnPointerCheck: TfrxPointerCheck;
    FLastPos: TPoint;
    FEditMode: Boolean;
    //FScreenScale: Single;
    procedure SetOffset(const Value: Integer);
    procedure SetScale(const Value: Extended);
    procedure SetStart(const Value: Integer);
    procedure SetUnits(const Value: TfrxRulerUnits);
    procedure SetPosition(const Value: Extended);
    procedure WMEraseBackground(var Message: TMessage); message WM_ERASEBKGND;
    procedure SetSize(const Value: Integer);
    procedure DrawArrow(aCanvas: TCanvas; aPos: Integer; IsSelected: Boolean; BrushColor: TColor; Sign: TfrxRulerSign);
    procedure DrawHint(aCanvas: TCanvas; aPos: Integer; Value: Extended);
  protected
    procedure DblClick; override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure DoEndDrag(Target: TObject; X, Y: Integer); override;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure DoPPIChanged(aNewPPI: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure MouseLeave;
    procedure Paint; override;
    property Guides: TStrings read FGuides write FGuides;
    property EditMode: Boolean read FEditMode write FEditMode;
  published
    property OnPointerAdded: TNotifyEvent read FOnPointerAdded write FOnPointerAdded;
    property OnUpdateVirtualLines: TfrxUpdateVirtualLines read FOnUpdateVirtualLines write FOnUpdateVirtualLines;
    property OnPointerChanged: TfrxPointerChanged read FOnPointerChanged write FOnPointerChanged;
    property OnPointerCheck: TfrxPointerCheck read FOnPointerCheck write FOnPointerCheck;
    property Offset: Integer read FOffset write SetOffset;
    property Scale: Extended read FScale write SetScale;
    property Start: Integer read FStart write SetStart;
    property Units: TfrxRulerUnits read FUnits write SetUnits default ruPixels;
    property Position: Extended read FPosition write SetPosition;
    property Size: Integer read FSize write SetSize;
  end;

{$IFDEF DELPHI16}
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}

  { TfrxScrollBox }

  TfrxScrollBox = class(TScrollBox)
  {$IFDEF FPC}
  private
    FOnAfterScroll: TNotifyEvent;
  {$ENDIF}
  protected
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
  {$IFDEF FPC}
    procedure ScrollBy(DeltaX, DeltaY: Integer); override;
  public
    property OnAfterScroll:TNotifyEvent read FOnAfterScroll write FOnAfterScroll;
  {$ENDIF}
  end;

  TfrxCustomSelector = class(TfrxDPIAwarePanel)
  private
    FclWidth: Integer;
    FclHeight: Integer;
    {$IFDEF FPC_NOPAINTOUTSIDEPAINTEVENT}
    FSavedIsDown: Boolean;
    FSavedX: Integer;
    FSavedY: Integer;
    {$ENDIF}
    procedure WMEraseBackground(var Message: TMessage); message WM_ERASEBKGND;
  protected
    procedure DrawEdge(X, Y: Integer; IsDown: Boolean); virtual; abstract;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    {$IFDEF FPC_NOPAINTOUTSIDEPAINTEVENT}
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    property SavedX: Integer read FSavedX;
    property SavedY: Integer read FSavedY;
    property SavedIsDown: Boolean read FSavedIsDown;
    {$ENDIF}
  public
    procedure Paint; override;
    constructor Create(AOwner: TComponent); override;
  end;

  TfrxColorSelector = class(TfrxCustomSelector)
  private
    FColor: TColor;
    FOnColorChanged: TNotifyEvent;
    FBtnCaption: String;
    FColorRows: Integer;
    FColorColumns: Integer;
    FGap: Integer;
    FColorCellSize: Integer;
    FButtonHeight: Integer;
  protected
    function CalcSize: TSize;
    procedure DrawEdge(X, Y: Integer; IsDown: Boolean); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Paint; override;
    property Color: TColor read FColor write FColor;
    property OnColorChanged: TNotifyEvent read FOnColorChanged write FOnColorChanged;
    property BtnCaption: String read FBtnCaption write FBtnCaption;
  end;

  TfrxLineSelector = class(TfrxCustomSelector)
  private
    FStyle: Byte;
    FOnStyleChanged: TNotifyEvent;
  protected
    procedure DrawEdge(X, Y: Integer; IsDown: Boolean); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    function GetScaledSize: Integer;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Paint; override;
    property Style: Byte read FStyle;
    property OnStyleChanged: TNotifyEvent read FOnStyleChanged write FOnStyleChanged;
  end;

  TfrxUndoBuffer = class(TObject)
  private
    FPictureCache: TfrxPictureCache;
    FRedo: TList;
    FUndo: TList;
    function GetRedoCount: Integer;
    function GetUndoCount: Integer;
    procedure SetPictureFlag(ReportComponent: TfrxComponent; Flag: Boolean);
    procedure SetPictures(ReportComponent: TfrxComponent);
  public
    constructor Create;
    destructor Destroy; override;
    procedure AddUndo(ReportComponent: TfrxComponent);
    procedure AddRedo(ReportComponent: TfrxComponent);
    procedure GetUndo(ReportComponent: TfrxComponent);
    procedure GetRedo(ReportComponent: TfrxComponent);
    procedure ClearUndo;
    procedure RemoveLastUndo;
    procedure ClearRedo;
    property UndoCount: Integer read GetUndoCount;
    property RedoCount: Integer read GetRedoCount;
    property PictureCache: TfrxPictureCache read FPictureCache write FPictureCache;
  end;

  TfrxClipboard = class(TObject)
  private
    FDesigner: TfrxCustomDesigner;
    FPictureCache: TfrxPictureCache;
    function GetPasteAvailable: Boolean;
    function IsFrxPasteAvailable: Boolean;
  public
    constructor Create(ADesigner: TfrxCustomDesigner);
    procedure Copy;
    function Paste: Boolean;
    property PasteAvailable: Boolean read GetPasteAvailable;
    property PictureCache: TfrxPictureCache read FPictureCache write FPictureCache;
  end;

  TfrxFrameLineClickedEvent = procedure(Line: TfrxFrameType; state: Boolean) of object;

  TfrxFrameSampleControl = class(TCustomControl)
  private
    FFrame: TfrxFrame;
    FOnFrameLineClicked: TfrxFrameLineClickedEvent;
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    procedure Paint; override;
    property Frame: TfrxFrame read FFrame write FFrame;
    property OnFrameLineClicked: TfrxFrameLineClickedEvent read FOnFrameLineClicked write FOnFrameLineClicked;
  end;

  TfrxLineStyleControl = class(TfrxDPIAwareCustomControl)
  private
    FStyle: TfrxFrameStyle;
    FOnStyleChanged: TNotifyEvent;
    procedure WMEraseBackground(var Message: TMessage); message WM_ERASEBKGND;
    procedure SetStyle(const Value: TfrxFrameStyle);
  protected
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Paint; override;
    property Style: TfrxFrameStyle read FStyle write SetStyle;
    property OnStyleChanged: TNotifyEvent read FOnStyleChanged write FOnStyleChanged;
  end;

  TfrxColorComboBox = class(TCustomControl)
  private
    FCombo: TComboBox;
    FColor: TColor;
    FShowColorName: Boolean;
    FOnColorChanged: TNotifyEvent;
    FBlockPopup: Boolean;
    procedure SetColor(const Value: TColor);
    procedure SetShowColorName(const Value: Boolean);
    procedure ColorChanged(Sender: TObject);
  protected
    procedure SetEnabled(Value: Boolean); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Paint; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property Color: TColor read FColor write SetColor;
    property ShowColorName: Boolean read FShowColorName write SetShowColorName;
    property OnColorChanged: TNotifyEvent read FOnColorChanged write FOnColorChanged;
  end;

  procedure frxAssignToolbarSize(AToolBar: TToolBar; AImgList: TImageList; APPI: Integer);
{$IFDEF FPC}
//  procedure Register;
{$ENDIF}

implementation
end.
