
{******************************************}
{                                          }
{             FastReport VCL               }
{           Designer workspace             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxDesgnWorkspace1;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}
  Windows, Messages,
  {$ENDIF}
  Types, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Buttons, frxClass, frxDesgn,
  frxDesgnWorkspace, frPopupForm, frTreeView
  {$IFDEF FPC}
  ,LCLType, LCLIntf, LazHelper
  {$ENDIF}
{$IFDEF DELPHI6}
, Variants
{$ENDIF}
{$IFDEF Delphi12}
, Character
{$ENDIF};


type
  TfrxGuideItem = class(TCollectionItem)
  public
    Left, Top, Right, Bottom: Extended;
  end;

  TfrxVirtualGuides = class(TCollection)
  private
    function GetGuides(Index: Integer): TfrxGuideItem;
  public
    constructor Create;
    procedure Add(Left, Top, Right, Bottom: Extended);
    property Items[Index: Integer]: TfrxGuideItem read GetGuides; default;
  end;

  TDesignerWorkspace = class(TfrxDesignerWorkspace)
  private
    FAHGuides: TStrings;
    FAVGuides: TStrings;
    FDesigner: TfrxDesignerForm;
    FGuide: Integer;
    FShowGuides: Boolean;
    FAutoGuides: Boolean;
    FAutoGuidesH: Boolean;
    FAutoGuidesV: Boolean;
    FSimulateMove: Boolean;
    FTool: TfrxDesignTool;
    FVirtualGuides: TfrxVirtualGuides;
    FVirtualGuideObjects: TList;
    FGuidesObjects: TList;
    FGuidesObjectsSize: TList;
    FPopupFormVisible: Boolean;
    FMouseDownObject: TfrxComponent;
    FStickToGuides: Boolean;
    FGuidesAsAnchor: Boolean;
    FStickAccuracy: Extended;
    FVVirtualGuid: Extended;
    FHVirtualGuid: Extended;
    procedure CreateVirtualGuides;
    procedure SetShowGuides(const Value: Boolean);
    procedure SetAutoGuides(const Value: Boolean);
    procedure SetAutoGuidesH(const Value: Boolean);
    procedure SetAutoGuidesV(const Value: Boolean);
    procedure SetHGuides(const Value: TStrings);
    procedure SetVGuides(const Value: TStrings);
    function GetHGuides: TStrings;
    function GetVGuides: TStrings;
    function GetPageHGuides: TStrings;
    function GetPageVGuides: TStrings;
    procedure SetTool(const Value: TfrxDesignTool);
  protected
    function IsSupportsGuidlines(APage: TfrxPage): Boolean;
    procedure CheckGuides(var kx, ky: Extended; var Result: Boolean; IsMouseDown: Boolean); override;
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState;
      var Accept: Boolean); override;
    procedure DrawObjects; override;
    procedure DoFinishInPlace(Sender: TObject; Refresh, Modified: Boolean); override;
    procedure SetDefaultEventParams(var EventParams: TfrxInteractiveEventsParams); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    function DoMouseWheel(Shift: TShiftState; WheelDelta: Integer;
      MousePos: TPoint): Boolean; override;
    procedure DblClick; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure ClearLastView; override;
    destructor Destroy; override;
    procedure DeleteObjects; override;
    procedure DragDrop(Source: TObject; X, Y: Integer); override;
    procedure SimulateMove;
    procedure SetVirtualGuids(VGuid, HGuid: Extended); override;
    procedure SetInsertion(AClass: TfrxComponentClass;
      AWidth, AHeight: Extended; AFlag: Word); override;
    property HGuides: TStrings read GetHGuides write SetHGuides;
    property VGuides: TStrings read GetVGuides write SetVGuides;
    property PageHGuides: TStrings read GetPageHGuides;
    property PageVGuides: TStrings read GetPageVGuides;
    property ShowGuides: Boolean read FShowGuides write SetShowGuides;
    property AutoGuides: Boolean read FAutoGuides write SetAutoGuides;
    property AutoGuidesH: Boolean read FAutoGuidesH write SetAutoGuidesH;
    property AutoGuidesV: Boolean read FAutoGuidesV write SetAutoGuidesV;
    property StickToGuides: Boolean read FStickToGuides write FStickToGuides;
    property GuidesAsAnchor: Boolean read FGuidesAsAnchor write FGuidesAsAnchor;
    property StickAccuracy: Extended read FStickAccuracy write FStickAccuracy;
    property Tool: TfrxDesignTool read FTool write SetTool;
  end;

implementation
end.
