
{******************************************}
{                                          }
{             FastReport VCL               }
{              Tool controls               }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxDock;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}Windows, Messages, {$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, Buttons, frDPIAwareInt, frxDPIAwareBaseControls, IniFiles,
  frxBaseForm
{$IFDEF FPC}
  , LazarusPackageIntf, LazHelper, types
{$ENDIF}
{$IFDEF DELPHI6}
, Variants
{$ENDIF};


type
{$IFDEF DELPHI16}
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxTBPanel = class(TPanel)
  protected
    procedure SetParent(AParent:TWinControl); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Paint; override;
  end;

{$IFDEF DELPHI16}
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxDockSite = class(TfrxDPIAwarePanel)
  private
    FPanelSize: Integer;
    FSavedSize: Integer;
    FSplitter: TControl;
    FTopParentWin: TWinControl;
  protected
    procedure VisibleChanging; override;
    procedure DoPPIChanged(aNewPPI: Integer); override;
    {$IFNDEF FPC}
    function CreateDockManager: IDockManager; override;
    {$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    procedure DockDrop(Source: TDragDockObject; X, Y: Integer); override;
    procedure DockOver(Source: TDragDockObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean); override;
    function DoUnDock(NewTarget: TWinControl; Client: TControl
      {$IFDEF FPC}; KeepDockSiteSize: Boolean = true{$ENDIF}): Boolean; override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure ReloadDockedControl(const AControlName: string;
      var AControl: TControl); override;
    procedure UpdateAlign;
    property SavedSize: Integer read FSavedSize write FSavedSize;
    property TopParentWin: TWinControl read FTopParentWin write FTopParentWin;
  end;

  TfrxDockForm = class(TfrxBaseForm)
  protected
  {$IFNDEF FPC}
    procedure DoStartDock(var DragObject: TDragObject); override;
  {$ENDIF}
    function GetCaptionFromResources: string; virtual; abstract;
    property CaptionFromResources: string read GetCaptionFromResources;
  public
    procedure UpdateResouces; override;
  end;

{$IFNDEF FPC}
  TBlueForm = class(TForm)
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  end;

  TfrxDragDockObject = class {$IFDEF VER130}(TDragDockObject){$ELSE}(TDragDockObjectEx){$ENDIF}
  private
    BlueForm: TBlueForm;
  protected
    procedure DrawDragDockImage; override;
    procedure EraseDragDockImage; override;
  public
    constructor Create(AControl: TControl); override;
    destructor Destroy; override;
  end;

  TfrxDockToolBar = class(TToolBar)
  protected
    procedure DoStartDock(var DragObject: TDragObject); override;
  end;

const
  ColorFill = $ECDECE;
  ColorClose = $606060;

type

  TfrxDockTree = class(TDockTree, IfrxDPIAwareControl)
  private
    FFont:HFont;
    FFontV:HFont;
    FCurrentPPI: Integer;
    HeadSize, ClosePadding, CaptionPadding, CloseWidth, TextHeight, TextHide: Integer;
    FRectClose: TRect;
    procedure DrawCaption(Canvas: TCanvas; ARect: TRect; Control: TControl);
    procedure DrawRect(Canvas:TCanvas; R:TRect);
    procedure DrawClose(Canvas:TCanvas; R:TRect);
    procedure DoPPIChanged(aNewPPI: Integer);
    procedure FreeFont;
  protected
    procedure PaintDockFrame(Canvas: TCanvas; Control: TControl; const ARect: TRect); override;
    procedure AdjustDockRect(Control: TControl; var ARect: TRect); override;
    procedure CreateCaptionFont(var Font,FontV:HFont);
    {$IFDEF DELPHI9}
    function ZoneCaptionHitTest(const Zone: TDockZone; const MousePos: TPoint; var HTFlag: Integer): Boolean; override;
    {$ELSE}
    function HitTest(const MousePos: TPoint; out HTFlag: Integer): TControl; override;
    {$ENDIF}
  public
    constructor Create(DockSite: TWinControl); override;
    destructor Destroy; override;
  end;

{$ELSE FPC}
  TfrxDockToolBar = class(TToolBar)
  end;
{$ENDIF FPC}


procedure frxSaveToolbarPosition(Ini: TCustomIniFile; t: TToolBar; CurrentFormPPI: Integer);
procedure frxRestoreToolbarPosition(Ini: TCustomIniFile; t: TToolBar; CurrentFormPPI: Integer);
procedure frxSaveDock(Ini: TCustomIniFile; d: TfrxDockSite; FormPPIScale: Single);
procedure frxRestoreDock(Ini: TCustomIniFile; d: TfrxDockSite);
procedure frxSaveFormPosition(Ini: TCustomIniFile; f: TForm; FormPPIScale: Single);
procedure frxRestoreFormPosition(Ini: TCustomIniFile; f: TForm; FormPPIScale: Single);


{$IFDEF FPC}
//procedure Register;
{$ENDIF}

implementation
end.
