
{******************************************}
{                                          }
{             FastReport VCL               }
{        ERSI Shape File DBF Import        }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxERSIShapeDBFImport;

interface

{$I frx.inc}

uses
  Classes, SysUtils
{$IFDEF DELPHI12}
, AnsiStrings
{$ENDIF}
  ;

type
  // http://www.clicketyclick.dk/databases/xbase/format/dbf.html#DBF_NOTE_1_TARGET
  TERSIDBFFileHeader = packed record
    Signature: Byte; // Version number
    YY, MM, DD: Byte; // Date of last update
    NumberOfRecords: LongWord;
    LengthOfHeaderStructure: Word; // = 32
    LengthOfEachRecord: Word; // = Sum of lengths of all fields + 1 (deletion flag)
    Reserved0: Word;
    IncompleteTransaction: Byte;
    EncryptionFlag: Byte;
    FreeRecordThread: LongWord;
    ReservedForMultiUser: array[0..7] of Byte;
    MDXFlag: byte;
    LanguageDriver: byte; // ==> CodePage
    Reserved1: Word;
  end;

  TERSIDBFFieldDescriptor = packed record
    FieldName: array[0..10] of AnsiChar; //  in ASCII (terminated by 00h)
    FieldType: AnsiChar;
    Reserved0: LongWord;
    FieldLength: Byte;
    DecimalCount: Byte; // <= 15
    Reserved1: array[0..12] of Byte;
    IndexFieldFlag: Byte;
  end;

  TDBFFieldType = (dfUnhandled, dfChar, dfDate, dfNumeric, dfLogical);

  TAnsiCharSet = set of AnsiChar;

  TERSIDBFFileColumn = class
  private
    FFieldName: AnsiString;
    FFieldType: TDBFFieldType;
    FFieldLength: Byte;
    FData: array of AnsiString;
    function GetData(iRecord: integer): AnsiString;
  protected
    FBuffer: array of AnsiChar;
    function BufferToAnsiString(TrimLeft, TrimRight: TAnsiCharSet): AnsiString;
  public
    constructor Create(Stream: TStream; NumberOfRecords: LongWord);
    procedure Read(Stream: TStream; CurrentRecord: integer);

    property FieldName: AnsiString read FFieldName;
    property FieldType: TDBFFieldType read FFieldType;
    property FieldLength: Byte read FFieldLength;
    property Data[iRecord: integer]: AnsiString read GetData;
  end;

  TERSIDBFFile = class
  private
    function GetCodePage: integer;
    function GetLegend(FieldName: AnsiString; iRecord: Integer): AnsiString;
    function GetLegendByColumn(iColumn, iRecord: Integer): AnsiString;
  protected
    FDBFFileHeader: TERSIDBFFileHeader;
    FColumns: array of TERSIDBFFileColumn;
    FCPG: string;

    procedure ReadFromStream(Stream: TStream; RecordCount: integer);
  public
    constructor Create(const FileName: string; RecordCount: integer);
    destructor Destroy; override;
    procedure GetColumnList(List: TStrings);
    function IsUTF8: boolean;

    property CodePage: integer read GetCodePage;
    property Legend[FieldName: AnsiString; iRecord: Integer]: AnsiString read GetLegend;
    property LegendByColumn[iColumn, iRecord: Integer]: AnsiString read GetLegendByColumn;
  end;

implementation
end.
