
{******************************************}
{                                          }
{             FastReport VCL               }
{               Memo editor                }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxEditMemo;

interface

{$I frx.inc}

uses
  SysUtils, {$IFNDEF FPC}Windows, Messages,{$ENDIF} Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls, ToolWin, frxClass,
  frxEditFormat, frxEditHighlight, frxBaseFormEditorDataTree, frSynMemo
{$IFDEF FPC}
  , LResources, LCLType, LMessages
{$ENDIF}

{$IFDEF DELPHI6}
, Variants
{$ENDIF};
  

type
  TfrxMemoController = class
  private
    FUseSmartMemo: Boolean;
    FOldMemo: TMemo;
    FNewMemo: TfrBaseSmartSyntaxMemo;

    procedure SetDPIScale(AValue: Single); // Only New
    function GetText: String;
    procedure SetText(AText: String);
    procedure SetSelText(AText: String);
    function GetWordWrap: Boolean;
    procedure SetWordWrap(AWordWrap: Boolean);
    function GetMemo: TObject;
    function GetCaretPos: TPoint;
    function GetLines: TStrings;
    procedure SetParent(AParent: TWinControl);
    procedure SetAlign(AAlign: TAlign);
    procedure SetTabOrder(ATabOrder: Integer);
    procedure SetOnKeyDown(AOnKeyDown: TKeyEvent);
    procedure SetOnDragOver(AOnDragOver: TDragOverEvent);
    procedure SetOnDragDrop(AOnDragDrop: TDragDropEvent);
    function GetFont: TFont;
    procedure SetScrollBars(AScrollBars: TScrollStyle); // Only Old
  public
    procedure ReCreatedMemo(AOwner: TControl; AUseSmartMemo: Boolean);
    procedure InitializeFrom(AMemoView: TfrxCustomMemoView);
    procedure SelectAll;
    procedure SetFocus;
{$IFNDEF FPC}
    function Perform(Msg: Cardinal; WParam: WPARAM; LParam: LPARAM): LRESULT; // Only Old
{$ENDIF}

    property DPIScale: Single write SetDPIScale; // Only New
    property UseSmartMemo: Boolean read FUseSmartMemo;
    property Text: String read GetText write SetText;
    property SelText: String write SetSelText;
    property WordWrap: Boolean read GetWordWrap write SetWordWrap;
    property Memo: TObject read GetMemo;
    property CaretPos: TPoint read GetCaretPos;
    property Lines: TStrings read GetLines;
    property Parent: TWinControl write SetParent;
    property Align: TAlign write SetAlign;
    property TabOrder: Integer write SetTabOrder;
    property OnKeyDown: TKeyEvent write SetOnKeyDown;
    property OnDragOver: TDragOverEvent write SetOnDragOver;
    property OnDragDrop: TDragDropEvent write SetOnDragDrop;
    property Font: TFont read GetFont;
    property ScrollBars: TScrollStyle write SetScrollBars; // Only Old
  end;

  TfrxMemoEditorForm = class(TfrxBaseFormEditorDataTree)
    PageControl1: TPageControl;
    TextTS: TTabSheet;
    FormatTS: TTabSheet;
    HighlightTS: TTabSheet;
    ToolBar: TToolBar;
    ExprB: TToolButton;
    AggregateB: TToolButton;
    WordWrapB: TToolButton;
    UseSmartMemoB: TToolButton;
    OkB: TButton;
    CancelB: TButton;
    procedure FormShow(Sender: TObject);
    procedure WordWrapBClick(Sender: TObject);
    procedure UseSmartMemoBClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure MemoKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ExprBClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure AggregateBClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PageControl1Change(Sender: TObject);
    procedure ExprMemoDragDrop(Sender, Source: TObject; X, Y: Integer);
  private
    FFormat: TfrxFormatEditorForm;
    FHighlight: TfrxHighlightEditorForm;
    FMemoView: TfrxCustomMemoView;
    FUseSmartMemo: Boolean;
    FText: WideString;
    FMemoController: TfrxMemoController;
    FDPIScale: Single;
  protected
    procedure LoadFormPreferences(PreferencesStorage: TObject; DefPreferencesStorage: TObject); override;
    procedure SaveFormPreferences(PreferencesStorage: TObject; DefPreferencesStorage: TObject); override;
    procedure OnDataTreeDblClick(Sender: TObject); override;
  public
    procedure UpdateResouces; override;
    procedure UpdateFormPPI(aNewPPI: Integer); override;
    property MemoView: TfrxCustomMemoView read FMemoView write FMemoView;
    property Text: WideString read FText write FText;
    property MemoController: TfrxMemoController read FMemoController;
  end;


implementation
end.

