
{******************************************}
{                                          }
{             FastReport VCL               }
{               SQL editor                 }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxEditSQL;

interface

{$I frx.inc}

uses
  SysUtils,
  {$IFNDEF FPC}
  Windows, Messages,
  {$ENDIF}
  Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls, ToolWin, frTypes,
  frBaseMemo, frSynMemo, frxSimpleMemo, frxCustomDB, frxBaseForm,
  frBaseGraphicsTypes
  {$IFDEF FPC}
  , LResources, LCLType
  {$ENDIF}
  {$IFDEF DELPHI6}
  , Variants
  {$ENDIF}
;


type
  TfrxSQLMemoController = class
  private
    FUseSmartMemo: Boolean;
    FBaseMemo: TfrBaseSyntaxMemo;

    function OldMemo: TfrxSyntaxMemo;
    function NewMemo: TfrBaseSmartSyntaxMemo;

    procedure SetDPIScale(AValue: Single); // Only New
    function GetSyntax: String; // Only Old
    procedure SetSyntax(const Value: String); // Only Old
    function GetHightlightingType: TfrxHightlightingsTypes; // Only New
    procedure SetHightlightingType(const Value: TfrxHightlightingsTypes); // Only New
  public
    procedure ReCreatedMemo(AOwner: TControl; AUseSmartMemo: Boolean);
    property Memo: TfrBaseSyntaxMemo read FBaseMemo;
    property DPIScale: Single write SetDPIScale; // Only New
    property HightlightingType: TfrxHightlightingsTypes read GetHightlightingType
      write SetHightlightingType;
    property Syntax: String read GetSyntax write SetSyntax;
  end;

  { TfrxSQLEditorForm }

  TfrxSQLEditorForm = class(TfrxBaseLoadSavePrefForm)
    ToolBar: TToolBar;
    OkB: TToolButton;
    CancelB: TToolButton;
    QBB: TToolButton;
    ParamsB: TToolButton;
    ToolButton2: TToolButton;
    CBPanel: TPanel;
    CBDialect: TComboBox;
    TBEditDialect: TToolButton;
    ToolButton3: TToolButton;
    LDialect: TLabel;
    TBSaveDialect: TToolButton;
    TBLoadDialect: TToolButton;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    UseSmartMemoB: TToolButton;
    procedure OkBClick(Sender: TObject);
    procedure CancelBClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure UseSmartMemoBClick(Sender: TObject);
    procedure MemoKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormHide(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure QBBClick(Sender: TObject);
    procedure ParamsBClick(Sender: TObject);
    procedure TBEditDialectClick(Sender: TObject);
    procedure CBDialectChange(Sender: TObject);
    procedure TBSaveDialectClick(Sender: TObject);
    procedure TBLoadDialectClick(Sender: TObject);
  private
    { Private declarations }
    FMemoController: TfrxSQLMemoController;
    FQuery: TfrxCustomQuery;
    FSaveSQL: TStrings;
    FSaveSchema: String;
    FSaveParams: TfrxParams;
    FUseSmartMemo: Boolean;
    FDPIScale: Single;
    function GetQueryAssistant: TfrQueryAssistant;
    procedure SetQuery(const Value: TfrxCustomQuery);
    procedure UpdateDialectsCB;
  protected
    procedure LoadFormPreferences(PreferencesStorage: TObject; DefPreferencesStorage: TObject); override;
    procedure SaveFormPreferences(PreferencesStorage: TObject; DefPreferencesStorage: TObject); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure UpdateResouces; override;
    procedure UpdateFormPPI(aNewPPI: Integer); override;

    property Query: TfrxCustomQuery read FQuery write SetQuery;
    property QueryAssistant: TfrQueryAssistant read GetQueryAssistant;
  end;


implementation
end.

