{******************************************}
{                                          }
{             FastReport VCL               }
{              Report engine               }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxEngine;

interface

{$I frx.inc}

uses
  SysUtils, {$IFNDEF FPC}Windows, Messages,{$ENDIF}
  Types, Classes, Graphics, Controls, Forms, Dialogs,
  frxClass, frxAggregate, frXML, frxDMPClass, frxStorage,
  frxVariables, Variants;

type
  { TfrxHeaderList holds a set of bands that should appear on each new page.
    This includes page header, column header and header bands with
    "Reprint on new page" setting }

  TfrxHeaderListItem = class(TObject)
  public
    Band: TfrxBand;
    Left: Extended;
    IsInKeepList: Boolean;
  end;

  TfrxShiftEngine = class(TObject)
  private
    FContainers: TList;
    FDestroyQueue: TList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure ClearDestroyQueue;
    procedure ClearContainer(Container: TfrxReportComponent);
    procedure PrepareShiftTree(Container: TfrxReportComponent);
    procedure ShiftObjects(Container: TfrxReportComponent);
    procedure InitShiftAmount(AObject: TfrxReportComponent; ShiftAmount: Extended);
    function ReShiftNodeUP(AContainer: TfrxReportComponent; AObject: TfrxReportComponent; ANewHeight: Extended): TfrxReportComponent;
    procedure ContainerToDestroyQueue(AContainer: TObject);
    procedure RestoreState(AContainer: TfrxReportComponent);
  end;

  TfrxHeaderList = class(TObject)
  private
    FList: TList;
    function GetCount: Integer;
    function GetItems(Index: Integer): TfrxHeaderListItem;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure AddItem(ABand: TfrxBand; ALeft: Extended; AInKeepList: Boolean);
    procedure RemoveItem(ABand: TfrxBand);
    property Count: Integer read GetCount;
    property Items[Index: Integer]: TfrxHeaderListItem read GetItems; default;
  end;

  TfrxEngine = class(TfrxCustomEngine)
  private
    FAggregates: TfrxAggregateList;
    FCallFromAddPage: Boolean;
    FCallFromEndPage: Boolean;
    FCurBand: TfrxBand;
    FLastBandOnPage: TfrxBand;
    FDontShowHeaders: Boolean;
    FHeaderList: TfrxHeaderList; { list of header bands }
    FFirstReportPage: Boolean;   { needed for correct setting of PreviewPages.FirstPage }
    FFirstColumnY: Extended;     { position of the first column }
    FIsFirstBand: Boolean;       { needed for KeepTogether }
    FIsFirstPage: Boolean;       { first and last page flags }
    FIsLastPage: Boolean;        {                           }
    FTitlePrinted: Boolean;
    FHBandNamesTree: TStrings;   { need for correct work with drill down in master-detail-subtetail-... }
    FKeepBand: TfrxBand;
    FKeepFooter: Boolean;
    FKeeping: Boolean;
    FKeepHeader: Boolean;
    FKeepCurY: Extended;         { need when group doesn't fit on the whole page}
    FPrevFooterHeight: Extended; {need for correct freespace calculation when use printOnPreviousPage}
    FKeepOutline: TfrXMLItem;
    FKeepPosition: Integer;
    FKeepAnchor: Integer;
    FCallFromPHeader: Boolean;   { endless loop fix }
    FOutputTo: TfrxNullBand;     { used in the subreports }
    FPage: TfrxReportPage;       { currently proceeded page }
    FPageCurX: Extended;
    FStartNewPageBand: TfrxBand; { needed in addpage }
    FVHeaderList: TList;         { list of vheader bands }
    FVMasterBand: TfrxBand;      { master hband for vbands }
    FVPageList: TList;           { list of page breaks for vbands }

    { need for subreports and keep }
    FSubSavePageN: Integer;
    FSubSaveCurY: Extended;
    { Bands sequance started with mcmTillPageEnds doesn't fit on the page }
    { and we need to brake bands output }
    FBreakShowBandTree: Boolean;
    { in case of mcmTillPageEnds footers printed before addition sequence }
    { so we need to reset Aggregates for footer anly after this sequence }
    FLockResetAggregates: Boolean;
    FShiftEngine: TfrxShiftEngine;
    procedure AddBandOutline(Band: TfrxBand);
    procedure AddColumn;
    procedure AddPage;
    procedure AddPageOutline;
    procedure AddToHeaderList(Band: TfrxBand);
    procedure AddToVHeaderList(Band: TfrxBand);
    procedure CheckBandColumns(Band: TfrxDataBand; ColumnKeepPos: Integer; var HeaderKeepPos: Integer;
      SaveCurY, SaveHeaderY: Extended);
    procedure CheckDrill(Master: TfrxDataBand; Band: TfrxGroupHeader);
    procedure CheckGroups(Master: TfrxDataBand; Band: TfrxGroupHeader;
      ColumnKeepPos: Integer; SaveCurY: Extended);
    procedure CheckSubReports(Band: TfrxBand);
    procedure CheckSuppress(Band: TfrxBand);
    procedure DoShow(Band: TfrxBand);
    procedure DrawSplit(Band: TfrxBand);
    procedure EndColumn;
    procedure EndKeep(Band: TfrxBand);
    procedure InitGroups(Master: TfrxDataBand; Band: TfrxGroupHeader;
      Index: Integer; ResetLineN: Boolean = False);
    procedure InitPage;
    procedure NotifyObjects(Band: TfrxBand);
    procedure OutlineRoot;
    procedure OutlineUp(Band: TfrxBand);
    procedure PreparePage(ErrorList: TStrings; PrepareVBands: Boolean);
    procedure RemoveFromHeaderList(Band: TfrxBand);
    procedure RemoveFromVHeaderList(Band: TfrxBand);
    procedure ResetSuppressValues(Band: TfrxBand);
    procedure RunPage(Page: TfrxReportPage);
    procedure RunReportPages(APage: TfrxReportPage);
    procedure RunSubreport(ASubreport: TfrxSubreport);
    procedure ShowGroupFooters(Band: TfrxGroupHeader; Index: Integer; Master: TfrxDataBand);
    procedure ShowVBands(HBand: TfrxBand);
    procedure StartKeep(Band: TfrxBand; Position: Integer = 0);
    function CanShow(Obj: TObject; PrintIfDetailEmpty: Boolean): Boolean;
    function FindBand(Band: TfrxBandClass): TfrxBand;
    function RunDialogs: Boolean;
    procedure RestoreVBandsObjects;
    function IsMaxHeightMode(View: TfrxView): Boolean;
  protected
    function GetPageHeight: Extended; override;
    procedure DoProcessState(aBand: TfrxBand; aState: TfrxProcessAtMode);
    function GetAggregates: TfrxCustomAggregateList; override;
    function GetCurColumnPos: Extended; override;
    function GetCurColumnWidth: Extended; override;
  public
    constructor Create(AReport: TfrxReport); override;
    destructor Destroy; override;
    procedure EndPage; override;
    procedure NewColumn; override;
    procedure NewPage; override;
    function IsMultiColumnPage: Boolean; override;
    procedure InitializeSplit(Container: TfrxReportComponent; SavedObjects: TList; SplitObject: TList); override;
    function SplitObjects(Container: TfrxReportComponent; SavedObjects: TList; SplitObject: TList; AHeight: Extended): Boolean; override;
    procedure FinalizeSplit(Container: TfrxReportComponent; SavedObjects: TList; SplitObject: TList; SplitHeight: Extended); override;
    function Run(ARunDialogs: Boolean; AClearLast: Boolean = False; APage: TfrxPage = nil): Boolean; override;
    function ShowBand(Band: TfrxBand): TfrxBand; overload; override;
    procedure ShowBand(Band: TfrxBandClass); overload; override;
    procedure Stretch(Container: TfrxReportComponent; SkipGetData: Boolean = False); override;
    procedure UnStretch(Container: TfrxReportComponent); override;
    function HeaderHeight(AddReprintOnNewPage: Boolean = False): Extended; override;
    function FooterHeight: Extended; override;
    function FreeSpace: Extended; override;
    procedure BreakAllKeep; override;   { used in crosstab }
    procedure PrepareShiftTree(Container: TfrxReportComponent); override;
    procedure ProcessObject(ReportObject: TfrxView); override;
    function GetAggregateValue(const Name, Expression: String;
      DataRow: TfrxComponent; Flags: Integer): Variant; override;
    function GetCurrentPage: TfrxPage; override;
    function Initialize: Boolean;
    procedure Finalize;
  end;


implementation
end.
