
{******************************************}
{                                          }
{             FastReport VCL               }
{            Gauge View Object             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxGaugeView;

interface

{$I frx.inc}

uses
  Types, {$IFNDEF FPC}Windows,{$ENDIF} Graphics, Classes, Controls,
  {$IFDEF FPC}LCLType, LCLIntf, LCLProc,{$ENDIF}
  frxClass, frxGauge;

type
{$IFDEF DELPHI16}
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxGaugeObject = class(TComponent)  // fake component
  end;
  
  TfrxBaseGaugeView = class(TfrxStyledView)
  private
    FBaseGauge: TfrxBaseGauge;
  protected
    function InnerPoint(X, Y: Integer): TPoint;
    procedure CreateGauge; virtual; abstract;
    procedure ContentChanged(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    procedure UpdateInspector;

    function DoMouseDown(X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams): Boolean; override;
    procedure DoMouseMove(X, Y: Integer;
      Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams); override;
    procedure DoMouseUp(X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams); override;
    function DoMouseWheel(Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var EventParams: TfrxInteractiveEventsParams): Boolean; override;

    property BaseGauge: TfrxBaseGauge read FBaseGauge;
  published
    property Editable default [ferAllowInPreview];
    property FillType;
    property Fill;
    property Frame;
    property Cursor;
    property Style;
    property TagStr;
  end;

  {$IFDEF DELPHI16}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
  {$ENDIF}
  TfrxGaugeView = class(TfrxBaseGaugeView)
  private
    FGauge: TfrxGauge;
    FExpression: String;
    FMacroLoaded: Boolean;
    procedure SetGauge(const Value: TfrxGauge);
  protected
    procedure CreateGauge; override;
  public
    procedure GetData; override;
    procedure SaveContentToDictionary(aReport: TfrxReport; PostProcessor: TfrxPostProcessor); override;
    function LoadContentFromDictionary(aReport: TfrxReport; aItem: TfrxMacrosItem): Boolean; override;
    procedure ProcessDictionary(aItem: TfrxMacrosItem; aReport: TfrxReport; PostProcessor: TfrxPostProcessor); override;
  published
    property Expression: String read FExpression write FExpression;
    property Gauge: TfrxGauge read FGauge write SetGauge;
    property Processing;
  end;

  {$IFDEF DELPHI16}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
  {$ENDIF}
  TfrxIntervalGaugeView = class(TfrxBaseGaugeView)
  private
    FIntervalGauge: TfrxIntervalGauge;
    FStartExpression: String;
    FEndExpression: String;
    procedure SetIntervalGauge(const Value: TfrxIntervalGauge);
  protected
    procedure CreateGauge; override;
  public
    procedure GetData; override;
  published
    property StartExpression: String read FStartExpression write FStartExpression;
    property EndExpression: String read FEndExpression write FEndExpression;
    property IntervalGauge: TfrxIntervalGauge read FIntervalGauge write SetIntervalGauge;
  end;


implementation
end.
