
{******************************************}
{                                          }
{             FastReport VCL               }
{             Gradient object              }
{                                          }
{  (former RoundRect plus Add-in object)   }
{     (C) Guilbaud Olivier for FR 2.4      }
{        mailto: golivier@free.fr          }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxGradient;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}
  Windows, Messages,
  {$ENDIF}
  SysUtils, Classes, Graphics, frCore, frxClass
  {$IFDEF FPC}
  ,LCLType, LCLProc, LCLIntf,LazarusPackageIntf
  {$ENDIF}
{$IFDEF DELPHI6}
, Variants
{$ENDIF}
{$IFDEF DELPHI16}
 , Vcl.Controls
{$ENDIF};

type
{$IFDEF DELPHI16}
/// <summary>
///   The TfrxGradientObject allows the use of the Gradient component in your
///   report. TfrxGradientObject is an empty component. It is used to add the
///   frxGradient.pas file to the "uses" list. The main component is
///   TfrxGradientView.
/// </summary>
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxGradientObject = class(TComponent);  // fake component

  /// <summary>
  ///   The TfrxGradientView component represents a gradient fill. Deprecated,
  ///   use Object.Fill instead of TfrxGradientView.
  /// </summary>
  TfrxGradientView = class(TfrxView)
  private
    FBeginColor: TColor;
    FEndColor: TColor;
    FStyle: TfrxGradientStyle;
    procedure DrawGradient(X, Y, X1, Y1: Integer);
    function GetColor: TColor;
    procedure SetColor(const Value: TColor);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    class function GetDescription: String; override;
  published
    /// <summary>
    ///   Begin color of the gradient.
    /// </summary>
    property BeginColor: TColor read FBeginColor write FBeginColor default clWhite;
    /// <summary>
    ///   End color of the gradient.
    /// </summary>
    property EndColor: TColor read FEndColor write FEndColor default clGray;
    /// <summary>
    ///   Style of the gradient.
    /// </summary>
    property Style: TfrxGradientStyle read FStyle write FStyle;
    property Frame;
    property Color: TColor read GetColor write SetColor;
  end;


  {$IFDEF FPC}
 // procedure Register;
  {$ENDIF}

implementation
end.
