{******************************************}
{                                          }
{             FastReport VCL               }
{    Basic visual controls for Editors     }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxGraphicControls;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}Windows, Messages,{$ENDIF}
  {$IFDEF FPC} LCLProc, LMessages,{$ENDIF}
  SysUtils, Variants, Classes, Graphics;

type
  TfrxSwithcButtonStyle = (sbOn, sbOff);
  TfrxSwithcButton = Class
  private
    FSwitch: Boolean;
    FWidth: Integer;
    FHeight: Integer;
    FOriginWidth: Integer;
    FOriginHeight: Integer;
    FButtonColor: TColor;
    FFrameColor: TColor;
    FFrameWidth: Integer;
    FFillActivateColor: TColor;
    FFillDeactivateColor: TColor;
    FBitmap: TBitmap;
    FOnStyleBitmap: TBitmap;
    FOffStyleBitmap: TBitmap;
    FBackColor: TColor;
    FNeedUpdate: Boolean;
    FTag: Integer;
    FColorTag: TColor;
    procedure SetOriginHeight(const Value: Integer);
    procedure SetSwitch(const Value: Boolean);
    procedure SetBackColor(const Value: TColor);
    procedure SetFillActivateColor(const Value: TColor);
    procedure SetFillDeactivateColor(const Value: TColor);
    procedure SetFrameColor(const Value: TColor);
  public
    constructor Create; overload;
    constructor Create(aOnStyleBitmap: TBitmap; aOffStyleBitmap: TBitmap); overload;
    destructor Destroy; override;
    procedure Draw(aCanvas: TCanvas; aLeft, aTop: Integer);
    property Switch: Boolean read FSwitch write SetSwitch;
    property ButtonColor: TColor read FButtonColor write FButtonColor;
    property BackColor: TColor read FBackColor write SetBackColor;
    property FrameColor: TColor read FFrameColor write SetFrameColor;
    property FillActivateColor: TColor read FFillActivateColor write SetFillActivateColor;
    property FillDeactivateColor: TColor read FFillDeactivateColor write SetFillDeactivateColor;
    property Width: Integer read FOriginWidth;
    property Height: Integer read FOriginHeight write SetOriginHeight;
    property Tag: Integer read FTag write FTag;
    property ColorTag: TColor read FColorTag write FColorTag;
  end;


  TfrxSwitchButtonsPanel = class
  private
    FButtons: TStringList;
    FSwitchOnStyle: TBitmap;
    FSwitchOffStyle: TBitmap;
    FShowCaption: Boolean;
    FShowColors: Boolean;
    FButtonOffsetY: Integer;
    FButtonOffsetX: Integer;
    FColorRectWidth: Integer;
    FColorRectGap: Integer;
    FOnButtonClick: TNotifyEvent;
    FButtonsHeight: Integer;
    FTextWidth: Integer;
    FBackColor: TColor;
    FFont: TFont;
    function GetButton(Index: Integer): TfrxSwithcButton;
    procedure SetShowCaption(const Value: Boolean);
    procedure SetShowColors(const Value: Boolean);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Draw(aCanvas: TCanvas; aLeft, aTop: Integer);
    function CalcHeight: Integer;
    function CalcWidth: Integer;
    function AddButton(sCaption: String): TfrxSwithcButton;
    function IsButtonClicked(X, Y: Integer): TfrxSwithcButton;
    function DoClick(X, Y: Integer): TfrxSwithcButton;
    procedure Clear;
    function Count: Integer;
    procedure SetButtonsHeight(Height: Integer);
    property ShowCaption: Boolean read FShowCaption write SetShowCaption;
    property ShowColors: Boolean read FShowColors write SetShowColors;
    property Button[Index: Integer]: TfrxSwithcButton read GetButton; default;
    property OnButtonClick: TNotifyEvent read FOnButtonClick write FOnButtonClick;
  end;


implementation
end.
