{
HtmlViewer Version 11.9
Copyright (c) 2010-2018 by Bernd Gabriel

This source module is based on code of CodeChangerDecode.pas written by SchwarzKopf-M (SchwarzKopf-M@yandex.ru)
and the source code library libiconv, which is published under the GNU Library General Public License.

Thanks to SchwarzKopf-M for his extensive work.

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Note that the source modules HTMLGIF1.PAS and DITHERUNIT.PAS
are covered by separate copyright notices located in those modules.
}

//This is the original copyright notice in CodeChangerDecode.pas

//This is charset Windows-1251:
//        " ".
//       
//     .
//     - SchwarzKopf-M (SchwarzKopf-M@yandex.ru).
//       
//      .
//      
//      libiconv.
//
//  HTMLViewer (http://code.google.com/p/thtmlviewer)

// Translated by "http://translate.google.de/#ru|en|" it is:
// Text conversion module is freeware and is distributed "as is".
// If you modify or partial use of source code links
// To the original module is required.
// Copyright reserve the - SchwarzKopf-M (SchwarzKopf-M@yandex.ru).
// This module uses the information found on the Internet
// Without affecting the original copyright holder.
// For the conversion of some Asian languages ??used in freestyle
// Interpretation of the source code library libiconv.
//
// Especially for HTMLViewer (http://code.google.com/p/thtmlviewer)

{$I frxHTMLCons.inc}

unit frxHTMLBuffConv;

interface

uses
{$ifdef MSWindows}
  Windows,
{$endif}
{$ifdef LCL}
  LclIntf, LclType,
{$endif}
  SysUtils,
  frxHTMLBuffConvArrays,
  frxHTMLGlobals,
  frxHTMLBuffer;

type

  // BG, 06.10.2012: converts UTF7 to unicode.
  TBuffConvUTF7 = class(TBuffConverter)
  private
    State: Byte;
  protected
    procedure Assign(Source: TBuffConverter); override;
  public
    function NextChar: TBuffChar; override;
  end;

  // BG, 26.09.2012: converts UTF8 to unicode.
  TBuffConvUTF8 = class(TBuffConverter)
  public
    function NextChar: TBuffChar; override;
  end;

  // BG, 28.01.2013: converts several single byte code pages using an AnsiCharMap to unicode.
  TBuffConvSingleByteMap = class(TBuffConverter)
  private
    FMap: ThtAnsiCharMap;
    procedure UpdateMap;
  protected
    procedure Assign(Source: TBuffConverter); override;
  public
    procedure AfterConstruction; override;
    function NextChar: TBuffChar; override;
  end;

  // BG, 02.10.2012: converts several single byte code pages using an AnsiCharHighMap to unicode.
  TBuffConvHighMap = class(TBuffConverter)
  private
    FMap: ThtAnsiCharHighMap;
    procedure UpdateMap;
  protected
    procedure Assign(Source: TBuffConverter); override;
  public
    procedure AfterConstruction; override;
    function NextChar: TBuffChar; override;
  end;

  // BG, 02.10.2012: converts the single byte code of ISO 8859 using an AnsiCharIsoMap to unicode.
  TBuffConvIsoMap = class(TBuffConverter)
  private
    FMap: ThtAnsiCharISOMap;
    procedure UpdateMap;
  protected
    procedure Assign(Source: TBuffConverter); override;
  public
    procedure AfterConstruction; override;
    function NextChar: TBuffChar; override;
  end;

  TBuffBaseConverter = class(TBuffConverter)
  protected
    function ASCIIDecodeChar(c1: Byte): TBuffChar;
    function CP922EstonianDecodeChar(c1: Byte): TBuffChar;
    function GB2312DecodeChar(c1, c2: Byte): TBuffChar;
    function GBKDecodeChar(c1, c2: Byte): TBuffChar;
    function ISO8859_11DecodeChar(c1: Byte): TBuffChar;
    function ISO8859_15DecodeChar(c1: Byte): TBuffChar;
    function ISO8859_5DecodeChar(c1: Byte): TBuffChar;
    function ISO8859_6DecodeChar(c1: Byte): TBuffChar;
    function ISO8859_7DecodeChar(c1: Byte): TBuffChar;
    function ISO8859_8DecodeChar(c1: Byte): TBuffChar;
    function ISO8859_9DecodeChar(c1: Byte): TBuffChar;
    function JIS_X0201DecodeChar(c1: Byte): TBuffChar;
    function JIS_X0208DecodeChar(c1, c2: Byte): TBuffChar;
    function JIS_X0212DecodeChar(c1, c2: Byte): TBuffChar;
    function KSC5601DecodeChar(c1, c2: Byte): TBuffChar;
    function Latin1EuroDecodeChar(c1: Byte): TBuffChar;
    function MultiByteToWideCharDecodeChar(c1: Byte): TBuffChar;
  end;

  // BG, 02.10.2012: converts several single byte code pages using one of the *DecoderChar functions to unicode.
  TBuffSingleByteDecoder = function(c1: Byte): TBuffChar of object;
  TBuffConvSingleByte = class(TBuffBaseConverter)
  private
    FDecoder: TBuffSingleByteDecoder;
    procedure UpdateDecoder;
  protected
    procedure Assign(Source: TBuffConverter); override;
  public
    procedure AfterConstruction; override;
    function NextChar: TBuffChar; override;
  end;

  // BG, 02.10.2012: converts several single byte code pages using one of the *DecoderChar functions to unicode.
  TBuffDoubleByteDecoder = function(c1, c2: Byte): TBuffChar of object;
  TBuffConvDoubleByte = class(TBuffBaseConverter)
  private
    FDecoder: TBuffDoubleByteDecoder;
    procedure UpdateDecoder;
  protected
    procedure Assign(Source: TBuffConverter); override;
  public
    procedure AfterConstruction; override;
    function NextChar: TBuffChar; override;
  end;

  // BG, 26.09.2012: converts shift-jis (codepages 932, 943) to unicode.
  TBuffConvShiftJis932 = class(TBuffConverter)
  public
    function NextChar: TBuffChar; override;
  end;

  // BG, 03.10.2012: converts simplified Chinese (PRC, Singapore) / Chinese simplified (GB2312) to unicode.
  TBuffConvIBM936 = class(TBuffBaseConverter)
  public
    function NextChar: TBuffChar; override;
  end;

  // BG, 04.10.2012: converts hangeul (codepage 949 to unicode.
  TBuffConvHangeul949 = class(TBuffBaseConverter)
  public
    function NextChar: TBuffChar; override;
  end;

  // BG, 04.10.2012: converts Traditional Chinese (code page 950) to unicode.
  TBuffConvBig5CP950 = class(TBuffConverter)
  public
    function NextChar: TBuffChar; override;
  end;

  TBuffConvGB18030 = class(TBuffBaseConverter)
  public
    function NextChar: TBuffChar; override;
  end;

  TBuffConvEUC_JP = class(TBuffBaseConverter)
  public
    function NextChar: TBuffChar; override;
  end;

  TBuffConvISO2022CN = class(TBuffBaseConverter)
  private
    CNState1: (CNSTATE_ASCII, CNSTATE_TWOBYTE);
    CNState2: (CNSTATE2_NONE, STATE2_DESIGNATED_GB2312, STATE2_DESIGNATED_CNS11643_1);
    CNState3: (CNSTATE3_NONE, STATE3_DESIGNATED_CNS11643_2);
    function CNS11643_1DecodeChar(c1, c2: Word): TBuffChar;
    function CNS11643_2DecodeChar(c1, c2: Word): TBuffChar;
  protected
    procedure Assign(Source: TBuffConverter); override;
  public
    function NextChar: TBuffChar; override;
  end;

  // BG, 27.09.2012: converts stateful ISO-2022-JP, -1, -2 (codepage 50220) to unicode.
  TBuffConvISO2022JP = class(TBuffBaseConverter)
  private
    JPState: (
      JPS_ASCII,
      JPS_JISX0201ROMAN,
      JPS_JISX0208,
      JPS_JISX0212,
      JPS_GB2312,
      JPS_KSC5601,
      JPS_8859_1,
      JPS_8859_7);
  protected
    procedure Assign(Source: TBuffConverter); override;
  public
    function NextChar: TBuffChar; override;
  end;

  TBuffConvISO2022KR = class(TBuffBaseConverter)
  private
    KRState1: (KRSTATE_ASCII, KRSTATE_TWOBYTE);
    KRState2: (KRSTATE2_NONE, STATE2_DESIGNATED_KSC5601);
  protected
    procedure Assign(Source: TBuffConverter); override;
  public
    function NextChar: TBuffChar; override;
  end;

implementation
end.
