{
Version   11.7
Copyright (c) 1995-2008 by L. David Baldwin
Copyright (c) 2008-2016 by HtmlViewer Team

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Note that the source modules HTMLGIF1.PAS and DITHERUNIT.PAS
are covered by separate copyright notices located in those modules.
}

{$I frxHTMLCons.inc}

unit frxHTMLStylePars;

interface

uses
{$ifdef VCL}
  Windows,  // needed to expand inline function htUpCase
{$endif}
{$ifdef LCL}
  variants,
{$endif}
  Classes, Graphics, SysUtils,
  frxHTMLGlobals, frxHTMLBuffer, frxHTMLURLSubs, frxHTMLStyleUn, frxHTMLStyleTypes;


{---------  Detect Shorthand syntax }
type
  EParseError = class(Exception);

  ThtTermCharKind = (tckNone, tckGood, tckBad);

  { THtmlStyleParser }

  THtmlStyleParser = class
  private
    Doc: TBuffer;
    LCh: ThtChar;
    LinkPath: ThtString;
    // parser methods
    procedure GetCh(WantLfChar: Boolean = False);
    procedure SkipWhiteSpace(WantLfChar: Boolean = False);
    function GetIdentifier(out Identifier: ThtString): Boolean;
    function GetString(out Str: ThtString): Boolean;
    //
    function AddPath(const S: ThtString): ThtString;
    procedure ProcessShortHand(Index: TShortHand; const Prop, OrigValue, StrippedValue: ThtString; IsImportant: Boolean);
  protected

    procedure GetCollection(const GoodTermChars, BadTermChars: ThtString);
    procedure ProcessProperty(const Prop, Value: ThtString; IsImportant: Boolean); virtual; abstract;
    procedure ProcessPropertyOrShortHand(Prop, Value: ThtString; IsImportant: Boolean);
  end;

  THtmlStyleTagParser = class(THtmlStyleParser)
  private
    FOnMatchMediaQuery: ThtMediaQueryEvent;
    Selectors: ThtStringList;
    Styles: TfrxHtStyleList;
    procedure GetCollection;
    procedure GetSelectors;
  protected
    procedure ProcessProperty(const Prop, Value: ThtString; IsImportant: Boolean); override;
  public
    constructor Create;
    destructor Destroy; override;
    procedure DoStyle(Styles: TfrxHtStyleList; var C: ThtChar; Doc: TBuffer; const APath, AMedia: ThtString; FromLink: boolean);
    property OnMatchMediaQuery: ThtMediaQueryEvent read FOnMatchMediaQuery write FOnMatchMediaQuery;
  end;

  THtmlStyleAttrParser = class(THtmlStyleParser)
  private
    Propty: TfrxHTProperties;
  protected
    procedure ProcessProperty(const Prop, Value: ThtString; IsImportant: Boolean); override;
  public
    procedure ParseProperties(Doc: TBuffer; Propty: TfrxHTProperties);
  end;

procedure ParsePropertyStr(const PropertyStr: ThtString; Propty: TfrxHTProperties);
function SortContextualItems(const S: ThtString): ThtString;

implementation
end.
