{
Version   11.7
Copyright (c) 1995-2008 by L. David Baldwin
Copyright (c) 2008-2010 by HtmlViewer Team
Copyright (c) 2011-2016 by Bernd Gabriel

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Note that the source modules HTMLGIF1.PAS and DITHERUNIT.PAS
are covered by separate copyright notices located in those modules.
}

{$I frxHTMLCons.inc}

unit frxHTMLStyleTypes;

interface

uses
{$ifdef MSWindows}
  Windows,
{$else}
  Types,
{$endif}
{$ifdef LCL}
  LclIntf, LclType,
{$endif}
  SysUtils, Math, Forms, Variants, Graphics,
  //
  frxHTMLGlobals;

const
  varInt = [varInteger, varByte, varSmallInt, varShortInt, varWord, varLongWord];
  varFloat = [varSingle, varDouble, varCurrency];
  varNum = varInt + varFloat;

type
  ThtAlignmentStyle = (
    aNone,
    aTop,
    aMiddle,
    aBaseline,
    aBottom,
    aLeft,
    aRight,
    aJustify,
    aSub,
    aSuper);

const
  CAlignmentStyle: array[ThtAlignmentStyle] of ThtString = (
    'none',
    'top',
    'middle',
    'baseline',
    'bottom',
    'left',
    'right',
    'justify',
    'sub',
    'super');

type
  ThtBoxSizing = (
    ContentBox,
    BorderBox);

const
  CBoxSizing: array[ThtBoxSizing] of ThtString = (
    'content-box',
    'border-box');

type
  ThtBackgroundPosition = (
    bpTop,
    bpCenter,
    bpBottom,
    bpLeft,
    bpRight,
    bpPercent,
    bpDim);

  PositionRec = record
    PosType: ThtBackgroundPosition;
    Value: Integer;
    RepeatD: Boolean;
    Fixed: Boolean;
  end;
  PPositionRec = ^PositionRec;
  PtPositionRec = record
    X, Y: PositionRec;
  end;

type
  ThtBoxFloatStyle = (
    flNone,
    flLeft,
    flRight,
    flTop,
    flBottom);
const
  CBoxFloatStyle: array[ThtBoxFloatStyle] of ThtString = (
    'none',
    'left',
    'right',
    'top',
    'bottom');

type
  ThtBoxPositionStyle = (
    posStatic,
    posRelative,
    posAbsolute,
    posFixed);
const
  CBoxPositionStyle: array[ThtBoxPositionStyle] of ThtString = (
    'static',
    'relative',
    'absolute',
    'fixed');

type
  ThtBorderStyle = (
    bssNone,
    bssSolid,
    bssInset,
    bssOutset,
    bssGroove,
    bssRidge,
    bssDashed,
    bssDotted,
    bssDouble);
const
  CBorderStyle: array[ThtBorderStyle] of ThtString = (
    'none',
    'solid',
    'inset',
    'outset',
    'groove',
    'ridge',
    'dashed',
    'dotted',
    'double');

type
  ThtBulletStyle = (
    lbBlank,
    lbCircle,
    lbDecimal,
    lbDisc,
    lbLowerAlpha,
    lbLowerRoman,
    lbNone,
    lbSquare,
    lbUpperAlpha,
    lbUpperRoman);
const
  CBulletStyle: array[ThtBulletStyle] of ThtString = (
    'blank',
    'circle',
    'decimal',
    'disc',
    'lower-alpha',
    'lower-roman',
    'none',
    'square',
    'upper-alpha',
    'upper-roman');

type
  ThtClearStyle = (
    clrNone,
    clrLeft,
    clrRight,
    clrBoth);
const
  CClearStyle: array[ThtClearStyle] of ThtString = (
    'none',
    'left',
    'right',
    'both');

type
  ThtDisplayStyle = (
    pdUnassigned,
    pdInline,
    pdBlock,
    pdInlineBlock,
    pdListItem,
    pdRunIn,
    pdCompact,
    pdTable,
    pdInlineTable,
    pdTableRowGroup,
    pdTableHeaderGroup,
    pdTableFooterGroup,
    pdTableRow,
    pdTableColumnGroup,
    pdTableColumn,
    pdTableCell,
    pdTableCaption,
    pdNone);
const
  CDisplayStyle: array [ThtDisplayStyle] of ThtString = (
    '',
    'inline',
    'block',
    'inline-block',
    'list-item',
    'run-in',
    'compact',
    'table',
    'inline-table',
    'table-row-group',
    'table-header-group',
    'table-footer-group',
    'table-row',
    'table-column-group',
    'table-column',
    'table-cell',
    'table-caption',
    'none');

  CRootDisplayStyle: array[ThtDisplayStyle] of ThtDisplayStyle = (
    pdUnassigned,
    pdBlock,
    pdBlock,
    pdBlock,
    pdBlock,
    pdBlock,
    pdBlock,
    pdTable,
    pdTable,
    pdBlock,
    pdBlock,
    pdBlock,
    pdBlock,
    pdBlock,
    pdBlock,
    pdBlock,
    pdBlock,
    pdNone);

function ToRootDisplayStyle(Display: ThtDisplayStyle): ThtDisplayStyle; {$ifdef UseInline} inline; {$endif}

type
  ThtTextTransformStyle = (
    txNone,
    txUpper,
    txLower,
    txCaps);

type
  ThtVisibilityStyle = (
    viInherit,
    viHidden,
    viVisible);

type
  ThtWhiteSpaceStyle = (
    wsNormal,
    wsNoWrap,
    wsPre,
    wsPreWrap,
    wsPreLine{, wsInherit});

const
  CWhiteSpace: array [ThtWhiteSpaceStyle] of ThtString = (
    'normal',
    'nowrap',
    'pre',
    'pre-wrap',
    'pre-line');

//BG, 16.09.2010: CSS2.2: same sizes like html font size:
type
  ThtFontSizeIncrement = -6..6;
  ThtFontConvBase = array[0..7] of Double;
  ThtFontConv = array[1..7] of Double;
const
  FontConvBase:    ThtFontConvBase = (0.5833, 0.75,   0.8333, 1.0,    1.1667, 1.5,  2.0,    3.0);
  PreFontConvBase: ThtFontConvBase = (0.5,    0.5833, 0.75,   0.8333, 1.0,    1.25, 1.6667, 2.5);

type
  ThtRectEdge     = (reLeft, reTop, reRight, reBottom);
  ThtRectIntegers = packed array[ThtRectEdge] of Integer;
  ThtRectColors   = packed array[ThtRectEdge] of TColor;
  ThtRectStyles   = packed array[ThtRectEdge] of ThtBorderStyle;
const
  NullIntegers: ThtRectIntegers = (0, 0, 0, 0);
  NoneColors: ThtRectColors = (clNone, clNone, clNone, clNone);
  NoneStyles: ThtRectStyles = (bssNone, bssNone, bssNone, bssNone);

function htRectIntegers(Left, Top, Right, Bottom: Integer): ThtRectIntegers;
function htRectColors(Left, Top, Right, Bottom: TColor): ThtRectColors;
function htRectStyles(Left, Top, Right, Bottom: ThtBorderStyle): ThtRectStyles;

//------------------------------------------------------------------------------
// media types
//------------------------------------------------------------------------------

type
  // http://www.w3.org/TR/2010/WD-CSS2-20101207/media.html
  ThtMediaType = (
    mtAll,        // Suitable for all devices.
    mtBraille,    // Intended for braille tactile feedback devices.
    mtEmbossed,   // Intended for paged braille printers.
    mtHandheld,   // Intended for handheld devices (typically small screen, limited bandwidth).
    mtPrint,      // Intended for paged material and for documents viewed on screen in print preview mode. Please consult the section on paged media for information about formatting issues that are specific to paged media.
    mtProjection, // Intended for projected presentations, for example projectors. Please consult the section on paged media for information about formatting issues that are specific to paged media.
    mtScreen,     // Intended primarily for color computer screens.
    mtSpeech,     // Intended for speech synthesizers. Note: CSS2 had a similar media type called 'aural' for this purpose. See the appendix on aural style sheets for details.
    mtTty,        // Intended for media using a fixed-pitch character grid (such as teletypes, terminals, or portable devices with limited display capabilities). Authors should not use pixel units with the "tty" media type.
    mtTv          // Intended for television-type devices (low resolution, color, limited-scrollability screens, sound available).
  );

const
  CMediaTypes: array[ThtMediaType] of ThtString = (
    'all',
    'braille',
    'embossed',
    'handheld',
    'print',
    'projection',
    'screen',
    'speech',
    'tty',
    'tv'
  );

type
  ThtMediaOperator = (
    moUnknown,
    moIs,         // Is true
    moEq,         //  =
    moLe,         // <=
    moGe);        // >=

type
  ThtMediaFeature = (
    mfUnknown,
    mfWidth,
    mfHeight,
    mfOrientation,
    mfAspectRatio,
    mfDeviceWidth,
    mfDeviceHeight,
    mfDeviceAspectRatio,
    mfColor,
    mfColorIndex,
    mfMonochrome,
    mfResolution,
    mfScan,
    mfGrid
  );

const
  CMediaFeatures: array[ThtMediaFeature] of record
    Name: ThtString;
  end = (
    (Name:''                    ),
    (Name:'width'               ),
    (Name:'height'              ),
    (Name:'orientation'         ),
    (Name:'aspect-ratio'        ),
    (Name:'device-width'        ),
    (Name:'device-height'       ),
    (Name:'device-aspect-ratio' ),
    (Name:'color'               ),
    (Name:'color-index'         ),
    (Name:'monochrome'          ),
    (Name:'resolution'          ),
    (Name:'scan'                ),
    (Name:'grid'                )
  );

//------------------------------------------------------------------------------
// media queries
//------------------------------------------------------------------------------

type

  ThtMediaExpression = record
    Oper: ThtMediaOperator;
    Feature: ThtMediaFeature;
    Expression: Variant;
  end;

  ThtMediaExpressions = array of ThtMediaExpression;

  ThtMediaQuery = record
    MediaType: ThtMediaType;
    Negated: Boolean;
    Expressions: ThtMediaExpressions;
  end;

  ThtMediaQueries = array of ThtMediaQuery;
  ThtMediaQueryEvent = procedure(Sender: TObject; const MediaQuery: ThtMediaQuery; var MediaMatchesQuery: Boolean) of object;

function MediaQueryToStr(const MediaQuery: ThtMediaQuery): ThtString;
function MediaQueriesToStr(const MediaQueries: ThtMediaQueries): ThtString;
function TryStrToMediaType(const Str: ThtString; out MediaType: ThtMediaType): Boolean;
function TryStrToMediaFeature(const Str: ThtString; out Feature: ThtMediaFeature; out Oper: ThtMediaOperator): Boolean;

function StrToFontName(const Str: ThtString): ThtString;
function StrToFontSize(const Str: ThtString; const FontConvBase: ThtFontConvBase; DefaultFontSize, Base, Default: Double): Double; overload;
function StrToFontSize(const Str: ThtString; const FontConv: ThtFontConv; Base, Default: Double): Double; overload;
function StrToLength(const Str: ThtString; Relative: Boolean; Base, EmBase, Default: Double): Double;

function TryStrToAlignmentStyle(const Str: ThtString; out AlignmentStyle: ThtAlignmentStyle): Boolean;
function TryStrToBoxSizing(const Str: ThtString; out ABoxSizing: ThtBoxSizing): Boolean;
function TryStrToBorderStyle(const Str: ThtString; out BorderStyle: ThtBorderStyle): Boolean;
function TryStrToBoxFloatStyle(const Str: ThtString; out Float: ThtBoxFloatStyle): Boolean;
function TryStrToBoxPositionStyle(const Str: ThtString; out Position: ThtBoxPositionStyle): Boolean;
function TryStrToBulletStyle(const Str: ThtString; out BulletStyle: ThtBulletStyle): Boolean;
function TryStrToClearStyle(const Str: ThtString; out Clear: ThtClearStyle): Boolean;
function TryStrToDisplayStyle(const Str: ThtString; out Display: ThtDisplayStyle): Boolean;
function TryStrToWhiteSpace(const Str: ThtString; out AWhiteSpace: ThtWhiteSpaceStyle): Boolean;
function GetPositionInRange(Which: ThtBackgroundPosition; Where, Range: Integer): Integer;
{
 Returns a positon according to the given settings.
 Which: which position in the range to get. pLeft and pTop return 0, pBottom and pRight return Range.
 Where: percentage or pixels for Which = pPercentage resp. pDim.
 Range: the range in which the result can vary.
 In the usual alignment calculations the range is outer size - inner size.
 If you have to consider an offset to outer's parent, add it to the function result afterwards.
}
procedure AdjustForTiling(Tiled: Boolean; TileAreaMin, TileAreaMax, TileSize: Integer;
  var Pos: Integer; out TiledEnd: Integer);
{
 Returns the start and end value for tiling an object of given size.
 Tiled: if false returns a TiledEnd according to unmodified Pos, that allows to pass the tiling
   process and depending on the visibility of the object in the cliparea the untiled object is
   processes at most once. If true, Pos is moved to a position between ClipMin - ObjectSize and
   ClipMin so that the tiling process will put one of the tiles to original Pos.
 TileAreaMin, TileAreaMax: the area in with the object is to tile.
 TileSize: the size of the tile.
 Pos: on input: the position to consider for tiling. on output the new position shifted by multiples
   of the object size to where the object covers the tile area minimum.
 TiledEnd: a position on and after which no more tiles are processed.
}
procedure CalcBackgroundLocationAndTiling(const PRec: PtPositionRec; ARect: TRect;
  XOff, YOff, IW, IH, BW, BH: Integer; out X, Y, X2, Y2: Integer);
{
 PRec has the CSS information on the background image, it's starting location and
 whether it is tiled in x, y, neither, or both.
 ARect is the cliprect, no point in drawing tiled images outside it.
 XOff, YOff are offsets which allow for the fact that the viewable area may not be at 0,0.
 IW, IH are the total width and height of the document if you could see it all at once.
 BW, BH are bitmap dimensions used to calc tiling.
 X, Y are the position (window coordinates) where the first background iamge will be drawn.
 X2, Y2 are tiling limits.  X2 and Y2 may be such that 0, 1, or many images will
   get drawn.  They're calculated so that only images within ARect are drawn.
}

{Internal stuff we expose for inlining}

const
// CSS 2.1 defines a fixed ratio between pt and px at 96 dpi: 1px = 0.75pt.
// (see http://www.w3.org/TR/2010/WD-CSS2-20101207/syndata.html#value-def-length for details).
  HTMLViewerPixelsPerInch = 96.0; // fixed assumption in CSS 2.1 as lots of designs rely on it.
  HTMLViewerPointsPerInch = 72.0;
  //BG, 16.06.2015: Issue 399: StyleTypes.pas: Code improvement for C++Builder
  // HTMLViewerPointsPerPixel = HTMLViewerPointsPerInch / HTMLViewerPixelsPerInch;
  // produces idiotic "static const System::Extended HTMLViewerPointsPerPixel = 7.500000E-01;"
  // in StyleTypes.hpp which then produces "unused" warnings.
  // The following declaration produces just another #define:
  HTMLViewerPointsPerPixel = 0.75; // HTMLViewerPointsPerInch / HTMLViewerPixelsPerInch;

type
  ThtLengthUnitInfo = record
    Name: ThtString;
    Factor: Double; // factor of length unit
    Index: Integer; // index of font size
    IsAbsolute: Boolean;
  end;
  ThtUnit = (
    luNone, luEm, luEx, luPercent, luPt, luPx, luPc, luIn, luCm, luMm,
    fsNone, fsSmaller, fsLarger,
    fsXxSmall, fsXSmall, fsSmall, fsMedium, fsLarge, fsXLarge, fsXxLarge);

  ThtLengthUnit = luNone..luMm;
  ThtFontSize = fsNone..fsXxLarge;

const
  CUnitInfo: array [ThtUnit] of ThtLengthUnitInfo = (
    // length units
    (Name: '';          Factor: 1.00; Index:  0; IsAbsolute: True),
    (Name: 'em';        Factor: 1.00; Index:  0; IsAbsolute: False),
    (Name: 'ex';        Factor: 0.50; Index:  0; IsAbsolute: False),
    (Name: '%' ;        Factor: 0.01; Index:  0; IsAbsolute: False),
    (Name: 'pt';        Factor: 0.75; Index:  0; IsAbsolute: True),
    (Name: 'px';        Factor: 1.00; Index:  0; IsAbsolute: True),
    (Name: 'pc';        Factor: 9.00; Index:  0; IsAbsolute: True),
    (Name: 'in';        Factor: HTMLViewerPixelsPerInch       ; Index: 0; IsAbsolute: True),
    (Name: 'cm';        Factor: HTMLViewerPixelsPerInch / 2.54; Index: 0; IsAbsolute: True),
    (Name: 'mm';        Factor: HTMLViewerPixelsPerInch / 25.4; Index: 0; IsAbsolute: True),
    // css font sizes
    (Name: '';          Factor: 1.00; Index:  3; IsAbsolute: True),
    (Name: 'smaller';   Factor: 1.00; Index: -1; IsAbsolute: False),
    (Name: 'larger';    Factor: 1.00; Index:  1; IsAbsolute: False),
    (Name: 'xx-small';  Factor: 1.00; Index:  0; IsAbsolute: True),
    (Name: 'x-small';   Factor: 1.00; Index:  1; IsAbsolute: True),
    (Name: 'small';     Factor: 1.00; Index:  2; IsAbsolute: True),
    (Name: 'medium';    Factor: 1.00; Index:  3; IsAbsolute: True),
    (Name: 'large';     Factor: 1.00; Index:  4; IsAbsolute: True),
    (Name: 'x-large';   Factor: 1.00; Index:  5; IsAbsolute: True),
    (Name: 'xx-large';  Factor: 1.00; Index:  6; IsAbsolute: True)
  );

implementation
end.
