{
Version   11.7
Copyright (c) 2014-2016 by HtmlViewer Team

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Note that the source modules HTMLGIF1.PAS and DITHERUNIT.PAS
are covered by separate copyright notices located in those modules.
}

{$I frxHTMLCons.inc}

unit frxHTMLSymb;

interface

uses
{$ifdef MSWINDOWS}
  Windows, SysUtils,
{$endif}
  frxHTMLGlobals;

type
  TElemSymb = (
    OtherChar, CommandSy, StringSy {was TextSy}, EolSy, EofSy,

    { doc type tags }
    XmlSy,     DocTypeSy,

    { html elements }

    HtmlSy,    HeadSy,
    HtmlEndSy, HeadEndSy,

    { frame set elements }

    FrameSetSy, FrameSetEndSy,
    FrameSy,
    NoFramesSy, NoFramesEndSy,

    { metadata elements }

    TitleElemSy, TitleEndSy,
    BaseSy,
    LinkElemSy,
    MetaSy,
    StyleSy, StyleEndSy,
    ScriptSy, ScriptEndSy,

    { misc elements }

    BaseFontSy,
    BgSoundSy,

    HRSy,
    BRSy,

    ImageSy,
    PanelSy,
    IFrameSy,   IFrameEndSy,
    ProgressSy, ProgressEndSy,
    MeterSy,    MeterEndSy,

    MapSy, MapEndSy,
    AreaSy,

    PageSy,

    ObjectSy, ObjectEndSy,
    ParamSy,

    { inline elements }

    SpanSy,    NoBrSy,
    SpanEndSy, NoBrEndSy,

    WbrSy,

    FontSy,    BSy,    ISy,    SSy,    StrikeSy,    USy,    SubSy,    SupSy,    BigSy,    SmallSy,    TTSy,
    FontEndSy, BEndSy, IEndSy, SEndSy, StrikeEndSy, UEndSy, SubEndSy, SupEndSy, BigEndSy, SmallEndSy, TTEndSy,

    EmSy,    StrongSy,    CodeSy,    KbdSy,    SampSy,    CiteSy,    VarSy,    AbbrSy,    AcronymSy,    DfnSy,    
    EmEndSy, StrongEndSy, CodeEndSy, KbdEndSy, SampEndSy, CiteEndSy, VarEndSy, AbbrEndSy, AcronymEndSy, DfnEndSy,

    DelSy,    InsSy,    MarkSy,    TimeSy,
    DelEndSy, InsEndSy, MarkEndSy, TimeEndSy,

    ASy,
    AEndSy,

    { block elements }

    BodySy,    PSy,    DivSy,    CenterSy,
    BodyEndSy, PEndSy, DivEndSy, CenterEndSy,

    ArticleSy,    SectionSy,    MainSy, NavSy,    AsideSy,
    ArticleEndSy, SectionEndSy, MainEndSy, NavEndSy, AsideEndSy,

    {Keep order} H1Sy,    H2Sy,    H3Sy,    H4Sy,    H5Sy,    H6Sy,    {end order}
    {Keep order} H1EndSy, H2EndSy, H3EndSy, H4EndSy, H5EndSy, H6EndSy, {end order}

    HGroupSy,    HeaderSy,    FooterSy,    AddressSy,    BlockQuoteSy,    PreSy,
    HGroupEndSy, HeaderEndSy, FooterEndSy, AddressEndSy, BlockQuoteEndSy, PreEndSy,

    OLSy,    LISy,    ULSy,    DirSy,    MenuSy,    DLSy,    DDSy,    DTSy,
    OLEndSy, LIEndSy, ULEndSy, DirEndSy, MenuEndSy, DLEndSy, DDEndSy, DTEndSy,

    liAloneSy,

    { table elements }

    TableSy, TableEndSy,
    ColGroupSy, ColGroupEndSy, ColSy,
    CaptionSy, CaptionEndSy,
    THeadSy,    TBodySy,    TFootSy,
    THeadEndSy, TBodyEndSy, TFootEndSy,
    TRSy, TREndSy,
    THSy,    TDSy,
    THEndSy, TDEndSy,

    { form elements }

    FormSy, FormEndSy,
    FieldsetSy, FieldsetEndSy,
    LegendSy, LegendEndSy,
    LabelSy, LabelEndSy,
    TextAreaSy,    SelectSy,     OptionSy,    ButtonSy, ButtonEndSy, InputSy,
    TextAreaEndSy, SelectEndSy,  OptionEndSy
  );
  TElemSymbSet = set of TElemSymb;

  TAttrSymb = (
    OtherAttribute,
    ActionSy,
    ActiveSy,
    AlignSy,
    AltSy,
    BackgroundSy,
    BGColorSy,
    BGPropertiesSy,
    BorderSy,
    BorderColorSy,
    BorderColorDarkSy,
    BorderColorLightSy,
    CellPaddingSy,
    CellSpacingSy,
    CharSetSy,
    CheckBoxSy,
    CheckedSy,
    ClassSy,
    ClearSy,
    ColorSy,
    ColsSy,
    ColSpanSy,
    ContentSy,
    CoordsSy,
    atDirSy,
    DisabledSy,
    EncodingSy,
    EncTypeSy,
    FaceSy,
    FrameAttrSy,
    FrameBorderSy,
    HeightSy,
    HighSy,
    HrefSy,
    HSpaceSy,
    HttpEqSy,
    IDSy,
    IsMapSy,
    LabelAttrSy,
    LanguageSy,
    LeftMarginSy,
    LinkSy,
    LoopSy,
    LowSy,
    MarginHeightSy,
    MarginWidthSy,
    MaxSy,
    MaxLengthSy,
    MediaSy,
    MethodSy,
    MinSy,
    MultipleSy,
    NameSy,
    NoHrefSy,
    NoResizeSy,
    NoShadeSy,
    NoWrapSy,
    OLinkSy,
    OnBlurSy,
    OnChangeSy,
    OnClickSy,
    OnFocusSy,
    OptimumSy,
    PlaceholderSy,
    PlainSy,
    RadioSy,
    RatioSy,
    ReadonlySy,
    RelSy,
    RevSy,
    RowsSy,
    RowSpanSy,
    RulesSy,
    ScrollingSy,
    SelectedSy,
    ShapeSy,
    SizeSy,
    SpanAttrSy,
    SpellCheckSy,
    SrcSy,
    StartSy,
    StyleAttrSy,
    TabIndexSy,
    TargetSy,
    TextSy,
    TitleSy,
    TopMarginSy,
    TypeSy,
    UseMapSy,
    VAlignSy,
    ValueSy,
    VersionSy,
    VLinkSy,
    VSpaceSy,
    WidthSy,
    WrapSy);
  TAttrSymbSet = set of TAttrSymb;

  PEntity = ^TEntity;
  TEntity = record
    Name: ThtString;
    Value: Integer;
  end;

  PResWord = ^TResWord;
  TResWord = record
    Name: ThtString;
    Symbol: TElemSymb;
    EndSym: TElemSymb; // CommandSy == no end symbol
  end;

  PSymbol = ^TSymbolRec;
  TSymbolRec = record
    Name: ThtString;
    Value: TAttrSymb;
  end;

var
  Entities: ThtStringList;
  ElementNames: ThtStringList;
  AttributeNames: ThtStringList;

procedure SetSymbolName(Sy: TElemSymb; Name: ThtString); overload;
procedure SetSymbolName(Sy: TAttrSymb; Name: ThtString); overload;
function SymbToStr(Sy: TElemSymb): ThtString; overload;
function SymbToStr(Sy: TAttrSymb): ThtString; overload;
function EndSymbToStr(Sy: TElemSymb): ThtString;
function EndSymbToSymb(Sy: TElemSymb): TElemSymb;
function EndSymbFromSymb(Sy: TElemSymb): TElemSymb;

implementation
end.
