{
Version   11.7
Copyright (c) 1995-2008 by L. David Baldwin,
Copyright (c) 2008-2016 by HtmlViewer Team

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Note that the source modules HTMLGIF1.PAS and DITHERUNIT.PAS
are covered by separate copyright notices located in those modules.
}
unit frxHTMLURLSubs;

{$I frx.inc}
{.$I frxHTMLCons.inc}

interface

uses
{$ifdef MSWindows}
  Windows,
{$else MSWindows}
  Types,
{$endif MSWindows}
  frxHTMLGlobals;

type
  ThtDocType = (HTMLType, TextType, ImgType, XHtmlType, OtherType);

{***************************************************************************************************
 * URL processing methods
 **************************************************************************************************}

procedure ParseURL(const Url: ThtString; out Proto, User, Pass, Host, Port, Path: ThtString);
{Franois PIETTE's URL parsing procedure}

procedure SplitString(var Str: ThtString; Sep: ThtChar; out Spall: ThtString); overload;
{$ifndef UNICODE}
procedure SplitString(var Str: String; Sep: Char; out Spall: String); overload;
{$endif}
{Split Str at first appearance of Sep into Str and Spall. Spall starts with Sep.}

function IsValidSchemeChar(const Ch: ThtChar): Boolean;
{ Returns true, if Ch is a valid scheme char (a..z, A..Z, 0..9, +, -, .)}
function FindSchemeSep(const Src: ThtString; var Index: Integer): Boolean;
{Find the ':' after a series of valid scheme chars starting with Src[Index].
 Returns true, if there is a ':' only preceeded by valid scheme chars.
 If returns true Index is advanced to the position of ':'}

procedure SplitScheme(const Src: ThtString; out Name, SchemeSpecific: ThtString); overload;
procedure SplitScheme(var Src: ThtString; out SchemeSpecific: ThtString); overload;
function  SplitScheme(var Src: ThtString): ThtString; overload;
{Split an URL into scheme and scheme specific part at first appearance of ':'}

procedure SplitDest(const Src: ThtString; out Name, Dest: ThtString); overload;
procedure SplitDest(var Src: ThtString; out Dest: ThtString); overload;
function  SplitDest(var Src: ThtString): ThtString; overload;
{Split local destination from an URL at first appearance of '#', returns Dest with separating '#'}

procedure SplitQuery(const Src: ThtString; out Name, Query: ThtString); overload;
procedure SplitQuery(var Src: ThtString; out Query: ThtString); overload;
function  SplitQuery(var Src: ThtString): ThtString; overload;
{$ifndef UNICODE}
procedure SplitQuery(var Src: String; out Query: String); overload;
function  SplitQuery(var Src: String): String; overload;
{$endif}
{Split query from an URL at first appearance of '?', returns Query with separating '?'}

function GetBase(const URL: ThtString): ThtString; deprecated;
function GetURLBase(const URL: ThtString): ThtString;
{Given an URL, get the base directory}

function Combine(Base, APath: ThtString): ThtString; deprecated;
function CombineURL(Base, APath: ThtString): ThtString;
{combine a base and a path taking into account that overlap might exist}
{needs work for cases where directories might overlap}

function IsFullURL(const URL: ThtString): boolean;
{set if contains a protocol}

function GetProtocol(const URL: ThtString): ThtString;
{return the http, mailto, etc in lower case}

function GetURLExtension(const URL: ThtString): ThtString;
{returns extension without the '.', mixed case}

function GetURLFilenameAndExt(const URL: ThtString): ThtString;
{returns mixed case after last /}

function DecodeURL(const URL: ThtString): ThtString;
{return % encoded URL converted to UnicodeString}

function EncodeUrl(const Url: ThtString): ThtString;


{***************************************************************************************************
 * both URL and DOS processing methods
 **************************************************************************************************}

function Normalize(const URL: ThtString): ThtString;
{lowercase, trim, and make sure a '/' terminates a hostname, adds http://}

function DosToHTML(const FName: ThtString): ThtString;
{convert an Dos style filename to one for HTML.  Does not add the file:///}

function DosToHtmlNoSharp(FName: ThtString): ThtString;
{convert a Dos style filename to one for HTML.  Does not add the file:///.
 use where "#" is not being used as a local position but rather is part of filename.}

function HTMLToDos(const FName: ThtString): ThtString;
{convert an HTML style filename to one for Dos}

function HTMLServerToDos(const FName, Root: ThtString): ThtString;
{convert an HTML style filename to one for Dos. Prefix with Root, if name starts with a single backslash}

{***************************************************************************************************
 * DOS processing methods
 **************************************************************************************************}

function IsAbsolutePath(const Filename: ThtString): Boolean;
{set if contains a drive or server}

function CombineDos(const Base, Path: ThtString): ThtString;
{combine a base and a path}

{***************************************************************************************************
 * both URL and Resource processing methods
 **************************************************************************************************}

function HTMLToRes(const URL: ThtString; out AType: ThtString): ThtString;
{convert an (% encoded) URL like res:[/]*<name>.<type> to resource name and type}

{**************************************************************************************************}
implementation
end.

