unit frxHash;

{$I frx.inc}

interface

uses
  {$IFNDEF Linux}
  Windows,
  {$ENDIF}
  SysUtils, Classes;

const
  MaxDigestSize = 64;

type
  TfrxHashDigest = array[0 .. MaxDigestSize - 1] of Byte;

  TfrxHash = class
  protected
    FDigest: TfrxHashDigest;
    FData: TBytes;
    FKey: TBytes;

    function GetDigestToAnsi: AnsiString;
  public
    constructor Create;
    procedure CalcHMAC;
    procedure Initialize; virtual; abstract;
    procedure Update(const Buf; Len: Cardinal); virtual; abstract;
    procedure UpdateByAnsi(const Str: AnsiString);
    procedure SetData(const AData; const ALen: Cardinal);
    procedure SetDataByAnsi(const AData: AnsiString);
    procedure SetKey(const AKey; const ALen: Cardinal);
    procedure SetKeyByAnsi(const AKey: AnsiString);
    procedure Finalize; virtual; abstract;
    procedure MoveDigestTo(var Destination; Size: Integer = -1);
    procedure Iterate(const Str: AnsiString; Count: Cardinal);
    function DigestFromAnsiToAnsi(const Str: AnsiString): AnsiString;
    class function DigestSize: Cardinal; virtual; abstract;
    class function BlockSize: Byte; virtual; abstract;

    class function SHA256(const AData: AnsiString): AnsiString; overload;
    class function SHA256(const AData: TStream): AnsiString; overload;

    function GetHMAC(const AData: AnsiString; const AKey: TBytes): TBytes; overload;
    function GetHMAC(const AData: AnsiString; const AKey: AnsiString): AnsiString; overload;

    property DigestToAnsi: AnsiString read GetDigestToAnsi;
  end;

  TfrxHashClass = class of TfrxHash;

  TfrxMD5Hash = class(TfrxHash)
  private
    FCount: array[0..1] of DWord;
    FState: array[0..3] of DWord;
    FBuffer: array[0..63] of Byte;
    FBufLen: DWord;
    procedure Transform(var Accu; const Buf);
  public
    procedure Initialize; override;
    procedure Update(const Buf; Len: Cardinal); override;
    procedure Finalize; override;
    class function DigestSize: Cardinal; override;
    class function BlockSize: Byte; override;
  end;

  TfrxSHA1Hash = class(TfrxHash)
  private
    Size: Int64; //Cardinal;
    CA, CB, CC, CD, CE: DWord;
    Buffer: array [0..63] of byte;
    W: array [0..79] of DWord;
    BufSize: Cardinal;
    procedure Transform(Chunk: Pointer);
  public
    procedure Initialize; override;
    procedure Update(const Buf; Len: Cardinal); override;
    procedure Finalize; override;
    class function DigestSize: Cardinal; override;
    class function BlockSize: Byte; override;
  end;

  TfrxSHA256Hash = class(TfrxHash)
  private
    FLenHi, FLenLo: DWord;
    FIndex: Cardinal;
    FHash: array[0..7] of DWord;
    FBuffer: array[0..63] of Byte;
    procedure Transform;
  public
    procedure Initialize; override;
    procedure Update(const Buf; Len: Cardinal); override;
    procedure Finalize; override;
    class function DigestSize: Cardinal; override;
    class function BlockSize: Byte; override;
  end;

implementation
end.
