
{******************************************}
{                                          }
{             FastReport VCL               }
{             FTP Save Filter              }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxIOTransportFTP;

interface

{$I frx.inc}

uses
  Classes, Forms, Controls, StdCtrls, ComCtrls,
  frxIOTransportHelpers, frxBaseTransportConnection, frxTransportIndyConnector,
  IdFTP, IdComponent, IdTCPConnection;

type
  TfrxFTPIOTransportForm = class(TfrxBaseTransportDialog)
    CancelB: TButton;
    OkB: TButton;
    RequiredLabel: TLabel;
    RememberPropertiesCheckBox: TCheckBox;
    PageControl: TPageControl;
    GeneralTabSheet: TTabSheet;
    HostLabel: TLabel;
    PortLabel: TLabel;
    UserNameLabel: TLabel;
    PasswordLabel: TLabel;
    RemoteDirLabel: TLabel;
    HostComboBox: TComboBox;
    PortComboBox: TComboBox;
    PasswordEdit: TEdit;
    UserNameEdit: TEdit;
    PassiveCheckBox: TCheckBox;
    RemoteDirComboBox: TComboBox;
    ProxyTabSheet: TTabSheet;
    ProxyPasswordEdit: TEdit;
    ProxyUserNameEdit: TEdit;
    ProxyPortComboBox: TComboBox;
    ProxyHostComboBox: TComboBox;
    ProxyPasswordLabel: TLabel;
    ProxyUserNameLabel: TLabel;
    ProxyPortLabel: TLabel;
    ProxyHostLabel: TLabel;
    ProxyTypeLabel: TLabel;
    ProxyTypeComboBox: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure OkBClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure PortComboBoxKeyPress(Sender: TObject; var Key: Char);
  private
    procedure RequireIf(L: TLabel; Flag: Boolean; MR: integer = mrNone);
  public
    procedure InitControlsFromFilter(TransportFilter: TfrxInternetIOTransport); override;
    procedure InitFilterFromDialog(TransportFilter: TfrxInternetIOTransport); override;
    { Public declarations }
  end;

  TfrxTransportIndyFTPConnector = class(TfrxTransportIndyConnector)
  protected
    function GetProxyHost: String; override;
    function GetProxyLogin: String; override;
    function GetProxyPassword: String; override;
    function GetProxyPort: Integer; override;
    procedure SetProxyHost(const Value: String); override;
    procedure SetProxyLogin(const Value: String); override;
    procedure SetProxyPassword(const Value: String); override;
    procedure SetProxyPort(const Value: Integer); override;
  public
    procedure SetDefaultParametersWithToken(AToken: String); override;
  end;

{$IFDEF DELPHI16}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxFTPIOTransport = class(TfrxInternetIOTransport)
  private
    FHost: String;
    FPort: integer;
    FUserName: String;
    FPassword: String;
    FPassive: Boolean;
    FProxyType: TIdFtpProxyType;
    FRemoteDir: String;

    procedure FTPDeleteFile(Name: String);
    procedure FTPDeleteFolder(Name: String);

    procedure FTPSetHost;
    procedure FTPSetProxy;
  protected
    FFTP: TidFTP;
    FConnector: TfrxTransportIndyFTPConnector;

    function Connection: TfrxBaseTransportConnection; override;
    function FilterSection: String; override;
    procedure DialogDirChange(Name, Id: String; DirItems: TStrings); override;
    procedure DialogDirCreate(Name: String; DirItems: TStrings); override;
    procedure DialogFileDelete(Name, Id: String; DirItems: TStrings); override;
    procedure DialogDirDelete(Name, Id: String; DirItems: TStrings); override;
    function CreateConnector: Boolean; override;
    procedure DisposeConnector; override;
    function DoConnectorConncet: Boolean; override;
    function DoBeforeSent: Boolean; override;
    procedure Upload(const Source: TStream; DestFileName: String = ''); override;
    procedure Download(const SourceFileName: String; const Source: TStream); override;
    procedure CreateRemoteDir(DirName: String; ChangeDir: Boolean = True); override;
    procedure ChangeDirUP; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function GetDescription: String; override;
    procedure GetDirItems(DirItems: TStrings; aFilter: String = ''); override;
    class function TransportDialogClass: TfrxBaseTransportDialogClass; override;
    function GetConnectorInstance: TfrxBaseTransportConnectionClass; override;
  published
    property Host: String read FHost write FHost;
    property Port: integer read FPort write FPort;
    property UserName: String read FUserName write FUserName;
    property Password: String read FPassword write FPassword;
    property Passive: Boolean read FPassive write FPassive;

    property ProxyType: TIdFtpProxyType read FProxyType write FProxyType;

    property RemoteDir: String read FRemoteDir write FRemoteDir;
  end;

implementation
end.
