
{******************************************}
{                                          }
{              FastReport v6.0             }
{            Save Filter Helpers           }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxIOTransportHelpers;

interface

{$I frx.inc}

uses
  frxClass, frxProgress, Classes, IniFiles, Controls, Forms, StdCtrls,
  frxBaseTransportConnection, frxBaseForm, frxProtocolFactory,
  frxBaseTransportDialogForm;

const
  frxTransportTokenName = 'AccessToken';

type
  TfrxInternetIOTransport = class;
  TDirStack = class;

  TfrxBaseTransportDialog = class(TfrxBaseForm)
  protected
    FIni: TCustomIniFile;
    FFilter: TfrxInternetIOTransport;
    function GetIniFile(TransportFilter: TfrxInternetIOTransport): TCustomIniFile;
    procedure InitDialog; virtual;
    function Decode64(Text: String): String;
    function Encode64(Text: String): String;
    procedure IniLoadComboBox(ComboBox: TComboBox);
    procedure IniLoadComboBoxWithItems(ComboBox: TComboBox);
    procedure IniLoadEdit(Edit: TEdit);
    procedure IniLoadCheckBox(CheckBox: TCheckBox);
    procedure IniSaveComboBoxItem(ComboBox: TComboBox);
    procedure IniSaveComboBox(ComboBox: TComboBox);
    procedure IniSaveEdit(Edit: TEdit);
    procedure IniSaveCheckBox(CheckBox: TCheckBox);
  public
    destructor Destroy; override;
    procedure UpdateResouces; override;
    procedure InitControlsFromFilter(TransportFilter: TfrxInternetIOTransport); virtual;
    procedure InitFilterFromDialog(TransportFilter: TfrxInternetIOTransport); virtual;
  end;

  TfrxBaseOAuthTransportDialog = class(TfrxBaseTransportDialog)
  protected
    FErrorMessage: String;
    FRedirectURI: String;
    function GetTokenDialog: Boolean; virtual;
    function GetClientID: String; virtual; abstract;
    function GetClientSecret: String; virtual; abstract;
    function IniLoadStringCrypt(const SectionName: String): String;
    procedure IniSaveStringCrypt(const SectionName, Data: String);
    function IniLoadBoolCrypt(const SectionName: String): Boolean;
    procedure IniSaveBoolCrypt(const SectionName: String; Data: Boolean);
    function IniLoadIntegerCrypt(const SectionName: String): Integer;
    procedure IniSaveIntegerCrypt(const SectionName: String; Data: Integer);
  public
    procedure InitControlsFromFilter(TransportFilter: TfrxInternetIOTransport); override;
    property ErrorMessage: String read FErrorMessage;
  end;


  TfrxBaseTransportDialogClass = class of TfrxBaseTransportDialog;
  TfrxBaseFormClass = class of TfrxBaseTransportDialogForm;

  TfrxInternetIOTransport = class(TfrxCustomIOTransport)
  private
    FUseIniFile: Boolean;
    FShowProgress: Boolean;
    procedure ConnectionWorkBegin(Sender: TObject; AWorkMode: TfrxHTTPWorkMode; AWorkCount: Int64);
    procedure ConnectionWork(Sender: TObject; AWorkMode: TfrxHTTPWorkMode; AWorkCount: Int64);
  protected
    FIsFilterOpened: Boolean;
    FProgress: TfrxProgress;
    FWorkBeginPosition: Int64;
    FDefaultProxyPort: Integer;
    FProxyHost: String;
    FProxyPort: integer;
    FProxyUserName: String;
    FProxyPassword: String;
    FUserName: String;
    FPassword: String;
    function Connection: TfrxBaseTransportConnection; virtual; abstract;
    function FormShowModal: TModalResult;
    procedure CreateProgress(ProgressCaption: String);

    function FilterSection: String; virtual; abstract;
    function PropertiesSection: String;
    procedure ProcessFiles;

    function SizeOfFiles: Int64;
    function IsSelectFileName: Boolean;
    function IsSelectDirectory: Boolean;
    function SendFiles: String;

    function DoCreateStream(var aFullFilePath: String; aFileName: String): TStream; override;
    class function GetProtocolClass: TfrxCustomDataLinkProtocolClass; virtual;
    { connector }
    { initialize connector - called before sending files }
    function CreateConnector: Boolean; virtual; abstract;
    { free connector }
    procedure DisposeConnector; virtual; abstract;
    { establish connection  }
    function DoConnectorConncet: Boolean; virtual; abstract;
    { prepare connection for file sending }
    function DoBeforeSent: Boolean; virtual;
    { send file stream }
    procedure Upload(const Source: TStream; DestFileName: String = ''); virtual; abstract;
    procedure Download(const SourceFileName: String; const Source: TStream); virtual;
    { creates remote directory }
    procedure CreateRemoteDir(DirName: String; ChangeDir: Boolean = True); virtual;
    { change current directory to parent }
    procedure ChangeDirUP; virtual;

    function IsDeleteSupported: Boolean; virtual;

    procedure SetName(const NewName: TComponentName); override;

    { Dialog callback's }
    procedure DialogDirChange(Name, Id: String; DirItems: TStrings); virtual; abstract;
    procedure DialogDirCreate(Name: String; DirItems: TStrings); virtual; abstract;
    procedure DialogDirDelete(Name, Id: String; DirItems: TStrings); virtual; abstract;
    procedure DialogFileDelete(Name, Id: String; DirItems: TStrings); virtual; abstract;

    procedure BeforeOpenFilter;
    function  DoOpenFilter: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    procedure AssignFilter(Source: TfrxCustomIOTransport); override;
    function OpenFilter: Boolean; override;
    procedure CloseFilter; override;
    { get directory items }
    function GetConnectorInstance: TfrxBaseTransportConnectionClass; virtual; abstract;
    procedure GetDirItems(DirItems: TStrings; aFilter: String = ''); virtual; abstract;
    class function TransportDialogClass: TfrxBaseTransportDialogClass; virtual;
    class function TransportPostDialogClass(DM: TfrxIOInternetDialogMode): TfrxBaseFormClass; virtual;
    procedure TestToken; virtual;
    function TestWithOutToken: String; virtual;
    procedure SilentAuthorize; virtual;
  published
    property UseIniFile: Boolean read FUseIniFile write FUseIniFile;
    property ShowProgress: Boolean read FShowProgress write FShowProgress;
    property DefaultProxyPort: Integer read FDefaultProxyPort;
    property ProxyHost: String read FProxyHost write FProxyHost;
    property ProxyPort: integer read FProxyPort write FProxyPort;
    property ProxyUserName: String read FProxyUserName write FProxyUserName;
    property ProxyPassword: String read FProxyPassword write FProxyPassword;
    property UserName: String read FUserName write FUserName;
    property Password: String read FPassword write FPassword;
  end;
  // TODO: split to HTTP and OAuth
  TfrxHTTPIOTransport = class(TfrxInternetIOTransport)
  private
    FClientID: String;
    FUseProxyServer: Boolean;
    FConnected: Boolean;
    FClientSecret: String;
    FEncryptionKey: AnsiString;
    FBlockedType: Boolean;
    FHost: String;
    function BaseLoadFromIni: Boolean;
  protected
    FDirStack: TDirStack;
    FHTTP: TfrxBaseTransportConnection;
    FAccessToken: String;
    FRefreshToken: String;
    FRemoteDir: String;
    FAccessTokens: TStrings;
    FListenerPort: Integer;
    function Connection: TfrxBaseTransportConnection; override;
    procedure SetProxy;
    procedure Upload(const Source: TStream; DestFileName: String = ''); override;
    procedure ChangeDirUP; override;
    procedure TestRemoteDir; virtual; // Empty
    function CreateConnector: Boolean; override;
    procedure DisposeConnector; override;
    function DoConnectorConncet: Boolean; override;
    procedure AddToDirItems(DirItems: TStrings; IsFolder, IsFile: Boolean;
      Name: String; Id: String = ''); virtual;
    procedure SetRemoteDir(const Value: String); virtual;
    function GetRemoteDir: String; virtual;
    procedure LoadFieldFromIni(const AIni: TCustomIniFile; const AIdentName: String; var ASaveTo: String); overload;
    procedure LoadFieldFromIni(const AIni: TCustomIniFile; const AIdentName: String; var ASaveTo: Boolean); overload;
    procedure LoadFieldFromIni(const AIni: TCustomIniFile; const AIdentName: String; var ASaveTo: Integer); overload;
    function LoadFromIni(const AIni: TCustomIniFile): Boolean; virtual;
    function AlwaysUseIniFile: Boolean; virtual;
    function GetPrefix: String; virtual;
    procedure SetPrefix(const APrefix: String); virtual;
    function GetHost: String;
    function DoRefreshToken: Boolean; virtual;
    function RefreshAllToken: Boolean;
    procedure ReSaveTokens;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AssignSharedProperties(Source: TfrxCustomIOTransport); override;
    procedure AssignFilter(Source: TfrxCustomIOTransport); override;
    procedure DownloadToStream(const SourceFileName: String; const Source: TStream);
    function OpenFilter: Boolean; override;
    function GetRedirectURI: String; virtual;
    function GetOAuthURI: String; virtual; abstract;
    function GetAccessToken(const AuthorizationCode: String; var ErrorMsg: String): String; virtual;
    function IsFinalURLTest(const URL: String; var Errors: String): Boolean; virtual;
    property RemoteDir: String read GetRemoteDir write SetRemoteDir;
    property EncryptionKey: AnsiString read FEncryptionKey write FEncryptionKey;
    property BlockedType: Boolean read FBlockedType write FBlockedType;
    property Host: String read GetHost write FHost;
    property Prefix: String read GetPrefix write SetPrefix;
    property DirStack: TDirStack read FDirStack write FDirStack;
  published
    property AccessToken: String read FAccessToken write FAccessToken;
    property RefreshToken: String read FRefreshToken write FRefreshToken;
    property ClientID: String read FClientID write FClientID;
    property ClientSecret: String read FClientSecret write FClientSecret;
    property UseProxyServer: Boolean read FUseProxyServer write FUseProxyServer;
    property ListenerPort: Integer read FListenerPort write FListenerPort default 9898;
  end;

  TIdObject = class
  private
    FId: String;
  public
    constructor Create(AId: String);
    property Id: String read FId;
  end;

  TDirStack = class
  protected
    FStringList: TStringList;
    FPrefix: String;
  public
    constructor Create; overload;
    constructor Create(APrefix: String); overload;
    destructor Destroy; override;
    procedure Assign(ASourceDS: TDirStack);

    procedure Push(st: String);
    function Pop: String;
    function Top: String;
    function Full: String;
    function GetCount: Integer;
    function isEmpty: Boolean;

    property Prefix: String read FPrefix write FPrefix;
    property Count: Integer read GetCount;
    property StringList: TStringList read FStringList;
  end;

procedure ClearLabelsFontStyle(Form: TForm);
function CopySubstring(const Source: String; Left, Right: Array of String): String;
function JsonEncode(str: String): String;
function SureUTF8(str: String): String;
function SureAnsi(str: String): AnsiString;
function PathFirstSlash(const Path: String): String;
function PathChangeDir(const Path, Dir: String): String;
procedure ClearWithObjects(DirItems: TStrings);

implementation
end.
