
{******************************************}
{                                          }
{              FastReport v6.0             }
{            Outlook base Filter           }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxIOTransportOutlookBase;

interface

{$I frx.inc}

uses
  Classes, Forms, Controls, StdCtrls, ComCtrls,
  frxIOTransportHelpers, frxBaseTransportConnection, frxIOTransportOAuthDialog,
  frxBaseMailApi;

const
  frx_Outlook_InitializeBase_URL = 'https://login.microsoftonline.com/';
  frx_Outlook_ListLabels_URL = 'https://graph.microsoft.com/v1.0/me/mailFolders';
  frx_Outlook_ListMessagesBase_URL = 'https://graph.microsoft.com/v1.0/me/mailFolders/';
  frx_Outlook_ListAttachmentsBase_URL = 'https://graph.microsoft.com/v1.0/me/messages/';
  frx_Outlook_Upload_URL = 'https://graph.microsoft.com/v1.0/me/sendMail';
  frx_Outlook_Test = 'https://graph.microsoft.com/v1.0/me';

  frx_Outlook_Scope = 'scope=offline_access Mail.Read Mail.Send User.Read';

type

  TfrxTenant = (t_common, t_organizations, t_consumers);

  TfrxOutlookIOTransportForm = class(TfrxOAuthTransportDialog)
  protected
    function GetHelpLink: String; override;
  public
    procedure UpdateResouces; override;
  end;

{$IFDEF DELPHI16}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxBaseOutlookIOTransport = class(TfrxBaseMailIOTransport)
  protected
    FfrxTenant: TfrxTenant;
    function FilterSection: String; override;

    function GetListLabels(aFilter: String = ''): String; virtual; abstract;
    function GetListMessages(aFilter: String = ''): String; virtual; abstract;
    function GetListMessagesContinue(NextPageToken: String): String; virtual; abstract;
    function GetMessageInfo(id: String; aFilter: String = ''): String; virtual; abstract;
    function GetListAttachments(aFilter: String = ''): String; virtual;
    function GetListDownloads(frxMessageStack: TfrxMessageStack; Id: String; aFilter: String = ''): String; virtual; abstract;
    function PrepareName(const Name: String): String;
    function PrepareUploadStream(const Source: TStream; CT: AnsiString; DestFileName: String): AnsiString;

    procedure Download(const SourceFileName: String; const Source: TStream); override;
  public
    class function TransportDialogClass: TfrxBaseTransportDialogClass; override;
    class function GetDescription: String; override;
    procedure GetDirItems(DirItems: TStrings; aFilter: String = ''); override;
    function GetOAuthURI: String; override;
    property frxTenant: TfrxTenant read FfrxTenant write FfrxTenant default t_common;
  end;

  function frx_Outlook_Initialize(ft: TfrxTenant): String;
  function frx_Outlook_Authorize_URL(ft: TfrxTenant): String;
  function frx_Outlook_GetToken_URL(ft: TfrxTenant): String;
  function frx_Outlook_ListMessages_URL(frxLabel: TfrxLabel; q: String = ''): String;
  function frx_Outlook_ListAttachments_URL(id: String): String;
  function frx_Outlook_Download_URL(frxMessage: TfrxMessage; Id: String): String;
  function frxTenantToString(ft: TfrxTenant): String;

implementation
end.
