
{******************************************}
{                                          }
{             FastReport VCL               }
{             S3 Save Filter               }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxIOTransportS3Base;

interface

{$I frx.inc}

uses
  Classes, IniFiles,
  frxIOTransportHelpers, frxBaseTransportConnection, frxIOTransportOAuthDialog,
  frxIOTransportS3Dialog, frxBaseTransportDialogForm, Dialogs;

type
  TfrxS3RequestType = (xrtS3Get, xrtS3Delete, xrtS3Put);

type
  TS3Signer = class;

  TfrxS3IOTransportForm = class(TfrxS3TransportDialog)
  protected
    function GetHelpLink: String; override;
  public
    procedure UpdateResouces; override;
  end;

  TfrxBucketStyle = (bsVirtualHosted = 0, bsPath = 1);

{$IFDEF DELPHI16}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxBaseS3IOTransport = class(TfrxHTTPIOTransport)
  protected
    FSigner: TS3Signer;
    FRegion: String;
    FBucketStyle: TfrxBucketStyle;

    // Processing FDirStack
    procedure DialogDirChange(Name, Id: String; DirItems: TStrings); override;
    procedure DialogDirCreate(Name: String; DirItems: TStrings); override;
    procedure DialogFileDelete(Name, Id: String; DirItems: TStrings); override;
    procedure DialogDirDelete(Name, Id: String; DirItems: TStrings); override;
    procedure CreateRemoteDir(DirName: String; ChangeDir: Boolean = True); override;
    procedure ChangeDirUP; override;
    procedure AddToDirItems(DirItems: TStrings; IsFolder, IsFile: Boolean;
      Name: String; Id: String = ''); override;

    // Parent override
    function FilterSection: String; override;
    function AlwaysUseIniFile: Boolean; override;
    function LoadFromIni(const AIni: TCustomIniFile): Boolean; override;

    // Support FDirStack
    function GetPathPrefix: String;

    // Get/Set
    function GetBucket: String;
    procedure SetBucket(const ABucket: String);
    function GetHostWithBucket: String;

    // Query
    procedure CreateFolder(const ADir: String);
    procedure DeleteFileOrFolder(const AId: String);
    function GetListFolderBucket: String;
    function GetListFolderFiles(const ANextMarker: String = ''): String;
    procedure Upload(const Source: TStream; DestFileName: String = ''); override;
    procedure Download(const SourceFileName: String; const Source: TStream); override;
    
    // Parsing
    procedure GetDirItemsBucket(const DirItems: TStrings; const aFilter: String);
    procedure GetDirItemsFolder(const DirItems: TStrings; const aFilter: String);
    
    // Query realization
    function DoQuery(const AReqType: TfrxS3RequestType; const ABucket: String = '';
      const APostfixHost: String = ''; const AQuery: String = '';
      const AContentStream: TStream = nil; const AIsNeedContentLength: Boolean = False): String; virtual; abstract;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    
    class function TransportDialogClass: TfrxBaseTransportDialogClass; override;
    class function GetDescription: String; override;
    
    function TestWithOutToken: String; override;
    procedure GetDirItems(DirItems: TStrings; aFilter: String = ''); override;
    function GetOAuthURI: String; override;

    property Signer: TS3Signer read FSigner write FSigner;
    property HostWithBucket: String read GetHostWithBucket;
  published
    property Host;
    property Bucket: String read GetBucket write SetBucket;
    property Region: String read FRegion write FRegion;
    property BucketStyle: TfrxBucketStyle read FBucketStyle write FBucketStyle;
  end;

  TS3Signer = class
  private
    FHost: String;
    FAccessKeyId: String;
    FSecretAccessKey: String;
    FRegion: String;
    FService: String;

    function GetAmzDate(const ADateTime: TDateTime): String;
    function GetURLDate(const ADateTime: TDateTime): String;
    function CalcPayloadHash(const AContentStream: TStream): String;
    function GetCanonicalUri(const AURLPostFix: String): String;
    function GetCanonicalHeaders(const AHeaders: TStrings; var ASignedHeaders: String): String;
    function CalcSignature(const AStringToSign, ANowDate: String): String;
  public
    procedure SetData(const AHost, AAccessKeyId, ASecretAccessKey, ARegion: String;
      const AService: String = 's3'); overload;
    procedure SetData(const AHost: String); overload;
    function Sign(const AHeaders: TStrings; const AReqType: TfrxS3RequestType;
      const AURLPostFix, AQuery: String; const AContentStream: TStream): String;
  end;

implementation
end.
