{******************************************}
{                                          }
{             FastReport VCL               }
{           Basic InPlace Editors          }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxInPlaceEditors;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}
  Windows,
  {$ELSE}
  LCLType, LazHelper,
  {$ENDIF}
  Types, Classes, SysUtils, Graphics, Controls, StdCtrls, ComCtrls, Forms,
  Menus, TypInfo, Dialogs, frTreeView, frxClass, frxUtils, frxGraphicControls,
  frxRes, frxUnicodeCtrls, frUnicodeUtils, Buttons, frPopupForm, Variants;

type
  TfrOwnerDrawState = set of (fodSelected, fodGrayed, fodDisabled, fodChecked, fodDefault); // TODO: move to frxGraphicControls or base Controls

  TfrxInPlaceMemoEditorBase = class(TfrxInPlaceEditor)
  private
    procedure LinesChange(Sender: TObject);
    procedure MemoKeyPress(Sender: TObject; var Key: Char);
    procedure DoExit(Sender: TObject);
    procedure MemoKeyDown(Sender: TObject; var Key: Word;
    Shift: TShiftState);
  protected
    FInPlaceMemo: TCustomMemo;
    FEdited: Boolean;
    procedure InitControlFromComponent; virtual;
    procedure InitComponentFromControl; virtual;
    procedure EditDone;
    procedure CreateMemo; virtual; abstract;
  public
    constructor Create(aClassRef: TfrxComponentClass;
      aOwner: TWinControl); override;
    destructor Destroy; override;
    function HasCustomEditor: Boolean; override;
    function DoMouseUp(X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams): Boolean; override;
    procedure EditInPlace(aParent: TComponent; aRect: TRect); override;
    function EditInPlaceDone: Boolean; override;
    function DoMouseWheel(Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var EventParams: TfrxInteractiveEventsParams): Boolean; override;
    procedure FinalizeUI(var EventParams: TfrxInteractiveEventsParams); override;
  end;

  TfrxInPlaceTextEditorBase = class(TfrxInPlaceMemoEditorBase)
  protected
    procedure CreateMemo; override;
  end;

  TfrxInPlaceMemoEditor = class(TfrxInPlaceTextEditorBase)
  protected
    procedure InitControlFromComponent; override;
    procedure InitComponentFromControl; override;
  end;

  TfrxInPlaceBaseListBoxEditor = class(TfrxInPlaceEditor)
  private
    FListBox: TListBox;
    procedure DoListBoxClick(Sender: TObject);
    procedure DoDrawItem(AControl: TWinControl; AIndex: Integer; ARect: TRect;
      AState: TOwnerDrawState);
    procedure SetListItems(const Value: TStrings);
    function GetListItems: TStrings;
    function GetListItemIndex: Integer;
    procedure SetListItemIndex(const Value: Integer);
  protected
    FDrawDragDrop: Boolean;
    FDrawDropDown: Boolean;
    FDrawButton: Boolean;
    FModified: Boolean;
    FRect: TRect;
    FPopUpRect: TRect;
    FPopupForm: TfrPopupForm;
    FButtonSize: Integer;
    function ListBoxClicked(Sender: TObject): Boolean; virtual; abstract;
    procedure ListBoxDrawItem(ACanvas: TCanvas; AControl: TWinControl; AIndex: Integer; ARect: TRect;
      AState: TfrOwnerDrawState); virtual; abstract;
    procedure DoPopupHide(Sender: TObject);
    function GetButtonSize: Integer; virtual;
    function GetButtonTopLeft: TPoint; virtual;
    function GetButtonBevelWidth: Integer;
    function GetButtonRect: TRect;
    function GetItemHeight: Integer;
    function IsFillListBox: Boolean; virtual; abstract;
    function IsDropDownButtonVisible: Boolean; virtual;
    procedure CalcPopupBounds(out ATopLeft: TPoint; out AWidth, AHeight: Integer; ADPI: Integer); virtual; abstract;
    function ListBoxCreate: TListBox; virtual;
    procedure UpdateRect; virtual;
    property ListItems: TStrings read GetListItems write SetListItems;
    property ListItemIndex: Integer read GetListItemIndex write SetListItemIndex;
    procedure DrawButton(ACanvas: TCanvas; ARect: TRect); virtual;
  public
    constructor Create(aClassRef: TfrxComponentClass; aOwner: TWinControl); override;
    function GetActiveRect: TRect; override;
    function ShowPopup(aParent: TComponent; aRect: TRect; X, Y: Integer): Boolean; virtual;
    procedure DrawCustomEditor(ACanvas: TCanvas; ARect: TRect); override;
    procedure InitializeUI(var EventParams: TfrxInteractiveEventsParams); override;
    procedure FinalizeUI(var EventParams: TfrxInteractiveEventsParams); override;
    function DoMouseDown(X, Y: Integer; Button: TMouseButton; Shift: TShiftState;
      var EventParams: TfrxInteractiveEventsParams): Boolean; override;
    function DoMouseMove(X, Y: Integer; Shift: TShiftState;
      var EventParams: TfrxInteractiveEventsParams): Boolean; override;
  end;

  TfrxInPlaceCustomListBoxEditor = class(TfrxInPlaceBaseListBoxEditor)
  private
  protected
    procedure CalcPopupBounds(out ATopLeft: TPoint; out AWidth, AHeight: Integer; ADPI: Integer); override;
    function ListBoxCreate: TListBox; override;
  end;

  TfrxInPlaceDataFiledEditor = class(TfrxInPlaceCustomListBoxEditor)
  private
    function GetParentDS: TfrxDataSet;
  protected
    function ListBoxClicked(Sender: TObject): Boolean; override;
    function IsFillListBox: Boolean; override;
    procedure ListBoxDrawItem(ACanvas: TCanvas; AControl: TWinControl; AIndex: Integer; ARect: TRect;
      AState: TfrOwnerDrawState); override;
    function IsDropDownButtonVisible: Boolean; override;
  public
    function DoCustomDragOver(Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean;
      var EventParams: TfrxInteractiveEventsParams): Boolean; override;
    function DoCustomDragDrop(Source: TObject; X, Y: Integer;
      var EventParams: TfrxInteractiveEventsParams): Boolean; override;
  end;

  TfrxInPlaceBasePanelEditor = class(TfrxInPlaceEditor)
  private
    FButtonsPanel: TfrxSwitchButtonsPanel;
    FSwitchMode: Boolean;
    FPosition: TPoint;
    function DestroyPanel: Boolean;
  protected
    FMouseDown: Boolean;
    function GetItem(Index: Integer): Boolean; virtual; abstract;
    procedure SetItem(Index: Integer; const Value: Boolean); virtual; abstract;
    function Count: Integer; virtual; abstract;
    function GetName(Index: Integer): String; virtual; abstract;
    function GetColor(Index: Integer): TColor; virtual; abstract;
  public
    destructor Destroy; override;
    procedure AfterConstruction; override;
    procedure InitializeUI(var EventParams: TfrxInteractiveEventsParams); override;
    procedure FinalizeUI(var EventParams: TfrxInteractiveEventsParams); override;
    function GetActiveRect: TRect; override;
    function DoMouseDown(X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams): Boolean; override;
    function DoMouseUp(X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams): Boolean; override;
    procedure DrawCustomEditor(aCanvas: TCanvas; aRect: TRect); override;
    property Item[Index: Integer]: Boolean read GetItem write SetItem;
  end;

implementation
end.
