
{******************************************}
{                                          }
{             FastReport VCL               }
{             Object Inspector             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxInsp;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}
  Windows, Messages,
  {$ELSE}
  LResources, LCLType, LMessages, LazHelper, LCLProc, LCLIntf,
  {$ENDIF}
  {$IFDEF UseTabset} Tabs,{$ENDIF}
  {$IFDEF DELPHI16} System.UITypes, {$ENDIF}
  Types, SysUtils, Classes, Graphics, Controls, Forms, Variants,
  Dialogs, ExtCtrls, StdCtrls, Buttons, frxDsgnIntf, frPopupForm,
  frxClass, Menus, ComCtrls, frxBaseForm, frxCtrls, frxDock, frTypes;

type
  { TfrxObjectInspector }

  TfrxObjectInspector = class(TfrxDockForm)
    ObjectsCB: TComboBox;
    PopupMenu1: TPopupMenu;
    N11: TMenuItem;
    N21: TMenuItem;
    N31: TMenuItem;
    BackPanel: TPanel;
    Box: TScrollBox;
    PB: TPaintBox;
    Edit1: TEdit;
    EditPanel: TPanel;
    EditBtn: TSpeedButton;
    ComboPanel: TPanel;
    ComboBtn: TSpeedButton;
    HintPanel: TScrollBox;
    Splitter1: TSplitter;
    PropL: TLabel;
    DescrL: TLabel;
    N41: TMenuItem;
    N51: TMenuItem;
    N61: TMenuItem;
    LInstruction: TLabel;
    procedure PBPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure PBMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PBMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PBMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Edit1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Edit1KeyPress(Sender: TObject; var Key: Char);
    procedure EditBtnClick(Sender: TObject);
    procedure ComboBtnClick(Sender: TObject);
    procedure Edit1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ObjectsCBClick(Sender: TObject);
    procedure ObjectsCBDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure PBDblClick(Sender: TObject);
    procedure FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure FormEndDock(Sender, Target: TObject; X, Y: Integer);
    procedure ComboBtnMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure TabChange(Sender: TObject);
    procedure N11Click(Sender: TObject);
    procedure N21Click(Sender: TObject);
    procedure N31Click(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ObjectsCBDropDown(Sender: TObject);
  private
    { Private declarations }
    FDesigner: TfrxCustomDesigner;
    FDisableDblClick: Boolean;
    FDisableUpdate: Boolean;
    FDown: Boolean;
    FEventList: TfrxPropertyList;
    FHintWindow: THintWindow;
    FItemIndex: Integer;
    FLastPosition: String;
    FList: TfrxPropertyList;
    FPopupForm: TfrPopupForm;
    FPopupLB: TListBox;
    FPopupLBVisible: Boolean;
    FPropertyList: TfrxPropertyList;
    FPanel: TPanel;
    FRowHeight: Integer;
    FSelectedObjects: TfrxSelectedObjectsList;
    FSplitterPos: Integer;
{$IFDEF UseTabset}
    FTabs: TTabSet;
{$ELSE}
    FTabs: TTabControl;
{$ENDIF}
    {$IFNDEF FPC}
    FTempBMP: TBitmap;
    {$ENDIF}
    FTempList: TfrxSelectedObjectsList;
    FTickCount: UInt;
    FUpdatingObjectsCB: Boolean;
    FUpdatingPB: Boolean;
    FOnSelectionChanged: TNotifyEvent;
    FOnModify: TNotifyEvent;
    FComboBtnGlyph: TBitmap;
    FEditBtnGlyph: TBitmap;
    FFilter: String;
    FFilterEdit: TfrxFilterEdit;
    FFileterMinWidth: Integer;
    FFileterMaxWidth: Integer;
    FColor: TColor;
    FSavedPositions: TStringList;
    FSavePosLocked: Boolean;
    FMaxSavedPos: Integer;
    FFavorPress: Boolean;
    FIsFavorZone: Boolean;
    FFavorites: TStringList;
    FFullList, FFavoritesList: TfrxPropertyList;
    function Count: Integer;
    function GetItem(Index: Integer): TfrxPropertyItem;
    function GetOffset(Index: Integer): Integer;
    function GetType(Index: Integer): TfrxPropertyAttributes;
    function GetValue(Index: Integer): String;
    procedure AdjustControls;
    procedure CMMouseLeave(var Msg: TMessage); message CM_MouseLeave;
    procedure DrawOneLine({$IFDEF FPC}ACanvas: TCanvas;{$ENDIF}
      i: Integer; Selected: Boolean);
    procedure DoModify;
    procedure SetObjects(Value: TList);
    procedure SetItemIndex(Value: Integer);
    procedure SetSelectedObjects(Value: TfrxSelectedObjectsList);
    procedure SetValue(Index: Integer; Value: String);
    procedure LBClick(Sender: TObject);
    procedure FilterChange(Sender: TObject);
    function GetSplitter1Pos: Integer;
    procedure SetSplitter1Pos(const Value: Integer);
    procedure SetFilter(const Value: String);
    procedure SetFileterMaxWidth(const Value: Integer);
    procedure SetFileterMinWidth(const Value: Integer);
    procedure PBOnDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure PBOnDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure RestorePosition;
    procedure SavePosition;
    procedure SetFavoritesActive(const Value: Boolean);
    function GetFavoritesActive: Boolean;
    procedure RebildFavorites;
    procedure PBRePaint;
    function FindProp(PropertyList: TfrxPropertyList; FullName: String): Integer;
  protected
    function GetCaptionFromResources: string; override;
    procedure LoadFormPreferences(PreferencesStorage: TObject; DefPreferencesStorage: TObject); override;
    procedure SaveFormPreferences(PreferencesStorage: TObject; DefPreferencesStorage: TObject); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DisableUpdate;
    procedure EnableUpdate;
    function GetName(Index: Integer): String;
    procedure Inspect(AObjects: array of TPersistent);
    procedure HostControls(Host: TWinControl); override;
    procedure SetColor(Color: TColor);
    procedure UpdateProperties;
    procedure UpdateFormPPI(aNewPPI: Integer); override;
    procedure UpdateResouces; override;
    procedure UnhostControls(AModalResult: TModalResult); override;
    procedure SetActiveProperty(const PropName: String);
    property Objects: TList write SetObjects;
    property ItemIndex: Integer read FItemIndex write SetItemIndex;
    property Filter: String read FFilter write SetFilter;
    property FileterMinWidth: Integer read FFileterMinWidth write SetFileterMinWidth;
    property FileterMaxWidth: Integer read FFileterMaxWidth write SetFileterMaxWidth;
    property FavoritesAcrive: Boolean read GetFavoritesActive write SetFavoritesActive;
    property SelectedObjects: TfrxSelectedObjectsList read FSelectedObjects write SetSelectedObjects;
    property SplitterPos: Integer read FSplitterPos write FSplitterPos;
    property Splitter1Pos: Integer read GetSplitter1Pos write SetSplitter1Pos;
    property OnModify: TNotifyEvent read FOnModify write FOnModify;
    property OnSelectionChanged: TNotifyEvent read FOnSelectionChanged write FOnSelectionChanged;
  end;

  const FavorButt = ssCtrl;

implementation
end.
