{******************************************}
{                                          }
{             FastReport VCL               }
{         ListBox Add-In Object            }
{                                          }
{         Copyright (c) 1998-2022          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxListBox;

interface

{$I frx.inc}

uses
  {$IFNDEF Linux}
    Windows,
  {$ELSE}
    LCLType, LCLIntf, LCLProc,
  {$ENDIF}
  Classes, Graphics, Types, Controls,
  frxClass, frxListControl;

type
  TScrollBoxMousePlace = (cpAbove, cpCenter, cpBelow);

  TfrxVerticalScrollBar = class(TfrxRightEdge)
  private
    FContentHeight: Extended;
    FControlHeight: Extended;
    FContentLineCount: Integer;
    FTopLineIndex: Integer;
    FDownTopLineIndex: Integer;
    FMouseCaptured: Boolean;
    FDownPlace: TScrollBoxMousePlace;
    FDownY: Integer;
    FCurrentY: Integer;
    FScrollBoxTop: Extended;
    FScrollBoxBottom: Extended;
    FScrollBoxStep: Extended;
    FMaxTopLineIndex: Integer;
    FWheelStepNumber: Integer;
    FInitialized: Boolean;

    procedure SetTopLineIndex(const Value: Integer);
  protected
    function CalcPlace(Y: Integer): TScrollBoxMousePlace;
    procedure SquareLineTo(Canvas: TCanvas; X, Y: Integer);
    function MixColor(C1, C2: TColor): TColor;
  public
    procedure Init(AContentHeight, AControlHeight: Extended; Rect: TRect; AContentLineCount: Integer);
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY: Extended); override;
    function IsVisible: Boolean;

    function DoMouseDown(X, Y: Integer): Boolean;
    procedure DoMouseUp;
    procedure DoMouseMove(X, Y: Integer);
    function DoMouseWheel(WheelDelta: Integer): Boolean;

    property MouseCaptured: Boolean read FMouseCaptured;
    property TopLineIndex: Integer read FTopLineIndex write SetTopLineIndex;
    property Initialized: Boolean read FInitialized;
  end;

{$IFDEF DELPHI16}
  /// <summary>
  ///   The TfrxListBoxObject allows the use of the TfrxListBoxView component in
  ///   your report. TfrxListControlObject is an empty component. It is used to
  ///   add the frxListBox.pas file to the "uses" list.
  /// </summary>
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxListBoxObject = class(TComponent)  // fake component
  end;

  /// <summary>
  ///   TfrxListBoxView displays a collection of items in a scrollable list.
  ///   Use TfrxListBoxView to display a scrollable list of items that users can
  ///   select, add, or delete.
  /// </summary>
  TfrxListBoxView = class(TfrxCustomListControlView)
  private
    function GetTopLineIndex: Integer;
  protected
    FContentHeight: Extended;
    FMouseCaptured: Boolean;
    FScrollBar: TfrxVerticalScrollBar;

    function IsChangeItemIndex(const Y: Integer): Boolean;
    function IsScrollBarAllowed: Boolean;
    function NoScrollBar(Rect: TRect): TRect; override;
    procedure DrawText;
    procedure SetDrawParams(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended);
    procedure FillMemoText(Memo: TfrxCustomMemoView); override;

    function DoMouseDown(X, Y: Integer; Button: TMouseButton; Shift: TShiftState;
      var EventParams: TfrxInteractiveEventsParams): Boolean; override;
    procedure DoMouseUp(X, Y: Integer; Button: TMouseButton; Shift: TShiftState;
      var EventParams: TfrxInteractiveEventsParams); override;
    procedure DoMouseMove(X, Y: Integer; Shift: TShiftState;
      var EventParams: TfrxInteractiveEventsParams); override;
    function DoMouseWheel(Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint;
      var EventParams: TfrxInteractiveEventsParams): Boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function GetDescription: string; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;

    property TopLineIndex: Integer read GetTopLineIndex;
  published
    property AllowExpressions;
    property BrushStyle;
    property Color;
    property Cursor;
    property DataField;
    property DataSet;
    property DataSetName;
    property Editable default [ferAllowInPreview];
    property ExpressionDelimiters;
    property Font;
    property Frame;
    property FillType;
    property Fill;
    property ItemIndex; // hidden
    property Items;
    property ItemsText; // hidden
    property ParentFont;
    property Style;
    property TagStr;
  end;

implementation
end.
