
{******************************************}
{                                          }
{             FastReport VCL               }
{        ListControl Add-In Object         }
{                                          }
{         Copyright (c) 1998-2022          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxListControl;

interface

{$I frx.inc}

uses
{$IFDEF FPC}
  LazHelper,
{$ELSE}
  WideStrings,
{$ENDIF}
  Classes, Types, Graphics, Controls, 
  frCoreClasses, frUnicodeUtils, frxClass, frGraphicUtils;

const
  DefaultExpressionDelimiters = '[,]';

type
  /// <summary>
  ///   TfrxCustomListControlView is the base class for TfrxListBoxView and
  ///   TfrxComboBoxView
  /// </summary>
  TfrxCustomListControlView = class(TfrxStyledView)
  private
    FAllowExpressions: Boolean;
    FExpressionDelimiters: String;
    FItemIndex: Integer;

    procedure SetItems(const Value: TWideStrings);
    function IsExprDelimitersStored: Boolean;
    function GetItemsText: WideString;
    procedure SetItemsText(const Value: WideString);
  protected
    FItems: TWideStrings;
    FDrawText: TfrxDrawText;
    FLineY: TIntegerDynArray;
    FLineSpacing: Integer;
    procedure SetItemIndex(const Value: Integer); virtual;

    function XCorrected(const X: Extended): Integer;
    function YCorrected(const Y: Extended): Integer;

    procedure DrawTextInit; // Before use
    procedure DrawTextDimensions(ScaleX, ScaleY, OffsetX, OffsetY: Extended);
    function CalcTextRect(OffsetX, OffsetY, ScaleX, ScaleY: Extended): TRect;
    function NoScrollBar(Rect: TRect): TRect; virtual;

    procedure LoadFromDataField;
    procedure GetExpressionDelimiters(out LeftDlm, RightDlm: WideString);
    function IsPreview(EventParams: TfrxInteractiveEventsParams): Boolean;
    function IsLeft(Button: TMouseButton): Boolean;

    procedure FillMemoText(Memo: TfrxCustomMemoView); virtual; abstract;
    /// <summary>
    ///   Determines if the object may contain expressions inside the text.
    ///   Expressions are enclosed in the square brackets (by default). The
    ///   ExpressionDelimiters defines which symbols to use for detecting the
    ///   expression. Default value is True.
    /// </summary>
    property AllowExpressions: Boolean read FAllowExpressions write FAllowExpressions default True;
    property ItemIndex: Integer read FItemIndex write SetItemIndex;
    property ItemsText: WideString read GetItemsText write SetItemsText;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GetData; override;

    procedure FillMemo(Memo: TfrxCustomMemoView);

    property Items: TWideStrings read FItems write SetItems;
    /// <summary>
    ///   Set of symbols, designating the expression. Default value is '[,]'.
    ///   The comma divides opening and closing symbols. There is one
    ///   limitation however: the opening and closing symbols cannot be
    ///   similar, so '%,%' will not work. One can set several symbols, for
    ///   example '&lt;%,%&gt;'.
    /// </summary>
    property ExpressionDelimiters: string read FExpressionDelimiters
      write FExpressionDelimiters stored IsExprDelimitersStored;
  end;

  TfrxRightEdge = class
  protected
    FRect: TRect;

    function IsInside(X, Y: Integer): Boolean;
  public
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY: Extended); virtual;
    procedure Init(Rect: TRect);
  end;

implementation
end.
