
{******************************************}
{                                          }
{             FastReport VCL               }
{                Map Object                }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxMap;

interface

{$I frx.inc}

uses
  Types,
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLType, LCLIntf, LCLProc,
{$ENDIF}
  Graphics, Classes, Controls, frxClass, frxMapLayer,
  frxMapHelpers, frxMapRanges, frxMapShape, frxMapLayerForm;

type
{$IFDEF DELPHI16}
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxMapObject = class(TComponent)  // fake component
  end;
{$IFDEF DELPHI16}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxMapView = class(TfrxStyledView)
  private
    FZoom: Extended;
    FMinZoom: Extended;
    FMaxZoom: Extended;
    FMapOffsetX: Extended;
    FMapOffsetY: Extended;
    FKeepAspectRatio: Boolean;
    FMercatorProjection: boolean;
{$IFDEF FRX_USE_BITMAP_MAP}
    FBitmapCache: TBitmap;
    FRepaintCache: Boolean;
{$ENDIF}
    procedure SetZoom(const Value: Extended);
    procedure SetMaxZoom(const Value: Extended);
    procedure SetMinZoom(const Value: Extended);
    procedure SetMercatorProjection(const Value: Boolean);
    function GetSelectedShapeName: String;
  protected
    FLayers: TMapLayerList;
    FHasPreviousOffset: Boolean;
    FPreviousOffset: TfrxPoint;
    FShowMoveArrow: Boolean;
    FConverter: TMapToCanvasCoordinateConverter;
    FFirstDraw: Boolean;
    FPreviousHyperlinkKind: TfrxHyperlinkKind;
    FNeedBuildVector: Boolean;
    FColorScale: TMapScale;
    FSizeScale: TMapScale;
    FOSMFileList: TOSMFileList;
    FClipMap: Boolean;
    oldLeft, oldTop, oldWidth, oldHeight, oldOffsetX, oldOffsetY, oldZoom: Extended;
    FModified: Boolean;
    FAddingLayer: Boolean;
{$IFDEF FRX_USE_BITMAP_MAP}
    FMapViewport: TfrxRect;
{$ENDIF}
    procedure DrawMap(Canvas: TCanvas; ScaleX, ScaleY: Extended);
    function IsMoveArrowArea(X, Y: Extended): Boolean;
    function CanvasSize: TfrxPoint;
    procedure DefineProperties(Filer: TFiler); override;
    procedure InitConverter;
    function IsAlignByZoomPolygon: Boolean;
    procedure ZoomRecenter(CenterX, CenterY, Factor: Extended);
    procedure EnableSupportedHyperlink;
    procedure DisableSupportedHyperlink;
    function ShowMoveArrow: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    procedure DrawClipped(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    function GetContainerObjects: TList; override;
    function IsContain(X, Y: Extended): Boolean; override;
    procedure GetData; override;
    procedure ExpandVar(var Expr: String);
    procedure AddLayer(LayerType: TLayerType; IsEmbed: Boolean; AMapFileName: string; DefaultReport: TfrxReport = nil);
    procedure GeometrySave;
    procedure GeometryChange(ALeft, ATop, AWidth, AHeight: Extended);
    procedure GeometryRestore;
    procedure ZoomByRect(ZoomRect: TfrxRect);
    procedure ZoomByFactor(Factor: Extended);
    procedure SwapConverter(EditedMap: TfrxMapView);

    function DoMouseWheel(Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var EventParams: TfrxInteractiveEventsParams): Boolean; override;
    function DoMouseDown(X, Y: Integer; Button: TMouseButton; Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams): Boolean; override;
    procedure DoMouseMove(X, Y: Integer; Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams); override;
    procedure DoMouseUp(X, Y: Integer; Button: TMouseButton; Shift: TShiftState; var EventParams: TfrxInteractiveEventsParams); override;
    procedure DoMouseEnter(aPreviousObject: TfrxComponent; var EventParams: TfrxInteractiveEventsParams); override;
    procedure DoMouseLeave(aNextObject: TfrxComponent; var EventParams: TfrxInteractiveEventsParams); override;

    property Converter: TMapToCanvasCoordinateConverter read FConverter;
    property SelectedShapeName: String read GetSelectedShapeName;
    property Layers: TMapLayerList read FLayers;
    property OSMFileList: TOSMFileList read FOSMFileList;
    property ClipMap: Boolean read FClipMap write FClipMap;
    property NeedBuildVector: Boolean read FNeedBuildVector write FNeedBuildVector;
{$IFDEF FRX_USE_BITMAP_MAP}
    property MapViewport: TfrxRect read FMapViewport;
{$ENDIF}
  published
    property Font;
    property FillType;
    property Fill;
    property Frame;
    property Cursor;
    property Editable default [ferAllowInPreview];

    property Zoom: Extended read FZoom write SetZoom;
    property MaxZoom: Extended read FMaxZoom write SetMaxZoom;
    property MinZoom: Extended read FMinZoom write SetMinZoom;
    property OffsetX: Extended read FMapOffsetX write FMapOffsetX;
    property OffsetY: Extended read FMapOffsetY write FMapOffsetY;
    property KeepAspectRatio: Boolean read FKeepAspectRatio write FKeepAspectRatio;
    property MercatorProjection: Boolean read FMercatorProjection write SetMercatorProjection;

    property ColorScale: TMapScale read FColorScale;
    property SizeScale: TMapScale read FSizeScale;
    property Style;
  end;

implementation
end.
