
{******************************************}
{                                          }
{             FastReport VCL               }
{             Map Color Ranges             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxMapColorRangeForm;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}
  Windows, Messages,
  {$ELSE}
  LCLType, LMessages, LCLIntf, LCLProc, ColorBox,
  {$ENDIF}
  SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons, frxBaseForm,
  frxMapRanges, frxMapHelpers, frxAnaliticGeometry;

type
  TColorRangeItem = class(TMapRangeItem)
  private
    FAutoColor: Boolean;
    FColor: TColor;

    procedure SetColor(const Value: TColor);
    procedure SetColorByForce(const Value: TColor);
  protected
    procedure AssignTo(Dest: TPersistent); override;
  public
    constructor Create(Collection: TCollection); override;
    procedure Read(Reader: TReader); override;
    procedure Write(Writer: TWriter); override;
    function IsValueInside(Value: Extended; var OutColor: TColor): Boolean;
    function AsString(FValueFormat: String): String; override;
  published
    property AutoColor: Boolean read FAutoColor write FAutoColor;
    property Color: TColor read FColor write SetColor;
    property ColorByForce: TColor read FColor write SetColorByForce;
  end;
(******************************************************************************)
  TColorRangeCollection = class(TMapRangeCollection)
  protected
    function GetItem(Index: Integer): TColorRangeItem;
    procedure SetItem(Index: Integer; const Value: TColorRangeItem);

    function GetMixedColor(C1, C2: TColor; p: Extended): TColor;
  public
    constructor Create;
    function Add: TColorRangeItem;
    function GetColor(Value: Extended): TColor;
    function GetPeerColor(StartColor, MiddleColor, EndColor: TColor; Value: Extended): TColor;
    function GetPeerPartColor(StartColor, MiddleColor, EndColor: TColor; Part: Extended): TColor;
    procedure FillRangeColors(StartIndex, EndIndex: Integer);

    property Items[Index: Integer]: TColorRangeItem read GetItem write SetItem; default;
  end;
(******************************************************************************)
  TMapColorRangeForm = class(TfrxBaseForm)
    CollectionListBox: TListBox;
    CancelB: TButton;
    OkB: TButton;
    ColorColorBox: TColorBox;
    StartEdit: TEdit;
    EndEdit: TEdit;
    ColorCheckBox: TCheckBox;
    StartCheckBox: TCheckBox;
    EndCheckBox: TCheckBox;
    AutoLabel: TLabel;
    AddButton: TButton;
    DeleteButton: TButton;
    UpSpeedButton: TSpeedButton;
    DownSpeedButton: TSpeedButton;
    Bevel1: TBevel;

    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure CollectionListBoxClick(Sender: TObject);
    procedure OkBClick(Sender: TObject);
    procedure ColorCheckBoxClick(Sender: TObject);
    procedure StartEditChange(Sender: TObject);
    procedure AddButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure UpSpeedButtonClick(Sender: TObject);
    procedure DownSpeedButtonClick(Sender: TObject);
  private
    FEditedCollection: TColorRangeCollection;
    FValueFormat: String;
    FPreviousItemIndex: Integer;

    procedure RefillCollectionListBox(const ItemIndex: integer);
    procedure Syncronize;
    procedure UpdateControls;
  public
    procedure SetCollection(CRCollection: TColorRangeCollection);
    function GetCollection: TColorRangeCollection;
    procedure UpdateResouces; override;
    procedure UpdateFormPPI(aNewPPI: Integer); override;
    property ValueFormat: String read FValueFormat write FValueFormat;
  end;
(******************************************************************************)
  TfrxColorRanges = class(TMapRanges)
  private
    FColorRangeCollection: TColorRangeCollection;
    FStartColor: TColor;
    FMiddleColor: TColor;
    FEndColor: TColor;

    procedure SetColorRangeCollection(const Value: TColorRangeCollection);
  protected
    procedure DrawContent(Canvas: TCanvas); override;

    procedure FillRangeColors;
  public
    constructor Create(MapScale: TMapScale);
    procedure Fill(const Values: TDoubleArray);

    property ColorRangeCollection: TColorRangeCollection read FColorRangeCollection write SetColorRangeCollection;
  published
    function GetColor(Value: Extended): TColor;

    property StartColor: TColor read FStartColor write FStartColor;
    property MiddleColor: TColor read FMiddleColor write FMiddleColor;
    property EndColor: TColor read FEndColor write FEndColor;
  end;
(******************************************************************************)
implementation
end.
