
{******************************************}
{                                          }
{             FastReport VCL               }
{         Map Interactive Layer Editor     }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxMapILEditor;

interface

{$I frx.inc}

uses
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLType, LCLIntf, LCLProc,
{$ENDIF}
  Classes, Graphics, Controls, Forms, Buttons, ComCtrls, StdCtrls,
  ExtCtrls, ToolWin, Dialogs, Menus, Messages,
  frxClass, frxMapInteractiveLayer, frxMap, frxMapHelpers, frxMapShape,
  frxAnaliticGeometry, frxPolygonTemplate, frxBaseForm, frTypes;

type
  TCanvasMapPair = class
  private
    FCanvasPoint: TfrxPoint;
    FMapPoint: TfrxPoint;
    FConverter: TMapToCanvasCoordinateConverter;
    procedure SetCanvasPoint(const Value: TfrxPoint);
    procedure SetMapPoint(const Value: TfrxPoint);
  public
    constructor Create(Converter: TMapToCanvasCoordinateConverter);
    constructor CreateClone(CanvasMapPair: TCanvasMapPair);
    procedure CalcCanvas;
    procedure CalcMap;
    procedure CanvasShift(dX, dY: Extended);

    property CanvasPoint: TfrxPoint read FCanvasPoint write SetCanvasPoint;
    property MapPoint: TfrxPoint read FMapPoint write SetMapPoint;
  end;

  TGraphicView = record
    PenWidth: Integer;
    PenStyle: TPenStyle;
    PenColor: TColor;
    BrushStyle: TBrushStyle;
    BrushColor: TColor;
  end;

  TFigureCreateRecord = record
    Converter: TMapToCanvasCoordinateConverter;
    Canvas: TCanvas;
    X, Y: Integer;
    Shape: TShape;
  end;
function FigureCreateRecord(Converter: TMapToCanvasCoordinateConverter;
  Canvas: TCanvas; X, Y: Integer; Shape: TShape): TFigureCreateRecord;

type
  TVirtualCursor = (vcDefault, vcCanSelect,
    vcMoveMobile, vcFixAndAdd, vcFix, vcUnfix, vcUnfixSegment,
    vcSize, vcSizeNESW, vcSizeNS, vcSizeNWSE, vcSizeWE);

  TAbstractFigure = class
  protected
    FShape: TShape;
    FCanvas: TCanvas;
    FConverter: TMapToCanvasCoordinateConverter;

    FLastPoint: TCanvasMapPair;

    procedure ReadShapeData; virtual; abstract;

    procedure SetupGraphics(GV: TGraphicView);
    procedure DrawCircle(Center: TfrxPoint; GV: TGraphicView; Radius: Integer = Unknown);
    procedure DrawLine(P1, P2: TfrxPoint; GV: TGraphicView);
    procedure DrawRect(TopLeft, BottomRight: TfrxPoint; GV: TGraphicView);
    procedure DrawPolygon(Points: TPointArray; GV: TGraphicView);
    procedure DrawEllipse(TopLeft, BottomRight: TfrxPoint; GV: TGraphicView);

    procedure DrawMobilePoint;

    function IsNearLastPoint(const X, Y: Integer): boolean;
  public
    constructor Create(FCR: TFigureCreateRecord);
    destructor Destroy; override;
    procedure Draw; virtual; abstract;
    procedure SetConstrainProportions(AConstrainProportions: Boolean); virtual; // Empty
    function ShapeName: String; virtual; abstract;
    function MouseDown(const X, Y: Integer): boolean; virtual; abstract;
    function MouseMove(const X, Y: Integer): boolean; virtual; abstract;
    procedure MouseUp(const X, Y: Integer); virtual; // Empty
    procedure RecalcCanvas; virtual;
    function GetCursor(const X, Y: Integer): TVirtualCursor; virtual; abstract;
    function ShapeType: TShapeType; virtual; abstract;
    function GetShapeData(Strings: TStrings): TShapeData; virtual; abstract;
    function IsCanDeletePoint: boolean; virtual; // False
    function IsCanDeleteShape: boolean; virtual;
    function IsCanSave: boolean; virtual; // True;
    function IsCanVK_DELETE: boolean; virtual; // True;
    procedure DeletePoint; virtual; // Empty
  end;

{$IfNdef DELPHI10}
  TPaintBox = class(ExtCtrls.TPaintBox)
  private
    FOnMouseLeave: TNotifyEvent;
    FOnMouseEnter: TNotifyEvent;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  published
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
  end;
{$EndIf}

  TfrxMapILEditorForm = class(TfrxBaseForm)
    btnCancel: TButton;
    btnOk: TButton;
    ShapeGroupBox: TGroupBox;
    TagsMemo: TMemo;
    TagsLabel: TLabel;
    CancelShapeButton: TButton;
    SaveShapeButton: TButton;
    MapPanel: TPanel;
    MapPaintBox: TPaintBox;
    RemoveShapeButton: TButton;
    PolyGroupBox: TGroupBox;
    DeletePointButton: TButton;
    PictureGroupBox: TGroupBox;
    EditPictureButton: TButton;
    ConstrainProportionsCheckBox: TCheckBox;
    LegendGroupBox: TGroupBox;
    TextMemo: TMemo;
    LabelFontButton: TButton;
    LabelFontLabel: TLabel;
    FontDialog: TFontDialog;
    TopPanel: TPanel;
    SelectSpeedButton: TSpeedButton;
    PointSpeedButton: TSpeedButton;
    PointPopupMenu: TPopupMenu;
    Point1: TMenuItem;
    Polyline1: TMenuItem;
    Polygon1: TMenuItem;
    PointMenuSpeedButton: TSpeedButton;
    RectSpeedButton: TSpeedButton;
    RectMenuSpeedButton: TSpeedButton;
    RectPopupMenu: TPopupMenu;
    Rectangle1: TMenuItem;
    Ellipse1: TMenuItem;
    Diamond1: TMenuItem;
    PictureSpeedButton: TSpeedButton;
    PictureMenuSpeedButton: TSpeedButton;
    PicturePopupMenu: TPopupMenu;
    Picture1: TMenuItem;
    Legend1: TMenuItem;
    TemplateSpeedButton: TSpeedButton;
    TemplateMenuSpeedButton: TSpeedButton;
    TemplatePopupMenu: TPopupMenu;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: boolean);
    procedure FormResize(Sender: TObject);
    procedure MapPaintBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MapPaintBoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MapPaintBoxMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure MapPaintBoxPaint(Sender: TObject);
    procedure SaveShapeButtonClick(Sender: TObject);
    procedure CancelShapeButtonClick(Sender: TObject);
    procedure RemoveShapeButtonClick(Sender: TObject);
    procedure DeletePointButtonClick(Sender: TObject);
    procedure EditPictureButtonClick(Sender: TObject);
    procedure ConstrainProportionsCheckBoxClick(Sender: TObject);
    procedure TextMemoChange(Sender: TObject);
    procedure LabelFontButtonClick(Sender: TObject);
    procedure PointMenuSpeedButtonClick(Sender: TObject);
    procedure Point1Click(Sender: TObject);
    procedure Rectangle1Click(Sender: TObject);
    procedure RectMenuSpeedButtonClick(Sender: TObject);
    procedure Picture1Click(Sender: TObject);
    procedure PictureMenuSpeedButtonClick(Sender: TObject);
    procedure MapPaintBoxMouseLeave(Sender: TObject);
    procedure TemplateMenuSpeedButtonClick(Sender: TObject);
    procedure TemplateClick(Sender: TObject);
  private
    FMap: TfrxMapView;
    FOriginalMap: TfrxMapView;
    FReportDesigner: TfrxCustomDesigner;
    FLayer: TfrxMapInteractiveLayer;
    FFigure: TAbstractFigure;
    FPreviousLines: TStringList;
    FCurentCursor: TCursor;

    procedure Change;
    procedure SetLayer(const Value: TfrxMapInteractiveLayer);
    procedure FontDialogToLabel(Lbl: TLabel);
    procedure RefreshCursor(VirtualCursor: TVirtualCursor);
    procedure LoadTemplates;
  protected
    FPointNextFigure: TShapeType;
    FRectNextFigure: TShapeType;
    FPictureNextFigure: TShapeType;

    function NextTemplateName: String;
    function IsInsidePaintBox(const X, Y: Integer): boolean;

    procedure CancelEditing;

    procedure FigureCreate(const X, Y: Integer);
    function NextFigureShapeType: TShapeType;

    function OffsetPoint(X, Y: Extended): TfrxPoint; overload;
    function OffsetPoint(P: TfrxPoint): TfrxPoint; overload;

    procedure UpdateControls;
  public
    procedure UpdateResouces; override;
    procedure UpdateFormPPI(aNewPPI: Integer); override;
    property Layer: TfrxMapInteractiveLayer read FLayer write SetLayer;
    property ReportDesigner: TfrxCustomDesigner read FReportDesigner write FReportDesigner;
  end;

implementation
end.
