
{******************************************}
{                                          }
{             FastReport VCL               }
{                Map Layer                 }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxMapLayer;

interface

{$I frx.inc}

uses
  frxClass, Contnrs,
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLType, LCLIntf, LCLProc, LazHelper, frLazGraphicClasses,
{$ENDIF}
  Graphics, Classes, frxMapShape, frxMapHelpers, frxDsgnIntf,
  frxMapColorRangeForm, frxMapSizeRangeForm, frxMapRanges, frxAnaliticGeometry, frxProtocolFactory;

type
  TMapOperation = (opSum, opAverage, opMin, opMax, opCount);
  TMapLabelKind = (mlNone, mlName, mlValue, mlNameAndValue);
  TMapPalette = (mpNone, mpLight, mpPastel, mpGrayScale, mpEarth, mpSea, mpBrightPastel);
(******************************************************************************)
  TOperationCalculator = class
  private
    FValue: Extended;
    FCount: Integer;
    FOperation: TMapOperation;
  public
    constructor Create(AOperation: TMapOperation); reintroduce;
    procedure Add(AValue: Extended);
    function Get: Extended;
  end;
(******************************************************************************)
  TValuesList = class(TStringList)
  private
    FOperation: TMapOperation;
    function GetOperationCalculator(Index: integer): TOperationCalculator;
  public
    constructor Create(AOperation: TMapOperation); reintroduce;
    destructor Destroy; override;
    procedure AddValue(SpatialValue: string; AnalyticalValue: Extended);
    function MinValue: Extended;
    function MaxValue: Extended;

    property OperationCalculator[Index: integer]: TOperationCalculator read GetOperationCalculator;
  end;
(******************************************************************************)
  TfrxCustomLayer = class(TfrxComponent)
  private
    FLabelKind: TMapLabelKind;
    FHighlightColor: TColor;
    FSelectedShapeIndex: Integer;
    FAnalyticalValue: string;
    FSpatialValue: string;
    FMapPalette: TMapPalette;
    FOperation: TMapOperation;
    FDataSet: TfrxDataSet;
    FColorRanges: TfrxColorRanges;
    FValueFormat: string;
    FFilter: string;
    FDefaultShapeStyle: TShapeStyle;
    FSizeRanges: TfrxSizeRanges;
    FPointLabelsVisibleAtZoom: Extended;
    FActive: Boolean;
    FShowLines: Boolean;
    FShowPoints: Boolean;
    FShowPolygons: Boolean;

    function GetSelectedShape: TShape;
    function GetColorRangeData: TColorRangeCollection;
    function GetSizeRangeData: TSizeRangeCollection;
  protected
    FLabelColumn: string;
    FSpatialColumn: string;
    FMapAccuracy: Extended;
    FPixelAccuracy: Extended;
    FPreviousSelectedShapeIndex: Integer;

    FValuesList: TValuesList;
    FShapes: TShapeList;
    FConverter: TMapToCanvasCoordinateConverter;
    FMapView: TComponent;
    FActiveHyperlink: Boolean;
    FVectorGraphic: TGraphic;
    FClippingRect: TfrxClippingRect;
{$IFDEF FRX_USE_BITMAP_MAP}
    FClippingMapRect: TfrxClippingRect;
{$ENDIF}
    procedure Draw(Canvas: TCanvas);

    procedure DrawClippedPoint(Canvas: TCanvas; X, Y, Radius: Extended);
    procedure DrawPoint(Canvas: TCanvas; iRecord: Integer);
    procedure DrawMultiPoint(Canvas: TCanvas; iRecord: Integer);
    procedure DrawPointLegend(Canvas: TCanvas; iRecord: Integer);

    procedure DrawPolyLine(Canvas: TCanvas; iRecord: Integer);
    procedure DrawPolygon(Canvas: TCanvas; iRecord: Integer);
    procedure DrawPolyLegend(Canvas: TCanvas; iRecord: Integer);
    procedure DrawTemplate(Canvas: TCanvas; iRecord: Integer);
    procedure DrawRect(Canvas: TCanvas; iRecord: Integer);
    procedure DrawDiamond(Canvas: TCanvas; iRecord: Integer);
    procedure DrawEllipse(Canvas: TCanvas; iRecord: Integer);
    procedure DrawPicture(Canvas: TCanvas; iRecord: Integer);
    procedure DrawLegend(Canvas: TCanvas; iRecord: Integer);
    procedure DrawHighlightFrame(Canvas: TCanvas; iRecord: Integer);

    procedure TunePoint(Canvas: TCanvas; iRecord: Integer; out Radius: Extended);
    procedure TuneBrush(Brush: TBrush; iRecord: Integer);
    procedure TunePen(Pen: TPen; iRecord: Integer);
    procedure SetParent(AParent: TfrxComponent); override;
    procedure InitTransform(iRecord: Integer); virtual; // Empty
    function GetShapeValue(iRecord: Integer): string;
    function GetShapeName(FieldName: string; iRecord: Integer): string;
    function GetShapeLegeng(FieldName: string; iRecord: Integer): string;
    function GetSelectedShapeName: string; virtual;
    function GetSelectedShapeValue: string; virtual;
    function IsHighlightSelectedShape: boolean; virtual;
    function IsHiddenShape(iRecord: Integer): Boolean; virtual; // False;
    procedure GetDesigningData;
    procedure InitialiseData; virtual;
    function IsCanGetData: Boolean; virtual;
    procedure FinaliseData;
    procedure FillRanges(const Values: TDoubleArray);
    procedure ExpandVariables; virtual;
    procedure AddValueList(vaAnalyticalValue: Variant); virtual; abstract;
    function IsIncludeAsRegion(P: TfrxPoint): Boolean; virtual;
    function IsSpecialBorderColor(iRecord: Integer; out SpecialColor: TColor): Boolean; virtual; // False;
    function IsSpecialFillColor(iRecord: Integer; out SpecialColor: TColor): Boolean; virtual; // False;
    procedure DefineProperties(Filer: TFiler); override;
    procedure DefinePrimeProperties(Filer: TFiler); virtual; // Empty
    function GetMetaFile(CanvasSize: TfrxPoint; ActiveHyperlink: Boolean): TMetaFile;

    property SelectedShapeIndex: Integer read FSelectedShapeIndex write FSelectedShapeIndex;
    property MapAccuracy: Extended read FMapAccuracy write FMapAccuracy;
    property PixelAccuracy: Extended read FPixelAccuracy write FPixelAccuracy;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DrawOn(Canvas: TCanvas; ActiveHyperlink: Boolean; aClipRect: TfrxRect);
    procedure BuildGraphic(CanvasSize: TfrxPoint; ActiveHyperlink: Boolean);
    function IsInclude(P: TfrxPoint): Boolean; virtual;
    procedure GetData;
    function IsHasMapRect(out MapRect: TfrxRect): boolean;
    function IsHasZoomRect(out ZoomRect: TfrxRect): boolean; virtual;
    function IsSelectedShapeChanded: Boolean;
    procedure GetColumnList(List: TStrings); virtual;
    procedure ClearSelectedShape;
    function IsMercatorSuitable: Boolean;
    procedure SwapShapes(EditedLayer: TfrxCustomLayer);
    procedure CopyDatasets(CustomLayer: TfrxCustomLayer); virtual;

    property ClippingRect: TfrxClippingRect read FClippingRect;
    property SelectedShape: TShape read GetSelectedShape;
    property SelectedShapeName: string read GetSelectedShapeName;
    property SelectedShapeValue: string read GetSelectedShapeValue;
    property VectorGraphic: TGraphic read FVectorGraphic;
    property MapView: TComponent read FMapView;

    property ShowLines: Boolean read FShowLines write FShowLines default True;
    property ShowPoints: Boolean read FShowPoints write FShowPoints default True;
    property ShowPolygons: Boolean read FShowPolygons write FShowPolygons default True;
    property LabelColumn: string read FLabelColumn write FLabelColumn;
    property SpatialColumn: string read FSpatialColumn write FSpatialColumn;
    property SpatialValue: string read FSpatialValue write FSpatialValue;
  published
    property Active: Boolean read FActive write FActive;
    property Visible;
    property Font;
    property AnalyticalValue: string read FAnalyticalValue write FAnalyticalValue;
    property ColorRanges: TfrxColorRanges read FColorRanges;
    property ColorRangesData: TColorRangeCollection read GetColorRangeData;
    property DataSet: TfrxDataSet read FDataSet write FDataSet;
    property DefaultShapeStyle: TShapeStyle read FDefaultShapeStyle;
    property Filter: string read FFilter write FFilter;
    property HighlightColor: TColor read FHighlightColor write FHighlightColor;
    property Operation: TMapOperation read FOperation write FOperation; // function
    property ValueFormat: string read FValueFormat write FValueFormat; // LabelFormat
    property LabelKind: TMapLabelKind read FLabelKind write FLabelKind;
    property MapPalette: TMapPalette read FMapPalette write FMapPalette; // Palette
    property SizeRanges: TfrxSizeRanges read FSizeRanges;
    property SizeRangesData: TSizeRangeCollection read GetSizeRangeData;
    property PointLabelsVisibleAtZoom: Extended read FPointLabelsVisibleAtZoom write FPointLabelsVisibleAtZoom;
  end;
(******************************************************************************)
  TMapLayerList = class
  private
    FObjects: TList;
    FSelectedLayerIndex: Integer;
    function GetSelectedLayer: TfrxCustomLayer;
    function GetCount: Integer;
    function GetLayer(Index: Integer): TfrxCustomLayer;
  protected
    FPreviousSelectedLayerIndex: Integer;
  public
    constructor Create(AObjects: TList);
    procedure Exchange(Index1, Index2: Integer);
    function IndexOf(Item: Pointer): Integer;

    function IsSelectedShapeChanded: Boolean;
    function IsInclude(P: TfrxPoint): Boolean;
    procedure GetData;
    function IsMercatorSuitable: Boolean;

    property Count: Integer read GetCount;
    property Items[Index: Integer]: TfrxCustomLayer read GetLayer; default;
    property SelectedLayer: TfrxCustomLayer read GetSelectedLayer;
  end;
(******************************************************************************)
  TfrxMapFileLayer = class(TfrxCustomLayer, IfrxDataLinkObject)
  private
    FZoomPolygon: string;
    procedure SetLayerTags(const Value: TStringList);
    function GetFileTags: TfrxSumStringList;

    procedure ClearObjectData;
    function LoadDataStream(Stream: TStream; const NewLink: string): Boolean;
    function GetLink(LoadMethod: TfrxDataLinkLoadMethod): string;
    function IsExpressionLink: Boolean;

    function GetDataLink: TfrxDataLink;
    procedure SetDataLink(const Value: TfrxDataLink);
    function IsDataLinkStored: Boolean;
  protected
    FMapFileName: string;
    FLayerTags: TStringList;
    FFirstReading: Boolean;
    FDataLink: TfrxDataLink;

    function GetSelectedShapeName: string; override;
    function GetSelectedShapeValue: string; override;
    procedure InitTransform(iRecord: Integer); override;
    procedure InitialiseData; override;
    function IsCanGetData: Boolean; override;
    function IsHighlightSelectedShape: boolean; override;

    procedure SetMapFileName(AMapFileName: string);
    function GetFileExtension: string;

    procedure ExpandVariables; override;
    procedure AddValueList(vaAnalyticalValue: Variant); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function IsHasZoomRect(out ZoomRect: TfrxRect): boolean; override;
    procedure Embed;
    procedure ReRead;
    procedure JustAdded;

    property FileTags: TfrxSumStringList read GetFileTags;
  published
    property DataLink: TfrxDataLink read GetDataLink write SetDataLink stored IsDataLinkStored;
    property ShowLines;
    property ShowPoints;
    property ShowPolygons;
    property LabelColumn;
    property SpatialColumn;
    property LayerTags: TStringList read FLayerTags write SetLayerTags;
    property MapAccuracy;
    property MapFileName: string read FMapFileName write SetMapFileName;
    property PixelAccuracy;
    property SpatialValue;
    property ZoomPolygon: string read FZoomPolygon write FZoomPolygon;
  end;
(******************************************************************************)
  TfrxApplicationLayer = class(TfrxCustomLayer)
  private
    FLabelValue: string;
    FLatitudeValue: string;
    FLongitudeValue: string;
  protected
    procedure InitialiseData; override;
    function IsCanGetData: Boolean; override;

    procedure ExpandVariables; override;
    procedure AddValueList(vaAnalyticalValue: Variant); override;
    function ApplicationShapeData(X, Y: Extended; Name, Location: string): TShapeData;
  public
    constructor Create(AOwner: TComponent); override;
    function IsInclude(P: TfrxPoint): Boolean; override; // False
  published
    property LabelValue: string read FLabelValue write FLabelValue;
    property LatitudeValue: string read FLatitudeValue write FLatitudeValue;
    property LongitudeValue: string read FLongitudeValue write FLongitudeValue;
  end;
(******************************************************************************)
type
  TfrxLabelColumnProperty = class(TfrxPropertyEditor)
  public
    function GetValue: string; override;
    function GetAttributes: TfrxPropertyAttributes; override;
    procedure GetValues; override;
    procedure SetValue(const Value: string); override;
  end;
(******************************************************************************)
  procedure OperationGetList(List: TStrings);
  procedure PaletteGetList(List: TStrings);
  procedure PenStyleGetList(List: TStrings);
  procedure MapLabelKindGetList(List: TStrings);
  procedure GeoDataKindList(List: TStrings);
(******************************************************************************)
implementation
end.
