
{******************************************}
{                                          }
{             FastReport VCL               }
{               Map Ranges                 }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxMapRanges;

interface

{$I frx.inc}

uses
  Classes, frxMapHelpers, frxAnaliticGeometry, Graphics, Types;

type
  TMapRangeItem = class(TCollectionItem)
  private
    FAutoStart: Boolean;
    FStartValue: Extended;
    FAutoEnd: Boolean;
    FEndValue: Extended;

    procedure SetStartValue(const Value: Extended);
    procedure SetEndValue(const Value: Extended);
    procedure SetStartValueByForce(const Value: Extended);
    procedure SetEndValueByForce(const Value: Extended);
  protected
    procedure AssignTo(Dest: TPersistent); override;
    function IsInside(Value: Extended): Boolean;
  public
    constructor Create(Collection: TCollection); override;
    procedure Read(Reader: TReader); virtual;
    procedure Write(Writer: TWriter); virtual;
    function AsString(FValueFormat: String): String; virtual;
  published
    property AutoStart: Boolean read FAutoStart write FAutoStart;
    property StartValue: Extended read FStartValue write SetStartValue;
    property StartValueByForce: Extended read FStartValue write SetStartValueByForce;
    property AutoEnd: Boolean read FAutoEnd write FAutoEnd;
    property EndValue: Extended read FEndValue write SetEndValue;
    property EndValueByForce: Extended read FEndValue write SetEndValueByForce;
  end;
(******************************************************************************)
  TRangeFactor = (rfValue, rfPercentile, rfCluster, rfAutoCluster);

  TMapRangeCollection = class(TCollection)
  private
    FMinValue, FMaxValue: Extended;
  protected
    FValues: TDoubleArray;
    FRangeFactor: TRangeFactor;

    function GetItem(Index: Integer): TMapRangeItem;
    procedure SetItem(Index: Integer; const Value: TMapRangeItem);
    function Part(Value: Extended): Extended;
    function Ranges(const Values: TDoubleArray; RangeFactor: TRangeFactor): TDoubleArray;
    function RangesByValue(const Values: TDoubleArray): TDoubleArray;
    function RangesByCLuster(const Values: TDoubleArray): TDoubleArray;
    function RangesByAutoCLuster(const Values: TDoubleArray): TDoubleArray;
    function RangesByPercentile(const Values: TDoubleArray): TDoubleArray;
    function MedianValue: Double;
  public
    procedure ReadDFM(Stream: TStream);
    procedure WriteDFM(Stream: TStream);
    procedure Read(Reader: TReader); virtual;
    procedure Write(Writer: TWriter); virtual;
    procedure FillRangeValues(const Values: TDoubleArray; RangeFactor: TRangeFactor);
    procedure Swap(Index1, Index2: Integer);

    property Items[Index: Integer]: TMapRangeItem read GetItem write SetItem; default;
  end;
(******************************************************************************)
  TScaleDock = (sdTopLeft, sdTopCenter, sdTopRight, sdMiddleLeft, sdMiddleRight,
    sdBottomLeft, sdBottomCenter, sdBottomRight, sdMiddleCenter);

  TMapScale = class(TPersistent)
  private
    FVisible: Boolean;
    FBorderColor: TColor;
    FBorderWidth: Integer;
    FDock: TScaleDock;
    FFillColor: TColor;
    FFont: TFont;
    FTitleFont: TFont;
    FTitleText: String;
    FValueFormat: String;
  public
    constructor Create;
    function LeftTopPoint(ConstrivtedParentRect: TRect): TPoint;
    destructor Destroy; override;
  published
    property Visible: Boolean read FVisible write FVisible;
    property BorderColor: TColor read FBorderColor write FBorderColor;
    property BorderWidth: Integer read FBorderWidth write FBorderWidth;
    property Dock: TScaleDock read FDock write FDock;
    property FillColor: TColor read FFillColor write FFillColor;
    property Font: TFont read FFont;
    property TitleFont: TFont read FTitleFont;
    property TitleText: String read FTitleText write FTitleText;
    property ValueFormat: String read FValueFormat write FValueFormat;
  end;
(******************************************************************************)
  TMapRanges = class(TPersistent)
  private
    FVisible: Boolean;

    function GetRangeCount: Integer;
    procedure SetRangeCount(const Value: Integer);

    function GetTitleHeight: Integer;
    function GetValuesHeight: Integer;
    function GetWidth: Integer;
    function GetHeight: Integer;
  protected
    FRangeFactor: TRangeFactor;
    FMapRangeCollection: TMapRangeCollection;
    FMapScale: TMapScale;
    {$IFDEF FRX_USE_BITMAP_MAP}
     OfsetX, OfsetY: Integer;
    {$ENDIF}

    function GetSpaceWidth: Integer; virtual;
    function GetStepWidth: Integer; virtual;
    function GetContentHeight: Integer; virtual;

    procedure DrawContent(Canvas: TCanvas); virtual; abstract;
    procedure DrawValues(Canvas: TCanvas);
    function CalcTextHeight(Font: TFont; Text: String): Integer;

    property StepWidth: Integer read GetStepWidth;
    property SpaceWidth: Integer read GetSpaceWidth;
    property ContentHeight: Integer read GetContentHeight;
    property TitleHeight: Integer read GetTitleHeight;
    property ValuesHeight: Integer read GetValuesHeight;
  public
    constructor Create(MapScale: TMapScale);
    destructor Destroy; override;
    function GetGraphic: TGraphic;
    procedure Draw(Canvas: TCanvas{$IFDEF FRX_USE_BITMAP_MAP}; x, y: Integer{$ENDIF});

    property MapScale: TMapScale read FMapScale;
    property Width: Integer read GetWidth;
    property Height: Integer read GetHeight;
  published
    property RangeFactor: TRangeFactor read FRangeFactor write FRangeFactor;
    property RangeCount: Integer read GetRangeCount write SetRangeCount;
    property Visible: Boolean read FVisible write FVisible;
  end;
(******************************************************************************)
  function IsValidFloat(NeedTest: Boolean; stFloat: String; Quiet: Boolean = False): Boolean;
  procedure RangeFactorGetList(List: TStrings);
(******************************************************************************)
implementation
end.
