
{******************************************}
{                                          }
{             FastReport VCL               }
{             Map Size Ranges              }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxMapSizeRangeForm;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}
  Windows, Messages,
  {$ELSE}
  LCLType, LMessages, LCLIntf, LCLProc,
  {$ENDIF}
  SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons, frxBaseForm,
  frxMapRanges, frxMapHelpers, frxAnaliticGeometry;

type
  TSizeRangeItem = class(TMapRangeItem)
  private
    FAutoSize: Boolean;
    FSize: Extended;

    procedure SetSize(const Value: Extended);
    procedure SetSizeByForce(const Value: Extended);
  protected
    procedure AssignTo(Dest: TPersistent); override;
  public
    constructor Create(Collection: TCollection); override;
    procedure Read(Reader: TReader); override;
    procedure Write(Writer: TWriter); override;
    function IsValueInside(Value: Extended; var OutSize: Extended): Boolean;
    function AsString(FValueFormat: String): String; override;
  published
    property AutoSize: Boolean read FAutoSize write FAutoSize;
    property Size: Extended read FSize write SetSize;
    property SizeByForce: Extended read FSize write SetSizeByForce;
  end;
(******************************************************************************)
  TSizeRangeCollection = class(TMapRangeCollection)
  protected
    function GetItem(Index: Integer): TSizeRangeItem;
    procedure SetItem(Index: Integer; const Value: TSizeRangeItem);

    function GetMixedSize(Size1, Size2: Extended; p: Extended): Extended;
  public
    constructor Create;
    function Add: TSizeRangeItem;
    function GetSize(Value: Extended): Extended;
    function GetPeerSize(StartSize, EndSize, Value: Extended): Extended;
    procedure FillRangeSizes(StartIndex, EndIndex: Integer);

    property Items[Index: Integer]: TSizeRangeItem read GetItem write SetItem; default;
  end;
(******************************************************************************)
  TMapSizeRangeForm = class(TfrxBaseForm)
    CollectionListBox: TListBox;
    SizeEdit: TEdit;
    StartEdit: TEdit;
    EndEdit: TEdit;
    SizeCheckBox: TCheckBox;
    StartCheckBox: TCheckBox;
    EndCheckBox: TCheckBox;
    AutoLabel: TLabel;
    OkB: TButton;
    CancelB: TButton;
    Bevel1: TBevel;
    AddButton: TButton;
    DeleteButton: TButton;
    UpSpeedButton: TSpeedButton;
    DownSpeedButton: TSpeedButton;

    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure CollectionListBoxClick(Sender: TObject);
    procedure OkBClick(Sender: TObject);
    procedure SizeCheckBoxClick(Sender: TObject);
    procedure StartEditChange(Sender: TObject);
    procedure AddButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure UpSpeedButtonClick(Sender: TObject);
    procedure DownSpeedButtonClick(Sender: TObject);
  private
    FEditedCollection: TSizeRangeCollection;
    FValueFormat: String;
    FPreviousItemIndex: Integer;

    procedure RefillCollectionListBox(const ItemIndex: integer);
    procedure Syncronize;
    procedure UpdateControls;
  public
    procedure SetCollection(SRCollection: TSizeRangeCollection);
    function GetCollection: TSizeRangeCollection;
    procedure UpdateResouces; override;
    procedure UpdateFormPPI(aNewPPI: Integer); override;

    property ValueFormat: String read FValueFormat write FValueFormat;
  end;
(******************************************************************************)
  TfrxSizeRanges = class(TMapRanges)
  private
    FSizeRangeCollection: TSizeRangeCollection;
    FStartSize: Extended;
    FEndSize: Extended;

    procedure SetSizeRangeCollection(const Value: TSizeRangeCollection);
  protected
    procedure DrawContent(Canvas: TCanvas); override;

    procedure FillRangeSizes;

    function GetSpaceWidth: Integer; override;
    function GetStepWidth: integer; override;
    function GetContentHeight: Integer; override;
  public
    constructor Create(MapScale: TMapScale);
    procedure Fill(const Values: TDoubleArray);
    property SizeRangeCollection: TSizeRangeCollection read FSizeRangeCollection write SetSizeRangeCollection;
  published
    function GetSize(Value: Extended): Extended;

    property StartSize: Extended read FStartSize write FStartSize;
    property EndSize: Extended read FEndSize write FEndSize;
  end;
(******************************************************************************)
implementation
end.
