
{******************************************}
{                                          }
{             FastReport VCL               }
{               OLE object                 }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxOLE;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}
  Windows, Messages,
  {$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  {$IFNDEF FPC}
  OleCtnrs,
  {$ENDIF}
  StdCtrls, ExtCtrls, frxClass, frCore
  {$IFNDEF FPC}
  , ActiveX
  {$ENDIF}
  {$IFDEF FPC}
  , LCLType, LCLIntf, LazHelper
  {$ENDIF}
{$IFDEF DELPHI6}
, Variants
{$ENDIF}
{$IFDEF FR_COM}
, FastReport_TLB
{$ENDIF};


type
  TfrxSizeMode = (fsmClip, fsmScale, fsmAutoSize);
{$IFDEF DELPHI16}
/// <summary>
///   The TfrxOLEObject allows the use of OLE objects in your report.
///   TfrxOLEObject is an empty component. It is used to add thefrxOLE.pas file
///   to the "uses" list. The main component is TfrxOLEView.
/// </summary>
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxOLEObject = class(TComponent)  // fake component
  end;

{$IFDEF FR_COM}
  TfrxOLEView = class(TfrxView, IfrxOLEView)
{$ELSE}
  /// <summary>
  ///   The TfrxOLEView component represents an OLE object. Component can show
  ///   an OLE object from the DB field (you should set DataSet, DataField
  ///   properties).
  /// </summary>
  TfrxOLEView = class(TfrxStyledView)
{$ENDIF}
  private
    FOleContainer: TOleContainer;
    FSizeMode: TfrxSizeMode;
    FStretched: Boolean;
    FZoom: Extended;
    procedure ReadData(Stream: TStream);
    procedure SetStretched(const Value: Boolean);
    procedure WriteData(Stream: TStream);
    function GetOriginalSize(var OriginalSize: TPoint): Boolean;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure SetWidth(Value: Extended); override;
    procedure SetHeight(Value: Extended); override;
{$IFDEF FR_COM}
    function Get_OleContainer(out Value: IUnknown): HResult; stdcall;
    function Get_SizeMode(out Value: Integer): HResult; stdcall;
    function Set_SizeMode(Value: Integer): HResult; stdcall;
    function Get_Stretched(out Value: WordBool): HResult; stdcall;
    function Set_Stretched(Value: WordBool): HResult; stdcall;
{$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    procedure GetData; override;
    class function GetDescription: String; override;
    /// <summary>
    ///   Reference to internal TOleContainer object.
    /// </summary>
    property OleContainer: TOleContainer read FOleContainer;
    function IsEMFExportable: Boolean; override;
  published
    property BrushStyle;
    property Color;
    property Cursor;
    property DataField;
    property DataSet;
    property DataSetName;
    property FillType;
    property Fill;
    property Frame;
    property SizeMode: TfrxSizeMode read FSizeMode write FSizeMode default fsmClip;
    /// <summary>
    ///   Determines if it is necessary to stretch an OLE object to the area
    ///   occupied by the object in a report. It may be useful for showing MS
    ///   Excel objects.
    /// </summary>
    property Stretched: Boolean read FStretched write SetStretched default False;
    property Style;
    property Zoom: Extended read FZoom write FZoom;
    property TagStr;
    property URL;
  end;

/// <summary>
///   This method copies the contents from ContFrom to ContTo OLE container.
/// </summary>
procedure frxAssignOle(ContFrom, ContTo: TOleContainer);


implementation
end.

