unit frxPDFPFX;

interface

{$I frx.inc}

uses
  {$IFNDEF Linux}
  Windows,
  {$ENDIF}
  SysUtils, Classes, Math,
  frCoreClasses,
  frxHelpers, frxASN1, frxPDFX509Certificate, frxCertificateList;

type

  TfrxPKCS12Document = class
  private
    FContentInfo: AnsiString;

    FMacSalt: AnsiString;
    FMacDigest: AnsiString;
    FMacAlgorithmName: TOIDName;
    FMacIterations: Integer;

    FValidPassword: UTF8String;
    FPrivateKeys: TfrOwnObjList;
    FCertificates: TfrOwnObjList;
    FChain: TfrxX509Certificate;
    FChainLen: Integer;

    function GetCertificates(i: Integer): TfrxX509Certificate;
    function GetPrivateKeys(i: Integer): TfrxRSAPrivateKey;
  protected
    FStatus: TSignatureSatus;
    FDebugLog: TLogList;

    procedure Clear;
    procedure ClearMacData;
    procedure LoadMacData(MacData: TfrxASN1Container);
    procedure ProcessSafeBag(SafeBag: TfrxASN1Container);
    class procedure PartCopy(Source: AnsiString; var Dest: AnsiString; APos: Integer);
    class procedure PartCopy2(Source: AnsiString; var Dest: AnsiString; APos: Integer);
    class function NormalizePassword(Password: UTF8String): AnsiString;
    procedure ReadPrivateKey(BagId: TOIDName; BagValue, BagAttributes: TfrxASN1Container);
    procedure ReadCertificate(BagValue, BagAttributes: TfrxASN1Container);

    property Certificates[i: Integer]: TfrxX509Certificate read GetCertificates;
    property PrivateKeys[i: Integer]: TfrxRSAPrivateKey read GetPrivateKeys;
  public
    constructor Create(ADebugLog: TLogList = nil);
    destructor Destroy; override;
    procedure LoadFromFile(AFileName: string);
    procedure LoadFromStream(AStream: TStream);
    function IsCheckPassword(Password: UTF8String): Boolean;
    procedure Parse;
    class function DerivingKey(Password: UTF8String; Salt: AnsiString; ID: Byte;
      Algorithm: TOIDName; Iteration: Integer; Len: Cardinal): AnsiString;

    property Chain: TfrxX509Certificate read FChain;
    property ChainLen: Integer read FChainLen;
    property Status: TSignatureSatus read FStatus;
  end;

implementation
end.
