
{******************************************}
{                                          }
{             FastReport VCL               }
{             Report preview               }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxPreview;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}Windows, Messages,{$ENDIF}
  Types, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Buttons, StdCtrls, Menus, ComCtrls, ImgList, frxCtrls, frxDock,
  ToolWin, frxPreviewPages, frxClass, frxBaseForm, frxSearchForm, frxComCtrls, frCore,
{$IFDEF DELPHI17}
  System.Actions,
{$ENDIF}
{$IFDEF FPC}
  LResources, LCLType, LCLProc, LazUTF8, LCLIntf, LazHelper, frLazGraphicClasses,
{$ENDIF}
{$IFDEF DELPHI6}
  Variants, ActnList,
{$ENDIF}
  frTypes, frScrollWin, frTreeView;

const
  WM_UPDATEZOOM = WM_USER + 1;

type
  TfrxPreview = class;
  TfrxPreviewWorkspace = class;
  TfrxPageList = class;

  TfrxPreviewTool = (ptHand, ptZoom); // not implemented, backw compatibility only
  /// <summary>
  ///   This event is generated when the current page is being changed.
  /// </summary>
  TfrxPageChangedEvent = procedure(Sender: TfrxPreview; PageNo: Integer) of object;

  TfrxFindAllTextItem = record
    PageNo: Integer;
    Bounds: TRect;
  end;

  PfrxTrvData = ^TfrxTrvData;
  TfrxTrvData = record
    PageNo: Integer;
    TextBnd: TRect;
  end;

  TfrxFindAllTextCallbackFunc = function (Item: TfrxFindAllTextItem; Text: String; Data: Pointer): Boolean;

  TfrxPreviewTabItem = class(TCollectionItem)
  private
    FName: String;
    FTop: Integer;
    FLeft: Integer;
    FPageNo: Integer;
    FThumbTop: Integer;
    FReport: TfrxReport;
    FPreviewPages: TfrxCustomPreviewPages;
    FDetailPage: String;
    FZoom: Extended;
    FZoomMode: TfrxZoomMode;
    FOutlineItem: Integer;
    FFreeObjects: Boolean;
  public
    property Name: String read FName write FName;
    property Top: Integer read FTop write FTop;
    property Left: Integer read FLeft write FLeft;
    property PageNo: Integer read FPageNo write FPageNo;
    property ThumbTop: Integer read FThumbTop write FThumbTop;
    property Report: TfrxReport read FReport write FReport;
    property PreviewPages: TfrxCustomPreviewPages read FPreviewPages write FPreviewPages;
    property DetailPage: String read FDetailPage write FDetailPage;
    property Zoom: Extended read FZoom write FZoom;
    property ZoomMode: TfrxZoomMode read FZoomMode write FZoomMode;
    property OutlineItem: Integer read FOutlineItem write FOutlineItem;
    property FreeObjects: Boolean read FFreeObjects write FFreeObjects;
  end;

  TfrxPreviewTabs = class(TCollection)
  private
//    FOutline: TfrTreeView;
    FThumbnail: TfrxPreviewWorkspace;
    FWorkspace: TfrxPreviewWorkspace;
    FPreview: TfrxPreview;
    FCurTab: Integer;
    function GetItems(Index: Integer): TfrxPreviewTabItem;
  public
    constructor Create(APreview: TfrxPreview);
    property CurrentTab: Integer read FCurTab;
    property Items[Index: Integer]: TfrxPreviewTabItem read GetItems; default;
    procedure AddTab(AReport: TfrxReport; aDetailPage: String; const TabName: String; AFreeObjects: Boolean);
    procedure DeleteTab(Index: Integer);
    procedure SetCurrentTab(Index: Integer);
    procedure ClearItems;
  end;

{$IFDEF DELPHI16}
/// <summary>
///   The TfrxPreview component is designed for creation of custom preview
///   windows. To display a report, the link to this component should be
///   assigned to the "TfrxReport.Preview" property.
/// </summary>
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxPreview = class(TfrxCustomPreview)
  private
    DesignerFSearchFrmVisible: Boolean;
    FAllowF3: Boolean;
    {$IFNDEF FPC}
    FBorderStyle: TBorderStyle;
    {$ENDIF}
    FCancelButton: TButton;
    FSortButton: TfrxToolPanelButton;
    FSortPopUp: TPopupMenu;
    FLocks: Integer;
    FMessageLabel: TLabel;
    FMessagePanel: TPanel;
    FOnPageChanged: TfrxPageChangedEvent;
    FOutline: TfrxTreePanel;
    FSearchFrm: TfrxSearchForm;
    FFindSplitter: TSplitter;
    FOutlineColor: TColor;
    FOutlinePopup: TPopupMenu;
    FPageNo: Integer;
    FSavedPageNo: Integer;
    FRefreshing: Boolean;
    FRunning: Boolean;
    FScrollBars: TScrollStyle;
    FSplitter: TSplitter;
    FThumbnail: TfrxPreviewWorkspace;
    FTick: Cardinal;
    FTool: TfrxPreviewTool;
    FWorkspace: TfrxPreviewWorkspace;
    FZoom: Extended;
    FZoomMode: TfrxZoomMode;
    FTabs: TTabControl;
    FInitialized: Boolean;
    FCalledFromPreview: Boolean;
    FTabItems: TfrxPreviewTabs;
    FTabImgList: TImageList;
    FEachReportInTab: Boolean;
    FOutlineTreeSortType: TfrxTreeSortType;
    FUnion: Boolean;
    FGeneralDialog: Boolean;

    function GetActiveFrameColor: TColor;
    function GetBackColor: TColor;
    function GetFindFmVisible: Boolean;
    procedure TreeViewCompare(Sender: TObject; Node1, Node2: TfrTreeViewNode; Data: Integer; var Compare: Integer);
    procedure OnTrvFindChange(Sender: TObject; Node: TfrTreeViewNode);
    procedure OnFindClick(Sender: TObject);
    procedure SetFindFmVisible(Value: Boolean);
    function GetFrameColor: TColor;
    function GetOutlineVisible: Boolean;
    function GetOutlineWidth: Integer;
    function GetOutline: TfrTreeView;
    function GetPageCount: Integer;
    function GetThumbnailVisible: Boolean;
    function GetOnMouseDown: TMouseEvent;
    procedure AddCloseBtnToImageList;
    procedure EditTemplate;
    procedure ToolOnClick(Sender: TObject);
    procedure CollapseExpand(aExpand: Boolean = False);
    procedure OnCancel(Sender: TObject);
    procedure OnCollapseClick(Sender: TObject);
    procedure FillOutlineTree;
    procedure OnExpandClick(Sender: TObject);
    procedure OnMoveSplitter(Sender: TObject);
    procedure OnOutlineClick(Sender: TObject);
    procedure OnChangeTab(Sender: TObject);
    procedure OnTabMouseUP(Sender: TObject; Button: TMouseButton;
    Shift: TShiftState; X, Y: Integer);
    procedure SetActiveFrameColor(const Value: TColor);
    procedure SetBackColor(const Value: TColor);
    {$IFNDEF FPC}
    procedure SetBorderStyle(Value: TBorderStyle);
    {$ENDIF}
    procedure SetFrameColor(const Value: TColor);
    procedure SetOutlineColor(const Value: TColor);
    procedure SetOutlineWidth(const Value: Integer);
    procedure SetOutlineVisible(const Value: Boolean);
    procedure SetPageNo(Value: Integer);
    procedure SetActivePage(Value: Integer);
    procedure SetThumbnailVisible(const Value: Boolean);
    procedure SetZoom(const Value: Extended);
    procedure SetZoomMode(const Value: TfrxZoomMode);
    procedure SetOnMouseDown(const Value: TMouseEvent);
    procedure CreateSortPopup;
    procedure UpdateOutline;
    procedure UpdateFindPage;
    procedure UpdatePages;
    procedure UpdatePageNumbers;
    procedure WMEraseBackground(var Message: TMessage); message WM_ERASEBKGND;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure SetHighlightEditable(const Value: Boolean);
    function GetHighlightEditable: Boolean;
    function GetHideScrolls: Boolean;
    function GetOnScrollMaxChange: TfrxScrollMaxChangeEvent;
    function GetOnScrollPosChange: TfrxScrollPosChangeEvent;
    procedure SetHideScrolls(const Value: Boolean);
    procedure SetOnScrollMaxChange(const Value: TfrxScrollMaxChangeEvent);
    procedure SetOnScrollPosChange(const Value: TfrxScrollPosChangeEvent);
    procedure ClearPageList;
    function GetLocked: Boolean;
    procedure CompositeTabsExport(Filter: TfrxCustomExportFilter);
    procedure CompositePreviewPages(buffPrevPags: TfrxCustomPreviewPages);//union support
    procedure SeparateTabsExport(Filter: TfrxCustomExportFilter);
    procedure SetTabsVisible(b: Boolean);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    function GetReport: TfrxReport; override;
    function GetPreviewPages: TfrxCustomPreviewPages; override;
    procedure Resize; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure DoFinishInPlace(Sender: TObject; Refresh, Modified: Boolean); override;
	procedure RemoveTabs(aReport: TfrxReport);
    procedure DoPPIChanged(aNewPPI: Integer); override;
    procedure SetOutlineTreeSortType(const Value: TfrxTreeSortType);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ClearAllTabs;
    function Init(aReport: TfrxReport; aPrevPages: TfrxCustomPreviewPages): Boolean; override;
    procedure UnInit(aReport: TfrxReport); override;
    function Lock: Boolean; override;
    procedure Unlock(DoUpdate: Boolean = True); override;
    procedure RefreshReport; override;
    procedure InternalOnProgressStart(Sender: TfrxReport;
      ProgressType: TfrxProgressType; Progress: Integer); override;
    procedure InternalOnProgress(Sender: TfrxReport;
      ProgressType: TfrxProgressType; Progress: Integer); override;
    procedure InternalOnProgressStop(Sender: TfrxReport;
      ProgressType: TfrxProgressType; Progress: Integer); override;
    /// <summary>
    ///   Adds a blank page to the end of the report.
    /// </summary>
    procedure AddPage;
    /// <summary>
    ///   Deletes the current page.
    /// </summary>
    procedure DeletePage;
    /// <summary>
    ///   Prints a report.
    /// </summary>
    procedure Print;
    /// <summary>
    ///   Loads the current page for editing to the designer.
    /// </summary>
    procedure Edit;
    /// <summary>
    ///   Moves to the first page.
    /// </summary>
    procedure First;
    /// <summary>
    ///   Moves to the next page.
    /// </summary>
    procedure Next;
    /// <summary>
    ///   Moves to the previous page.
    /// </summary>
    procedure Prior;
    /// <summary>
    ///   Moves to the last page.
    /// </summary>
    procedure Last;
    /// <summary>
    ///   Displays the page setting dialogue.
    /// </summary>
    procedure PageSetupDlg;
    /// <summary>
    ///   Displays the page setting dialogue.
    /// </summary>
    procedure PageSetupDialog;
    /// <summary>
    ///   Displays the text searching dialogue.
    /// </summary>
    procedure Find;
    /// <summary>
    ///   Continues searching the text.
    /// </summary>
    procedure FindNext;
    /// <summary>
    ///   Aborts a report constructing.
    /// </summary>
    procedure Cancel;
    /// <summary>
    ///   Clears a report.
    /// </summary>
    procedure Clear;
    /// <summary>
    ///   Moves to the page PageN and top position Top on the page.
    /// </summary>
    procedure SetPosition(PageN, Top: Integer);
    procedure ShowMessage(const s: String);
    procedure HideMessage;
    /// <summary>
    ///   <para>
    ///     Scrolls the preview window. This method is used for
    ///     Form.OnMouseWheel event handler:
    ///   </para>
    ///   <code lang="Delphi">procedure TForm1.FormMouseWheel(Sender: TObject;
    ///   Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint;
    ///    var Handled: Boolean);
    ///    begin
    ///      frxPreview1.MouseWheelScroll(WheelDelta);
    ///    end;</code>
    /// </summary>
    procedure MouseWheelScroll(Delta: Integer; Shift: TShiftState; MousePos: TPoint; Horz: Boolean = False);
    /// <summary>
    ///   Return position on the current page.
    /// </summary>
    function GetTopPosition: Integer;
    /// <summary>
    ///   Displays the file loading dialogue.
    /// </summary>
    procedure LoadFromFile; overload;
    /// <summary>
    ///   Loads a file without displaying the dialogue.
    /// </summary>
    procedure LoadFromFile(FileName: String); overload;
    //procedure FindAllText(SearchString: String; IsCaseSensitive: Boolean; Callback : TfrxFindAllTextCallbackFunc; Data : Pointer);
    //procedure FindAllTextItemHighlight(Item : TfrxFindAllTextItem);  not used
    /// <summary>
    ///   Displays the file saving dialogue.
    /// </summary>
    procedure SaveToFile; overload;
    /// <summary>
    ///   Saves a file without displaying the dialogue.
    /// </summary>
    procedure SaveToFile(FileName: String); overload;
    /// <summary>
    ///   Exports the report using the specified export filter.
    /// </summary>
    procedure Export(Filter: TfrxCustomExportFilter;
      ExportsAllTabs: Boolean = False);
    procedure CompositeExport(Filter: TfrxCustomExportFilter);
    function FindText(SearchString: String; FromTop, IsCaseSensitive: Boolean): Boolean;
    function FindTextFound: Boolean;
    procedure FindTextClear;
    procedure AddPreviewTab(AReport: TfrxReport; const TabName: String; const TabCaption: String = ''; FreeObjects: Boolean = True; aDetailPage: String = ''); override;
    procedure AddPreviewTabOrSwitch(AReport: TfrxReport; const TabName: String; const TabCaption: String = ''; FreeObjects: Boolean = True; aDetailPage: String = ''); override;
    procedure RemoveTab(TabIndex: Integer); override;
    function HasTab(const TabName: String): Boolean; overload; override;
    function HasTab(const aReport: TfrxReport): Boolean; overload; override;
    function HasVisibleTabs: Boolean; override;
    procedure PreviewPagesChanged; override;
    procedure SwitchToTab(const TabName: String); overload; override;
    procedure SwitchToTab(const aReport: TfrxReport); overload; override;

    /// <summary>
    ///   Number of pages in a report.
    /// </summary>
    property PageCount: Integer read GetPageCount;
    /// <summary>
    ///   The current page number (starts from 1). To move to a required page,
    ///   assign a value to this property.
    /// </summary>
    property PageNo: Integer read FPageNo write SetPageNo;
    // not implemented, backw compatibility only
    property Tool: TfrxPreviewTool read FTool write FTool;
    /// <summary>
    ///   The scaling factor. "1" conforms 100% scale.
    /// </summary>
    property Zoom: Extended read FZoom write SetZoom;
    /// <summary>
    ///   Zoom mode. The following values are available: <br />zmDefault -
    ///   scale can be set with the help of the "Zoom" property; <br />
    ///   zmWholePage - the whole page fits; <br />zmPageWidth - the page fits
    ///   by width; <br />zmManyPages - two pages fit.
    /// </summary>
    property ZoomMode: TfrxZoomMode read FZoomMode write SetZoomMode;
    /// <summary>
    ///   Flag returns state of window blocking.
    /// </summary>
    property Locked: Boolean read GetLocked;
    /// <summary>
    ///   Reference to Outline Tree window.
    /// </summary>
    property OutlineTree: TfrTreeView read GetOutline;
    /// <summary>
    ///   Reference to spliter object <br />
    /// </summary>
    property Splitter: TSplitter read FSplitter;
    property TabItems: TfrxPreviewTabs read FTabItems;
    property FindSplitter: TSplitter read FFindSplitter;
    property SearchFrm: TfrxSearchForm read FSearchFrm;
    /// <summary>
    ///   Reference to Thumbnail window object.
    /// </summary>
    property Thumbnail: TfrxPreviewWorkspace read FThumbnail;
    /// <summary>
    ///   Reference to preview Workspace object.
    /// </summary>
    property Workspace: TfrxPreviewWorkspace read FWorkspace;
    property HighlightEditable: Boolean read GetHighlightEditable write SetHighlightEditable;
    property Outline: TfrxTreePanel read FOutline write FOutline;
  published
    property Align;
    /// <summary>
    ///   The color of active page frame.
    /// </summary>
    property ActiveFrameColor: TColor read GetActiveFrameColor write SetActiveFrameColor default $3CC7FF;
    /// <summary>
    ///   The workspace color.
    /// </summary>
    property BackColor: TColor read GetBackColor write SetBackColor default clGray;
    {$IFDEF FPC}
    property BorderStyle;
    {$ELSE}
    property BevelEdges;
    property BevelInner;
    property BevelKind;
    property BevelOuter;
    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle default bsSingle;
    {$ENDIF}
    property BorderWidth;
    property Constraints;
    property EachReportInTab: Boolean read FEachReportInTab write FEachReportInTab default False;
    property Enabled;
    /// <summary>
    ///   The page frame color.
    /// </summary>
    property FrameColor: TColor read GetFrameColor write SetFrameColor default $606060;
    property OutlineColor: TColor read FOutlineColor write SetOutlineColor default clWindow;
    /// <summary>
    ///   Report tree visibility.
    /// </summary>
    property OutlineVisible: Boolean read GetOutlineVisible write SetOutlineVisible;
    property OutlineWidth: Integer read GetOutlineWidth write SetOutlineWidth;
    property PopupMenu;
    /// <summary>
    ///   Thumbnails visibility.
    /// </summary>
    property ThumbnailVisible: Boolean read GetThumbnailVisible write SetThumbnailVisible;
    property FindFmVisible: Boolean read GetFindFmVisible write SetFindFmVisible;
    property Visible;
    property OnClick;
    property OnDblClick;
    /// <summary>
    ///   This event is generated when the current page is being changed.
    /// </summary>
    property OnPageChanged: TfrxPageChangedEvent read FOnPageChanged write FOnPageChanged;
    property OnMouseDown: TMouseEvent read GetOnMouseDown write SetOnMouseDown;
    property Anchors;
    property UseReportHints;
    property OutlineTreeSortType: TfrxTreeSortType read FOutlineTreeSortType write SetOutlineTreeSortType;
    property HideScrolls: Boolean read GetHideScrolls write SetHideScrolls;
    property OnScrollMaxChange: TfrxScrollMaxChangeEvent read GetOnScrollMaxChange write SetOnScrollMaxChange;
    property OnScrollPosChange: TfrxScrollPosChangeEvent read GetOnScrollPosChange write SetOnScrollPosChange;
  end;

  { TfrxPreviewForm }

  TfrxPreviewForm = class(TfrxBaseForm)
    ToolBar: TToolBar;
    OpenB: TToolButton;
    SaveB: TToolButton;
    PrintB: TToolButton;
    FindB: TToolButton;
    PageSettingsB: TToolButton;
    Sep3: TfrxTBPanel;
    SaveAllTabsB: TToolButton;
    ZoomCB: TfrxComboBox;
    Sep1: TToolButton;
    Sep2: TToolButton;
    FirstB: TToolButton;
    PriorB: TToolButton;
    Sep4: TfrxTBPanel;
    PageE: TEdit;
    NextB: TToolButton;
    LastB: TToolButton;
    StatusBar: TStatusBar;
    ZoomMinusB: TToolButton;
    Sep5: TToolButton;
    ZoomPlusB: TToolButton;
    DesignerB: TToolButton;
    frTBPanel1: TfrxTBPanel;
    CancelB: TSpeedButton;
    ExportPopup: TPopupMenu;
    ExportAllTabsPopup: TPopupMenu;
    HiddenMenu: TPopupMenu;
    Showtemplate1: TMenuItem;
    RightMenu: TPopupMenu;
    FullScreenBtn: TToolButton;
    EmailB: TToolButton;
    PdfB: TToolButton;
    OutlineB: TToolButton;
    ThumbB: TToolButton;
    N1: TMenuItem;
    ExpandMI: TMenuItem;
    CollapseMI: TMenuItem;
    OfNL: TLabel;
    HighlightEditableTB: TToolButton;
    PreviewActionList: TActionList;
    CopyCmd: TAction;
    PasteCmd: TAction;
    CopyCmd1: TMenuItem;
    PasteCmd1: TMenuItem;
    StylePan: TfrxTBPanel;
    StyleSheetCB: TfrxComboBox;
    SepStyle: TfrxTBPanel;
    StyleLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ZoomMinusBClick(Sender: TObject);
    procedure ZoomCBClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FirstBClick(Sender: TObject);
    procedure PriorBClick(Sender: TObject);
    procedure NextBClick(Sender: TObject);
    procedure LastBClick(Sender: TObject);
    procedure PageEClick(Sender: TObject);
    procedure PrintBClick(Sender: TObject);
    procedure OpenBClick(Sender: TObject);
    procedure SaveBClick(Sender: TObject);
    procedure FindBClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure DesignerBClick(Sender: TObject);
    procedure NewPageBClick(Sender: TObject);
    procedure DelPageBClick(Sender: TObject);
    procedure CancelBClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PageSettingsBClick(Sender: TObject);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure DesignerBMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Showtemplate1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FullScreenBtnClick(Sender: TObject);
    procedure PdfBClick(Sender: TObject);
    procedure EmailBClick(Sender: TObject);
    procedure ZoomPlusBClick(Sender: TObject);
    procedure OutlineBClick(Sender: TObject);
    procedure ThumbBClick(Sender: TObject);
    procedure CollapseAllClick(Sender: TObject);
    procedure ExpandAllClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure HighlightEditableTBClick(Sender: TObject);
    procedure CopyCmdExecute(Sender: TObject);
    procedure PasteCmdExecute(Sender: TObject);
    procedure RightMenuPopup(Sender: TObject);
    procedure ZoomCBKeyPress(Sender: TObject; var Key: Char);
    procedure StyleSheetCBChange(Sender: TObject);
  private
    FFilterList: TStringList;
    FFreeOnClose: Boolean;
    FIsClosing: Boolean;
    FPreview: TfrxPreview;
    FOldBS: TFormBorderStyle;
    FOldState: TWindowState;
    FFullScreen: Boolean;
    FPDFExport: TfrxCustomExportFilter;
    FEmailExport: TfrxCustomExportFilter;
    {$IFNDEF FPC}
    FStatusBarOldWindowProc: TWndMethod;
    {$ENDIF}
    procedure CreateExportMenu;
    procedure ExportMIClick(Sender: TObject);
    procedure ExportMIACTClick(Sender: TObject);
    procedure OnPageChanged(Sender: TfrxPreview; PageNo: Integer);
    procedure OnPreviewDblClick(Sender: TObject);
    procedure UpdateControls;
    procedure UpdateZoom;
    procedure WMUpdateZoom(var Message: TMessage); message WM_UPDATEZOOM;
    procedure WMActivateApp(var Msg: TWMActivateApp); message WM_ACTIVATEAPP;
    procedure WMSysCommand(var Msg: TWMSysCommand); message WM_SYSCOMMAND;
    procedure StatusBarWndProc(var Message: TMessage);
    function GetReport: TfrxReport;
    procedure OnSaveFilterExecute(Sender: TObject);
    procedure UpdateStyles;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Init;
    procedure SetMessageText(const Value: String; IsHint: Boolean = False);
    procedure SwitchToFullScreen;
    procedure UpdateResouces; override;
    procedure UpdateFormPPI(aNewPPI: Integer); override;
    property IsClosing: Boolean read FIsClosing write FIsClosing;
    property FreeOnClose: Boolean read FFreeOnClose write FFreeOnClose;
    property Preview: TfrxPreview read FPreview;
    property Report: TfrxReport read GetReport;
  end;

  TfrxPreviewWorkspace = class(TfrScrollWin)
  private
    FActiveFrameColor: TColor;
    FBackColor: TColor;
    FDefaultCursor: TCursor;
    FDisableUpdate: Boolean;
    FDown: Boolean;
    FEMFImage: TMetafile;
    FEMFImagePage: Integer;
    FFrameColor: TColor;
    FIsThumbnail: Boolean;
    FLastFoundPage: Integer;
    FLastPoint: TPoint;
    FLocked: Boolean;
    FOffset: TPoint;
    FTimeOffset: Cardinal;
    FPageList: TfrxPageList;
    FPageNo: Integer;
    FPreview: TfrxPreview;
    FLockVScroll: Boolean;
    //FPreviewPages: TfrxCustomPreviewPages;
    FZoom: Extended;
    FRTLLanguage: Boolean;
    FHighlightEditable: Boolean;
    FSelectRect: TRect;
    FCachedView: TBitmap;
    procedure DrawPages(BorderOnly: Boolean);
    procedure FindText;
    //procedure FindAllText(Callback : TfrxFindAllTextCallbackFunc; Data : Pointer);
    procedure HighlightText;
    procedure SetToPageNo(PageNo: Integer);
    procedure UpdateScrollBars;
    procedure SetLocked(const Value: Boolean);
    function GetPreviewPages: TfrxCustomPreviewPages;
    procedure LockVScroll;
    procedure UnLockVScroll;
  protected
    procedure PrevDblClick(Sender: TObject);
    procedure MouseDown(Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    function DoMouseWheel(Shift: TShiftState; WheelDelta: Integer;
      MousePos: TPoint): Boolean; override;
    procedure OnHScrollChange(Sender: TObject); override;
    procedure Resize; override;
    procedure OnVScrollChange(Sender: TObject); override;
    procedure CallIteractiveEvent(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer; var aEvent: TfrxPreviewIntEventParams);
    procedure SetEditMode(var aEvent: TfrxPreviewIntEventParams);
    function ScaleRect(const aRect: TRect; Scale, aOffsetX, aOffsetY: Extended): TfrxRect;
    procedure InitEventParams(AMouseIntEvent: TfrxMouseIntEvents; var AEventParams: TfrxPreviewIntEventParams);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure SetPosition(PageN, Top: Integer);
    function GetTopPosition: Integer;
    { page list }
    procedure AddPage(AWidth, AHeight: Integer);
    procedure ClearPageList;
    procedure CalcPageBounds(ClientWidth: Integer);

    property ActiveFrameColor: TColor read FActiveFrameColor write FActiveFrameColor default $804020;
    property BackColor: TColor read FBackColor write FBackColor default clGray;
    property FrameColor: TColor read FFrameColor write FFrameColor default clBlack;
    property IsThumbnail: Boolean read FIsThumbnail write FIsThumbnail;
    property Locked: Boolean read FLocked write SetLocked;
    property PageNo: Integer read FPageNo write FPageNo;
    property Preview: TfrxPreview read FPreview write FPreview;
    property PreviewPages: TfrxCustomPreviewPages read GetPreviewPages;
    // FPreviewPages
//      write FPreviewPages;
    property Zoom: Extended read FZoom write FZoom;
    property RTLLanguage: Boolean read FRTLLanguage write FRTLLanguage;
    property HighlightEditable: Boolean read FHighlightEditable write FHighlightEditable;
    property OnDblClick;
  end;

  TfrxPageItem = class(TCollectionItem)
  public
    Height: Integer;
    Width: Integer;
    OffsetX: Integer;
    OffsetY: Integer;
  end;

  TfrxPageList = class(TCollection)
  private
    FMaxWidth: Integer;
    function GetItems(Index: Integer): TfrxPageItem;
  public
    constructor Create;
    property Items[Index: Integer]: TfrxPageItem read GetItems; default;
    procedure AddPage(AWidth, AHeight: Integer; Zoom: Extended);
    procedure CalcBounds(ClientWidth: Integer);
    function FindPage(OffsetY: Integer; OffsetX: Integer = 0): Integer;
    function GetPageBounds(Index, ClientWidth: Integer; Scale: Extended; RTL: Boolean): TRect;
    function GetMaxBounds: TPoint;
  end;


implementation
end.
