
{******************************************}
{                                          }
{             FastReport VCL               }
{              Preview Pages               }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxPreviewPages;

interface

{$I frx.inc}

uses
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLType, LResources, LCLIntf, LazHelper,
  {$IFDEF LCLGTK2} CairoCanvas, {$ENDIF}
{$ENDIF}
  Messages, Types, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Printers, Variants,
  frxClass, frXML, frPictureCache, frBaseGraphicsTypes, frxVariables, frCore;

type
  TfrxOutline = class(TfrxCustomOutline)
  private
  protected
    function GetCount: Integer; override;
  public
    function Root: TfrXMLItem;
    procedure AddItem(const Text: String; Top: Integer); override;
    procedure DeleteItems(From: TfrXMLItem); override;
    procedure LevelDown(Index: Integer); override;
    procedure LevelRoot; override;
    procedure LevelUp; override;
    procedure GetItem(Index: Integer; var Text: String;
      var Page, Top: Integer); override;
    procedure ShiftItems(From: TfrXMLItem; NewTop: Integer); override;
    function GetCurPosition: TfrXMLItem; override;
  end;

  TfrxDictionary = class(TObject)
  private
    FNames: TStringList;
    FSourceNames: TStringList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(const Name, SourceName: String; Obj: TObject);
    procedure Clear;
    function AddUnique(const Base, SourceName: String; Obj: TObject): String;
    function CreateUniqueName(const Base: String): String;
    function GetSourceName(const Name: String): String;
    function GetObject(const Name: String): TObject;
    property Names: TStringList read FNames;
    property SourceNames: TStringList read FSourceNames;
  end;

  TfrxSignaturePages = class(TStringList)
  protected
    FActive: Boolean;
  public
    constructor Create;
    procedure Activate;
    procedure Deactivate;
    function IsContaine(Name: string): Boolean;

    property Active: Boolean read FActive;
  end;

  TfrxPreviewPages = class(TfrxCustomPreviewPages)
  private
    FAllowPartialLoading: Boolean;
    FDictionary: TfrxDictionary;   { list of all objects }
    FFirstObjectIndex: Integer;   { used in the ClearFirstPassPages }
    FFirstPageIndex: Integer;     { used in the ClearFirstPassPages }
    FFirstOutlineIndex: Integer;  { used in the ClearFirstPassPages }
    FLogicalPageN: Integer;
    FPageCache: TStringList;  { last 20 TfrxPreviewPage }
    FPagesItem: TfrXMLItem;  { shortcut to XMLDoc.Root.FindName('previewpages') }
    FPictureCache: TfrxPictureCache;
    FSourcePages: TList;      { list of source pages }
    FTempStream: TStream;
    FXMLDoc: TfrXMLDocument; { parsed FP3 document }
    FXMLSize: Integer;
    FLastY: Extended;         { last Y for composite report created with AddFrom }
//    FMouseInView: Boolean;    { used in the ObjectOver to deteterminate mouseleave/mouseenter events }
    FLastObjectOver: TfrxView;
    FLastDrill: TfrxGroupHeader;
    FLockedPage: TfrxReportPage;
    FCachedGraphicState: TfrxCachedGraphicType;
    FNeedRefresh: Boolean;
    procedure AfterLoad;
    procedure ApplyStyle(AComponent: TfrxComponent);
    procedure BeforeSave;
    procedure ClearSourcePages;
    function CurXMLPage: TfrXMLItem;
    function GetObject(const Name: String): TfrxComponent;
    procedure DoLoadFromStream;
    procedure DoSaveToStream;
    function ProcessHyperlink(Hyperlink: TfrxHyperlink): Boolean;
    procedure ThumbnailsUpdated(Sender: TObject; UpdateState: TfrxThumbnailsUpdateState);
  protected
    FSignaturePages: TfrxSignaturePages;
    procedure ClearPageCache; override;
    function GetCount: Integer; override;
    function GetPage(Index: Integer): TfrxReportPage; override;
    function GetPageSize(Index: Integer): TPoint; override;
    procedure UpdateCachedObject(CachedObject: TfrxComponent; IsLoading: Boolean = False);
    function GetPictureCacheOptions: TfrxPictureCacheOptions; override;
    // internal use in script code
    property NeedRefresh: Boolean read FNeedRefresh write FNeedRefresh;
  public
    constructor Create(AReport: TfrxReport); override;
    destructor Destroy; override;
    procedure Clear; override;
    procedure Initialize; override;

    { engine commands }
    procedure AddAnchor(const Text: String);
    procedure AddObject(Obj: TfrxComponent); override;
    procedure AddPage(Page: TfrxReportPage); override;
    procedure AddPicture(Picture: TfrxPictureView); override;
    function AddGraphic(Graphic: TGraphic): Integer; override;
    procedure AddSourcePage(Page: TfrxReportPage); override;
    procedure AddToSourcePage(Obj: TfrxComponent); override;
    procedure BeginPass; override;
    procedure ClearFirstPassPages; override;
    procedure CutObjects(APosition: Integer); override;
    procedure DeleteObjects(APosition: Integer); override;
    procedure DeleteAnchors(From: Integer); override;
    procedure Finish; override;
    procedure FireMouseLeave; override;
    procedure IncLogicalPageNumber; override;
    procedure ResetLogicalPageNumber; override;
    procedure PasteObjects(X, Y: Extended); override;
    procedure ShiftAnchors(From, NewTop: Integer); override;
    procedure UpdatePageDimensions(Page: TfrxReportPage; Width, Height: Extended);
    { fast update of page bounds used for endless width }
    procedure UpdatePageDimension(Index: Integer; Width, Height: Extended; AOrientation: TPrinterOrientation);
    procedure UpdatePageLastColumn(aColumn: Integer; aColumnPos: Extended; aReset: Boolean = False);
    procedure GetLastColumnPos(var aColumn: Integer; var ColumnPos: Extended);
    function BandExists(Band: TfrxBand): Boolean; override;
    function FindAnchor(const Text: String): TfrXMLItem;
    function FindAnchorRoot: TfrXMLItem;
    function GetPicture(CacheType: TfrxCachedGraphicType; ImageIndex: Integer): IfrxCachedGraphic; override;
    function GetAnchorPage(const Text: String): Integer;
    function GetAnchorCurPosition: Integer; override;
    function GetCurPosition: Integer; override;
    function GetLastY(ColumnPosition: Extended = 0): Extended; override;
    function GetLogicalPageNo: Integer; override;
    function GetLogicalTotalPages: Integer; override;
    { Lock/Unlock page uses to save-lock current exporting page }
    { in case export filter clears PageChache                   }
    procedure LockPage;
    procedure UnlockPage;

    { preview commands }
{$IFDEF ACADEMIC_ED}
    procedure DrawWaterMark(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY, aPaperWidth, aPaperHeight: Extended);
{$ENDIF}
    procedure DrawPage(Index: Integer; Canvas: TCanvas; ScaleX, ScaleY,
      OffsetX, OffsetY: Extended; bHighlightEditable: Boolean = False); override;
    procedure AddEmptyPage(Index: Integer); override;
    procedure DeletePage(Index: Integer); override;
    procedure ModifyPage(Index: Integer; Page: TfrxReportPage); override;
    procedure ModifyObject(Component: TfrxComponent); override;
    procedure AddFrom(Report: TfrxReport); overload; override;
    procedure AddFromPreviewPages(vPreviewPages: TfrxCustomPreviewPages); overload; override;
    procedure LoadFromStream(Stream: TStream;
      AllowPartialLoading: Boolean = False); override;
    function SaveToFilter(Filter: TfrxCustomIOTransport; const FileName: String): Boolean; override;
    procedure SaveToStream(Stream: TStream); override;
    function LoadFromFile(const FileName: String;
      ExceptionIfNotFound: Boolean = False): Boolean; override;
    procedure SaveToFile(const FileName: String); override;
    function Print: Boolean; override;
    function Export(Filter: TfrxCustomExportFilter): Boolean; override;
    procedure ObjectOver(Index: Integer; X, Y: Integer; Button: TMouseButton;
      Shift: TShiftState; Scale, OffsetX, OffsetY: Extended; var aEventParams: TfrxPreviewIntEventParams); override;
    property SourcePages: TList read FSourcePages;
  end;


implementation
end.
