
{ ****************************************** }
{                                            }
{             FastReport VCL                 }
{          Barcode Add-in object             }
{                                            }
{         Copyright (c) 1998-2021            }
{            by Fast Reports Inc.            }
{                                            }
{ ****************************************** }

unit frxQRCodeEPCPreset;

interface

{$I frx.inc}

uses
  SysUtils, Classes, Variants, frxClass, frxBarcode2D, Graphics, Types,
{$IFDEF FPC}
  LResources, LMessages, LCLType, LCLIntf, LazarusPackageIntf,
  LCLProc, FileUtil, LazHelper
{$ELSE}
  Windows
{$ENDIF};

const
  frxEPCVersionString: array[0..1] of string = ('001', '002');
  frxEPCCurrencyString: array[0..0] of string = ('EUR');

type
  TfrxEPCVersion = (V1 = 0, V2 = 1);
  TfrxEPCCurrency = (EUR = 0);
  TfrxEPCCharSet = (UTF8 = 1);
  TfrxEPCTypeRemittance = (iref, itext);

  TOneStringContent = (a, n);
  TStringContent = set of TOneStringContent;
  TArrayI = array of Integer;

  TfrxEPCHintType = (htEPCNone, htEPCLeft, htEPCRight, htEPCTop, htEPCBottom);
  TfrxEPCPaymentPreset = class;

  TfrxEPCDrawOptions = class(TPersistent)
  private
    FHint: String;
    FFrameWidth: Integer;
    FFrameColor: TColor;
    FFrameVisible: Boolean;
    FFillColor: TColor;
    FFont: TFont;
    FHintType: TfrxEPCHintType;
    FHintExpression: String;
    FOwner: TfrxEPCPaymentPreset;
    procedure SetFrameWidth(const Value: Integer);
    procedure SetFrameVisible(const Value: Boolean);
    procedure SetHintType(const Value: TfrxEPCHintType);
  public
    constructor Create(Owner: TfrxEPCPaymentPreset);
    destructor Destroy; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY: Extended; Area: TRect);
    function CalcQuietZone: Integer;
  published
    property FrameColor: TColor read FFrameColor write FFrameColor default clBlack;
    property FillColor: TColor read FFillColor write FFillColor default clWhite;
    property Hint: String read FHint write FHint;
    property HintExpression: String read FHintExpression write FHintExpression;
    property HintType: TfrxEPCHintType read FHintType write SetHintType;
    property FrameWidth: Integer read FFrameWidth write SetFrameWidth default 1;
    property FrameVisible: Boolean read FFrameVisible write SetFrameVisible default False;
  end;

  TfrxEPCPMoney = class(TPersistent)
  private
    FCurrency: TfrxEPCCurrency;
    FAmount: String;
  published
    property Currency: TfrxEPCCurrency read FCurrency write FCurrency;
    property Amount: String read FAmount write FAmount;
  end;

  TfrxEPCPRemittance = class(TPersistent)
  private
    FTypeRemittance: TfrxEPCTypeRemittance;
    FValue: String;
  published
    property Value: String read FValue write FValue;
    property TypeRemittance: TfrxEPCTypeRemittance read FTypeRemittance write FTypeRemittance;
  end;

  TfrxEPCPaymentPreset = class(TfrxCustomObjectPreset)
  private
    FServiceTag: String;
    FVersion: TfrxEPCVersion;
    FCharSet: TfrxEPCCharSet;
    FIdentification: String;
    FBIC: String;
    FName: String;
    FIBAN: String;
    FDrawOptions: TfrxEPCDrawOptions;
    FMoney: TfrxEPCPMoney;
    FPurpose: String;
    FRemittance: TfrxEPCPRemittance;
    FInformation: String;
    //
    FSaveErrorLevels: Integer;
    FSaveShowText: Boolean;
    FOwner: TfrxComponent;
    procedure CheckString(pref, s: String; variants: TArrayI; Content: TStringContent; Mandatory: Boolean); overload;
    //procedure CheckString(pref, s: String; Lng: Integer; Content: TStringContent; Mandatory: Boolean); overload;
    procedure CheckString(pref, s: String; LngMin, LngMax: Integer; Content: TStringContent; Mandatory: Boolean); overload;
    procedure StartCheck(pref, s: String; Mandatory: Boolean);
    procedure CheckContent(pref, s: String; Content: TStringContent);
    function AmountToString(aReport: TfrxReport): String;
    function RemittanceToString(aReport: TfrxReport): String;
    procedure CreateError(a: String);
  public
    constructor Create; override;
    destructor Destroy; override;
    function GetData(aReport: TfrxReport): String; override;
    procedure ApplySettings(aComponent: TfrxComponent); override;
    procedure SaveComponentState(aComponent: TfrxComponent); override;
    procedure RestoreComponentState(aComponent: TfrxComponent); override;
    procedure BeginDraw(Canvas: TCanvas; ScaleX, ScaleY: Extended; Area: TRect); override;
    procedure UpdateSettings;
  published
    property ServiceTag: String read FServiceTag;
    property Version: TfrxEPCVersion read FVersion write FVersion;
    property CharSet: TfrxEPCCharSet read FCharSet write FCharSet;
    property Identification: String read FIdentification;
    property BIC: String read FBIC write FBIC;
    property Name: String read FName write FName;
    property IBAN: String read FIBAN write FIBAN;
    property Money: TfrxEPCPMoney read FMoney write FMoney;
    property Purpose: String read FPurpose write FPurpose;
    property Remittance: TfrxEPCPRemittance read FRemittance write FRemittance;
    property Information: String read FInformation write FInformation;
    property DrawOptions: TfrxEPCDrawOptions read FDrawOptions write FDrawOptions;
  end;

  function GetValue(aReport: TfrxReport; const aFileld: String): String;
  function VersionToString(vers: TfrxEPCVersion): String;
  function CurrencyToString(cur: TfrxEPCCurrency): String;
  function TArrayIToString(arr: TArrayI): String;
  function FindInArray(arr: TArrayI; val: Integer): Boolean;

implementation
end.
