{ ****************************************** }
{                                            }
{             FastReport VCL                 }
{            QRCodeSberPreset                }
{                                            }
{         Copyright (c) 1998-2021            }
{            by Fast Reports Inc.            }
{                                            }
{ ****************************************** }

unit frxQRCodeSberPreset;

interface

{$I frx.inc}

uses
  SysUtils, Classes, Variants, frxClass, TypInfo, frxBarcode2D;

type

 TfrxSberEncoding = (ec_WIN1251 ,ec_UTF8 ,ec_KOI8_R);
 TfrxFieldTypesQR = (fdExpression, fdStatic);

 TfrxCustomPresetAttr = class(TPersistent)
  private
    FSeparator: Char;
    FEncoding: TfrxSberEncoding;
    FFieldTypes : TfrxFieldTypesQR;
    function GetValue(aReport: TfrxReport; const aFileld: String; const aFieldName: String): String;
  public
    function GetData(aReport: TfrxReport): String; virtual; abstract;
  published
    function GetEncoding: TfrxSberEncoding;
    function GetSeparator: Char;
    function GetFieldTypes: TfrxFieldTypesQR;
  end;

 TfrxSberPayee = class(TfrxCustomPresetAttr)
  private
    FName: String;
    FPersonalAcc: String;
    FBankName: String;
    FBIC: String;
    FCorrespAcc: String;
    procedure SetName(val: String);
    procedure SetPersonalAcc(val: String);
    procedure SetBankName(val: String);
    procedure SetBIC(val: String);
    procedure SetCorrespAcc(val: String);
  public
    function GetData(aReport: TfrxReport): String; override;
  published
    property Name: String read FName write SetName;
    property PersonalAcc: String read FPersonalAcc write SetPersonalAcc;
    property BankName: String read FBankName write SetBankName;
    property BIC: String read FBIC write SetBIC;
    property CorrespAcc: String read FCorrespAcc write SetCorrespAcc;
  end;

 TfrxSberPayeeAdditionalInfo = class(TfrxCustomPresetAttr)
  private
    FPayeeINN: String;
    FKPP: String;
    procedure SetPayeeINN(val: String);
    procedure SetKPP(val: String);
  public
    function GetData(aReport: TfrxReport): String; override;
  published
    property PayeeINN: String read FPayeeINN write SetPayeeINN;
    property KPP: String read FKPP write SetKPP;
  end;

  TfrxSberDepartmentalInfo = class(TfrxCustomPresetAttr)
  private
    FDrawerStatus: String;
    FCBC: String;
    FOKTMO: String;
    FPaytReason: String;
    FTaxPeriod: String;
    FDocNo: String;
    FDocDate: String;
    FTaxPaytKind: String;
    procedure SetDrawerStatus(val: String);
    procedure SetCBC(val: String);
    procedure SetOKTMO(val: String);
    procedure SetPaytReason(val: String);
    procedure SetTaxPeriod(val: String);
    procedure SetDocNo(val: String);
    procedure SetDocDate(val: String);
    procedure SetTaxPaytKind(val: String);
  public
    function GetData(aReport: TfrxReport): String; override;
  published
    property DrawerStatus: String read FDrawerStatus write SetDrawerStatus;
    property CBC: String read FCBC write SetCBC;
    property OKTMO: String read FOKTMO write SetOKTMO;
    property PaytReason: String read FPaytReason write SetPaytReason;
    property TaxPeriod: String read FTaxPeriod write SetTaxPeriod;
    property DocNo: String read FDocNo write SetDocNo;
    property DocDate: String read FDocDate write SetDocDate;
    property TaxPaytKind: String read FTaxPaytKind write SetTaxPaytKind;
  end;

  TfrxSberRequiredBankingDetails = class(TfrxCustomPresetAttr)
  private
    FPayee: TfrxSberPayee;
    procedure SetPayee(val: TfrxSberPayee);
  public
    constructor Create;
    destructor Destroy; override;
    function GetData(aReport: TfrxReport): String; override;
  published
    property Payee: TfrxSberPayee read FPayee write SetPayee;
  end;

  TfrxSberAdditionalBankingDetails = class(TfrxCustomPresetAttr)
  private
    FSum: String;
    FPurpose: String;
    FPayeeAdditionalInfo: TfrxSberPayeeAdditionalInfo;
    FPayerINN: String;
    FDepartmentalInfo: TfrxSberDepartmentalInfo;
    procedure SetSum(val: String);
    procedure SetPurpose(val: String);
    procedure SetPayeeAdditionalInfo(val: TfrxSberPayeeAdditionalInfo);
    procedure SetPayerINN(val: String);
    procedure SetDepartmentalInfo(val: TfrxSberDepartmentalInfo);
  public
    constructor Create;
    destructor Destroy; override;
    function GetData(aReport: TfrxReport): String; override;
  published
    property Sum: String read FSum write SetSum;
    property Purpose: String read FPurpose write SetPurpose;
    property PayeeAdditionalInfo: TfrxSberPayeeAdditionalInfo read FPayeeAdditionalInfo
      write SetPayeeAdditionalInfo;
    property PayerINN: String read FPayerINN write SetPayerINN;
    property DepartmentalInfo: TfrxSberDepartmentalInfo read FDepartmentalInfo
      write SetDepartmentalInfo;
  end;

  TfrxSberOtherAdditionalBankingDetails = class(TfrxCustomPresetAttr)
  private
    FLastName: String;
    FFirstName: String;
    FMiddleName: String;
    FPayerAddress: String;
    FPersonalAccount: String;
    FDocIdx: String;
    FPensAcc: String;
    FContract: String;
    FPersAcc: String;
    FFlat: String;
    FPhone: String;
    FPayerIdType: String;
    FPayerIdNum: String;
    FChildFio: String;
    FBirthDate: String;
    FPaymTerm: String;
    FPaymPeriod: String;
    FCategory: String;
    FServiceName: String;
    FCounterId: String;
    FCounterVal: String;
    FQuittId: String;
    FInstNum: String;
    FClassNum: String;
    FSpecFio: String;
    FAddAmount: String;
    FRuleId: String;
    FExecId: String;
    FRegType: String;
    FUIN: String;
    FTechCode: String;
  public
    function GetData(aReport: TfrxReport): String; override;
  published
    property LastName: String read FLastName write FLastName;
    property FirstName: String read FFirstName write FFirstName;
    property MiddleName: String read FMiddleName write FMiddleName;
    property PayerAddress: String read FPayerAddress write FPayerAddress;
    property PersonalAccount: String read FPersonalAccount write FPersonalAccount;
    property DocIdx: String read FDocIdx write FDocIdx;
    property PensAcc: String read FPensAcc write FPensAcc;
    property Contract: String read FContract write FContract;
    property PersAcc: String read FPersAcc write FPersAcc;
    property Flat: String read FFlat write FFlat;
    property Phone: String read FPhone write FPhone;
    property PayerIdType: String read FPayerIdType write FPayerIdType;
    property PayerIdNum: String read FPayerIdNum write FPayerIdNum;
    property ChildFio: String read FChildFio write FChildFio;
    property BirthDate: String read FBirthDate write FBirthDate;
    property PaymTerm: String read FPaymTerm write FPaymTerm;
    property PaymPeriod: String read FPaymPeriod write FPaymPeriod;
    property Category: String read FCategory write FCategory;
    property ServiceName: String read FServiceName write FServiceName;
    property CounterId: String read FCounterId write FCounterId;
    property CounterVal: String read FCounterVal write FCounterVal;
    property QuittId: String read FQuittId write FQuittId;
    property InstNum: String read FInstNum write FInstNum;
    property ClassNum: String read FClassNum write FClassNum;
    property SpecFio: String read FSpecFio write FSpecFio;
    property AddAmount: String read FAddAmount write FAddAmount;
    property RuleId: String read FRuleId write FRuleId;
    property ExecId: String read FExecId write FExecId;
    property RegType: String read FRegType write FRegType;
    property UIN: String read FUIN write FUIN;
    property TechCode: String read FTechCode write FTechCode;
  end;

 TfrxSberData = class(TfrxCustomPresetAttr)
  private
    FIdFormat: String;
    FVersion: String;
    FRequiredBankingDetails: TfrxSberRequiredBankingDetails;
    FAdditionalBankingDetails: TfrxSberAdditionalBankingDetails;
    FOtherAdditionalBankingDetails: TfrxSberOtherAdditionalBankingDetails;
    procedure SetIdFormat(val: String);
    procedure SetVersion(val: String);
    procedure SetEncoding(val: TfrxSberEncoding);
    procedure SetSeparator(val: Char);
    procedure SetFieldTypes(val: TfrxFieldTypesQR);
  public
    constructor Create;
    destructor Destroy; override;
    function GetData(aReport: TfrxReport): String; override;
  published
    property IdFormat: String read FIdFormat write SetIdFormat;
    property Version: String read FVersion write SetVersion;
    property Encoding: TfrxSberEncoding read GetEncoding write SetEncoding default ec_WIN1251;
    property Separator: Char read GetSeparator write SetSeparator default '|';
    property FieldTypes : TfrxFieldTypesQR read GetFieldTypes write SetFieldTypes default fdExpression;
  end;

   TfrxSberPaymentPreset = class(TfrxCustomObjectPreset)
  private
    FSberData: TfrxSberData;
    FSaveErrorLevels: Integer;
    FSaveMarker: Boolean;
    FSaveShowText: Boolean;
    function GetRequiredBankingDetails: TfrxSberRequiredBankingDetails;
    function GetAdditionalBankingDetails: TfrxSberAdditionalBankingDetails;
    function GetOtherAdditionalBankingDetails: TfrxSberOtherAdditionalBankingDetails;
  public
    constructor Create; override;
    destructor Destroy; override;
    function GetData(aReport: TfrxReport): String; override;
    procedure ApplySettings(aComponent: TfrxComponent); override;
    procedure SaveComponentState(aComponent: TfrxComponent); override;
    procedure RestoreComponentState(aComponent: TfrxComponent); override;
  published
    property SberData: TfrxSberData read FSberData;
    property RequiredBankingDetails: TfrxSberRequiredBankingDetails
      read GetRequiredBankingDetails;
    property AdditionalBankingDetails: TfrxSberAdditionalBankingDetails
      read GetAdditionalBankingDetails;
    property OtherAdditionalBankingDetails: TfrxSberOtherAdditionalBankingDetails
      read GetOtherAdditionalBankingDetails;
  end;

implementation
end.
