
{ ****************************************** }
{                                            }
{             FastReport VCL                 }
{          Barcode Add-in object             }
{                                            }
{         Copyright (c) 1998-2021            }
{            by Fast Reports Inc.            }
{                                            }
{ ****************************************** }

unit frxQRCodeSwissPreset;

interface

{$I frx.inc}

uses
  SysUtils, Classes, Variants, frxClass, frxBarcode2D
{$IFNDEF FPC}
     , Windows
{$ELSE}
  , LCLType
{$ENDIF}
  , Graphics;

type
 TfrxCustomPresetAttr = class(TPersistent)
  private
    function GetValue(aReport: TfrxReport; const aFileld: String): String;
  public
    function GetData(aReport: TfrxReport): String; virtual; abstract;
  end;

  TfrxSPCreditorInformation = class(TfrxCustomPresetAttr)
  private
    FIBAN: String;
  public
    function GetData(aReport: TfrxReport): String; override;
  published
    property IBAN: String read FIBAN write FIBAN;
  end;

  TfrxSPAddressType = (frAddT_S, frAddT_K);


  TfrxSPPersonInfo = class(TfrxCustomPresetAttr)
  private
    FTown: String;
    FName: String;
    FAddressTypeExp: String;
    FPostalCode: String;
    FAddressLine2: String;
    FAddressLine1: String;
    FAddressType: TfrxSPAddressType;
    FCountry: String;
  protected
    function GetAddrType(aReport: TfrxReport): String; virtual;
  public
    function GetData(aReport: TfrxReport): String; override;
    property AddressType: TfrxSPAddressType read FAddressType write FAddressType;
    property AddressTypeExp: String read FAddressTypeExp write FAddressTypeExp;
    property Name: String read FName write FName;
    property AddressLine1: String read FAddressLine1 write FAddressLine1;
    property AddressLine2: String read FAddressLine2 write FAddressLine2;
    property PostalCode: String read FPostalCode write FPostalCode;
    property Town: String read FTown write FTown;
    property Country: String read FCountry write FCountry;
  end;

  TfrxSPCreditor = class(TfrxSPPersonInfo)
  published
    property AddressType;
    property AddressTypeExp;
    property Name;
    property AddressLine1;
    property AddressLine2;
    property PostalCode;
    property Town;
    property Country;
  end;

  TfrxSPUltimateInfo = class(TfrxSPPersonInfo)
  protected
    function GetAddrType(aReport: TfrxReport): String; override;
  end;

  TfrxSPUltimateCreditor = class(TfrxSPUltimateInfo)
  published
    property AddressType;
    property AddressTypeExp;
    property Name;
    property AddressLine1;
    property AddressLine2;
    property PostalCode;
    property Town;
    property Country;
  end;

  TfrxSPPaymentAmountInfo = class(TfrxCustomPresetAttr)
  private
    FAmount: String;
    FCurrency: String;
  public
    function GetData(aReport: TfrxReport): String; override;
  published
    property Amount: String read FAmount write FAmount;
    property Currency: String read FCurrency write FCurrency;
  end;

  TfrxSPUltimateDebtor = class(TfrxSPUltimateInfo)
  published
    property AddressType;
    property AddressTypeExp;
    property Name;
    property AddressLine1;
    property AddressLine2;
    property PostalCode;
    property Town;
    property Country;
  end;

  TfrxSPReferenceType = (frRT_QRR, frRT_SCOR, frRT_NON);

  TfrxSPPaymentReference = class(TfrxCustomPresetAttr)
  private
    FReferenceTyp: TfrxSPReferenceType;
    FReference: String;
    FReferenceTypExp: String;
  public
    function GetData(aReport: TfrxReport): String; override;
  published
    property ReferenceTyp: TfrxSPReferenceType read FReferenceTyp write FReferenceTyp;
    property ReferenceTypExp: String read FReferenceTypExp write FReferenceTypExp;
    property Reference: String read FReference write FReference;
  end;

  TfrxSPAdditionalInformation = class(TfrxCustomPresetAttr)
  private
    FUnstructuredMessage: String;
    FBillInformation: String;
  public
    function GetData(aReport: TfrxReport): String; override;
  published
    property UnstructuredMessage: String read FUnstructuredMessage write FUnstructuredMessage;
    property BillInformation: String read FBillInformation write FBillInformation;
  end;

  TfrxSPAlternativeSchemes = class(TfrxCustomPresetAttr)
  private
    FSchemeParameterOne: String;
    FSchemeParameterTwo: String;
  public
    function GetData(aReport: TfrxReport): String; override;
  published
    property SchemeParameterOne: String read FSchemeParameterOne write FSchemeParameterOne;
    property SchemeParameterTwo: String read FSchemeParameterTwo write FSchemeParameterTwo;
  end;


  TfrxSwissPaymentPreset = class(TfrxCustomObjectPreset)
  private
    FAlternativeSchemes: TfrxSPAlternativeSchemes;
    FPaymentReference: TfrxSPPaymentReference;
    FPaymentAmountInfo: TfrxSPPaymentAmountInfo;
    FUltimateCreditor: TfrxSPUltimateCreditor;
    FCreditor: TfrxSPCreditor;
    FCreditorInformation: TfrxSPCreditorInformation;
    FUltimateDebtor: TfrxSPUltimateDebtor;
    FAdditionalInformation: TfrxSPAdditionalInformation;
    FSaveErrorLevels: Integer;
    FSaveGraphic: TGraphic;
  public
    constructor Create; override;
    destructor Destroy; override;
    function GetData(aReport: TfrxReport): String; override;
    procedure ApplySettings(aComponent: TfrxComponent); override;
    procedure SaveComponentState(aComponent: TfrxComponent); override;
    procedure RestoreComponentState(aComponent: TfrxComponent); override;
  published
    property CreditorInformation: TfrxSPCreditorInformation read FCreditorInformation;
    property Creditor: TfrxSPCreditor read FCreditor;
    property UltimateCreditor: TfrxSPUltimateCreditor read FUltimateCreditor;
    property PaymentAmountInfo: TfrxSPPaymentAmountInfo read FPaymentAmountInfo;
    property UltimateDebtor: TfrxSPUltimateDebtor read FUltimateDebtor;
    property PaymentReference: TfrxSPPaymentReference read FPaymentReference;
    property AdditionalInformation: TfrxSPAdditionalInformation read FAdditionalInformation;
    property AlternativeSchemes: TfrxSPAlternativeSchemes read FAlternativeSchemes;
  end;


implementation
end.
