{******************************************}
{                                          }
{             FastReport VCL               }
{          Result Table builder            }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

/// <summary>
///   This unit contains classes of Result Table report object.
/// </summary>
unit frxResultTableObject;

interface

{$I frx.inc}

uses
  SysUtils, Types, Classes, Graphics, Variants, Controls,
  frCore, frxClass, frxTableObject;

type
  TfrxHackTableCell = class(TfrxTableCell);
  TfrxHackCustomTableRow = class(TfrxCustomTableRow);
  TfrxResultTableObject = class;
  TfrxHackComponent = class(TfrxComponent);

  TfrxCellDataList = class(TfrxObjectsNotifyList)
  private
    FSourceTable: TfrxResultTableObject;
  protected
    function Get(Index: TfrListInteger): Pointer; override;
    procedure Put(Index: TfrListInteger; const Value: Pointer); override;
    function GetCount: TfrListInteger; override;
  end;

  TfrxResultTableObject = class(TfrxTableObject)
  private
    FFixedColumns: TList;
    FFooterColumns: TList;
    FFixedRows: TList;
    FNullCell: TfrxTableCell;
    FAggregatesList: TList;
    FReport: TfrxReport;
  protected
    function GetAggregateResult(Report: TfrxReport; AggCell: TfrxTableCell; ResultTableAddr: TPoint): Boolean;
    function GetReport: TfrxReport; override;
    function GetSplitRow(ARow: TfrxCustomTableRow; AHeight: Extended; var IsLastPart: Boolean): TfrxCustomTableRow; override;
    function GetCurColumnsHash: String;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetCellData(x, y: Integer): TfrxCellData;
    class function CreateTableRow: TfrxCustomTableRow; override;
  end;

  TfrxRowPart = class
  private
    FRowForSplit: TfrxCustomTableRow;
    FPartIndex: Integer;
    FRemainHeight: Extended;
  public
    constructor Create(ARow: TfrxCustomTableRow);
    destructor Destroy; override;
  end;

  TfrxTableRow = class(TfrxCustomTableRow)
  private
    FRowForSplit: TfrxCustomTableRow;
    FSplitParts: TStringList;
    FSplitRows: TList;
    procedure ClrearCellData;
    function GetColumnsHash: String;
    procedure CreatePartsList;
    function GetExistPart(const Hash: String): TfrxRowPart;
    function AddSplitPart(const Hash: String; RowPart: TfrxCustomTableRow): TfrxRowPart;
    procedure FreeSplitRows;
  protected
    function GetCellData(Index: Integer): TfrxCellData;
    function GetCell(Index: Integer): TfrxTableCell; override;
    class function GetObjectListClass: TfrxObjectsNotifyListClass; override;
    procedure SetParent(AParent: TfrxComponent); override;
  public
    destructor Destroy; override;
    function CreateTableCell(Index: Integer; bSetUniqueName: Boolean = False): TObject; override;
    function GetSplitRow(AHeight: Extended; Wrapped: Boolean; var IsLastPart: Boolean): TfrxCustomTableRow;
  end;

  TfrxResultTableBuilder = class(TfrxCustomTableBuilder)
  protected
    class function GetResultTableClass: TfrxTableObjectClass; override;
    procedure CopyCells(OriginalColumnIndex, OriginalRowIndex,
      ResultColumnIndex, ResultRowIndex: Integer); override;
    procedure AddFixedColumn(FixedColumn: TfrxTableColumn; DestTable: TfrxTableObject); override;
    procedure AddFixedRow(FixedRow: TfrxCustomTableRow; DestTable: TfrxTableObject); override;
    procedure AddFooterColumn(ColumnFooter: TfrxTableColumn; DestTable: TfrxTableObject); override;
    procedure ClearResultObjects(ATable: TfrxTableObject); override;
    function GetCellData(X, Y: Integer): TfrxCellData; override;
  public
    constructor Create(SourceTable: TfrxTableObject); override;
    destructor Destroy; override;
  end;


implementation
end.
