
{******************************************}
{                                          }
{             FastReport VCL               }
{     Signature and Certificate Dialog     }
{                                          }
{         Copyright (c) 1998-2023          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxSignatureCertificateDialog;

interface

{$I frx.inc}

uses
{$IFNDEF FPC}
{$ELSE}
  LCLType, LCLIntf, LCLProc, LazHelper, ColorBox,
{$ENDIF}
  Forms, Classes, Controls, StdCtrls, ExtCtrls, Buttons, Dialogs, SysUtils,
  frxBaseForm, frxSignatureEngine, frxCertificateList
{$IFDEF FPC}
  , EditBtn
{$ENDIF};

type
  TfrxSignatureCertificateDialog = class(TfrxBaseForm)
    CancelB: TButton;
    OkB: TButton;
    CertificatePropertiesMemo: TMemo;
    CertificateComboBox: TComboBox;
    FoundLabel: TLabel;
    PFXFileOpenDialog: TOpenDialog;
    PFXFileGroupBox: TGroupBox;
    PFXFileSpeedButton: TSpeedButton;
    PasswordLabel: TLabel;
    PFXFileEdit: TEdit;
    PasswordEdit: TEdit;
    CertificateStoreRadioGroup: TRadioGroup;
    TSAURLEdit: TEdit;
    TSAURLLabel: TLabel;
    SigningMethodRadioGroup: TRadioGroup;
    TimeStampCheckBox: TCheckBox;

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SignatureDataChanged(Sender: TObject);
    procedure CertificateDataChanged(Sender: TObject);
    procedure PFXFileSpeedButtonClick(Sender: TObject);
    procedure CertificateComboBoxChange(Sender: TObject);
  private
    function Format: TfrxSignatureFormat;
    function Signing: TfrxSigningMethod;
    function TSAURL: string;
    function Detached: Boolean;
    function Chain: Boolean;
    function GetSignatureData: TfrxSignatureData;
    procedure SetSignatureData(const Value: TfrxSignatureData);
  private
    function Kind: TfrxCertificateStoreKind;
    function PFXFile: TFileName;
    function Password: AnsiString;
    function IgnoreCase: Boolean;
    function SubjectSubStr: string;
    function IssuerSubStr: string;
    function AllowExpired: Boolean;
    function OnlyGOST: Boolean;
  private
    function GetIsEmpty: Boolean;
    function GetSelected: Integer;
    function GetIsGOSTSelected: Boolean;
    function GetCertificateData: TfrxCertificateData;
    procedure SetCertificateData(const Value: TfrxCertificateData);

    property IsEmpty: Boolean read GetIsEmpty;
    property Selected: Integer read GetSelected;
    property IsGOSTSelected: Boolean read GetIsGOSTSelected;
  protected
    FCertificateList: TfrxCertificateList;
    FUpdateEnabled: Boolean;

    procedure UpdateControls;
    procedure UpdateSignatureControls;
    procedure UpdateCertificateProperties;
  public
    procedure UpdateResouces; override;

    property SignatureData: TfrxSignatureData read GetSignatureData write SetSignatureData;
    property CertificateData: TfrxCertificateData read GetCertificateData write SetCertificateData;
  end;

function SignatureCertificateDialog(var SD: TfrxSignatureData;
  var Certificate: TfrxCertificate; var CD: TfrxCertificateData): TModalResult;

implementation
end.
