
{******************************************}
{                                          }
{             FastReport VCL               }
{         Signature Engine Crypto          }
{                                          }
{         Copyright (c) 1998-2023          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxSignatureEngineCrypto;

interface

{$I frx.inc}

uses
  SysUtils, Types,
{$IFNDEF NONWINFPC}
  Windows,
{$ENDIF}
  {Before frxWinCrypt}
  frxSignatureEngine, frxCades, frxWinCrypt, frxHelpers, frxCertificateList;

type
  /// <summary>
  /// Deprecated.
  /// The TCertificateStoreLookup object finds the first certificate context in
  /// a certificate store that matches a search criteria established by it
  /// properties.
  /// </summary>
  TCertificateStoreLookup = class // https://docs.microsoft.com/en-us/windows/win32/api/wincrypt/ns-wincrypt-cert_info
  private
    FIgnoreCase: Boolean;
    FIssuer: WideString;
    FNotBefore: string;
    FNotAfter: string;
    FSubject: WideString;
    FDateFormat: string;
    FCertificatePath: WideString;
  protected
    function IsCertificateSuitable(Certificate: PCERT_CONTEXT): Boolean;
    function IsNameSuitable(const NameBlob: TCertNameBlob; Name: WideString): Boolean;
    function IsWideInclude(Str, SubStr: WideString): Boolean;
    function IsTimeSuitable(const CertTime: TFileTime; SoughtFor: string): Boolean;
  public
    constructor Create;
    function LookupParams: string;
    function FindCertificate(hCertStore: HCERTSTORE): PCERT_CONTEXT;

    /// <summary>
    ///   Allows you to ignore case sensitivity for Issuer and Subject properies.
    ///   Default = False;
    /// </summary>
    property IgnoreCase: Boolean read FIgnoreCase write FIgnoreCase;
    /// <summary>
    ///   This substring (if specified) must be in the certificate Issuer field.
    /// </summary>
    property Issuer: WideString read FIssuer write FIssuer;
    /// <summary>
    ///   This string (if provided) must match the certificate's NotBefore field.
    ///   Note: in Console Root / Certificates dates may be rounded.
    /// </summary>
    property NotBefore: string read FNotBefore write FNotBefore;
    /// <summary>
    ///   This string (if provided) must match the certificate's NotAfter field.
    ///   Note: in Console Root / Certificates dates may be rounded.
    /// </summary>
    property NotAfter: string read FNotAfter write FNotAfter;
    /// <summary>
    ///   This substring (if specified) must be in the certificate Subject field.
    /// </summary>
    property Subject: WideString read FSubject write FSubject;
    /// <summary>
    ///   Determines date format for NotBefore and NotAfter properies.
    ///   Default = 'dd.mm.yy'
    /// </summary>
    property DateFormat: string read FDateFormat write FDateFormat;
    /// <summary>
    ///   pfx / p12 file path.
    /// </summary>
    property CertificatePath: WideString read FCertificatePath write FCertificatePath;
  end;

  TfrxSignatureEngineCrypto = class(TfrxSignatureEngine)
  protected
    FHashOID: LPStr;

    function IsCadesSDKFound(PFunction: Pointer; ErrorStatus: TSignatureSatus): Boolean;
    function GetCertificateCommonName: WideString; override;
    function GetCertificateDistinguishedName: WideString; override;
  protected
    FURL: PWideChar;

    function IsDoChain(context: PCCERT_CONTEXT; signPara: CRYPT_SIGN_MESSAGE_PARA): Boolean;
    procedure GetCadesSignPara(var cadesSignPara: CADES_SIGN_PARA;
      var tspConnectionPara: CADES_SERVICE_CONNECTION_PARA);
    procedure CopyMessage(out SignedMessage: TByteDynArray; pSignedMessage: PCRYPT_DATA_BLOB);

    function IsEnhanced: Boolean;

    function IsOpenCertificateStore(out hStoreHandle: HCERTSTORE): Boolean;
    function OpenPFXStore(Password: AnsiString): HCERTSTORE;

    function IsSignByCryptSignMessage(MessageData: TByteDynArray;
      signPara: CRYPT_SIGN_MESSAGE_PARA; out SignedMessage: TByteDynArray): Boolean;
    function IsStampByCryptRetrieveTimeStamp(var SignedMessage: TByteDynArray): Boolean;

    function IsSignByCadesSignMessage(MessageData: TByteDynArray;
      signPara: CRYPT_SIGN_MESSAGE_PARA; out SignedMessage: TByteDynArray): Boolean;
    function IsStampByCadesEnhanceMessage(var SignedMessage: TByteDynArray): Boolean;
  public
    constructor Create(ASD: TfrxSignatureData; ACertificate: TfrxCertificate; ADebugLog: TLogList);

    function Sign(MessageData: TByteDynArray; out SignedMessage: TByteDynArray): TSignatureSatus; override;

    procedure SignatureHashMD5; override;
    procedure SignatureHashSHA1; override;
    procedure SignatureHashSHA256; override;
  end;

function GetSpecialPath(CSIDL: word): string;
{$IFNDEF LINUX}
procedure SetCryptoProSDKPath(SDKPath: TFileName = '');
{$ENDIF}

implementation
end.
