
{******************************************}
{                                          }
{             FastReport VCL               }
{      Signature Engine FastReports        }
{                                          }
{         Copyright (c) 1998-2023          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxSignatureEngineFastReport;

interface

{$I frx.inc}

uses
  Types, Classes,
  frxHelpers, frxASN1, frxPDFPFX, frxSignatureEngine, frxCertificateList, frxHash;

type
  TfrxSignatureEngineFastReport = class(TfrxSignatureEngine)
  private
    FPFX: TfrxPKCS12Document;
    FDigest: TfrxASN1Data;
    FSignedDigest: TfrxASN1Data;
    FSign: TfrxASN1Base;

    FHashOIDName: TOIDName;
    FHashClass: TfrxHashClass;

    procedure FRInit(Stream: TStream; Password: AnsiString = '');
    procedure FRInitPFXFile;
    procedure FRInitCertificate;
    procedure PrepareSign;
    procedure CalcHash(MessageData: TByteDynArray);
  protected
    function GetCertificateCommonName: WideString; override;
    function GetCertificateDistinguishedName: WideString; override;
  public
    constructor Create(ASD: TfrxSignatureData; ACertificate: TfrxCertificate; ADebugLog: TLogList);
    destructor Destroy; override;

    function Sign(MessageData: TByteDynArray; out SignedMessage: TByteDynArray): TSignatureSatus; override;

    procedure SignatureHashMD5; override;
    procedure SignatureHashSHA1; override;
    procedure SignatureHashSHA256; override;
  end;

implementation
end.
