{******************************************}
{                                          }
{             FastReport VCL               }
{           Simple Memo control            }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxSimpleMemo;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}
  Windows, Imm,
  {$ELSE}
  LCLType, LMessages, LazHelper, LCLIntf, LCLProc,
  {$ENDIF}
  Messages, Types, SysUtils, Classes, Graphics, Controls, StdCtrls, ExtCtrls,
  Forms, Menus, IniFiles, frxClass, frxCtrls, fs_iparser, frPopupForm, fs_xml,
  fs_iinterpreter, frxRes, frTypes, frxHint, frGraphicUtils, frVectorCanvas,
  frUnicodeUtils, frBaseMemo, frDialectHelper
  {$IFDEF Delphi10}
  , WideStrings
  {$ENDIF}
  ;

const
  WM_FRX_SYNC_SCRIPT = WM_USER + 100;
  WM_FRX_UPDATE_CODE = WM_USER + 101;
  WM_FRX_FILL_CODE_COMPLETION = WM_USER + 102;

type

  TfrxCompletionList = class;
  //TCharAttributes = set of TCharAttr;
  TfrxCharAttributes = record
    StyleIndex: Byte;
    IsSelBlock: Boolean;
  end;

  TfrxCodeCompletionEvent = procedure(const Name: String; List: TfrxCompletionList) of object;

  TfrxByteArr = array of Byte;

  TfrxSynAttributes = class
  private
    FArray: array of TfrxByteArr;
    FCapacity: Integer;
    FCount: Integer;
    FUpdating: Boolean;
    FDialectStyles: TfrSynDialectStyles;
    FParser: TfsParser;
    procedure SetCapacity(NewCapacity: Integer);
    procedure SetCount(NewCount: Integer);
    function GetLine(Index: Integer): TfrxByteArr;
    procedure PutLine(Index: Integer; const Value: TfrxByteArr);
    procedure ActiveChanged(Sender: TObject);
  public
    constructor Create(Parser: TfsParser; DialectStyles: TfrSynDialectStyles);
    procedure SetLineLen(Index: Integer; NewLen: Integer);
    procedure Delete(Index: Integer);
    function GetAllAttributes: TfrxByteArr;
    procedure SetAllAttributes(Attr: TfrxByteArr);
    procedure UpdateSyntax(EndLine: Integer; Text: TStringList);
    procedure UpdateSyntaxDialect;
    property Line[Index: Integer]: TfrxByteArr read GetLine write PutLine; default;
    property Count: Integer read FCount write SetCount;
    property Updating: Boolean read FUpdating;
  end;

  TfrxBreakPoint = class
  private
    FCondition: String;
    FSpecialCondition: String;
    FEnabled: Boolean;
    FLine: Integer;
  public
    property Condition: String read FCondition write FCondition;
    property Enabled: Boolean read FEnabled write FEnabled;
    property Line: Integer read FLine write FLine;
    property SpecialCondition: String read FSpecialCondition write FSpecialCondition;
  end;

  TfrxItemType = (itVar, itProcedure, itFunction, itProperty, itIndex, itConstant, itConstructor, itType, itEvent);
  TfrxItemTypes = set of TfrxItemType;

  TfrxCompletionListType = (cltRtti, cltScript, cltAddon);
  TfrxCompletionListTypes = set of TfrxCompletionListType;

  TfrxCompletionItem = class
  private
    FParent: TfrxCompletionItem;
    FName: String;
    FType: String;
    FParams: String;
    FItemType: TfrxItemType;
    FStartVisible: Integer;
    FEndVisible: Integer;
  public
    property Name: String read FName write FName;
    property Typ: String read FType write FType;
    property Params: String read FParams write FParams;
    property ItemType: TfrxItemType read FItemType write FItemType;
    property StartVisible: Integer read FStartVisible write FStartVisible;
    property EndVisible: Integer read FEndVisible write FEndVisible;
  end;

  TfrxCompletionList = class
  private
    FConstants: TStringList;
    FVariables: TStringList;
    FFunctions: TStringList;
    FClasses: TStringList;
    FLocked: Boolean;
    function AddBaseVar(varList: TStrings; const Name, sType: String; VisibleStart: Integer = 0; VisibleEnd: Integer = -1; const ParentFunc: String = ''): TfrxCompletionItem;
    function GetItem(Index: Integer): TfrxCompletionItem;
  public
    constructor Create;
    destructor Destroy; override;
    procedure DestroyItems;
    function Count: Integer;
    function AddConstant(const Name, sType: String; VisibleStart: Integer = 0; VisibleEnd: Integer = -1; const ParentFunc: String = ''): TfrxCompletionItem;
    function AddVariable(const Name, sType: String; VisibleStart: Integer = 0; VisibleEnd: Integer = -1; const ParentFunc: String = ''): TfrxCompletionItem;
    function AddClass(const Name, sType: String; VisibleStart: Integer = 0; VisibleEnd: Integer = -1; const ParentFunc: String = ''): TfrxCompletionItem;
    function AddFunction(const Name, sType, Params: String; VisibleStart: Integer = 0; VisibleEnd: Integer = -1; const ParentFunc: String = ''): TfrxCompletionItem;
    function Find(const Name: String): TfrxCompletionItem;
    property Items[Index: Integer]: TfrxCompletionItem read GetItem; default;
    property Locked: Boolean read FLocked write FLocked;
  end;

  { TfrxCodeCompletionThread }

  TfrxCodeCompletionThread = class(TThread)
  private
    FText: TStringList;
    FScript: TfsScript;
    FOriginalScript: TfsScript;
    FILCode: TStream;
    FXML: TfsXMLDocument;
    FCompletionList: TfrxCompletionList;
    FSyntaxType: String;
    FMemoHandle: HWND;
    procedure SyncScript;
    procedure UpdateCode;
    procedure FreeScript;
    procedure FillCodeCompletion;
  public
    destructor Destroy; override;
    procedure Execute; override;
    property Script: TfsScript read FOriginalScript write FOriginalScript;
  end;

  {$IFDEF FPC}
  { TBrkStringList }

  TBrkStringList = class(TStringList)
    function AddObject(const S: string; AObject: TObject): Integer; override; overload;
  end;
  {$ENDIF}

  TfrxSimpleSyntaxMemo = class(TfrBaseSyntaxMemo)
  protected
    //Caret var
    FMultiByteLang: Boolean;
    FCharWidth: Integer;
    FCharHeight: Integer;

    //Syntax var
    FMaxLength: Integer;
    FSynAttributes: TfrxSynAttributes;
    FSyntax: String;
    //FSynStrings: TStrings;

    {$IFNDEF FPC}
    FTmpCanvas: TBitmap;
    {$ENDIF}
    // from base
    FOnCodeCompletion: TfrxCodeCompletionEvent;
    FRttiCompletionList: TfrxCompletionList;
    FScriptCompletionList: TfrxCompletionList;
    FAddonCompletionList: TfrxCompletionList;
    FClassCompletionList: TfrxCompletionList;
    FCodeComplitionFilter: TfrxItemTypes;
    FShowInCodeComplition: TfrxCompletionListTypes;
    FBookmarks: array[0..9] of Integer;
    FToggleBreakPointDown: Boolean;
    FBreakPoints: {$IFDEF FPC}TBrkStringList{$ELSE}TStringList{$ENDIF};
    FEnableHint: Boolean;
    FTimer: TTimer;
    FParser: TfsParser;
    FScriptRTTIXML: TfsXMLDocument;
    FScript: TfsScript;
    FCodeCompletionThread: TfrxCodeCompletionThread;
    FfrxHintShowEvent: TfrxHintShowEvent;
    procedure DoChange; override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure QuickComment; override;
    procedure SetEnableHint(vEnableHint: boolean);
    procedure SetShowInCodeComplition(const Value: TfrxCompletionListTypes);
    procedure FakeHintShow(var Msg: TCMHintShow);
    procedure RealHintShow(var Msg: TCMHintShow);
    procedure DoTimer(Sender: TObject);
    procedure WMFRXSyncScript(var Message: TMessage); message WM_FRX_SYNC_SCRIPT;
    procedure WMFRXUpdateCode(var Message: TMessage); message WM_FRX_UPDATE_CODE;
    procedure WMFRXFillCodeCompletion(var Message: TMessage); message WM_FRX_FILL_CODE_COMPLETION;
    procedure WMFRXHintShow(var Msg: TCMHintShow); message CM_HINTSHOW;
    procedure BuildCClist(const sName: String; aCodeCompList: TStringList; AllFunc: Boolean = False);
    procedure GetCurWord(Point: TPoint; var Prefix: String; var Header: String; var TextHint: String);
    function IsCursorInStringBlock: Boolean;
    function GetIdentEnd(aPos: Integer): Integer; override;
    procedure CorrectBookmark(Line, Delta: Integer); override;
    procedure CorrectBreakPoints(Line, Delta: Integer);
    function ItemToPrefix(Item: TfrxCompletionItem; var pref: String; c: TColor = clNone): TColor;
    procedure FillAddonCompletionLis(const AFilter: String);
    procedure CompletionLBDblClick(Sender: TObject); override;
    procedure CompletionLBDrawItem(Control: TWinControl; Index: Integer;
      ARect: TRect; State: TOwnerDrawState); override;
    procedure FilterCodeStringList(aFilterStr: String; aLBList: TStrings;
      aCodeList: TStrings; aDiffFunc: TDiffFunc); override;
    {$IFNDEF FPC}
    {  need for east languages carret correction  }
    function GetCharWidth(const Str: String): Integer;
    function GetCharXPos(X: Integer): Integer;
    {$ENDIF}

    function GetSelText: String; override;
    procedure SetSelText(const Value: String); override;

    //Caret func
    procedure DoCodeCompletion; override;
    procedure ShowCaretPos; override;
    function CurPosToSynPos(X, Y: Integer): TPoint; override;
    procedure EventFontChanged(var Message: TMessage); override;
    procedure EventSetFocus(var Msg: TWMSetFocus); override;

    //Syntax func
    function GetCharAttr(Pos: TPoint): TfrxCharAttributes;
    procedure SetSyntax(const Value: String);
    procedure ClearSyntax(ClearFrom: Integer); override;
    procedure CreateSynArray(EndLine: Integer);
    function GetAttributeStyles: TfrAttributeStyles;

    //Caret func
    procedure ShiftSelected(ShiftRight: Boolean); override;
    procedure DoReturn; override;
    procedure DoRight; override;
    procedure DoLeft; override;
    procedure Resize; override;
    procedure OnHScrollChange(Sender: TObject); override;
    procedure OnVScrollChange(Sender: TObject); override;

    procedure UpdateSyntaxType; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    function IsBookmark(Line: Integer): Integer; override;
    procedure AddBookmark(Line, Number: Integer); override;
    procedure DeleteBookmark(Number: Integer); override;
    procedure GotoBookmark(Number: Integer); override;
    procedure AddNewBreakPoint;
    procedure AddBreakPoint(Number: Integer; const Condition: String; const Special: String);
    procedure ToggleBreakPoint(Number: Integer; const Condition: String);
    procedure DeleteBreakPoint(Number: Integer);
    procedure DeleteF4BreakPoints;
    function IsBreakPoint(Number: Integer): Boolean;
    function IsActiveBreakPoint(Number: Integer): Boolean;
    function GetBreakPointCondition(Number: Integer): String;
    function GetBreakPointSpecialCondition(Number: Integer): String;
    procedure ClearBreakPoints;
    procedure SaveToIni(const IniPath: String; const Section: String; const FileName: String); override;
    procedure LoadFromIni(const IniPath: String; const Section: String; const FileName: String); override;
    procedure FillRtti;
    procedure CompletionClose; override;

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure SetPos(x, y: Integer; ToRight: Boolean); override;
    procedure SetPosByCoords(X, Y: Integer); override;

    property MultiByteLang: Boolean read FMultiByteLang write FMultiByteLang;
    property Syntax: String read FSyntax write SetSyntax;
    property AttributeStyles: TfrAttributeStyles read GetAttributeStyles;
    property BreakPoints:{$IFDEF FPC}TBrkStringList{$ELSE}TStringList{$ENDIF} read FBreakPoints;
    property ShowInCodeComplition: TfrxCompletionListTypes read FShowInCodeComplition write SetShowInCodeComplition;
    property CodeCompletionThread: TfrxCodeCompletionThread read FCodeCompletionThread;
    property Script: TfsScript read FScript write FScript;
    property ScriptRTTIXML: TfsXMLDocument read FScriptRTTIXML;
    property EnableHint: Boolean read FEnableHint write SetEnableHint default True;
    property CodeComplitionFilter: TfrxItemTypes read FCodeComplitionFilter write FCodeComplitionFilter;
    property OnCodeCompletion: TfrxCodeCompletionEvent read FOnCodeCompletion
          write FOnCodeCompletion;
  end;

  TfrxSyntaxMemo = TfrxSimpleSyntaxMemo;

implementation
end.
