
{******************************************}
{                                          }
{             FastReport VCL               }
{             XML serializer               }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxXMLSerializer;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}
  Windows, Messages,
  {$ELSE}
  LResources, LCLType, LCLProc, LazHelper,
  {$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Variants,
  TypInfo, frXML, frUnicodeUtils, frxClass, frxVariables, frCore;


type
  TfrxGetAncestorEvent = procedure(const ComponentName: String;
    var Ancestor: TPersistent) of object;


{ TfrxXMLSerializer is the XML analogue of the Delphi component streaming -
  TReader and TWriter }

  TfrxXMLSerializer = class(TObject)
  private
    FErrors: TStringList;
    FFixups: TList;
    FOwner: TfrxComponent;
    FReader: TReader;
    FReaderStream: TMemoryStream;
    FSerializeDefaultValues: Boolean;
    FStream: TStream;
    FOldFormat: Boolean;
    FOnGetAncestor: TfrxGetAncestorEvent;
    FGetCustomDataEvent: TfrxGetCustomDataEvent;
    FOnSaveCustomDataEvent: TfrxSaveCustomDataEvent;
    FHandleNestedProperties: Boolean;
    FSaveAncestorOnly: Boolean;
    FIgnoreName: Boolean;
    procedure AddFixup(Obj: TPersistent; p: PPropInfo; Value: String);
    procedure ClearFixups;
    procedure FixupReferences;

    {$IFDEF FPC}
    procedure PropertyNotFoundEvent(Reader: TReader; Instance: TPersistent;
      var PropName: string; IsPath: boolean; var Handled, Skip: Boolean);
    procedure ReadWriteStringPropertyEvent(Sender:TObject;
      const Instance: TPersistent; PropInfo: PPropInfo;
      var Content:string);
    procedure ReaderError(Reader: TReader; const Message: string;
      var Handled: Boolean);
    {$ENDIF}

  public
    constructor Create(Stream: TStream);
    destructor Destroy; override;
    function ObjToXML(Obj: TPersistent; const Add: String = ''; Ancestor: TPersistent = nil): String;
    function ReadComponent(Root: TfrxComponent): TfrxComponent;
    function ReadComponentStr(Root: TfrxComponent; s: String; DontFixup: Boolean = False): TfrxComponent;
    function WriteComponentStr(c: TfrxComponent; Ancestor: TPersistent = nil): String;
    procedure ReadRootComponent(Root: TfrxComponent; XMLItem: TfrXMLItem = nil);
    procedure CopyFixupList(FixList: TList);
    procedure ReadPersistentStr(Root: TComponent; Obj: TPersistent; const s: String);
    procedure WriteComponent(c: TfrxComponent; Ancestor: TPersistent = nil);
    procedure WriteRootComponent(Root: TfrxComponent; SaveChildren: Boolean = True;
      XMLItem: TfrXMLItem = nil);
    procedure XMLToObj(const s: String; Obj: TPersistent);
    property Errors: TStringList read FErrors;
    property Owner: TfrxComponent read FOwner write FOwner;
    property Stream: TStream read FStream;
    property Fixups: TList read FFixups;
    property SerializeDefaultValues: Boolean read FSerializeDefaultValues
      write FSerializeDefaultValues;
    property OnGetAncestor: TfrxGetAncestorEvent read FOnGetAncestor write FOnGetAncestor;
    property OnGetCustomDataEvent: TfrxGetCustomDataEvent read FGetCustomDataEvent write FGetCustomDataEvent;
    property OnSaveCustomDataEvent: TfrxSaveCustomDataEvent read FOnSaveCustomDataEvent write FOnSaveCustomDataEvent;
    property OldFormat: Boolean read FOldFormat write FOldFormat;
    property HandleNestedProperties: Boolean read FHandleNestedProperties write FHandleNestedProperties;
    property SaveAncestorOnly: Boolean read FSaveAncestorOnly write FSaveAncestorOnly;
    { ignore components name during loading }
    property IgnoreName: Boolean read FIgnoreName write FIgnoreName;
  end;

  TfrxFixupItem = class(TObject)
  public
    Obj: TPersistent;
    PropInfo: PPropInfo;
    Value: String;
  end;


implementation
end.

