
{******************************************}
{                                          }
{             FastReport VCL               }
{              ZipCode Object              }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxZipCode;

{$I frx.inc}

interface

uses
  Classes, Graphics, Types,
  frxClass
{$IFDEF DELPHI16}
 , Vcl.Controls
{$ENDIF};

type
  TZipCodeKind = (zcCustom, zcInternational, zcRF, zcSample);
  {$IFDEF DELPHI16}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
  {$ENDIF}
  TfrxZipCodeObject = class(TComponent);  // fake component

  TfrxZipCodeView = class(TfrxStyledView)
  private
    FmmDigitWidth: Extended;
    FmmDigitHeight: Extended;
    FmmSpacing: Extended;
    FmmLineWidth: Extended;
    FShowMarkers: Boolean;
    FShowGrid: Boolean;
    FExpression: String;
    FText: String;
    FDigitCount: Integer;
    FKind: TZipCodeKind;
    FColor: TColor;
    procedure SetKind(const Value: TZipCodeKind);
    procedure SetmmDigitWidth(const Value: Extended);
    procedure SetmmDigitHeight(const Value: Extended);
    procedure SetmmLineWidth(const Value: Extended);
    procedure SetmmSpacing(const Value: Extended);
    procedure SetDigitCount(const Value: Integer);
    procedure SetShowMarkers(const Value: Boolean);
    procedure SetShowGrid(const Value: Boolean);
    procedure SetText(const Value: String);
  protected
    FMarker: Integer; // if ShowMarkers then FMarker := 1 else FMarker := 0;
    F1mm: TfrxPoint;
    FSpacing: Extended;
    FMarkerHeight: Extended;
    FMarkerWidth: Extended;
    FGridPoint: TfrxPoint;
    FGridStep: TfrxPoint;
    FDigitLeftTop: TfrxPoint;
    FDigitWidth: Extended;
    FDigitHeight: Extended;
    FDigitPenThickness: Extended;
    procedure DrawSegment(Canvas: TCanvas; Number: Integer);
    procedure PaintPolyline(Canvas: TCanvas; const Ps: array of TPoint);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    procedure GetData; override;
  published
    property FillType;
    property Fill;
    property Cursor;

    property DataField;
    property DataSet;
    property DataSetName;
    property Expression: String read FExpression write FExpression;
    property Style;
    property Text: String read FText write SetText;

    property Kind: TZipCodeKind read FKind write SetKind;
    property mmDigitWidth: Extended read FmmDigitWidth write SetmmDigitWidth;
    property mmDigitHeight: Extended read FmmDigitHeight write SetmmDigitHeight;
    property mmLineWidth: Extended read FmmLineWidth write SetmmLineWidth;
    property mmSpacing: Extended read FmmSpacing write SetmmSpacing;
    property DigitCount: Integer read FDigitCount write SetDigitCount;
    property ShowMarkers: Boolean read FShowMarkers write SetShowMarkers;
    property ShowGrid: Boolean read FShowGrid write SetShowGrid;
    property Color: TColor read FColor write FColor;
  end;

implementation
end.
