unit frxDelphiZXingQRCode;


// ZXing QRCode port to Delphi, by Debenu Pty Ltd
// www.debenu.com

// Original copyright notice
(*
 * Copyright 2008 ZXing authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *)
{$I frx.inc}

interface

uses frxBarcode2DBase;


type
  TQRCodeEncoding = (qrAuto, qrNumeric, qrAlphanumeric, qrISO88591, qrUTF8NoBOM, qrUTF8BOM, qrShift_JIS);
  TQRErrorLevels = (ecL, ecM, ecQ, ecH);

  TErrorCorrectionLevel = class
  private
    FBits: Integer;
    FOrdinal: Integer;
  public
    procedure Assign(Source: TErrorCorrectionLevel);
    property Bits: Integer read FBits write FBits;
    property Ordinal: Integer read FOrdinal write FOrdinal;
  end;

  TDelphiZXingQRCode = class
  private
    procedure SetErrorLevels(const Value: TQRErrorLevels);
    procedure SetCodePage(const Value: Longint);
  protected
    FData: WideString;
    FRows: Integer;
    FColumns: Integer;
    FEncoding: TQRCodeEncoding;
    FQuietZone: Integer;
    FErrorLevels: TQRErrorLevels;
    FLevel: TErrorCorrectionLevel;
    FCodePage: Longint;
    procedure SetEncoding(NewEncoding: TQRCodeEncoding);
    procedure SetData(const NewData: WideString);
    procedure SetQuietZone(NewQuietZone: Integer);
    function GetIsBlack(Row, Column: Integer): Boolean;
    procedure Update;
  public
    FElements: T2DBooleanArray;
    constructor Create;
    destructor Destroy; override;
    property Data: WideString read FData write SetData;
    property Encoding: TQRCodeEncoding read FEncoding write SetEncoding;
    property QuietZone: Integer read FQuietZone write SetQuietZone;
    property Rows: Integer read FRows;
    property Columns: Integer read FColumns;
    property IsBlack[Row, Column: Integer]: Boolean read GetIsBlack;
    property ErrorLevels: TQRErrorLevels read FErrorLevels write SetErrorLevels;
    property CodePage: Longint read FCodePage write SetCodePage;
  end;

implementation
end.
