unit frxChartLaz;

{$I frx.inc}

interface

uses
  SysUtils, Classes, Graphics, Variants,
  frCoreClasses,
  frxClass,
  TAGraph, TACustomSource, TAIntervalSources, TACustomSeries, TASeries,
  TAChartTeeChart, TARadialSeries, TAMultiSeries,
  TASources, TATransformations, TADrawerCanvas, frPrinter;

const
   CntSeries = 11;
   cPointSeries = 'Point Series';
   cHorizBarSeries = 'HorizBar Series';

type

   TChartClass = class of TChart;
   TNSeries = array[1..CntSeries] of string;
   TfrxChartSeries = array[1..CntSeries] of TSeriesClass;
   TfrxSeriesDataType = (dtDBData, dtBandData, dtFixedData);

const
   NSeries:TNSeries = (
    'Area', 'Bar', 'HorizBar', 'Line', 'Point', 'Pie', 'Polar',
    'Bubble','OHLC','Box','ConstLine');
   CSeries:TfrxChartSeries = (
      TAreaSeries, TBarSeries, THorizBarSeries, TLineSeries,
      TPointSeries, TPieSeries, TPolarSeries, TBubbleSeries,
      TOpenHighLowCloseSeries, TBoxAndWhiskerSeries,
      TConstantLine
   );
   CTransform: array[1..4] of TAxisTransformClass = (
    TLinearAxisTransform, TAutoScaleAxisTransform,
    TLogarithmAxisTransform, TCumulNormDistrAxisTransform
   );
   NTransform: array[1..4] of string = (
     'Linear', 'AutoScale', 'Logarithm', 'CumulNormDistr'
   );

type


  TfrxChartObject = class(TComponent);  // fake component

  { TStyItem }

  TStyItem = class(TfrCollectionItem)
  private
    FBrush: TBrush;
    FPen: TPen;
    FFont: TFont;
    FRepeatCount: Cardinal;
    FText: String;
    FUseBrush: Boolean;
    FUsePen: Boolean;
    FUseFont: Boolean;
    FNmSer:string;
  protected
    function IsNameStored: Boolean;
  public
    constructor Create(Collection:TCollection);override;
    destructor Destroy;override;
  published
    property Brush: TBrush read FBrush write FBrush;
    property Font: TFont read FFont write FFont;
    property Pen: TPen read FPen write FPen;
    property RepeatCount: Cardinal
      read FRepeatCount write FRepeatCount default 1;
    property Text: String read FText write FText;
    property UseBrush: Boolean read FUseBrush write FUseBrush default true;
    property UseFont: Boolean read FUseFont write FUseFont default true;
    property UsePen: Boolean read FUsePen write FUsePen default true;
    property NmSer:string read FNmSer write FNmSer;
  end;

  { TStyData }

  TStyData = class(TfrCollection)
  private
    FReport:TfrxReport;
    function GetItems(Index: Integer): TStyItem;
  public
    constructor Create(Report:TfrxReport);
    function Add:TStyItem;
    property Items[Index: Integer]:TStyItem  read GetItems; default;
  end;


  { TLSItem }

  TLSItem = class(TfrCollectionItem)
  private
    FDataBand: TfrxDataBand;
    FDataSet: TfrxDataSet;
    FDataType: TfrxSeriesDataType;
    FDataSetName: string;
    FYCount:Cardinal;
    FDP:TStrings;
    FDDP:TStrings;
    FSorted:Boolean;
    //------------------
    FParams: TChartAxisIntervalParams;
    FDateTimeFormat: String;
    FSteps: TDateTimeSteps;
    FSuppressPrevUnit: Boolean;
    FNameAS: string;
    FTypeAS: Integer;
    procedure SetDataSet(const Value: TfrxDataSet);
    procedure SetDataSetName(const Value: String);
    function GetDataSetName: String;
  protected
    function IsNameStored: Boolean;
  public
    constructor Create(Collection:TCollection);override;
    destructor Destroy;override;
  published
    property DataType:TfrxSeriesDataType read FDataType write FDataType;
    property DataBand: TfrxDataBand read FDataBand write FDataBand;
    property DataSet: TfrxDataSet read FDataSet write SetDataSet;
    property DataSetName: string read GetDataSetName write SetDataSetName;
    property YCount:Cardinal read FYCount write FYCount;
    property DP:TStrings read FDP write FDP;
    property DDP:TStrings read FDDP write FDDP;
    property Sorted:Boolean read FSorted write FSorted;
    //----------------------------
    property Params: TChartAxisIntervalParams read FParams write FParams;
    property DateTimeFormat: String read FDateTimeFormat write FDateTimeFormat;
    property Steps: TDateTimeSteps
      read FSteps write FSteps default DATE_TIME_STEPS_ALL;
    property SuppressPrevUnit: Boolean
      read FSuppressPrevUnit write FSuppressPrevUnit default true;
    property NameAS: string read FNameAS write FNameAS;
    property TypeAS: Integer read FTypeAS write FTypeAS;
  end;

  { TLSData }

  TLSData = class(TfrCollection)
  private
    FReport:TfrxReport;
    function GetItems(Index: Integer): TLSItem;
  public
    constructor Create(Report:TfrxReport);
    function Add:TLSItem;
    property Items[Index: Integer]:TLSItem  read GetItems; default;
  end;

  { TTransItem }

  TTransItem = class(TfrCollectionItem)
  private
    FEnabled: Boolean;
    FOffest: Double;
    {TLinearAxisTransform}
    FOffset: Double;
    FScale: Double;
    {TAutoScaleAxisTransform}
    FMaxValue: Double;
    FMinValue: Double;
    {TLogarithmAxisTransform}
    FBase: Double;
    FNameTr:string;
  protected
    function IsNameStored: Boolean;
  public
    constructor Create(Collection:TCollection);override;
    destructor Destroy;override;
  published
    property Enabled:Boolean read FEnabled write FEnabled;
    {TLinearAxisTransform}
    property Offset:Double read FOffset write FOffest;
    property Scale:Double read FScale write FScale;
    {TAutoScaleAxisTransform}
    property MaxValue:Double read FMaxValue write FMaxValue;
    property MinValue:Double read FMinValue write FMinValue;
    {TLogarithmAxisTransform}
    property Base:Double read FBase write FBase;
    property NameTr:string read FNameTr write FNameTr;
  end;

  { TTransData }

  TTransData = class(TfrCollection)
  private
    FReport:TfrxReport;
    function GetItems(Index: Integer): TTransItem;
  public
    constructor Create(Report:TfrxReport);
    function Add:TTransItem;
    property Items[Index: Integer]:TTransItem  read GetItems; default;
  end;

  { TfrxChartView }

  TfrxChartView = class(TfrxView)
  private
    FChart:TChart;
    FLSData:TLSData;
    FStyData:TStyData;
    FTransData: TTransData;
    FChWidth:Integer;
    FChHeight:Integer;
    FIsDraw:Boolean;
    procedure ClearTrans;
    procedure ClearSeriesSource;
    procedure ClearSource;
    procedure ClearStripes;
    procedure ClearAll;
    procedure ReadData(Stream:TStream);
    procedure ReadData2(Reader: TReader);
    procedure ReadData3(Reader: TReader);
    procedure ReadData4(Reader: TReader);
    procedure WriteData(Stream: TStream);
    procedure WriteData2(Writer: TWriter);
    procedure WriteData3(Writer: TWriter);
    procedure WriteData4(Writer: TWriter);
    procedure FindComponentClass(
      AReader: TReader; const AClassName: String; var AClass: TComponentClass);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure ClearMarksSource; virtual;
    procedure CreateChart; virtual;
    function GetNeedSelectSeries: Boolean; virtual;
    class function GetChartClass: TChartClass; virtual;
    procedure FillChart; virtual;
  public
    constructor Create(AComponent:TComponent);override;
    destructor Destroy;override;
    class function GetDescription: String; override;
    procedure BeforeStartReport; override;
    procedure GetData; override;
    procedure Draw(Canvas: TCanvas;
      ScaleX, ScaleY, OffsetX, OffsetY: Extended);override;
    property Chart:TChart read FChart write FChart;
    property LSData:TLSData read FLSData;
    property StyData:TStyData read FStyData;
    property TransData:TTransData read FTransData;
 published
    property ChWidth:Integer read FChWidth write FChWidth;
    property ChHeight:Integer read FChHeight write FChHeight;
    property IsDraw:Boolean read FIsDraw write FIsDraw;
    property FillType;
    property Fill;
    property Cursor;
    property Frame;
    property TagStr;
    property URL;
  end;

 {$IFDEF Windows}
  TfrxPrinterDrawer = class(TScaledCanvasDrawer)
  public
    constructor Create(APrinter: TfrxCustomPrinter);
  end;
 {$ENDIF}

implementation
end.
