unit frxRichLaz;

{$I frx.inc}

interface

uses
  LCLIntf, SysUtils, Classes, Graphics, Variants,
  frxClass, RichMemo, frxRes, LazHelper, frCore, frTypes;

type

  TfrxRichObject = class(TComponent);  // fake component
  TRxRichMemo = class;

  TRxParaAttributes = class(TPersistent)
  private
    RichMemo: TRxRichMemo;
    ParaMetric: TParaMetric;
    ParaNumbering: TParaNumbering;
    function GetAlignment: TParaAlignment;
    function GetFirstIndent: Longint;
    function GetLeftIndent: Longint;
    function GetRightIndent: Longint;
    function GetNumbering: TParaNumStyle;

    procedure SetAlignment(Value: TParaAlignment);
    procedure SetFirstIndent(Value: Longint);
    procedure SetLeftIndent(Value: Longint);
    procedure SetRightIndent(Value: Longint);
    procedure SetNumbering(Value: TParaNumStyle);
  protected
    //procedure AssignTo(Dest: TPersistent); override;
  public
    constructor Create(AOwner: TRxRichMemo);
    //procedure Assign(Source: TPersistent); override;

    property Alignment: TParaAlignment read GetAlignment write SetAlignment;
    property FirstIndent: Longint read GetFirstIndent write SetFirstIndent;
    property LeftIndent: Longint read GetLeftIndent write SetLeftIndent;
    property RightIndent: Longint read GetRightIndent write SetRightIndent;
    property Numbering: TParaNumStyle read GetNumbering write SetNumbering;
  end;

  TRxRichMemo = class(TRichMemo)
  private
    FParagraph: TRxParaAttributes;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function FindText(const SearchStr: string; sStartPos, sLength: Integer): Integer;
    procedure SetSelection(StartPos, EndPos: Longint; ScrollCaret: Boolean = False);
    function GetSelection: TCharRange;
    function LoadRichText(path: String): Boolean; overload;
    function LoadRichFromString(AString: String): Boolean;
    procedure SaveSelection(st: TStream);

    property Paragraph: TRxParaAttributes read FParagraph;
  end;

  { TfrxRichView }

  TfrxRichView = class(TfrxStretcheable)
  private
    FAllowExpressions: Boolean;
    FExpressionDelimiters: String;
    FFlowTo: TfrxRichView;
    FGapX: Extended;
    FGapY: Extended;
    FParaBreak: Boolean;
    FRichMemo: TRxRichMemo;
    FTempStream: TMemoryStream;
    FTempStream1: TMemoryStream;
    FWysiwyg: Boolean;
    FHasNextDataPart: Boolean;
    FStopSplit: Boolean;
    FLastChar: Integer;
    FFileLink: String;

    procedure RichDraw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended);
    function IsExprDelimitersStored: Boolean;
    procedure ReadData(Stream: TStream);
    procedure WriteData(Stream: TStream);
    //function UsePrinterCanvas: Boolean;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AComponent:TComponent); override;
    destructor Destroy; override;
    class function GetDescription: String; override;
    procedure Draw(Canvas: TCanvas; ScaleX, ScaleY, OffsetX, OffsetY: Extended); override;
    procedure AfterPrint; override;
    procedure BeforePrint; override;
    procedure GetData; override;
    procedure InitPart; override;
    function CalcHeight: Extended; override;
    function DrawPart: Extended; override;
    function GetComponentText: String; override;
    function HasNextDataPart(aFreeSpace: Extended): Boolean; override;
    property RichMemo: TRxRichMemo read FRichMemo write FRichMemo;
  published
    property AllowExpressions: Boolean read FAllowExpressions write FAllowExpressions default True;
    property ExpressionDelimiters: String read FExpressionDelimiters write FExpressionDelimiters stored IsExprDelimitersStored;
    property FlowTo: TfrxRichView read FFlowTo write FFlowTo;
    property FileLink: String read FFileLink write FFileLink;
    property GapX: Extended read FGapX write FGapX;
    property GapY: Extended read FGapY write FGapY;
    property Wysiwyg: Boolean read FWysiwyg write FWysiwyg default True;
    property FillType;
    property Fill;
    property Frame;
    property BrushStyle;
    property Color;
    property Cursor;
    property DataField;
    property DataSet;
    property DataSetName;
    property TagStr;
    property URL;
  end;

procedure frxAssignRich(RichFrom, RichTo: TRichMemo);

implementation
end.
