{ ****************************************** }
{ }
{ FastReport VCL/FMX/LCL }
{ Graphics Library }
{ }
{ Copyright (c) 1998-2022 }
{ by Fast Reports Inc. }
{ }
{ ****************************************** }

unit frLazGraphicClasses;

{$I frVer.inc}

interface

uses
  SysUtils, Classes, Controls, LCLType, LMessages,
  Graphics, ExtCtrls, LazUTF8, LCLIntf, Types,
{$IFNDEF NONWINFPC}
  Windows, Messages, Registry,
{$ELSE}
  Masks, fpmimetypes,
{$ENDIF}
{$IFDEF LCLGTK2}
  gdk2pixbuf, glib2, gdk2, gtk2, Pango, Gtk2FontCache,
  Gtk2Globals, Gtk2Def, Gtk2Proc, CairoCanvas,
{$ENDIF}
  FPCanvas, FPImage, syncobjs;

type
  TlmfList = class;

  TlmfImage = class(TGraphic)
  private
    forgX, forgY, fWidth, fHeight: integer;
    kx, ky: double;
    fList: TlmfList;
    fCrs: TCriticalSection;
  protected
    procedure AssignTo(Dest: TPersistent); override;
    function GetWidth: integer; override;
    procedure SetWidth(AVal: integer); override;
    function GetHeight: integer; override;
    procedure SetHeight(AVal: integer); override;
    function GetEmpty: boolean; override;
    function GetTransparent: boolean; override;
    procedure SetTransparent(Value: boolean); override;
    // procedure Erase;override;

    function ScaleX(ax: integer): integer;
    function ScaleY(ay: integer): integer;
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure Clear; override;
    procedure SaveToStream(Stream: TStream); override;
    procedure LoadFromStream(Stream: TStream); override;
    procedure Draw(ACanvas: TCanvas; const Rect: TRect); override;
    property List: TlmfList read fList;
  end;

  TlmfList = class(TComponent)
  private
    fWidth, fHeight: integer;
  public
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    function GetChildOwner: TComponent; override;
  published
    property Width: integer read fWidth write fWidth;
    property Height: integer read fHeight write fHeight;
  end;

  TlmfObject = class(TComponent)
  public
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); virtual; abstract;
  end;

  TlmfAnchor = class(TlmfObject)
  private
    fPos: TPoint;
  public
    constructor Create(ax, ay: integer); virtual;
  published
    property px: integer read fPos.x write fPos.x;
    property py: integer read fPos.y write fPos.y;
  end;

  TlmfMoveTo = class(TlmfAnchor)
  public
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  end;

  TlmfLineTo = class(TlmfAnchor)
  public
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  end;

  TlmfLine = class(TlmfAnchor)
  private
    fEndPos: TPoint;
  public
    constructor Create(x1, y1, x2, y2: integer); overload;
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  published
    property px1: integer read fEndPos.x write fEndPos.x;
    property py1: integer read fEndPos.y write fEndPos.y;
  end;

  { TlmfText }

  TlmfText = class(TlmfAnchor)
  private
    FStrBounds: TRect;
    fText: string;
    // fHeight:integer;
    // fStyle:TTextStyle;
  protected
    procedure DefineProperties(Filer: TFiler); override;
  public
    constructor Create(x, y: integer; const AText: string); overload;
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
    property StrBounds: TRect Read FStrBounds Write FStrBounds;
  published
    property Text: string read fText write fText;
    // property Height:integer read fHeight write fHeight;
    // property Style:TTextStyle read fTextStyle write fTextStyle;
  end;

  TlmfColor = class(TlmfAnchor)
  private
    fColor: TfpColor;
  public
    constructor Create(x, y: integer; AColor: TfpColor); overload;
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  published
    property r: word read fColor.red write fColor.red;
    property g: word read fColor.green write fColor.green;
    property b: word read fColor.blue write fColor.blue;
    property a: word read fColor.alpha write fColor.alpha;
  end;

  TlmfClip = class(TlmfObject)
  private
    fClip: TRect;
  public
    constructor Create(AClip: TRect); virtual; overload;
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  published
    property Left: integer read fClip.Left write fClip.Left;
    property Top: integer read fClip.Top write fClip.Top;
    property Right: integer read fClip.Right write fClip.Right;
    property Bottom: integer read fClip.Bottom write fClip.Bottom;
  end;

  TlmfRect = class(TlmfClip)
  public
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  end;

  TlmfFillRect = class(TlmfRect)
  public
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  end;

  TlmfEllipse = class(TlmfClip)
  public
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  end;

  TlmfFont = class(TlmfObject)
  private
    fFont: TFont;
    fHeight, fRotation: integer;
    fName: string;
  public
    constructor Create(AnOwner: TComponent); override;
    destructor Destroy; override;
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  published
    property Font: TFont read fFont write fFont;
    property Height: integer read fHeight write fHeight;
    property Rotation: integer read fRotation write fRotation;
  end;

  TlmfBrush = class(TlmfObject)
  private
    fBrush: TBrush;
  public
    constructor Create(AnOwner: TComponent); override;
    destructor Destroy; override;
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  published
    property Brush: TBrush read fBrush write fBrush;
  end;

  TlmfPen = class(TlmfObject)
  private
    fPen: TPen;
  public
    constructor Create(AnOwner: TComponent); override;
    destructor Destroy; override;
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  published
    property Pen: TPen read fPen write fPen;
  end;

  TlmfGraph = class(TlmfClip)
  private
    fGraph: TPicture;
  public
    constructor Create(AnOwner: TComponent); override;
    destructor Destroy; override;
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  published
    property Graph: TPicture read fGraph write fGraph;
  end;

  TlmfPolyline = class(TlmfRect)
  private
    pts: array of TPoint;
  protected
    procedure StorePoints(AStream: TStream); virtual;
    procedure LoadPoints(AStream: TStream); virtual;
    procedure DefineProperties(Afiler: TFiler); override;
  public
    constructor Create(Points: PPoint; NumPts: integer); overload;
    destructor Destroy; override;
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  end;

  TlmfPolygon = class(TlmfPolyline)
  private
    fWinding: boolean;
  public
    constructor Create(Points: PPoint; NumPts: integer;
      Winding: boolean = false); overload;
    procedure Action(fImage: TlmfImage; ACanvas: TCanvas); override;
  published
    property Winding: boolean read fWinding write fWinding;
  end;

  TlmfCanvas = class(TCanvas)
  private
    fClipRect: TRect;
    fState: TCanvasState;
    fImage: TlmfImage;
  protected
    procedure CreateFont; override;
    procedure CreateBrush; override;
    procedure CreatePen; override;
    function DoCreateDefaultFont: TFPCustomFont; override;
    function DoCreateDefaultPen: TFPCustomPen; override;
    function DoCreateDefaultBrush: TFPCustomBrush; override;
    procedure DoGetTextSize(Text: string; var w, h: integer); override;
    function DoAllowBrush(ABrush: TFPCustomBrush): boolean; override;
    procedure DoMoveTo(x, y: integer); override;
    procedure DoLineTo(x, y: integer); override;
    procedure DoLine(x1, y1, x2, y2: integer); override;
    // procedure DoEllipseFill (const Bounds:TRect); override;
    procedure DoEllipse(const Bounds: TRect); override;
    procedure DoRectangleFill(Const Bounds: TRect); override;

    procedure SetPixel(x, y: integer; Value: TColor); override;
    procedure SetColor(x, y: integer; const Value: TfpColor); override;
    function GetColor(x, y: integer): TfpColor; override;
    procedure SetClipRect(const AValue: TRect); override;
    function GetClipRect: TRect; override;
    procedure RequiredState(ReqState: TCanvasState); override;
  public
    FCreateOnlyText: boolean;
    procedure CopyRect(const Dest: TRect; SrcCanvas: TCanvas;
      const Source: TRect); override;
    constructor Create(Almf: TlmfImage);
    procedure TextOut(x, y: integer; const Text: string); override;
    // already in fpcanvas
    function TextExtent(const Text: string): TSize; override;
    procedure TextRect(ARect: TRect; x, y: integer; const Text: string;
      const Style: TTextStyle); override;
    procedure StretchDraw(const DestRect: TRect; SrcGraphic: TGraphic);
      override;
    procedure Rectangle(x1, y1, x2, y2: integer); override;
    // already in fpcanvas
    procedure Polyline(Points: PPoint; NumPts: integer); override;
    procedure Polygon(Points: PPoint; NumPts: integer;
      Winding: boolean = false); override;
    procedure Ellipse(x1, y1, x2, y2: integer); override;
  end;

  { TMetafile }

  TMetafile = class(TlmfImage)
  private
    FEnhanced: boolean;
  published
    property Enhanced: boolean read FEnhanced write FEnhanced;
  end;

implementation
end.
