unit frAppearance;

interface
{$I frVer.inc}
//{$DEFINE USE_THEMES}

uses
{$IFDEF FPC}
  LResources, LCLType, LCLIntf,
{$ELSE}
  Windows, UxTheme,
{$ENDIF}
{$IFDEF DELPHI6}
  GraphUtil,
{$ENDIF}
{$IFDEF DELPHI18}
  System.UITypes,
{$ENDIF}
  Themes, Types, Classes, SysUtils, Controls, Forms, StdCtrls, Graphics, ImgList,
  frDPIAwareUtils, frUtils, frTypes, frResources, frXML, frCoreClasses, frGraphics;

type

  TfrPaintStyle = (psDefault, psStandard, psFlat, psNative);

  TfrPartState = (tsNormal, tsHot, tsPressed, tsDisabled);

  TfrArrowDirection = (adLeft, adRight, adUp, adDown);

  TfrButtonState = (bsDefault, bsNormal, bsHot, bsDisabled, bsPressed);
  TfrButtonStates = set of TfrButtonState;

  TfrGradientDirection = (
    gdNone,
    gdHorizontal,
    gdVertical,
    gdHorizontalCenter,
    gdVerticalCenter,
    gdDiagonal1,
    gdDiagonal2,
    gdDiagonal1Center,
    gdDiagonal2Center,
    gdCorner1,
    gdCorner2,
    gdCorner3,
    gdCorner4,
    gdCenter
  );

  { TfrScaleHelper }

  TfrScaleHelper = class
  strict private
    FDenominator: Integer;
    FNumerator: Integer;

    procedure ApplySourceScale(var ASourceNumerator, ASourceDenominator: Integer);
  public
    constructor Create; virtual;

    procedure Assign(ANumerator, ADenominator: Integer); overload;
    procedure Assign(ASource: TfrScaleHelper); overload;
    procedure Change(ANumerator, ADenominator: Integer);

    function Apply(const AValue: Integer): Integer; overload;
    function Apply(const AValue: TPoint): TPoint; overload;
    function Apply(const AValue: TRect): TRect; overload;
    function Apply(const AValue: TSize): TSize; overload;

    function Apply(const AValue: Integer; ASource: TfrScaleHelper): Integer; overload;
    function Apply(const AValue: TPoint; ASource: TfrScaleHelper): TPoint; overload;
    function Apply(const AValue: TRect; ASource: TfrScaleHelper): TRect; overload;
    function Apply(const AValue: TSize; ASource: TfrScaleHelper): TSize; overload;

    function Revert(const AValue: Integer): Integer; overload;
    function Revert(const AValue: TPoint): TPoint; overload;
    function Revert(const AValue: TRect): TRect; overload;
    function Revert(const AValue: TSize): TSize; overload;

    property Denominator: Integer read FDenominator;
    property Numerator: Integer read FNumerator;
  end;

  { TfrCustomCanvas }

  TfrCustomCanvas = class(TfrInterfacedObject)
  protected
    function GetFont: TFont; virtual; abstract;
    procedure SetFont(AValue: TFont); virtual; abstract;
  public
  // region
    procedure RestoreClipRegion; virtual; abstract;
    procedure SaveClipRegion; virtual; abstract;
    procedure SetClipRect(const ARect: TRect; AIntersect: Boolean = True); virtual; abstract;

    procedure FillGradient(const ARect: TRect; AStartColor, AEndColor: Cardinal; ADirection: TfrGradientDirection); virtual; abstract;
    procedure FillRect(const R: TRect; AColor: TColor); overload; virtual; abstract;
    procedure FillRect(const R: TRect; ABitmap: TBitmap); overload; virtual; abstract;
    procedure DrawFocusRect(const R: TRect); virtual; abstract;
    procedure DrawImageList(AImageList: TCustomImageList; AImageIndex: Integer; const ARect: TRect; AIsDisabled: Boolean = False); virtual; abstract;
    procedure DrawLine(const P1, P2: TPoint; AColor: TColor; APenStyle: TPenStyle); virtual; abstract;
    procedure FrameRect(const R: TRect; AColor: TColor; ALineWidth: Integer = 1); virtual; abstract;
    procedure Polygon(const APoints: array of TPoint; AColor: TColor); virtual; abstract;
    procedure Rectangle(X1, Y1, X2, Y2: Integer); overload; virtual; abstract;
    procedure Rectangle(X1, Y1, X2, Y2: Integer; AColor: TColor); overload; virtual; abstract;
    function TextHeight(const AText: string): Integer; virtual; abstract;
    function TextWidth(const AText: string): Integer; virtual; abstract;
    procedure DrawText(const R: TRect; const AText: string); overload; virtual; abstract;
    procedure DrawText(const R: TRect; const AText: string; AColor: TColor); overload; virtual; abstract;
    procedure DrawText(const R: TRect; const AText: string; AColor, ABgColor: TColor); overload; virtual; abstract;
    procedure DrawThemeElement(ADetails: TThemedElementDetails; const ARect: TRect; AScaleHelper: TfrScaleHelper); virtual; abstract;

    property Font: TFont read GetFont write SetFont;
  end;

  { TfrHRGNList }

  TfrHRGNList = class(TfrList)
  protected
    function GetItem(AIndex: Integer): HRGN;
    procedure Notify(const AItem: Pointer; AAction: TListNotification); override;
    procedure SetItem(AIndex: Integer; AItem: HRGN);
  public
    property Items[Index: Integer]: HRGN read GetItem write SetItem; default;
  end;

  { TfrCanvas }

  TfrCanvas = class(TfrCustomCanvas)
  strict private
    FCanvas: TCanvas;
    FList: TfrHRGNList;

    function GetDCOrigin: TPoint;
  protected
    function GetFont: TFont; override;
    procedure SetCanvas(AValue: TCanvas);
    procedure SetClipRegion(const ARegion: HRGN; AIntersect: Boolean);
    procedure SetFont(AValue: TFont); override;
  public
    constructor Create(ACanvas: TCanvas); virtual;
    destructor Destroy; override;

  // region
    function GetClipRegion: HRGN;
    procedure RestoreClipRegion; override;
    procedure SaveClipRegion; override;
    procedure SetClipRect(const ARect: TRect; AIntersect: Boolean = True); override;

    procedure FillGradient(const ARect: TRect; AStartColor, AEndColor: Cardinal; ADirection: TfrGradientDirection); override;
    procedure FillRect(const R: TRect; AColor: TColor); override;
    procedure FillRect(const R: TRect; ABitmap: TBitmap); override;
    procedure DrawFocusRect(const R: TRect); override;
    procedure DrawImageList(AImageList: TCustomImageList; AImageIndex: Integer; const ARect: TRect; AIsDisabled: Boolean = False); override;
    procedure DrawLine(const P1, P2: TPoint; AColor: TColor; APenStyle: TPenStyle); override;
    procedure FrameRect(const R: TRect; AColor: TColor; ALineWidth: Integer = 1); override;
    procedure Polygon(const APoints: array of TPoint; AColor: TColor); override;
    procedure Rectangle(X1, Y1, X2, Y2: Integer); override;
    procedure Rectangle(X1, Y1, X2, Y2: Integer; AColor: TColor); override;
    function TextHeight(const AText: string): Integer; override;
    function TextWidth(const AText: string): Integer; override;
    procedure DrawText(const R: TRect; const AText: string); override;
    procedure DrawText(const R: TRect; const AText: string; AColor: TColor); override;
    procedure DrawText(const R: TRect; const AText: string; AColor, ABgColor: TColor); override;
    procedure DrawThemeElement(ADetails: TThemedElementDetails; const ARect: TRect; AScaleHelper: TfrScaleHelper); override;

    property Canvas: TCanvas read FCanvas write SetCanvas;
    property DCOrigin: TPoint read GetDCOrigin;
  end;

  { TfrCustomPainter }

  TfrCustomPainter = class
  protected
    FUtilImages: TImageList;
    function GetUtilImages: TImageList;
    class function GetPostfix: String; virtual;
    function LoadBitmapRes(const ResName: string): TBitmap;

    property UtilImages: TImageList read GetUtilImages;
  public
    constructor Create; virtual;
    destructor Destroy; override;

    class function GetPaintStyle: TfrPaintStyle; virtual;

    procedure DrawArrow(ACanvas: TfrCustomCanvas; const ARect: TRect; ADir: TfrArrowDirection;
      ALineWidth: Integer; AColor: TColor); virtual;
    procedure DrawBackground(ACanvas: TfrCustomCanvas; const ARect: TRect; AColor: TColor); virtual;
    procedure DrawBorder(ACanvas: TfrCustomCanvas; const ARect: TRect; AColor: TColor); virtual;
    procedure DrawButton(ACanvas: TfrCustomCanvas; const ARect: TRect; AState: TfrButtonState; AColor: TColor = clDefault); virtual;
    procedure DrawFocusRect(ACanvas: TfrCustomCanvas; const ARect: TRect); virtual;
    procedure DrawFrame(ACanvas: TfrCustomCanvas; const ARect: TRect; AColor: TColor; AThickness: Integer = 1); virtual;
    procedure DrawImageList(ACanvas: TfrCustomCanvas; AImageList: TCustomImageList; AImageIndex: Integer; const ARect: TRect; AIsDisabled: Boolean = False); virtual;
    procedure DrawMinus(ACanvas: TfrCustomCanvas; const ARect: TRect; AColor: TColor); virtual;
    procedure DrawPlus(ACanvas: TfrCustomCanvas; const ARect: TRect; AColor: TColor); virtual;

    procedure DrawText(ACanvas: TfrCustomCanvas; const ARect: TRect; const AText: string; AFont: TFont); overload; virtual;
    procedure DrawText(ACanvas: TfrCustomCanvas; const ARect: TRect; const AText: string; AColor: TColor; AFont: TFont); overload; virtual;
    procedure DrawText(ACanvas: TfrCustomCanvas; const ARect: TRect; const AText: string; AColor, ABgColor: TColor; AFont: TFont); overload; virtual;

    procedure FillGradient(ACanvas: TfrCustomCanvas; const ARect: TRect;
      AStartColor, AEndColor: Cardinal; ADirection: TfrGradientDirection);

    function GetButtonColor(AState: TfrButtonState): TColor; virtual;
    function GetDefaultBackgroundColor: TColor; virtual;
    function GetDefaultBorderColor: TColor; virtual;
    function GetHighliteTextColor: TColor; virtual;

    //TreeView
    procedure DrawTreeLine(ACanvas: TfrCustomCanvas; const ARect: TRect); virtual;
    procedure DrawTreeViewExpandButton(ACanvas: TfrCustomCanvas; const ARect: TRect; AExpanded, AHotTrack: Boolean; AScaleHelper: TfrScaleHelper); virtual;
    procedure DrawTreeViewSelect(ACanvas: TfrCustomCanvas; const ARect: TRect; AControlNotFocused: Boolean = False); virtual;
    procedure DrawTreeViewHotTrack(ACanvas: TfrCustomCanvas; const ARect: TRect); virtual;
    function GetDefaultTreeViewBorderColor: TColor; virtual;
    function GetTreeViewExpandButtonSymbolColor(AHotTrack: Boolean): TColor; virtual;
    function GetTreeViewExpandButtonSize: TSize; virtual;
    function GetTreeViewNodeHighliteBorderColor: TColor; virtual;
    function GetTreeViewNodeHotTrackColor: TColor; virtual;
    function GetTreeViewNodeSelectedColor(AControlNotFocused: Boolean = False): TColor; virtual;
    function GetTreeViewTextOffset: Integer; virtual;

    //ScrollBar
    procedure DrawScrollBarBackground(ACanvas: TfrCustomCanvas; const ARect: TRect); virtual;
    procedure DrawScrollBarButton(ACanvas: TfrCustomCanvas; const ARect: TRect; AArrowDir: TfrArrowDirection; AState: TfrButtonState);
    procedure DrawScrollBarThumb(ACanvas: TfrCustomCanvas; const ARect: TRect; AState: TfrButtonState); virtual;
    function GetScrollBarArrowColor(AState: TfrButtonState): TColor; virtual;
    function GetScrollBarButtonColor(AState: TfrButtonState): TColor; virtual;
    function GetScrollBarColor: TColor; virtual;
    function GetScrollBarThumbColor(AState: TfrButtonState): TColor; virtual;
    function GetScrollBarThumbWidth: Integer; virtual;
    function GetScrollBarWidth: Integer; virtual;
  end;

  { TfrStandardPainter }

  TfrStandardPainter = class(TfrCustomPainter);

  { TfrFlatPainter }

  TfrFlatPainter = class(TfrCustomPainter)

  end;

  { TfrNativePainter }

  TfrNativePainter = class(TfrCustomPainter)
  strict private
    FThemesEnabled: Boolean;
    FSystemStyle: Boolean;
  protected
    class function GetSystemStyle: Boolean;
    class function GetThemeEnabled: Boolean;
    class function GetPostfix: String; override;
  public
    constructor Create; override;

    class function GetPaintStyle: TfrPaintStyle; override;

    //TreeView
    procedure DrawTreeViewExpandButton(ACanvas: TfrCustomCanvas; const ARect: TRect; AExpanded, AHotTrack: Boolean; AScaleHelper: TfrScaleHelper); override;
  end;


implementation
end.
