
{******************************************}
{                                          }
{          FastReport VCL/FMX/LCL          }
{             Graphics Library             }
{                                          }
{         Copyright (c) 1998-2022          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frBaseGraphicsTypes;

interface

{$I frVer.inc}

uses
{$IFDEF FPC}
  LCLType,
{$ELSE}
  Windows,
{$ENDIF}
  Classes, Graphics, Types;

type
  TfrxGraphicQuality = (gqDefault, gqFast, gqHiQuality, gqPrint);
  TfrxGraphicDrawProp = (fgdAutoSize, fgdCenter, fgdKeepAspectRatio, fgdStretch, fgdTransparent, fgdClip, fgdDefaultMaskColor, fgdHiQuality { backward compatibility });
  TfrxGraphicDrawProps = set of TfrxGraphicDrawProp;
  TfrxThumbnailBuildType = (tbtNone, tbtOriginal, tbtAtPrepare, tbtAtSave, tbtAtPreview);
  TfrxPercentage =  0 .. 100;
  TfrxCachedGraphicType = (cgNone, cgOriginal, cgThumbnail);
  TfrPictureHashMapResultType = (phmIndex, phmData);
  TfrxLoadPictureToMethod = function (APicture: TPicture; aGraphic: TGraphic; ImageIndex: Integer): TGraphic of object;

  IfrxCachedGraphic = interface
  ['{13DA72E5-E8D1-4DEA-931A-8A729CB877E1}']
    procedure Draw(Canvas: TCanvas; X, Y, X1, Y1: Integer);
    function GetGraphic(CacheType: TfrxCachedGraphicType): TGraphic;
    function GetCacheType: TfrxCachedGraphicType;
    procedure SetCacheType(CacheType: TfrxCachedGraphicType);
    function GetOriginalSize: TSize;
  end;

  IfrxPictureCache = interface
    ['{06FD066C-726D-4BC0-811D-94150807E798}']
    function AddPicture(aGraphic: TGraphic): Integer;
    procedure GetGraphic(aGraphic: TGraphic; ImageIndex: Integer); overload;
    function GetGraphic(ImageIndex: Integer): TGraphic; overload;
    procedure GetPicture(APicture: TPicture; ImageIndex: Integer);
    function GetCachedBitmap(CacheType: TfrxCachedGraphicType; ImageIndex: Integer): IfrxCachedGraphic;
    function IsAutoRefMode: Boolean;
    function GetSize(const CacheType: TfrxCachedGraphicType; const ImageIndex: Integer): TSize;
  end;

  IfrxCachedView = interface
    ['{1E87267F-436A-4BFF-A6E9-0D03DD69F56C}']
    //function GetIndex: Integer;
    procedure SetCachedGraphic(const PictureCache: IfrxPictureCache);
    procedure GetCachedGraphic(ACacheType: TfrxCachedGraphicType; const PictureCache: IfrxPictureCache);
    function IsUpdateRequired(NewCacheType: TfrxCachedGraphicType): Boolean;
  //  function SetCachedGraphic(Index: Integer; cGraphic: IfrxCachedGraphic): Boolean;
//    function LoadSourceGraphic(Index: Integer; ALoadPictureTo: TfrxLoadPictureToMethod): Boolean;
    procedure ReleaseCachedGraphic;
  end;

  TfrxPictureConverterOptions = class(TPersistent)
  private
    FReducePercent: TfrxPercentage;
    FMinHeight: Integer;
    FMaxWidth: Integer;
    FMinWidth: Integer;
    FMaxHeight: Integer;
    procedure SetMinHeight(const Value: Integer);
    procedure SetMinWidth(const Value: Integer);
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
  published
    property MinWidth: Integer read FMinWidth write SetMinWidth default 32;
    property MaxWidth: Integer read FMaxWidth write FMaxWidth default MaxInt;
    property MinHeight: Integer read FMinHeight write SetMinHeight default 32;
    property MaxHeight: Integer read FMaxHeight write FMaxHeight default MaxInt;
    property ReducePercent: TfrxPercentage read FReducePercent write FReducePercent;
  end;

  TfrxPictureCacheOptions = class;

  TfrxPictureCacheOptions = class(TPersistent)
  private
    FCachedImagesBuildType: TfrxThumbnailBuildType;
    FThumbnailQualityReducer: TfrxPictureConverterOptions;
    FOriginalQualityReducer: TfrxPictureConverterOptions;
    FCalculateHash: Boolean;
    procedure SetOriginalQualityReducer(
      const Value: TfrxPictureConverterOptions);
    procedure SetThumbnailQualityReducer(
      const Value: TfrxPictureConverterOptions);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetPercentFor(const CacheType: TfrxCachedGraphicType; const Width, Height: Integer): TfrxPercentage;
  published
    property CalculateHash: Boolean read FCalculateHash write FCalculateHash default False;
    property OriginalQualityReducer: TfrxPictureConverterOptions read FOriginalQualityReducer write SetOriginalQualityReducer;
    property CachedImagesBuildType: TfrxThumbnailBuildType read FCachedImagesBuildType write FCachedImagesBuildType default tbtNone;
    property ThumbnailQualityReducer: TfrxPictureConverterOptions read FThumbnailQualityReducer write SetThumbnailQualityReducer;
  end;
  /// <summary>
  ///   Underlines text mode for "Text" object.
  /// </summary>
  TfrxUnderlinesTextMode = (
    /// <summary>
    ///   Disabled.
    /// </summary>
    ulmNone,
    /// <summary>
    ///   Underline whole object.
    /// </summary>
    ulmUnderlinesAll,
    /// <summary>
    ///   Underline only lines with text.
    /// </summary>
    ulmUnderlinesText,
    /// <summary>
    ///   Underline text lines and empty lines.
    /// </summary>
    ulmUnderlinesTextAndEmptyLines);
  /// <summary>
  ///   The horizontal alignment of the text in the "Text" object.
  /// </summary>
  TfrxHAlign = (haLeft, haRight, haCenter, haBlock);
  /// <summary>
  ///   The vertical alignment of the text in the "Text" object.
  /// </summary>
  TfrxVAlign = (vaTop, vaBottom, vaCenter);

  TfrxInterpretationType = (itHTML, itBBCode);
  TfrxHightlightingType = (ihHTML, ihBBCode, ihExpressions, ihSQL);
  TfrxInterpretationsTypes = set of TfrxInterpretationType;
  TfrxHightlightingsTypes = set of TfrxHightlightingType;
{$IFDEF NONWINFPC}
//PDeviceMode added just until compiles ok
  PDeviceMode = ^TDeviceMode;
  TDeviceMode =  packed Record
    dmDeviceName      : array[0..31] of AnsiChar;
    dmSpecVersion     : Word;
    dmDriverVersion   : Word;
    dmSize            : Word;
    dmDriverExtra     : Word;
    dmFields          : DWORD;
    dmOrientation     : SHORT;
    dmPaperSize       : SHORT;
    dmPaperLength     : SHORT;
    dmPaperWidth      : SHORT;
    dmScale           : SHORT;
    dmCopies          : SHORT;
    dmDefaultSource   : SHORT;
    dmPrintQuality    : SHORT;
    dmColor           : SHORT;
    dmDuplex          : SHORT;
    dmYResolution     : SHORT;
    dmTTOption        : SHORT;
    dmCollate         : SHORT;
    dmFormName        : Array[0..31] of AnsiChar;
    dmLogPixels       : Word;
    dmBitsPerPel      : DWORD;
    dmPelsWidth       : DWORD;
    dmPelsHeight      : DWORD;
    dmDisplayFlags    : DWORD;
    dmDisplayFrequency: DWORD;
    dmICMMethod       : DWORD;
    dmICMIntent       : DWORD;
    dmMediaType       : DWORD;
    dmDitherType      : DWORD;
    dmICCManufacturer : DWORD;
    dmICCModel        : DWORD;
    dmPanningWidth    : DWORD;
    dmPanningHeight   : DWORD;
  end;
{$ENDIF}
{$IFDEF FPC}
const
  DMBIN_UPPER = 1;
  DMBIN_FIRST = DMBIN_UPPER;
  DMBIN_ONLYONE = 1;
  DMBIN_LOWER = 2;
  DMBIN_MIDDLE = 3;
  DMBIN_MANUAL = 4;
  DMBIN_ENVELOPE = 5;
  DMBIN_ENVMANUAL = 6;
  DMBIN_AUTO = 7;
  DMBIN_TRACTOR = 8;
  DMBIN_SMALLFMT = 9;
  DMBIN_LARGEFMT = 10;
  DMBIN_LARGECAPACITY = 11;
  DMBIN_CASSETTE = 14;
  DMBIN_LAST = DMBIN_CASSETTE;
  DMBIN_USER = 256;

  MAC_CHARSET = 77;
  VIETNAMESE_CHARSET = 163;
  JOHAB_CHARSET = 130;

  DMPAPER_LETTER=1;//	US Letter 8 1/2 x 11 in
  DMPAPER_LETTERSMALL=2;//	US Letter Small 8 1/2 x 11 in
  DMPAPER_TABLOID=3;//	US Tabloid 11 x 17 in
  DMPAPER_LEDGER=4;//	US Ledger 17 x 11 in
  DMPAPER_LEGAL=5;//	US Legal 8 1/2 x 14 in
  DMPAPER_STATEMENT=6;//	US Statement 5 1/2 x 8 1/2 in
  DMPAPER_EXECUTIVE=7;//	US Executive 7 1/4 x 10 1/2 in
  DMPAPER_A3=8;//	A3 297 x 420 mm
  DMPAPER_A4=9;//	A4 210 x 297 mm
  DMPAPER_A4SMALL=10;//	A4 Small 210 x 297 mm
  DMPAPER_A5=11;//	A5 148 x 210 mm
  DMPAPER_B4=12;//	B4 (JIS) 257 x 364 mm
  DMPAPER_B5=13;//	B5 (JIS) 182 x 257 mm
  DMPAPER_FOLIO=14;//	Folio 8 1/2 x 13 in
  DMPAPER_QUARTO=15;//	Quarto 215 x 275 mm

  DMPAPER_10X14=16;//	10 x 14 in
  DMPAPER_11X17=17;//	11 x 17 in
  DMPAPER_NOTE=18;//	US Note 8 1/2 x 11 in
  DMPAPER_ENV_9=19;//	US Envelope #9 3 7/8 x 8 7/8
  DMPAPER_ENV_10=20;//	US Envelope #10 4 1/8 x 9 1/2
  DMPAPER_ENV_11=21;//	US Envelope #11 4 1/2 x 10 3/8
  DMPAPER_ENV_12=22;//	US Envelope #12 4 3/4 x 11 in
  DMPAPER_ENV_14=23;//	US Envelope #14 5 x 11 1/2
  DMPAPER_CSHEET=24;//	C size sheet
  DMPAPER_DSHEET=25;//	D size sheet
  DMPAPER_ESHEET=26;//	E size sheet
  DMPAPER_ENV_DL=27;//	Envelope DL 110 x 220mm
  DMPAPER_ENV_C5=28;//	Envelope C5 162 x 229 mm
  DMPAPER_ENV_C3=29;//	Envelope C3 324 x 458 mm
  DMPAPER_ENV_C4=30;//	Envelope C4 229 x 324 mm
  DMPAPER_ENV_C6=31;//	Envelope C6 114 x 162 mm
  DMPAPER_ENV_C65=32;//	Envelope C65 114 x 229 mm
  DMPAPER_ENV_B4=33;//	Envelope B4 250 x 353 mm
  DMPAPER_ENV_B5=34;//	Envelope B5 176 x 250 mm
  DMPAPER_ENV_B6=35;//	Envelope B6 176 x 125 mm
  DMPAPER_ENV_ITALY=36;//	Envelope 110 x 230 mm
  DMPAPER_ENV_MONARCH=37;//	US Envelope Monarch 3.875 x 7.5 in
  DMPAPER_ENV_PERSONAL=38;//	6 3/4 US Envelope 3 5/8 x 6 1/2 in
  DMPAPER_FANFOLD_US=39;//	US Std Fanfold 14 7/8 x 11 in
  DMPAPER_FANFOLD_STD_GERMAN=40;//	German Std Fanfold 8 1/2 x 12 in
  DMPAPER_FANFOLD_LGL_GERMAN=41;//	German Legal Fanfold 8 1/2 x 13 in
  DMPAPER_ISO_B4=42;//	B4 (ISO) 250 x 353 mm
  DMPAPER_JAPANESE_POSTCARD=43;//	Japanese Postcard 100 x 148 mm
  DMPAPER_9X11=44;//	9 x 11 in
  DMPAPER_10X11=45;//	10 x 11 in
  DMPAPER_15X11=46;//	15 x 11 in
  DMPAPER_ENV_INVITE=47;//	Envelope Invite 220 x 220 mm
  DMPAPER_RESERVED_48=48;//	RESERVED--DO NOT USE
  DMPAPER_RESERVED_49=49;//	RESERVED--DO NOT USE
  DMPAPER_A2 = 66;
  DMPAPER_A6 = 70;

  DMPAPER_FIRST = DMPAPER_LETTER;
  DMPAPER_LAST = DMPAPER_FANFOLD_LGL_GERMAN;
  DMPAPER_USER = 256;

  DMORIENT_PORTRAIT = 1;
  DMORIENT_LANDSCAPE = 2;

  DM_PAPERSIZE = 2;//1;
  DM_ORIENTATION = 1;//8;
  DM_COPIES = 16;
  DM_DUPLEX = 32;
  DM_DEFAULTSOURCE = 64;
  DM_PAPERLENGTH = 128;
  DM_PAPERWIDTH = 255;

  DMDUP_SIMPLEX = $01;
  DMDUP_VERTICAL = $02;
  DMDUP_HORIZONTAL = $03;

  DMCOLLATE_FALSE = 0;
  DMCOLLATE_TRUE = 1;

  srNone = '(None)';
  srUnknown = '(Unknown)';
  EM_REPLACESEL          = $00C2;

  E_INVALIDARG = 80070057;

  CP_ACP = 0;

  MEM_COMMIT = $1000;
  MEM_RESERVE = $2000;
  MEM_RESET = $80000;
  MEM_LARGE_PAGES = $20000000;
  MEM_PHYSICAL = $400000;
  MEM_TOP_DOWN = $100000;
  MEM_WRITE_WATCH = $200000;

  MEM_DECOMMIT = $4000;
  MEM_RELEASE = $8000;

  PAGE_EXECUTE = $10;
  PAGE_EXECUTE_READ = $20;
  PAGE_EXECUTE_READWRITE = $40;
  PAGE_EXECUTE_WRITECOPY = $80;
  PAGE_NOACCESS = $01;
  PAGE_READONLY = $02;
  PAGE_READWRITE = $04;
  PAGE_WRITECOPY = $08;
  PAGE_GUARD = $100;
  PAGE_NOCACHE = $200;
  PAGE_WRITECOMBINE = $400;
{$ENDIF}

const
  fr1pxToTwips = 15;

procedure frShowErrorMsgBox(const Text: String);

implementation
end.
