
{******************************************}
{                                          }
{          FastReport VCL/FMX/LCL          }
{             Graphics Library             }
{                                          }
{         Copyright (c) 1998-2022          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

{$IFNDEF FMX}
unit frDialectHelper;
{$ENDIF}

interface

{$I frVer.inc}

uses
  Types, Graphics, Classes,
{$IFNDEF FPC}
  Windows,
{$ELSE}
  frLazGraphicClasses,
{$ENDIF}
  SysUtils;

const
  SQLKeywords =
    'active,after,all,alter,and,any,as,asc,ascending,at,auto,' +
    'base_name,before,begin,between,by,cache,call,cast,check,column,commit,' +
    'committed,computed,conditional,constraint,containing,count,create,' +
    'current,cursor,database,debug,declare,default,delete,desc,descending,' +
    'distinct,do,domain,drop,else,end,entry_point,escape,exception,execute,' +
    'exists,exit,external,extract,filter,for,foreign,from,full,function,' +
    'generator,grant,group,having,if,in,inactive,index,inner,insert,into,is,' +
    'isolation,join,key,left,level,like,merge,names,no,not,null,of,on,only,' +
    'or,order,outer,parameter,password,plan,position,primary,privileges,' +
    'procedure,protected,read,retain,returns,revoke,right,rollback,schema,' +
    'select,set,shadow,shared,snapshot,some,suspend,table,then,to,' +
    'transaction,trigger,uncommitted,union,unique,update,user,using,values,' +
    'view,wait,when,where,while,with,work';
  SQLCommentLine1 = '--';
  SQLCommentBlock1 = '/*,*/';
  SQLStringQuotes = '"';
  SQLHexSequence = '0x';

type
  TCharAttr = (caNo, caText, caComment, caKeyword, caString,
    caNumber);

  TfrAttributeStyle = class(TCollectionItem)
  private
    FStyleID: Byte;
    FFontColor: TColor;
    FFontStyle: TFontStyles;
    FKeywords: TStrings;
    FAttrType: TCharAttr;
  protected
    procedure SetCollection(Value: TCollection); override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    property StyleID: Byte read FStyleID;
  published
    property AttrType: TCharAttr read FAttrType write FAttrType;
    property FontColor: TColor read FFontColor write FFontColor;
    property FontStyle: TFontStyles read FFontStyle write FFontStyle;
    property Keywords: TStrings read FKeywords;
  end;

  TfrAttributeStyles = class(TCollection)
  private
    { null object }
    FDefaultAttribute: TfrAttributeStyle;
    FIndexedList: TStringList;
    function GetItem(Index: Integer): TfrAttributeStyle;
  public
    function GetUniqueID: Byte;
    constructor Create;
    destructor Destroy; override;
    function Add: TfrAttributeStyle;
    function GetStyleByID(ID: Byte): TfrAttributeStyle;
    procedure AssignStyleByID(ID: Byte; aFont: TFont);
    procedure AssignStyle(aStyle: TfrAttributeStyle; aFont: TFont);
    function FindStyleIDByKeyword(const Name: String; AttrType: TCharAttr): Byte;
    property Items[Index: Integer]: TfrAttributeStyle read GetItem; default;
  end;

  TfrSynDialectStyle = class(TPersistent)
  private
    FAttributeStyles: TfrAttributeStyles;
    FKeywords: String;
    FCommentLine1: String;
    FCommentLine2: String;
    FCommentBlock1: String;
    FCommentBlock2: String;
    FStringQuotes: String;
    FHexSequence: String;
    FName: String;
    FOwner: TObject;
    procedure SetName(const Value: String);
  public
    constructor Create(AOwner: TObject);
    destructor Destroy; override;
    { TODO: we need one contract for all setings. I.e. need interface for this }
    procedure SaveTo(const Section: String; Ini: TObject);
    procedure LoadFrom(const Section: String; Ini: TObject);
    property AttributeStyles: TfrAttributeStyles read FAttributeStyles;
    property Name: String read FName write SetName;
  published
    property Keywords: String read FKeywords write FKeywords;
    property CommentLine1: String read FCommentLine1 write FCommentLine1;
    property CommentLine2: String read FCommentLine2 write FCommentLine2;
    property CommentBlock1: String read FCommentBlock1 write FCommentBlock1;
    property CommentBlock2: String read FCommentBlock2 write FCommentBlock2;
    property StringQuotes: String read FStringQuotes write FStringQuotes;
    property HexSequence: String read FHexSequence write FHexSequence;
  end;

  TfrSynDialectStyles = class(TPersistent)
  private
    FLIst: TList;
    FDefSynDialectStyle: TfrSynDialectStyle;
    FActiveIndex: Integer;
    FActiveChanged: TNotifyEvent;
    function GetItem(Index: Integer): TfrSynDialectStyle;
    function GetActiveDialicet: TfrSynDialectStyle;
    function GetActiveStyles: TfrAttributeStyles;
    procedure SetActiveIndex(const Value: Integer);
   public
    constructor Create;
    destructor Destroy; override;
    function Add: TfrSynDialectStyle;
    procedure Clear;
    function Count: Integer;
    { TODO: we need one contract for all setings. I.e. need interface for this }
    procedure SaveTo(const Section: String; Ini: TObject);
    procedure LoadFrom(const Section: String; Ini: TObject);
    property Items[Index: Integer]: TfrSynDialectStyle read GetItem; default;
    property ActiveStyles: TfrAttributeStyles read GetActiveStyles;
    property ActiveIndex: Integer read FActiveIndex write SetActiveIndex;
    property OnActiveChanged: TNotifyEvent read FActiveChanged write FActiveChanged;
  published
    property ActiveDialicet: TfrSynDialectStyle read GetActiveDialicet;
  end;

implementation
end.
