{******************************************}
{                                          }
{          FastReport VCL/FMX/LCL          }
{             Graphics Library             }
{                                          }
{         Copyright (c) 1998-2022          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

      {******************************************************************}
      { GDIPKerning                                                      }
      {                                                                  }
      { home page : http://www.mwcs.de                                   }
      { email     : martin.walter@mwcs.de                                }
      {                                                                  }
      { date      : 20-11-2007                                           }
      {                                                                  }
      { version   : 1.1                                                  }
      {                                                                  }
      { Use of this file is permitted for commercial and non-commercial  }
      { use, as long as the author is credited.                          }
      { This file (c) 2007 Martin Walter                                 }
      {                                                                  }
      { This Software is distributed on an "AS IS" basis, WITHOUT        }
      { WARRANTY OF ANY KIND, either express or implied.                 }
      {                                                                  }
      { *****************************************************************}
      {               Adapted for FR 7 by Oleg Adibekov                  }
      { *****************************************************************}

unit frGDIPKerning;

interface

{$I frVer.inc}

uses
  Windows,
  Classes,
{$IFDEF DELPHI16}
  Winapi.GDIPOBJ,
  Winapi.GDIPAPI,
{$ELSE}
  frGDIPOBJ,
  frGDIPAPI,
{$ENDIF}
  Types,
  frSVGCanvas;

type
  TKerningPairs = array of TKerningPair;
  PKerningPairs = ^TKerningPairs;

  TKerning = record
    Chars: Cardinal;
    Cell: Integer;
    Kerning: Integer;
  end;
  
  PKerning = ^TKerning;

  TGPKerningText = class(TObject)
  private
    FKerningList: TList;
    FKerningPairs: TKerningPairs;
    FPairCount: Integer;
    FLastFont: WideString;
    FLastWeight: Integer;
    FLastStyle: Integer;
    FFontSizeFactor: Double;

    FFont: HFont;
    FOldFont: HFont;
    FDC: HDC;
    FGPFont: TGPFont;

    procedure PrepareDC;
    procedure UnprepareDC;
    procedure PrepareFont(const LF: TLogFontW);
    procedure UnprepareFont;
    procedure PrepareKerning(const Font: TGPFont); overload;
    procedure PrepareKerning(const Font: TGPFont; const Graphics: TGPGraphics;
      WithFactor: Boolean); overload;
    procedure UnprepareKerning;
    procedure ClearKerningList;
    procedure AddToKerningList(const First, Second: Word;
      Cell, Kerning: Integer);
    function IndexOfKerning(const First, Second: Word): Integer;

    function GetDistance(const Index: Integer;
      const DistanceFactor, KerningFactor: Single): Single;

    procedure Clear;

    function AddGlyphToPath(const Path: TGPGraphicsPath; const Char: WideChar;
      const Family: TGPFontFamily; const Style: Integer; const Size: Single;
      const Origin: TGPPointF; const Format: TGPStringFormat): TStatus;

    function AddGlyphToGraphics(const Graphics: TGPGraphics;
      const Char: WideChar; const Font: TGPFont; const Origin: TGPPointF;
      const Format: TGPStringFormat; const Brush: TGPBrush): TStatus;

    procedure AddDecoration(const Path: TGPGraphicsPath;
      const Left, Top, Width: Single; const Font: TGPFont; dY: Single);
  public
    constructor Create;
    destructor Destroy; override;

  	function AddToPath(const Path, UPath, LPath, OPath: TGPGraphicsPath;
      const Text: WideString; const Family: TGPFontFamily; Style: Integer;
      const Size: Single; TextOrigin: TTextOrigin;
      const DistanceFactor: Single = 1;
      const KerningFactor: Single = 1): TStatus;

  	function AddToGraphics(const Graphics: TGPGraphics;
      const Text: WideString; const Font: TGPFont; Origin: TGPPointF;
      const Format: TGPStringFormat; const Brush: TGPBrush;
      const DistanceFactor: Single = 1;
      const KerningFactor: Single = 1): TStatus; overload;

  	function AddToGraphics(const Graphics: TGPGraphics;
      const Text: WideString; const Font: TGPFont; Origin: TGPPoint;
      const Format: TGPStringFormat; const Brush: TGPBrush;
      const DistanceFactor: Single = 1;
      const KerningFactor: Single = 1): TStatus; overload;

    function MeasureText(const Text: WideString; const Font: TGPFont;
      const DistanceFactor: Single = 1;
      const KerningFactor: Single = 1): Single;

    function GetCellWidth(const First, Second: Word;
      const DistanceFactor: Single = 1;
      const KerningFactor: Single = 1): Single;

    procedure Prepare(const Family: TGPFontFamily; Style: Integer;
      const Size: Single; const Format: TGPStringFormat);

    procedure Unprepare;
  end;

function KerningText: TGPKerningText;

implementation
end.
