
{******************************************}
{                                          }
{             FastReport VCL               }
{                 Printer                  }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frPrinter;

interface

{$I frVer.inc}

uses 
{$IFNDEF NONWINFPC}
  Windows,
{$ELSE}
  {$IFDEF LCLGTK2}
  CairoCanvas,
  {$ENDIF}
{$ENDIF}
  SysUtils, Types, Classes, Graphics, Forms, Printers, frCore
{$IFDEF FPC}
  ,LCLType, LCLIntf
{$ENDIF}
  ,SyncObjs
{$IFDEF DELPHI6}
, Variants
{$ENDIF};


type
  TfrxCustomPrinter = class(TObject)
  protected
    FBin: Integer;
    FDuplex: Integer;
    FBins: TStrings;
    FCanvas: TCanvas;
    FDefOrientation: TPrinterOrientation;
    FDefPaper: Integer;
    FDefPaperHeight: Extended;
    FDefPaperWidth: Extended;
    FDefDuplex: Integer;
    FDefBin: Integer;
    FDPI: TPoint;
    FFileName: String;
    FHandle: TfrHandle;
    FInitialized: Boolean;
    FName: String;
    FPaper: Integer;
    FPapers: TStrings;
    FPaperHeight: Extended;
    FPaperWidth: Extended;
    FLeftMargin: Extended;
    FTopMargin: Extended;
    FRightMargin: Extended;
    FBottomMargin: Extended;
    FOrientation: TPrinterOrientation;
    FPort: String;
    FPrinting: Boolean;
    FTitle: String;
    FIsCopy: Boolean;
  public
    constructor Create(const AName, APort: String); virtual;
    destructor Destroy; override;
    function CreateCopy: TfrxCustomPrinter;
    procedure Assign(APrinter: TfrxCustomPrinter);
    function BinIndex(ABin: Integer): Integer;
    function PaperIndex(APaper: Integer): Integer;
    function BinNameToNumber(const ABin: String): Integer;
    function PaperNameToNumber(const APaper: String): Integer;
    function IsDefaultDuplexActive: Boolean; virtual;

    procedure Init; virtual; abstract;
    procedure Abort; virtual; abstract;
    procedure BeginDoc; virtual; abstract;
    procedure BeginPage; virtual; abstract;
    procedure BeginRAWDoc; virtual; abstract;
    procedure EndDoc; virtual; abstract;
    procedure EndPage; virtual; abstract;
    procedure EndRAWDoc; virtual; abstract;
    procedure WriteRAWDoc(const buf: AnsiString); virtual; abstract;
    procedure SetViewParams(APaperSize: Integer;
      APaperWidth, APaperHeight: Extended;
      AOrientation: TPrinterOrientation); virtual; abstract;
    procedure SetPrintParams(APaperSize: Integer;
      APaperWidth, APaperHeight: Extended; AOrientation: TPrinterOrientation;
      ABin, ADuplex, ACopies: Integer); virtual; abstract;
    procedure PropertiesDlg; virtual; abstract;
    class function GetDefaultPrinter: String; virtual; abstract;
    class procedure FillPrinters(APrinterList: TList; APrinters: TStrings); virtual; abstract;

    property Bin: Integer read FBin write FBin;
    property Duplex: Integer read FDuplex;
    property Bins: TStrings read FBins;
    property Canvas: TCanvas read FCanvas;
    property DefOrientation: TPrinterOrientation read FDefOrientation;
    property DefPaper: Integer read FDefPaper;
    property DefPaperHeight: Extended read FDefPaperHeight;
    property DefPaperWidth: Extended read FDefPaperWidth;
    property DefBin: Integer read FDefBin;
    property DefDuplex: Integer read FDefDuplex;
    property DPI: TPoint read FDPI;
    property FileName: String read FFileName write FFileName;
    property Handle: TfrHandle read FHandle;
    property Name: String read FName;
    property Paper: Integer read FPaper;
    property Papers: TStrings read FPapers;
    property PaperHeight: Extended read FPaperHeight;
    property PaperWidth: Extended read FPaperWidth;
    property Printing: Boolean read FPrinting;
    property LeftMargin: Extended read FLeftMargin;
    property TopMargin: Extended read FTopMargin;
    property RightMargin: Extended read FRightMargin;
    property BottomMargin: Extended read FBottomMargin;
    property Orientation: TPrinterOrientation read FOrientation;
    property Port: String read FPort;
    property Title: String read FTitle write FTitle;
    property Initialized: Boolean read FInitialized;
    property IsCopy: Boolean read FIsCopy;
  end;

  TfrxCustomVirtualPrinter = class(TfrxCustomPrinter)
  public
    class function GetDefaultPrinter: String; override;
    class procedure FillPrinters(APrinterList: TList; APrinters: TStrings); override;
    procedure FillPapers; virtual;
    procedure Init; override;
    procedure Abort; override;
    procedure BeginDoc; override;
    procedure BeginPage; override;
    procedure BeginRAWDoc; override;
    procedure EndDoc; override;
    procedure EndPage; override;
    procedure EndRAWDoc; override;
    procedure WriteRAWDoc(const buf: AnsiString); override;
    procedure SetViewParams(APaperSize: Integer;
      APaperWidth, APaperHeight: Extended;
      AOrientation: TPrinterOrientation); override;
    procedure SetPrintParams(APaperSize: Integer;
      APaperWidth, APaperHeight: Extended; AOrientation: TPrinterOrientation;
      ABin, ADuplex, ACopies: Integer); override;
    procedure PropertiesDlg; override;
  end;

  TfrxPrinters = class(TObject)
  protected
    FHasPhysicalPrinters: Boolean;
    FPrinters: TStrings;
    FPrinterIndex: Integer;
    FPrinterList: TList;
    FCriticalSection: TCriticalSection;
    FChangeLockCount: Integer;
    function GetItem(Index: Integer): TfrxCustomPrinter;
    function GetCurrentPrinter: TfrxCustomPrinter;
    function GetDefaultPrinter: String;
    procedure SetPrinterIndex(Value: Integer);
    procedure Lock;
    procedure Unlock;
  public
    constructor Create;
    destructor Destroy; override;
    function IndexOf(AName: String): Integer;
    procedure Clear;
    procedure FillPrinters;
    function GetPrinterByName(const AName: string): TfrxCustomPrinter;
    procedure LockPrinterChange;
    procedure UnlockPrinterChange;

    property Items[Index: Integer]: TfrxCustomPrinter read GetItem; default;
    property HasPhysicalPrinters: Boolean read FHasPhysicalPrinters;
    property Printer: TfrxCustomPrinter read GetCurrentPrinter;
    property PrinterIndex: Integer read FPrinterIndex write SetPrinterIndex;
    property Printers: TStrings read FPrinters;
  end;

  TfrxPrinterCanvas = class({$IFDEF LCLGTK2}TCairoPrinterCanvas{$ELSE}TCanvas{$ENDIF})
  private
{$IFDEF LCLGTK2}
    function GetPrinter(): TPrinter;
{$ELSE}
    FPrinter: TfrxCustomPrinter;
{$ENDIF}
  public
    procedure Changing; override;
    procedure UpdateFont;
{$IFDEF LCLGTK2}
    property Printer: TPrinter read GetPrinter;
{$ELSE}
    property Printer: TfrxCustomPrinter read FPrinter write FPrinter;
{$ENDIF}
  end;

function frxPrinters: TfrxPrinters;
function frxGetPaperDimensions(PaperSize: Integer; var Width, Height: Extended): Boolean;

type
  TPaperInfo = {packed} record
    Typ: Integer;
    Name: String;
    X, Y: Integer;
    LTyp:Integer; //Use Only in Linux
  end;

const
  PAPERCOUNT = 67;
  PaperInfo: array[0..PAPERCOUNT - 1] of TPaperInfo = (
  (Typ:1;  Name: ''; X:2159; Y:2794; LTyp:0),
  (Typ:2;  Name: ''; X:2159; Y:2794; LTyp:0),
  (Typ:3;  Name: ''; X:2794; Y:4318; LTyp:0),
  (Typ:4;  Name: ''; X:4318; Y:2794; LTyp:0),
  (Typ:5;  Name: ''; X:2159; Y:3556; LTyp:0),
  (Typ:6;  Name: ''; X:1397; Y:2159; LTyp:0),
  (Typ:7;  Name: ''; X:1842; Y:2667; LTyp:0),
  (Typ:8;  Name: ''; X:2970; Y:4200; LTyp:0),
  (Typ:9;  Name: ''; X:2100; Y:2970; LTyp:0),
  (Typ:10; Name: ''; X:2100; Y:2970; LTyp:0),
  (Typ:11; Name: ''; X:1480; Y:2100; LTyp:0),
  (Typ:12; Name: ''; X:2500; Y:3540; LTyp:0),
  (Typ:13; Name: ''; X:1820; Y:2570; LTyp:0),
  (Typ:14; Name: ''; X:2159; Y:3302; LTyp:0),
  (Typ:15; Name: ''; X:2150; Y:2750; LTyp:0),
  (Typ:16; Name: ''; X:2540; Y:3556; LTyp:0),
  (Typ:17; Name: ''; X:2794; Y:4318; LTyp:0),
  (Typ:18; Name: ''; X:2159; Y:2794; LTyp:0),
  (Typ:19; Name: ''; X:984;  Y:2254; LTyp:0),
  (Typ:20; Name: ''; X:1048; Y:2413; LTyp:0),
  (Typ:21; Name: ''; X:1143; Y:2635; LTyp:0),
  (Typ:22; Name: ''; X:1207; Y:2794; LTyp:0),
  (Typ:23; Name: ''; X:1270; Y:2921; LTyp:0),
  (Typ:24; Name: ''; X:4318; Y:5588; LTyp:0),
  (Typ:25; Name: ''; X:5588; Y:8636; LTyp:0),
  (Typ:26; Name: ''; X:8636; Y:11176; LTyp:0),
  (Typ:27; Name: ''; X:1100; Y:2200; LTyp:0),
  (Typ:28; Name: ''; X:1620; Y:2290; LTyp:0),
  (Typ:29; Name: ''; X:3240; Y:4580; LTyp:0),
  (Typ:30; Name: ''; X:2290; Y:3240; LTyp:0),
  (Typ:31; Name: ''; X:1140; Y:1620; LTyp:0),
  (Typ:32; Name: ''; X:1140; Y:2290; LTyp:0),
  (Typ:33; Name: ''; X:2500; Y:3530; LTyp:0),
  (Typ:34; Name: ''; X:1760; Y:2500; LTyp:0),
  (Typ:35; Name: ''; X:1760; Y:1250; LTyp:0),
  (Typ:36; Name: ''; X:1100; Y:2300; LTyp:0),
  (Typ:37; Name: ''; X:984;  Y:1905; LTyp:0),
  (Typ:38; Name: ''; X:920;  Y:1651; LTyp:0),
  (Typ:39; Name: ''; X:3778; Y:2794; LTyp:0),
  (Typ:40; Name: ''; X:2159; Y:3048; LTyp:0),
  (Typ:41; Name: ''; X:2159; Y:3302; LTyp:0),
  (Typ:42; Name: ''; X:2500; Y:3530; LTyp:0),
  (Typ:43; Name: ''; X:1000; Y:1480; LTyp:0),
  (Typ:44; Name: ''; X:2286; Y:2794; LTyp:0),
  (Typ:45; Name: ''; X:2540; Y:2794; LTyp:0),
  (Typ:46; Name: ''; X:3810; Y:2794; LTyp:0),
  (Typ:47; Name: ''; X:2200; Y:2200; LTyp:0),
  (Typ:50; Name: ''; X:2355; Y:3048; LTyp:0),
  (Typ:51; Name: ''; X:2355; Y:3810; LTyp:0),
  (Typ:52; Name: ''; X:2969; Y:4572; LTyp:0),
  (Typ:53; Name: ''; X:2354; Y:3223; LTyp:0),
  (Typ:54; Name: ''; X:2101; Y:2794; LTyp:0),
  (Typ:55; Name: ''; X:2100; Y:2970; LTyp:0),
  (Typ:56; Name: ''; X:2355; Y:3048; LTyp:0),
  (Typ:57; Name: ''; X:2270; Y:3560; LTyp:0),
  (Typ:58; Name: ''; X:3050; Y:4870; LTyp:0),
  (Typ:59; Name: ''; X:2159; Y:3223; LTyp:0),
  (Typ:60; Name: ''; X:2100; Y:3300; LTyp:0),
  (Typ:61; Name: ''; X:1480; Y:2100; LTyp:0),
  (Typ:62; Name: ''; X:1820; Y:2570; LTyp:0),
  (Typ:63; Name: ''; X:3220; Y:4450; LTyp:0),
  (Typ:64; Name: ''; X:1740; Y:2350; LTyp:0),
  (Typ:65; Name: ''; X:2010; Y:2760; LTyp:0),
  (Typ:66; Name: ''; X:4200; Y:5940; LTyp:0),
  (Typ:67; Name: ''; X:2970; Y:4200; LTyp:0),
  (Typ:68; Name: ''; X:3220; Y:4450; LTyp:0),
  (Typ:70; Name: ''; X:1050; Y: 1480; LTyp:0));

implementation
end.

