
{******************************************}
{                                          }
{             FastReport VCL               }
{              SVG Elements                }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}
unit frSVGElement;

interface

{$I frVer.inc}

uses
  Classes, Types,
  frSVGCanvas, frSVGHelpers, frSVGComponents, frCSSStyle, frSVGAttribute,
  frSVGColor, frSVGParse;

type
  TSVGElementObj = class (TPersistent)
  private
    FParent: TSVGElementObj;
    FItems: TList;
    FHostSVG: TSVGElementObj;
    FStylesApplyingSequence: TList;
    FAttrObj: array[TSVGAttribute] of TSVGAttributeObj;
    FIsAttrEnabled: array[TSVGAttribute] of Boolean;

    function GetCount: Integer;
    function GetItem(const Index: Integer): TSVGElementObj;
    function GetIsAttrEnabled(a: TSVGAttribute): Boolean;
    procedure SetIsAttrEnabled(a: TSVGAttribute; const Value: Boolean);
    function GetAttrObj(a: TSVGAttribute): TSVGAttributeObj;
  protected
    FID: string;
    FRoot: TSVGElementObj;
    FPattern: TSVGElementObj;
    FElement: TSVGElement;
    FInlineStyle: TfrxCSSStyle;
    FAttributeStyle: TfrxCSSStyle;
    FClasses: TStrings;
    FSwitchedOn: Boolean; // to show / hide switch children
    FCompleteMatrix: TSVGTransform; // from root
    FInnerMatrix: TSVGTransform; // inside nearest svg
    FObjBounds: TSingleBounds;

    procedure AssignTo(Dest: TPersistent); override;
    procedure AssignStylesApplyingSequenceTo(Obj: TSVGElementObj); virtual;
    function New(Parent: TSVGElementObj): TSVGElementObj; virtual; abstract;
    procedure ReadIn(const Node: TfrxSVGXMLItem);
    procedure ReadClasses(st: string);
    procedure ReadChildren(const Node: TfrxSVGXMLItem);
    procedure GetValueFromStyles(aObj: TSVGAttributeObj);
    procedure ConstructPath; virtual; // Empty
    procedure ClarifyStyleSequences;
    procedure ClarifyOwnShorthandProperties; virtual;
    procedure ClarifyShorthandProperties;
    procedure ConstructPathes;
    procedure SwitchChildren; virtual; // Empty
    procedure SwitchAll;
    procedure TryAddToSequence(StyleName: string); overload;
    procedure TryAddToSequence(Style: TfrxCSSStyle); overload;
    procedure ConstructStyleSequence;
    procedure ConstructAttributes(AElement: TSVGElement);
    function GetCanvasClass: TSVGCanvasClass; virtual;
    function CalcUnitedChildrenBounds: TSingleBounds;
    function CalcSelfBounds: TSingleBounds; virtual; // raise Exception
    function IsOpt(eo: TSVGElementOptions): Boolean;
    function IsAttrDefault(a: TSVGAttribute): Boolean;
    function GetPatternFactor: Single; virtual;
    function GetIsFullOpacity: Boolean; virtual;
    procedure SetIsFullOpacity(const Value: Boolean); virtual;

    procedure LogStructure(S: string);
  public
    constructor Create; overload; virtual;
    constructor Create(AParent: TSVGElementObj); overload;
    destructor Destroy; override;
    procedure Clear; virtual;
    procedure ClearItems;
    procedure ClarifyUses;
    function Clone(Parent: TSVGElementObj): TSVGElementObj;
    procedure CalculateMatrices;
    procedure ClarifyBounds;
    function IsRoot: Boolean; virtual;
    function IsNeedsPainting: Boolean;
    function SerialNumber: Integer;
    function IsInPattern: Boolean;

    procedure CalcMatrix;

    function ElementName: string;
    function FindByID(const SoughtID: string): TSVGElementObj;

    function GetHostWidth: Single; virtual;
    function GetHostHeight: Single; virtual;
    function GetHostDiagonal: Single; virtual;
    function GetDirectionalHostSize(Direction: TSVGDirection): Single;

    function GetWidth: Single; virtual;
    function GetHeight: Single; virtual;
    function GetBounds: TSingleBounds;
    function GetDiagonal: Single;
    function GetDirectionalSize(Direction: TSVGDirection): Single;
    function GetTransform: TSVGTransform; virtual;

    function CalcFillOpacity: Single; virtual;
    function CalcStrokeOpacity: Single; virtual;

    function atAlpha(a: TSVGAttribute): Single;
    function atIsDefault(a: TSVGAttribute): Boolean;
    function atIsPercent(a: TSVGAttribute): Boolean;

    function atLength(a: TSVGAttribute; Base: Single = DefaultBase): Single;
    function atLengthAuto(a: TSVGAttribute; Base: Single = DefaultBase): Single;
    procedure atSetLengthAuto(a: TSVGAttribute; Value: Single);
    function atLengthInherit(a: TSVGAttribute): Single;
    function atLengthList(a: TSVGAttribute): TSingleDynArray;
    function atLengthListNoneInherit(a: TSVGAttribute): TSingleDynArray;
    function IsForceLengthToPercent: Boolean; virtual;

    function atNumber(a: TSVGAttribute): Single;
    function atNumberInherit(a: TSVGAttribute): Single;
    function atFontWeight: Single;

    function atColorCurrent(a: TSVGAttribute): TSVGColor;
    function atColorInherit(a: TSVGAttribute): TSVGColor;

    function atSpecificWord(a: TSVGAttribute): TSVGSpecificWord;
    function atSpecificWordSet(a: TSVGAttribute): TSVGSpecificWordSet;

    function atString(a: TSVGAttribute): string;
    function atStringInherit(a: TSVGAttribute): string;
    function atURI(a: TSVGAttribute): string;

    function atPAR: TSVGPreserveAspectRatio;

    function atPaint(a: TSVGAttribute): TSVGPaint;

    function atPoints: TSinglePointDynArray;

    function atMatrix(a: TSVGAttribute): TSVGTransform;

    property Items[const Index: Integer]: TSVGElementObj read GetItem; default;
    property Count: Integer read GetCount;

    property Element: TSVGElement read FElement;
    property Parent: TSVGElementObj read FParent;
    property Pattern: TSVGElementObj read FPattern;
    property PatternFactor: Single read GetPatternFactor;
    property CanvasClass: TSVGCanvasClass read GetCanvasClass;
    property ID: string read FID;

    property IsFullOpacity: Boolean read GetIsFullOpacity write SetIsFullOpacity;

    property SwitchedOn: Boolean read FSwitchedOn write FSwitchedOn;

    property CompleteMatrix: TSVGTransform read FCompleteMatrix;
    property InnerMatrix: TSVGTransform read FInnerMatrix;

    property IsAttrEnabled[a: TSVGAttribute]: Boolean read GetIsAttrEnabled write SetIsAttrEnabled;
    property AttrObj[a: TSVGAttribute]: TSVGAttributeObj read GetAttrObj;
  end;

function IsElementByName(Name: string; out Element: TSVGElement): Boolean;

implementation
end.
