
{******************************************}
{                                          }
{             FastReport VCL               }
{             SVG GDI+ Canvas              }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frSVGGDIPCanvas;

interface

{$I frVer.inc}

uses
  Windows,
  Graphics,
  Classes,
  Types,
{$IFDEF DELPHI16}
  System.UITypes,
  Winapi.GDIPOBJ,
  Winapi.GDIPAPI,
{$ELSE}
  frGDIPOBJ,
  frGDIPAPI,
{$ENDIF}
  frGDIPPathText,
  frSVGColor,
  frSVGCanvas,
  frSVGHelpers,
  frSVGComponents;

type
  TSVGGDIPCanvas = class(TSVGCanvas)
  private
    FPen: TGPPen;
    FGPMatrix: TGPMatrix;
    FGraphics: TGPGraphics;
  protected
    FColors: packed array of ARGB;
    FPositions: packed array of Single;

    procedure CalcGradientData(SVGGradientArray: TSVGGradientArray);
    function GetStringSize(WS: WideString; Font: TGPFont): TSingleSize;
    procedure SetElementsGPMatrix(const Matrix: TSVGTransform);

    class function GetGPFont(FontData: TSVGFontData; out FontHeight: Single): TGPFont;
    class function GDIPLineJoin(SVGLineJoin: TSVGSpecificWord): TLineJoin;
    class function GDIPLineCap(SVGLineCap: TSVGSpecificWord): TLineCap;
    class function GDIPFontStyle(FontData: TSVGFontData): TFontStyle;
    class function GDIPFillMode(FillRule:  TSVGSpecificWord): TFillMode;
    class function ConvertColor(SVGColor: TSVGColor): Cardinal;
    class function ProjectionOfPointToLine(L1, L2, P: TSinglePoint): TSinglePoint;
    class function Distance(const x1, y1, x2, y2: Single): Single; overload;
    class function Distance(const P1, P2: TSinglePoint): Single; overload;
  protected
    FSolidColor: array [TSVGDrawType] of TGPSolidBrush;
    FLinearGradient: array [TSVGDrawType] of TGPLinearGradientBrush;
    FRadialGradient: array [TSVGDrawType] of TGPPathGradientBrush;
    FPattern: array [TSVGDrawType] of TGPTextureBrush;

    procedure ChangeSolidColorFiller(DrawType: TSVGDrawType); override;

    procedure ChangeLinearGradientFiller(DrawType: TSVGDrawType); override;
    procedure WidenTile(var GA: TSVGGradientArray;
      const WidenAddStart, WidenAddFinish: Integer;
      var WidenStart, WidenFinish: TSinglePoint);

    procedure ChangeRadialGradientFiller(DrawType: TSVGDrawType); override;
    procedure WidenRadius(var GA: TSVGGradientArray; out Radius: TSinglePoint;
      const ObjBounds: TSingleBounds; const cx, cy, r: Single);

    procedure ChangePatternFiller(DrawType: TSVGDrawType); override;

    procedure StrokePath(Path: TSVGCanvasPath); override;
    procedure FillPath(Path: TSVGCanvasPath); override;

    function CreateGPBitmapFromBitmap(const Bitmap: TBitmap; Offset: TPoint): TGPBitmap; overload;
    function CreateGPBitmapFromBitmap(const Bitmap: TBitmap): TGPBitmap; overload;
    function LoadGPImageFromGraphic(const Graphic: TGraphic; out Stream: TMemoryStream): TGpImage;
  public
    constructor Create(DC: HDC); override;
    destructor Destroy; override;

    procedure SetTransform(const Matrix: TSVGTransform); override;
    procedure GetTransform(out Matrix: TSVGTransform); override;
    procedure ResetTransform; override;

    procedure SetClip(Path: TSVGCanvasPath); override;
    procedure IntersectClip(Path: TSVGCanvasPath); override;
    procedure ResetClip; override;

    procedure SetStroke(StrokeData: TSVGStrokeData); override;

    function MeasureString(st: string; FontData: TSVGFontData;
      out FontHeight: Single): TSingleSize; override;

    procedure DrawImage(FImage: TSVGCanvasImage; SVGRect: TSingleBounds; FillOpacity: Single); override;

    class procedure AddStringToPath(st: string; Path: TSVGCanvasPath; Decorations: TTextDecorations;
      TextOrigin: TTextOrigin; FontData: TSVGFontData); override;

    class function GetFontFamily(const FontName: string): TGPFontFamily;

    class function CreateImage(SA: TStreamAdapter): TSVGCanvasImage; override;
    class function CreatePath: TSVGCanvasPath; override;
    class function CreatePathText(Path: TSVGCanvasPath): TSVGCanvasPathText; override;
    class function GetPathLength(const Path: TSVGCanvasPath): Single; override;

    class function GetFont(FontData: TSVGFontData): TFont; override;
  end;

implementation
end.
