
{******************************************}
{                                          }
{             FastReport VCL               }
{       Parse of SVG property values       }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frSVGParse;

interface

{$I frVer.inc}

uses
  Windows, Classes,
  frSVGHelpers, frSVGComponents, frCSSStyle;

type
  // https://www.w3.org/TR/css-values/#lengths
  TSVGUnit = (suNone, suPX, suPT, suPC, suIN, suMM, suCM, suEM, suEX, suPercent,
  // https://webref.ru/css/value/angle
    suDeg, suGrad, suRad, suTurn);

const
  SVGAbsoluteUnits = [suNone .. suCM, suDeg..suTurn];
  SVGFontRelativeUnits = [suEM .. suEX];

type
  TSVGLength = record
    Value: Single;
    SVGUnit: TSVGUnit;
  end;

  TSVGLengthSW = record
    SW: TSVGSpecificWord;
    Value: Single;
    SVGUnit: TSVGUnit;
  end;

  TNumberSW = record
    SW: TSVGSpecificWord;
    Value: Single;
  end;

  TSVGLengthArray = array of TSVGLength;

  TSVGLengthArraySW = record
    Arr: TSVGLengthArray;
    SW: TSVGSpecificWord;
  end;

  TSVGStringSW = record
    SW: TSVGSpecificWord;
    S: string;
  end;

  TSVGUnits = record
    Name: string;
    Factor: Single;
  end;

  TSVGPreserveAspectRatio = record
    Align, MeetOrSlice: TSVGSpecificWord;
  end;

procedure ParseSingle(const S: string; var Value: Single);

procedure ParseAlpha(const S: string; var Value: Single); 

procedure ParsePercentage(const S: string; var Value: Single);

procedure ParsePoints(const S: string; var Value: TSinglePointDynArray);

procedure ParseString(const S: string; var Value: string); //overload;
procedure ParseStringInherit(const S: string; var Value: TSVGStringSW);

procedure ParseLength(const S: string; var Value: TSVGLength; const Units: TSVGSpecificWord = svg_userSpaceOnUse);
procedure ParseLengthSW(const S: string; SW: TSVGSpecificWord; var Value: TSVGLengthSW);
procedure ParseLengthArray(const S: string; var Value: TSVGLengthArray);
procedure ParseLengthArrayNoneInherit(const S: string; var Value: TSVGLengthArraySW);

procedure ParseNumber(const S: string; var Value: Single);
procedure ParseNumberInherit(const S: string; var Value: TNumberSW);

procedure ParseViewBox(const S: string; var Value: TSingleBounds);

procedure ParseURI(const URI: string; var Value: string);

procedure ParseTextDecorations(const S: string; var Value: TSVGSpecificWordSet);

procedure ParseFontWeight(const S: string; var Value: TNumberSW);

procedure SplitShorthandProperties(const Style: TfrxCSSStyle; const SW: TSVGSpecificWord);
function ToSVGLength(Value: Single; SVGUnit: TSVGUnit = suNone): TSVGLength; overload;
function ToSVGLength(Value: TSVGLengthSW): TSVGLength; overload;
function ToSVGLength(Value: TSVGLengthSW; AutoLength: TSVGLength): TSVGLength; overload;
function ToLengthSW(SW: TSVGSpecificWord; Value: Single; SVGUnit: TSVGUnit = suNone): TSVGLengthSW;
function ToStringInherit(S: string): TSVGStringSW;
function RelativeFontWeight(SW: TSVGSpecificWord; BaseWeight: Single): Single;

procedure ParsePreserveAspectRatio(const S: string; var Value: TSVGPreserveAspectRatio);

function CalcScale(Width, Height: Single; ViewBox: TSingleBounds;
  PreserveAspectRatio: TSVGPreserveAspectRatio): TSinglePoint;

function CalcOffset(Scale: TSinglePoint; Width, Height: Single; ViewBox: TSingleBounds;
  PreserveAspectRatio: TSVGPreserveAspectRatio): TSinglePoint;

implementation
end.
