{$IFNDEF FMX}
unit frLocalizationEditor;

interface

{$I frVer.inc}

uses
{$IFDEF DELPHI16}
  System.UITypes,
{$ENDIF}
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLIntf, LCLType, LMessages,
{$ENDIF}
  Types, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs, StdCtrls, ComCtrls, ToolWin, Menus,
{$IFDEF DELPHI17}
  System.Actions,
{$ENDIF}
{$IFDEF DELPHI22}
  System.ImageList,
{$ENDIF}
  ActnList, Grids, ExtCtrls, ImgList, Contnrs,
  frCore, frCoreClasses, frXML, frResources, frBaseForm;

type

  TfrLocalizationEditorGrid = class;
  TfrEditingResources = class;

  { TfrmLocalizationEditor }

  TfrmLocalizationEditor = class(TfrBaseForm)
    ToolBar: TToolBar;
    tbSaveAs: TToolButton;
    tbSaveToXML: TToolButton;
    ToolButton3: TToolButton;
    tbUndo: TToolButton;
    tbRedo: TToolButton;
    ToolButton6: TToolButton;
    cbLanguages: TComboBox;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    ActionList: TActionList;
    pnlGrid: TPanel;
    acLoadFromXML: TAction;
    ilActions: TImageList;
    acSaveAs: TAction;
    acSaveToXML: TAction;
    acUndo: TAction;
    acRedo: TAction;
    pnlButtons: TPanel;
    btnClose: TButton;
    tbLoadFromXML: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    edFilter: TEdit;
    ToolButton1: TToolButton;
    tbTranslated: TToolButton;
    tbNotTranslated: TToolButton;
    acTranslated: TAction;
    acNotTranslated: TAction;
    tbClearFilter: TToolButton;
    acClearFilter: TAction;
    ilSort: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure cbLanguagesChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure acLoadFromXMLExecute(Sender: TObject);
    procedure acSaveToXMLExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure acTranslatedExecute(Sender: TObject);
    procedure acNotTranslatedExecute(Sender: TObject);
    procedure edFilterEnter(Sender: TObject);
    procedure edFilterExit(Sender: TObject);
    procedure edFilterChange(Sender: TObject);
    procedure acUndoExecute(Sender: TObject);
    procedure acRedoExecute(Sender: TObject);
    procedure acSaveAsExecute(Sender: TObject);
    procedure acClearFilterExecute(Sender: TObject);
  private type
    THistory = record
      EditKey: string;
      EditValue: string;
      IsTranslated: Boolean;
      FilterValue: string;
      NextEditKey: string;
      NextEditValue: string;
      NotTranslated: Boolean;
      SelIndex: Integer;
      SortCol: Integer;
      SortDescending: Boolean;
      Translated: Boolean;
    end;
  strict private
    FGrid: TfrLocalizationEditorGrid;
    FHistory: array of THistory;
    FHistoryIndex: Integer;
    FFileName: string;
    FLockHistory: Boolean;
    FPath: string;
    FResources: TfrEditingResources;

    procedure AddHistory; overload;
    procedure AddHistory(const AKey, AValue, ANewValue: string; AIsTranslated: Boolean); overload;
    procedure ClearHistory;
    procedure ExitFromFilter;
    procedure LoadFromHistory(AIndex: Integer);
    procedure PopulateCurrentHistory(var AHistory: THistory);
    procedure SetHistoryIndex(AValue: Integer);

    procedure Redo;
    procedure Undo;

    function CheckingNeedSave: Integer;
    procedure GridChangedEditingValueHandler(Sender: TObject;
      const AKey, ANewValue, AOldValue: string; ATranslated: Boolean);
    procedure LoadFromXML;
    procedure LoadLanguages;
    procedure PopulateGrid;
    procedure PopulateValues;
    procedure SaveNewValuesFromGrid;
    function SaveToXML(AFileName: string): Boolean;
    procedure SelectLanguage(AIndex: Integer);
    procedure SortParamChangedHandler(ASender: TObject);

    property HistoryIndex: Integer read FHistoryIndex write SetHistoryIndex;
  end;

{$ENDIF FMX}

  TfrGridSortParameter = (spKey, spOriginalValue, spEditingValue);

  TfrGridDataController = class
  strict private
    FDescending: Boolean;
    FEditingValues: TStringList;
    FFilteredArray: array of Integer;
    FFilterText: string;
    FKeys: TStringList;
    FOnlyTranslated: Boolean;
    FOnlyNotTranslated: Boolean;
    FOriginalValues: TStringList;
    FSortParam: TfrGridSortParameter;
    FTranslatedArray: array of Boolean;

    procedure Filter;
    function GetCount: Integer;
    function GetEditingValues(AIndex: Integer): string;
    function GetIndexByKey(const AKey: string): Integer;
    function GetFilteredCount: Integer;
    function GetKeys(AIndex: Integer): string;
    function GetOriginalValues(AIndex: Integer): string;
    function GetVisibleIndex(AIndex: Integer): Integer;

    procedure SetEditingValues(AIndex: Integer; const AValue: string);

    procedure Sort;
    procedure SortByStringList(ASortingList: TStringList; ALists: array of TStringList);
  public
    constructor Create; virtual;
    destructor Destroy; override;

    procedure AddRecord(const AKey, AOriginalValue, AEditingValue: string; ATranslated: Boolean);
    procedure GetRecord(AIndex: Integer; out AKey, AOriginalValue, AEditingValue: string; out ATranslated: Boolean);
    function GetVisibleIndexByKey(const AKey: string): Integer;
    procedure InsertEditingValue(const AKey, AEditingValue: string; ATranslated: Boolean);
    function IsFilteredRecord(AIndex: Integer): Boolean;
    function IsTranslated(AKey: string): Boolean;
    procedure Update;

    property Count: Integer read GetCount;
    property EditingValues[AIndex: Integer]: string read GetEditingValues write SetEditingValues;
    property FilteredCount: Integer read GetFilteredCount;
    property FilterText: string read FFilterText write FFilterText;
    property Keys[AIndex: Integer]: string read GetKeys;
    property OnlyTranslated: Boolean read FOnlyTranslated write FOnlyTranslated;
    property OnlyNotTranslated: Boolean read FOnlyNotTranslated write FOnlyNotTranslated;
    property OriginalValues[AIndex: Integer]: string read GetOriginalValues;
    property Descending: Boolean read FDescending write FDescending;
    property SortParam: TfrGridSortParameter read FSortParam write FSortParam;
  end;

  TfrLocalizationEditorGridChangedEditingValueEvent = procedure(Sender: TObject;
    const AKey, ANewValue, AOldValue: string; ATranslated: Boolean) of object;

  TfrLocalizationEditorGrid = class(TStringGrid)
  const
    FResourceStringNameCol: Integer = 0;
    FResourceOriginValueCol: Integer = 1;
    FResourceEditingValueCol: Integer = 2;
  strict private
  {$IFDEF FMX}
    FColumns: array[0..2] of TStringColumn;
  {$ENDIF}
    FController: TfrGridDataController;
    FLockCount: Integer;
    FSortImageList: TImageList;
    FUpdatingData: Boolean;

    FOnSortParamsChanged: TNotifyEvent;
    FOnChangedEditingValue: TfrLocalizationEditorGridChangedEditingValueEvent;

    procedure ClickColumnHeader(ACol, AOldSortCol: Integer);
  {$IFDEF FMX}
    procedure DrawColumnHeader(const Canvas: TCanvas; const Column: TColumn; const Bounds: TRectF);
  {$ENDIF}

    function GetRecordCount: Integer;
    function GetFilterText: string;
    function GetIndexByRow(ARow: Integer): Integer;
    function GetKeys(AIndex: Integer): string;
    function GetOnlyTranslated: Boolean;
    function GetOnlyNotTranslated: Boolean;
    function GetOriginValues(AIndex: Integer): string;
    function GetRowByIndex(AIndex: Integer): Integer;
    function GetSelectedIndex: Integer;
    function GetSortColumnIndex: Integer;
    function GetSortDescending: Boolean;
    function GetEditingValues(AIndex: Integer): string;
    function GetVisibleRecordCount: Integer;
    procedure Initialize;
    procedure SetFilterText(const AValue: string);
    procedure SetOnlyTranslated(AValue: Boolean);
    procedure SetOnlyNotTranslated(AValue: Boolean);
    procedure SetSelectedIndex(AValue: Integer);
    procedure SetSortColumnIndex(AValue: Integer);
    procedure SetSortDescending(AValue: Boolean);
    procedure SetEditingValues(AIndex: Integer; AValue: string);
  {$IFDEF FMX}
    procedure ApplyStyleLookupHandler(Sender: TObject);
    procedure DrawHeaderHandler(Sender: TObject; const Canvas: TCanvas; const Column: TColumn; const Bounds: TRectF);
    procedure EditingDoneHandler(Sender: TObject; const ACol, ARow: Integer);
    procedure HeaderClickHandler(Column: TColumn);
    procedure SelectCellHandler(Sender: TObject; const ACol, ARow: Integer; var CanSelect: Boolean);
  {$ENDIF}
  protected
    procedure BeginDataUpdate;
    procedure EndDataUpdate;
    procedure DoChangedEditingValue(const AKey, ANewValue, AOldValue: string; ATranslated: Boolean);
    procedure DoSortParamsChanged;
    procedure DrawSortIcon({$IFDEF FMX}const ACaptionRect: TRectF{$ELSE}var ACaptionRect: TRect{$ENDIF});
    procedure SetEditingValue(AIndex: Longint; const AValue: string);
  {$IFDEF FMX}
    procedure DoSelectCell(Col, Row: Integer; var CanSelect: Boolean);
  {$ELSE}
    procedure DrawCell(ACol: Integer; ARow: Integer; ARect: TRect; AState: TGridDrawState); override;
    procedure FixedCellClick(ACol, ARow: Longint); {$IFNDEF FPC} override; {$ENDIF}
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    function SelectCell(ACol, ARow: Longint): Boolean; override;
    procedure SetEditText(ACol, ARow: Longint; const Value: string); override;
  {$ENDIF}
    procedure UpdateData;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure AddRecord(const AKey, AOriginalValue, AEditingValue: string; ATranslated: Boolean);
    procedure GetRecord(AIndex: Integer; out AKey, AOriginalValue, AEditingValue: string; out ATranslated: Boolean);
    procedure InsertEditingValue(const AKey, AEditingValue: string; ATranslated: Boolean);
    function IsTranslated(const AKey: string): Boolean;

    property RecordCount: Integer read GetRecordCount;
    property FilterText: string read GetFilterText write SetFilterText;
    property Keys[AIndex: Integer]: string read GetKeys;
    property OnlyTranslated: Boolean read GetOnlyTranslated write SetOnlyTranslated;
    property OnlyNotTranslated: Boolean read GetOnlyNotTranslated write SetOnlyNotTranslated;
    property OriginValues[AIndex: Integer]: string read GetOriginValues;
    property SelectedIndex: Integer read GetSelectedIndex write SetSelectedIndex;
    property SortColumnIndex: Integer read GetSortColumnIndex write SetSortColumnIndex;
    property SortImageList: TImageList read FSortImageList write FSortImageList;
    property SortDescending: Boolean read GetSortDescending write SetSortDescending;
    property EditingValues[AIndex: Integer]: string read GetEditingValues write SetEditingValues;
    property VisibleRecordCount: Integer read GetVisibleRecordCount;

    property OnSortParamsChanged: TNotifyEvent read FOnSortParamsChanged write FOnSortParamsChanged;
    property OnChangedEditingValue: TfrLocalizationEditorGridChangedEditingValueEvent
      read FOnChangedEditingValue write FOnChangedEditingValue;
  end;

//ToDo: Compared
  TfrEditingResources = class
  const
    FXMLRootItemName = 'Resources';
    FXMLCodePagePropName = 'CodePage';
    FXMLLanguagePropName = 'Language';
    FXMStrResItemName = 'StrRes';
    FXMStrResKeyPropName = 'Name';
    FXMStrResTextPropName = 'Text';
  strict private
    FCodePage: Integer;
    FEditingLanguage: string;
    FEditingValues: TfrStringDictionary;
    FLanguages: TStringList;
    FOriginalValues: TfrStringDictionary;

    procedure AddOrSetEditingValue(const AName: string; const AValue: string);
    procedure ClearEditingValues;
    function CreateXMLDocument: TfrXMLDocument;
    function GetEditingValues(const AKey: string): string;
    function GetLanguageCount: Integer;
    function GetLanguages: TStringList;
    function GetValues: TfrStringDictionary;
    procedure PopulateValues(const ALanguage: string; var AValues: TfrStringDictionary);
    procedure SetEditingLanguage(const AValue: string);
    procedure SetEditingValues(const AKey, AValue: string);
  public
    constructor Create;
    destructor Destroy; override;

    function IsTranslated(const AKey: string): Boolean;
    procedure LoadFromStream(AStream: TStream);
    procedure LoadFromXML(const AFileName: string);
    procedure SaveToXML(const AFileName: string);

    property EditingLanguage: string read FEditingLanguage write SetEditingLanguage;
    property EditingValues[const AKey: string]: string read GetEditingValues write SetEditingValues;
    property LanguageCount: Integer read GetLanguageCount;
    property Languages: TStringList read GetLanguages;
    property OriginalValues: TfrStringDictionary read FOriginalValues;
    property Values: TfrStringDictionary read GetValues;
  end;

procedure frShowLocalizationEditor;

implementation
end.
