{******************************************}
{                                          }
{             FastReport VCL               }
{           Smart Memo control             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frSmartMemo;

interface

{$I frVer.inc}

uses
  {$IFNDEF FPC}
  Windows, Imm,
  {$ELSE}
  LCLType, LMessages, LCLIntf, LCLProc,
  {$ENDIF}
  Messages, Types, SysUtils, Classes, Graphics, Controls, StdCtrls,ExtCtrls,
  Forms, Menus, IniFiles, frPopupForm, frTypes, frBaseGraphicsTypes,
  frVectorCanvas, frUnicodeUtils, frGraphicUtils, frSynMemo, frBaseMemo
  {$IFDEF Delphi10}
  , WideStrings
  {$ENDIF}
  ;

type
  TfrSmartSyntaxMemo = class(TfrBaseSmartSyntaxMemo)
  protected
    FInterpretationType: TfrxInterpretationsTypes;
    FDrawText: TfrxDrawText;
    FCaretHeight: Integer;
    FPrevPos: TPoint;

    //Caret var
    FFirstPaint: Boolean;

    function GetWideText: TfrWideStrings;
    procedure UpdateDrawParamsBase(PrePaint: Boolean = False; VC: TVectorCanvas = nil);
    procedure UpdateDrawParams(PrePaint: Boolean = False; VC: TVectorCanvas = nil); overload;
    function UpdateDrawParams(mode: TfrxTextAddMode; p: TPoint; PrePaint: Boolean = False; VC: TVectorCanvas = nil): TPoint; overload;
    procedure SetWordWrap(val: Boolean); override;
    function LineAt(Index: Integer; UseTrim: Boolean = True): String; override;
    function GetSelText: String; override;
    procedure SetSelText(const Value: String); override;
    procedure FilterCodeStringList(aFilterStr: String; aLBList: TStrings;
      aCodeList: TStrings; aDiffFunc: TDiffFunc); override;
    procedure DoCodeCompletion; override;
    procedure CompletionLBDrawItem(Control: TWinControl; Index: Integer;
      ARect: TRect; State: TOwnerDrawState); override;
    procedure CompletionLBDblClick(Sender: TObject); override;
    function GetIdentEnd(aPos: Integer): Integer; override;

    //Caret func
    function CurPosToSynPos(X, Y: Integer): TPoint; override;
    function SynPosToCurPos(X, Y: Integer): TPoint;
    procedure DoReturn; override;
    procedure DoDown; override;
    procedure DoRightInText;
    procedure DoLeftInText;
    procedure DoRight; override;
    procedure DoLeft; override;
    procedure DoUp; override;
    procedure Resize; override;
    procedure UpdateScrollBar; override;
    procedure OnHScrollChange(Sender: TObject); override;
    procedure OnVScrollChange(Sender: TObject); override;
    // supports cater func
    function OriginalPosToDrawPos(p: TPoint): TPoint;
    function DrawPosToCanvasPos(p: TPoint): TPoint;
    function DrawPosToOriginalPos(p: TPoint): TPoint;
    function CanvasPosToDrawPos(p: TPoint): TPoint;
    //
    function IsCursorInStringBlock(var AInDrawPos: TPoint): Boolean;
    procedure ShowCaretPos; override;
    procedure EventSetFocus(var Msg: TWMSetFocus); override;
    function CaclChangeWheel(val: Integer): Integer;
    procedure CheckSurrogatePairsInDraw(var x: Integer; var y: Integer; ToRight: Boolean = True);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean); override;
    procedure MouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean); override;
    procedure CorrectDrawPos(var PointTo: TPoint; x, y: Integer; ToRight: Boolean = True);
    procedure CorrectOriginalPos(var PointTo: TPoint; x, y: Integer; ToRight: Boolean = True);
    procedure SetPos(x, y: Integer; ToRight: Boolean); override;
    procedure SetPosByCoords(X, Y: Integer); override;

    property DrawText: TfrxDrawText read FDrawText;
  published
    property SynDialectStyles;
    property InterpretationType: TfrxInterpretationsTypes read FInterpretationType write FInterpretationType;
    property HightlightingType;
    property WordWrap;
    property UseDefaultCharset;
    property RTLReading;
    property ExpressionDelimiters;
    property Color;
    property Font;
    property GutterWidth;
    property Lines;
    property ShowGutter;
    property CommentAttr;
    property KeywordAttr;
    property NumberAttr;
    property StringAttr;
    property TextAttr;
    property OnChangePos;
    property OnChangeText;
    property OnDragDrop;
    property OnDragOver;
    property OnKeyDown;
  end;

implementation
end.
